@@ -6,6 +6,8 @@ require 5.000;
 
 use strict;
 use English;
+use Fcntl;
+use File::Basename;
 
 my ($TRUE, $FALSE) = (1,0);
 
@@ -18,6 +20,14 @@ my $cpCommand;
 #
 #*****************************************************************************
 
+sub autoFlushStdout() {
+    my $oldFh = select(STDOUT); 
+    $OUTPUT_AUTOFLUSH = $TRUE;
+    select($oldFh);
+}
+
+
+
 sub prompt($$) {
 
     my ($prompt, $default) = @_;
@@ -191,9 +201,9 @@ sub getBinDir($) {
 
 
 
-sub installProgram($$) {
+sub installProgram($$$) {
 
-    my ($pkgdir, $prefix) = @_;
+    my ($pkgdir, $prefix, $bindirR) = @_;
 
     my $binDir = getBinDir($prefix);
 
@@ -207,6 +217,7 @@ sub installProgram($$) {
     } else {
         print("Done.\n");
     }
+    $$bindirR = $binDir;
 }
 
 
@@ -300,12 +311,19 @@ execLdconfig() {
 
 
 
+sub ldconfigExists() {
+
+    return (system("ldconfig -? 2>/dev/null") >> 8) != 127;
+}
+
+
+
 sub
 doLdconfig() {
 #-----------------------------------------------------------------------------
 #  Run Ldconfig where appropriate.
 #-----------------------------------------------------------------------------
-    if ($OSNAME eq "linux" || system("ldconfig -? 2>/dev/null") != 127) {
+    if ($OSNAME eq "linux" || ldconfigExists()) {
         # This is a system where Ldconfig makes sense
 
         print("In order for the Netpbm shared library to be found when " .
@@ -342,9 +360,9 @@ doLdconfig() {
 
 
 
-sub installSharedLib($$) {
+sub installSharedLib($$$) {
 
-    my ($pkgdir, $prefix) = @_;
+    my ($pkgdir, $prefix, $libdirR) = @_;
 
     if (-d("$pkgdir/lib")) {
         my $libDir = getLibDir($prefix);
@@ -361,6 +379,7 @@ sub installSharedLib($$) {
             print("\n");
             doLdconfig();
         }
+        $$libdirR = $libDir;
     } else {
         print("You did not build a shared library, so I will not " .
               "install one.\n");
@@ -404,9 +423,9 @@ sub getLinkDir($) {
 
 
 
-sub installStaticLib($$) {
+sub installStaticLib($$$) {
 
-    my ($pkgdir, $prefix) = @_;
+    my ($pkgdir, $prefix, $linkdirR) = @_;
 
     if (-d("$pkgdir/link")) {
         my $linkDir = getLinkDir($prefix);
@@ -421,6 +440,7 @@ sub installStaticLib($$) {
         } else {
             print("done.\n");
         }
+        $$linkdirR = $linkDir;
     } else {
         print("You did not build a static library, so I will not " .
               "install one \n");
@@ -497,9 +517,9 @@ sub getHdrDir($) {
 
 
 
-sub installDataFile($$) {
+sub installDataFile($$$) {
 
-    my ($pkgdir, $prefix) = @_;
+    my ($pkgdir, $prefix, $datadirR) = @_;
 
     my $dataDir = getDataDir($prefix);
 
@@ -512,15 +532,16 @@ sub installDataFile($$) {
               "failed.\n");
         print("cp exit code is $rc\n");
     } else {
+        $$datadirR = $dataDir;
         print("done.\n");
     }
 }
 
 
 
-sub installHeader($$) {
+sub installHeader($$$) {
 
-    my ($pkgdir, $prefix) = @_;
+    my ($pkgdir, $prefix, $includedirR) = @_;
 
     my $hdrDir = getHdrDir($prefix);
 
@@ -535,6 +556,7 @@ sub installHeader($$) {
     } else {
         print("done.\n");
     }
+    $$includedirR = $hdrDir;
 }
 
 
@@ -727,12 +749,12 @@ sub makeInManwebPath($) {
 
 
 
-sub installManPage($$) {
+sub installManPage($$$) {
 
 
 # Note: This installs the pointer man pages and the netpbm.url file for Manweb.
 
-    my ($pkgdir, $prefix) = @_;
+    my ($pkgdir, $prefix, $mandirR) = @_;
 
     my $manDir = getManDir($prefix);
 
@@ -752,11 +774,109 @@ sub installManPage($$) {
     removeObsoleteManPage($manDir);
 
     makeInManwebPath("$manDir/web");
-
+    
+    $$mandirR = $manDir;
 }
 
 
 
+sub 
+processTemplate($$$$$$$$$) {
+    my ($templateR, $version, $bindir, $libdir, $linkdir, $datadir, 
+        $includedir, $mandir, $outputR) = @_;
+
+    my @output;
+
+    foreach (@{$templateR}) {
+        if (m{^@}) {
+            # Comment -- ignore it.
+        } else {
+            if (defined($version)) {
+                s/\@VERSION\@/$version/;
+            }
+            if (defined($bindir)) {
+                s/\@BINDIR@/$bindir/;
+            }
+            if (defined($libdir)) {
+                s/\@LIBDIR@/$libdir/;
+            }
+            if (defined($linkdir)) {
+                s/\@LINKDIR@/$linkdir/;
+            }
+            if (defined($datadir)) {
+                s/\@DATADIR@/$datadir/;
+            }
+            if (defined($includedir)) {
+                s/\@INCLUDEDIR@/$includedir/;
+            }
+            if (defined($mandir)) {
+                s/\@MANDIR@/$mandir/;
+            }
+            push(@output, $_);
+        }
+    }
+    $$outputR = \@output;
+}
+
+
+
+sub
+installConfig($$$$$$$) {
+    my ($pkgdir, 
+        $bindir, $libdir, $linkdir, $datadir, $includedir, $mandir) = @_;
+
+    my $error;
+
+    my $configTemplateFilename = dirname($0) . "/config_template";
+
+    my $templateOpened = open(TEMPLATE, "<$configTemplateFilename");
+    if (!$templateOpened) {
+        $error = "Can't open template file '$configTemplateFilename'.\n";
+    } else {
+        my @template = <TEMPLATE>;
+
+        close(TEMPLATE);
+
+        my $versionOpened = open(VERSION, "<$pkgdir/VERSION");
+
+        my $version;
+        if (!$versionOpened) {
+            $error = "Unable to open $pkgdir/VERSION for reading.  " .
+                "Errno=$ERRNO\n";
+        } else {
+            $version = <VERSION>;
+            chomp($version);
+            close(VERSION);
+
+            processTemplate(\@template, $version, $bindir, $libdir,
+                            $linkdir, $datadir, $includedir, $mandir,
+                            \my $fileContentsR);
+
+            # TODO: Really, this ought to go in an independent directory,
+            # because you might want to have the Netpbm executables in
+            # some place not in the PATH and use this program, via the
+            # PATH, to find them.
+            
+            my $filename = "$bindir/netpbm-config";
+            
+            my $success = open(NETPBM_CONFIG, ">$filename");
+            if ($success) {
+                chmod(0755, $filename);
+                foreach (@{$fileContentsR}) { print NETPBM_CONFIG; }
+                close(NETPBM_CONFIG);
+            } else {
+                $error = "Unable to open the file " .
+                    "'$filename' for writing.  Errno=$ERRNO\n";
+            }
+        }
+    }
+    if ($error) {
+        print(STDERR "Failed to create the Netpbm configuration program.  " .
+              "$error\n");
+    }
+}
+
+
 
 #******************************************************************************
 #
@@ -764,7 +884,7 @@ sub installManPage($$) {
 #
 #*****************************************************************************
 
-
+autoFlushStdout();
 
 print("Welcome to the Netpbm install dialogue.  We will now proceed \n");
 print("to interactively install Netpbm on this system.\n");
@@ -779,24 +899,27 @@ my $prefix = getPrefix();
 
 $cpCommand = getCpCommand();
 
-installProgram($pkgdir, $prefix);
+installProgram($pkgdir, $prefix, \my $bindir);
 print("\n");
 
-installSharedLib($pkgdir, $prefix);
+installSharedLib($pkgdir, $prefix, \my $libdir);
 print("\n");
 
-installStaticLib($pkgdir, $prefix);
+installStaticLib($pkgdir, $prefix, \my $linkdir);
 print("\n");
 
-installDataFile($pkgdir, $prefix);
+installDataFile($pkgdir, $prefix, \my $datadir);
 print("\n");
 
-installHeader($pkgdir, $prefix);
+installHeader($pkgdir, $prefix, \my $includedir);
 print("\n");
 
-installManPage($pkgdir, $prefix);
+installManPage($pkgdir, $prefix, \my $mandir);
 print("\n");
 
+installConfig($pkgdir, 
+              $bindir, $libdir, $linkdir, $datadir, $includedir, $mandir);
+
 print("Installation is complete (except where previous error messages have\n");
 print("indicated otherwise).\n");
 
