@@ -2,7 +2,7 @@
                                  libopt
 ------------------------------------------------------------------------------
   This is a program to convert link library filepaths to linker options
-  that select them.  E.g. ../pbm/libpbm.so becomes -L../pbm -lpbm   .
+  that select them.  E.g. ../lib/libnetpbm.so becomes -L../lib -lnetpbm   .
 
   Each argument is a library filepath.  The option string to identify
   all of those library filepaths goes to Standard Output.
@@ -23,8 +23,8 @@
   If you specify the option "-runtime", the output includes a -R
   option in addition, for every library after "-runtime" in the
   arguments.  -R tells the buildtime linker to include the specified
-  library in the search path that the runtime linker uses to find
-  dynamically linked libraries.
+  library's directory in the search path that the runtime linker uses
+  to find dynamically linked libraries.
 
   But if you compile this program with the EXPLICIT macro defined, and
   a library filepath contains a slash, then it skips all that -L/-l
@@ -36,12 +36,12 @@
   On some systems, the -L/-l output of this program has exactly the
   same effect as the filepath input when used in the arguments to a
   link command.  A GNU/Linux system, for example.  On others (Solaris,
-  for example), if you include /tmp/lib/libpbm.so in the link as a
+  for example), if you include /tmp/lib/libnetpbm.so in the link as a
   link object, the executable gets built in such a way that the system
-  accesses the shared library /tmp/lib/libpbm.so at run time.  On the
-  other hand, if you instead put the options -L/tmp/lib -lpbm on the
+  accesses the shared library /tmp/lib/libnetpbm.so at run time.  On the
+  other hand, if you instead put the options -L/tmp/lib -lnetpbm on the
   link command, the executable gets built so that the system accesses
-  libpbm.so in its actual installed directory at runtime (that
+  libnetpbm.so in its actual installed directory at runtime (that
   location might be determined by a --rpath linker option or a
   LD_LIBRARY_PATH environment variable at run time).
 
@@ -50,13 +50,13 @@
   the rule's command uses to identify its input.  Here is an example
   of using libopt for that:
 
-     PBMLIB=../pbm/libpbm.so
+     NETPBMLIB=../lib/libnetpbm.so
      ...
      pbmmake: pbmmake.o $(PBMLIB)
              ld -o pbmmake pbmmake.o `libopt $(PBMLIB)` --rpath=/lib/netpbm
 
-  Caveat: "-L../pbm -lpbm" is NOT exactly the same as
-  "../pbm/libpbm.so" on any system.  All of the -l libraries are
+  Caveat: "-L../lib -lnetpbm" is NOT exactly the same as
+  "../pbm/libnetpbm.so" on any system.  All of the -l libraries are
   searched for in all of the -L directories.  So you just might get a
   different library with the -L/-l version than if you specify the
   library file explicitly.
@@ -65,8 +65,9 @@
   handle explicit file names.
 
 -----------------------------------------------------------------------------*/
-#define _BSD_SOURCE 1
-    /* Make sure strdup() is in stdio.h */
+#define _BSD_SOURCE 1      /* Make sure strdup() is in stdio.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
+
 #define MAX_PREFIXES 10
 
 /* Here's how to use SHLIBPREFIXLIST:  Use a -D compile option to pass in
@@ -74,7 +75,7 @@
 
    It's a blank-delimited list of prefixes that library names might
    have.  "lib" is traditionally the only prefix, as in libc or
-   libpbm.  However, on Windows there is a convention of using
+   libnetpbm.  However, on Windows there is a convention of using
    different prefixes to distinguish different co-existent versions of
    the same library (kind of like a major number in some unices). 
    E.g. the value "cyg lib" is appropriate for a Cygwin system.
@@ -350,6 +351,8 @@ parse_filename(const char *  const filename,
     }
 }   
 
+
+
 static void
 parse_filepath(const char *  const filepath,
                const char ** const directory_p, 
@@ -452,11 +455,9 @@ processOneLibrary(const char *  const filepath,
    linker option string as a newly malloced null-terminated string at
    *optionsP.
 -----------------------------------------------------------------------------*/
-    unsigned int const MAX_OPT_SIZE = 1024;
-
-    const char *directory;
+    const char * directory;
         /* Directory component of 'filepath' */
-    const char *filename;
+    const char * filename;
         /* Filename component of 'filepath' */
 
     parse_filepath(filepath, &directory, &filename, errorP);
@@ -476,11 +477,13 @@ processOneLibrary(const char *  const filepath,
                        &libname, &valid_library, &staticlib, errorP);
         
         if (!*errorP) {
-            if (valid_library) 
+            if (valid_library) {
                 doOptions(filepath, directory, libname, 
                           runtime, explicit, staticlib, optionsP);
-            
-            strfree(libname);
+
+                strfree(libname);
+            } else
+                *optionsP = strdup(filepath);
         }
         strfree(directory); 
         strfree(filename);
@@ -494,23 +497,25 @@ main(int argc, char **argv) {
 
     bool error;
     bool runtime;  /* -runtime option has been seen */
+    bool quiet;    /* -quiet option has been seen */
     int retval;
     unsigned int arg;  /* Index into argv[] of argument we're processing */
     char outputLine[1024];
 
     strcpy(outputLine, "");  /* initial value */
     runtime = FALSE;  /* initial value */
+    quiet = FALSE;   /* initial value */
     error = FALSE;  /* no error yet */
     for (arg = 1; arg < argc && !error; arg++) {
         if (strcmp(argv[arg], "-runtime") == 0)
             runtime = TRUE;
+        else if (strcmp(argv[arg], "-quiet") == 0)
+            quiet = TRUE;
         else {
-            if (strlen(argv[arg]) > 200)
-                error = TRUE;
-            else {
-                const char * options;
-                processOneLibrary(argv[arg], runtime, explicit, 
-                                  &options, &error);
+            const char * options;
+            processOneLibrary(argv[arg], runtime, explicit, 
+                              &options, &error);
+            if (!error) {
                 if (strlen(outputLine) + strlen(options) + 1 + 1 > 
                     sizeof(outputLine))
                     error = TRUE;
@@ -528,7 +533,6 @@ main(int argc, char **argv) {
         retval = 10;
     } else {
         fputs(outputLine, stdout);
-        fprintf(stderr, "libopt results: '%s'\n", outputLine);
         retval = 0;
     }
     return retval;
