@@ -18,10 +18,15 @@
    describe the Lempel-Ziv base.
 */
 
+#define _BSD_SOURCE   /* Make sure strcasecmp() is in string.h */
+
 #include <string.h>
+#include <assert.h>
+
 #include "pnm.h"
 #include "shhopt.h"
 #include "mallocvar.h"
+#include "nstring.h"
 
 #define GIFMAXVAL 255
 #define MAXCOLORMAPSIZE 256
@@ -36,17 +41,28 @@
 #define LOCALCOLORMAP  0x80
 #define BitSet(byte, bit)      (((byte) & (bit)) == (bit))
 
-#define        ReadOK(file,buffer,len) (fread(buffer, len, 1, file) != 0)
+static __inline__ bool
+ReadOK(FILE *          const fileP,
+       unsigned char * const buffer,
+       size_t          const len) {
+
+    size_t bytesRead;
+
+    bytesRead = fread(buffer, len, 1, fileP);
+
+    return (bytesRead != 0);
+}
+
 
 #define LM_to_uint(a,b)                        (((b)<<8)|(a))
 
 static int const maxnum_lzwCode = (1<<MAX_LZW_BITS);
 
-struct cmdline_info {
+struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
-    const char *input_filespec;  /* Filespecs of input files */
+    const char * input_filespec;  /* Filespecs of input files */
     unsigned int verbose;    /* -verbose option */
     unsigned int comments;   /* -comments option */
     bool all_images;  /* He wants all the images */
@@ -54,34 +70,41 @@ struct cmdline_info {
         /* image number he wants from input, starting at 0.  Undefined
            if all_images is TRUE
         */
-    const char *alpha_filename;
+    const char * alpha_filename;
+    unsigned int quitearly;
 };
 
 
+
 static void
-parse_command_line(int argc, char ** argv,
-                   struct cmdline_info *cmdlineP) {
+parseCommandLine(int argc, char ** argv,
+                 struct cmdlineInfo * const cmdlineP) {
 /*----------------------------------------------------------------------------
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
-    optEntry *option_def = malloc(100*sizeof(optEntry));
+    optEntry * option_def;
         /* Instructions to optParseOptions3 on how to parse our options.
          */
-    optStruct3 opt;
     
+    optStruct3 opt;
+
     unsigned int alphaSpec, imageSpec;
     const char * image;
 
     unsigned int option_def_index;
 
+    MALLOCARRAY_NOFAIL(option_def, 100);
+    
     option_def_index = 0;   /* incremented by OPTENTRY */
     OPTENT3(0, "verbose",     OPT_FLAG, NULL, 
             &cmdlineP->verbose,         0);
     OPTENT3(0, "comments",    OPT_FLAG, NULL,
             &cmdlineP->comments,        0);
+    OPTENT3(0, "quitearly",    OPT_FLAG, NULL,
+            &cmdlineP->quitearly,       0);
     OPTENT3(0, "image",       OPT_STRING, &image,
-            &imageSpec,                  0);
+            &imageSpec,                 0);
     OPTENT3(0, "alphaout",    OPT_STRING, &cmdlineP->alpha_filename, 
             &alphaSpec,                 0);
 
@@ -138,6 +161,7 @@ struct gifScreen {
     unsigned int    Height;
     gifColorMap     ColorMap;
     unsigned int    ColorMapSize;
+        /* Number of colors in the color map. */
     unsigned int    ColorResolution;
     unsigned int    Background;
     unsigned int    AspectRatio;
@@ -183,6 +207,8 @@ readColorMap(FILE *ifP, const int colormapsize,
     int             i;
     unsigned char   rgb[3];
 
+    assert(colormapsize <= MAXCOLORMAPSIZE);
+
     *hasGrayP = FALSE;  /* initial assumption */
     *hasColorP = FALSE;  /* initial assumption */
 
@@ -263,6 +289,13 @@ getDataBlock(FILE *          const ifP,
 
 static void
 readThroughEod(FILE * const ifP) {
+/*----------------------------------------------------------------------------
+  Read the file 'ifP' through the next EOD marker.  An EOD marker is a
+  a zero length data block.
+
+  If there is no EOD marker between the present file position and EOF,
+  we read to EOF and issue warning message about a missing EOD marker.
+-----------------------------------------------------------------------------*/
     unsigned char buf[260];
     bool eod;
 
@@ -696,9 +729,17 @@ lzwInit(struct decompressor * const decompP,
     decompP->ifP = ifP;
     decompP->init_codeSize = init_codeSize;
 
+    assert(decompP->init_codeSize < sizeof(decompP->max_dataVal) * 8);
+
     decompP->max_dataVal = (1 << init_codeSize) - 1;
     decompP->clear_code = decompP->max_dataVal + 1;
     decompP->end_code = decompP->max_dataVal + 2;
+
+    if (verbose)
+        pm_message("Initial code size is %u bits; clear code = 0x%x, "
+                   "end code = 0x%x",
+                   decompP->init_codeSize, 
+                   decompP->clear_code, decompP->end_code);
     
     /* The entries in the translation table for true data codes are
        constant throughout the stream.  We set them now and they never
@@ -717,7 +758,7 @@ lzwInit(struct decompressor * const decompP,
     
     decompP->fresh = TRUE;
     
-    initStack(&decompP->stack, maxnum_lzwCode*2);
+    initStack(&decompP->stack, maxnum_lzwCode * 2);
 }
 
 
@@ -813,19 +854,21 @@ lzwReadByte(struct decompressor * const decompP) {
   Return the next data element of the decompressed image.  In the context
   of a GIF, a data element is the color table index of one pixel.
 
-  We read and return the next byte of the decompressed image:
+  We read and return the next byte of the decompressed image, or:
 
     Return -1 if we hit EOF prematurely (i.e. before an "end" code.  We
     forgive the case that the "end" code is followed by EOF instead of
     an EOD marker (zero length DataBlock)).
 
-    Return -2 if there are no more bytes in the image.
+    Return -2 if there are no more bytes in the image.  In that case,
+    make sure the file is positioned immediately after the image (i.e.
+    after the EOD marker that marks the end of the image or EOF).
 
     Return -3 if we encounter errors in the LZW stream.
 -----------------------------------------------------------------------------*/
     int retval;
 
-    if (!stackIsEmpty(&decompP->stack)) 
+    if (!stackIsEmpty(&decompP->stack))
         retval = popStack(&decompP->stack);
     else if (decompP->fresh) {
         decompP->fresh = FALSE;
@@ -838,27 +881,35 @@ lzwReadByte(struct decompressor * const decompP) {
                 getCode(decompP->ifP, decompP->codeSize, FALSE);
             decompP->prevcode = decompP->firstcode;
         } while (decompP->firstcode == decompP->clear_code);
-        retval = decompP->firstcode;
+        if (decompP->firstcode == decompP->end_code) {
+            if (!zeroDataBlock)
+                readThroughEod(decompP->ifP);
+            retval = -2;
+        } else
+            retval = decompP->firstcode;
     } else {
         int code;
         code = getCode(decompP->ifP, decompP->codeSize, FALSE);
-        if (code < 0)
-            retval = code;
+        if (code == -1)
+            retval = -1;
         else {
+            assert(code >= 0);  /* -1 is only possible error return */
             if (code == decompP->clear_code) {
                 resetDecompressor(decompP);
-                retval = lzwReadByte( decompP );
-            } else if (code == decompP->end_code) {
-                if (!zeroDataBlock)
-                    readThroughEod(decompP->ifP);
-                retval = -2;
+                retval = lzwReadByte(decompP);
             } else {
-                bool error;
-                expandCodeOntoStack(decompP, code, &error);
-                if (error)
-                    retval = -3;
-                else
-                    retval = popStack(&decompP->stack);
+                if (code == decompP->end_code) {
+                    if (!zeroDataBlock)
+                        readThroughEod(decompP->ifP);
+                    retval = -2;
+                } else {
+                    bool error;
+                    expandCodeOntoStack(decompP, code, &error);
+                    if (error)
+                        retval = -3;
+                    else
+                        retval = popStack(&decompP->stack);
+                }
             }
         }
     }
@@ -867,87 +918,169 @@ lzwReadByte(struct decompressor * const decompP) {
 
 
 
+enum pass {MULT8PLUS0, MULT8PLUS4, MULT4PLUS2, MULT2PLUS1};
 
+static void
+bumpRowInterlace(unsigned int * const rowP,
+                 unsigned int   const rows,
+                 enum pass *    const passP) {
+/*----------------------------------------------------------------------------
+   Move *pixelCursorP to the next row in the interlace pattern.
+-----------------------------------------------------------------------------*/
+    /* There are 4 passes:
+       MULT8PLUS0: Rows 8, 16, 24, 32, etc.
+       MULT8PLUS4: Rows 4, 12, 20, 28, etc.
+       MULT4PLUS2: Rows 2, 6, 10, 14, etc.
+       MULT2PLUS1: Rows 1, 3, 5, 7, 9, etc.
+    */
+    
+    switch (*passP) {
+    case MULT8PLUS0:
+        *rowP += 8;
+        break;
+    case MULT8PLUS4:
+        *rowP += 8;
+        break;
+    case MULT4PLUS2:
+        *rowP += 4;
+        break;
+    case MULT2PLUS1:
+        *rowP += 2;
+        break;
+    }
+    /* Set the proper pass for the next read.  Note that if there are
+       more than 4 rows, the sequence of passes is sequential, but
+       when there are fewer than 4, we may skip e.g. from MULT8PLUS0
+       to MULT4PLUS2.
+    */
+    while (*rowP >= rows && *passP != MULT2PLUS1) {
+        switch (*passP) {
+        case MULT8PLUS0:
+            *passP = MULT8PLUS4;
+            *rowP = 4;
+            break;
+        case MULT8PLUS4:
+            *passP = MULT4PLUS2;
+            *rowP = 2;
+            break;
+        case MULT4PLUS2:
+            *passP = MULT2PLUS1;
+            *rowP = 1;
+            break;
+        case MULT2PLUS1:
+            /* We've read the entire image */
+            break;
+        }
+    }
+}
+
+
+
+struct pnmBuffer {
+    xel ** xels;
+    unsigned int col;
+    unsigned int row;
+};
+
+static void
+addPixelToRaster(unsigned int       const cmapIndex,
+                 struct pnmBuffer * const pnmBufferP,
+                 unsigned int       const cols,
+                 unsigned int       const rows,
+                 gifColorMap              cmap, 
+                 unsigned int       const cmapSize,
+                 bool               const interlace,
+                 int                const transparentIndex,
+                 bit **             const alphabits,
+                 enum pass *        const passP) {
+
+    if (cmapIndex >= cmapSize)
+        pm_error("Invalid color index %u in an image that has only "
+                 "%u colors in the color map.", cmapIndex, cmapSize);
+    
+    assert(cmapIndex < MAXCOLORMAPSIZE);
+    
+    PPM_ASSIGN(pnmBufferP->xels[pnmBufferP->row][pnmBufferP->col], 
+               cmap[CM_RED][cmapIndex],
+               cmap[CM_GRN][cmapIndex],
+               cmap[CM_BLU][cmapIndex]);
+    
+    if (alphabits) 
+        alphabits[pnmBufferP->row][pnmBufferP->col] =
+            (cmapIndex == transparentIndex) ? PBM_BLACK : PBM_WHITE;
+    
+    ++pnmBufferP->col;
+    if (pnmBufferP->col == cols) {
+        pnmBufferP->col = 0;
+        if (interlace)
+            bumpRowInterlace(&pnmBufferP->row, rows, passP);
+        else
+            ++pnmBufferP->row;
+    }
+}
 
 
 
 static void
-readImageData(FILE * const ifP, 
-              xel ** const xels, 
-              int    const len, 
-              int    const height, 
-              gifColorMap  cmap, 
-              bool   const interlace) {
+readImageData(FILE *       const ifP, 
+              xel **       const xels, 
+              unsigned int const cols,
+              unsigned int const rows,
+              gifColorMap        cmap, 
+              unsigned int const cmapSize,
+              bool         const interlace,
+              int          const transparentIndex,
+              bit **       const alphabits) {
 
     unsigned char lzwMinCodeSize;      
-    int v;
-    int xpos, ypos, pass;
+    enum pass pass;
     struct decompressor decomp;
+    struct pnmBuffer pnmBuffer;
+    bool gotMinCodeSize;
 
-    pass = 0;
-    xpos = 0;
-    ypos = 0;
+    pass = MULT8PLUS0;
 
-    
-    /*
-    **  Initialize the Compression routines
-    */
-    if (! ReadOK(ifP,&lzwMinCodeSize,1))
+    pnmBuffer.xels = xels;
+    pnmBuffer.col  = 0;
+    pnmBuffer.row  = 0;
+
+    gotMinCodeSize =  ReadOK(ifP, &lzwMinCodeSize, 1);
+    if (!gotMinCodeSize)
         pm_error("GIF stream ends (or read error) "
                  "right after an image separator; no "
                  "image data follows.");
 
+    if (lzwMinCodeSize > MAX_LZW_BITS)
+        pm_error("Invalid minimum code size value in image data: %u.  "
+                 "Maximum allowable code size in GIF is %u", 
+                 lzwMinCodeSize, MAX_LZW_BITS);
+
     lzwInit(&decomp, ifP, lzwMinCodeSize);
 
-    if (verbose)
-        pm_message("reading %d by %d%s GIF image",
-                   len, height, interlace ? " interlaced" : "" );
+    while (pnmBuffer.row < rows) {
+        int const rc = lzwReadByte(&decomp);
 
-    while ((v = lzwReadByte(&decomp)) >= 0 ) {
-        PPM_ASSIGN(xels[ypos][xpos], 
-                   cmap[CM_RED][v], cmap[CM_GRN][v], cmap[CM_BLU][v]);
-
-        ++xpos;
-        if (xpos == len) {
-            xpos = 0;
-            if (interlace) {
-                switch (pass) {
-                case 0:
-                case 1:
-                    ypos += 8; break;
-                case 2:
-                    ypos += 4; break;
-                case 3:
-                    ypos += 2; break;
-                }
-                
-                if (ypos >= height) {
-                    ++pass;
-                    switch (pass) {
-                    case 1:
-                        ypos = 4; break;
-                    case 2:
-                        ypos = 2; break;
-                    case 3:
-                        ypos = 1; break;
-                    default:
-                        goto fini;
-                    }
-                }
-            } else {
-                ++ypos;
-            }
-        }
-        if (ypos >= height)
+        switch (rc) {
+        case -3:
+            pm_error("Error in GIF input stream");
             break;
+        case -2:
+            pm_error("Error in GIF image: Not enough raster data to fill "
+                     "%u x %u dimensions.  Ran out of raster data in "
+                     "row %u", cols, rows, pnmBuffer.row);
+            break;
+        case -1:
+            pm_error("Premature end of file; no proper GIF closing");
+            break;
+        default:
+            addPixelToRaster(rc, &pnmBuffer, cols, rows, cmap, cmapSize,
+                             interlace, transparentIndex, alphabits, &pass);
+        }
     }
-fini:
-    if (v == -3)
-        pm_error("Error in GIF input stream");
-    else {
-        if (lzwReadByte(&decomp)>=0)
-            pm_message("too much input data, ignoring extra...");
-    }
+    if (lzwReadByte(&decomp) >= 0)
+        pm_message("Extraneous data at end of image.  "
+                   "Skipped to end of image");
+
     lzwTerm(&decomp);
 }
 
@@ -987,79 +1120,29 @@ writePnm(FILE *outfile, xel ** const xels,
 
 
 static void
-transparencyMessage(int const transparent_index, 
+transparencyMessage(int const transparentIndex, 
                     gifColorMap cmap) {
 /*----------------------------------------------------------------------------
    If user wants verbose output, tell him that the color with index
-   'transparent_index' is supposed to be a transparent background color.
+   'transparentIndex' is supposed to be a transparent background color.
    
-   If transparent_index == -1, tell him there is no transparent background
+   If transparentIndex == -1, tell him there is no transparent background
    color.
 -----------------------------------------------------------------------------*/
     if (verbose) {
-        if (transparent_index == -1)
+        if (transparentIndex == -1)
             pm_message("no transparency");
         else
             pm_message("transparent background color: rgb:%02x/%02x/%02x "
                        "Index %d",
-                       cmap[CM_RED][transparent_index],
-                       cmap[CM_GRN][transparent_index],
-                       cmap[CM_BLU][transparent_index],
-                       transparent_index
+                       cmap[CM_RED][transparentIndex],
+                       cmap[CM_GRN][transparentIndex],
+                       cmap[CM_BLU][transparentIndex],
+                       transparentIndex
                 );
     }
 }
 
-
-
-static void
-outputAlpha(FILE *alpha_file, pixel ** const xels, 
-            const int cols, const int rows, const int transparent_index,
-            unsigned char cmap[3][MAXCOLORMAPSIZE]) {
-/*----------------------------------------------------------------------------
-   Output to file 'alpha_file' (unless it is NULL) the alpha mask for the
-   image 'xels', given that the color whose index in the color map 'cmap' is
-   'transparent_index' is the transparent color.  The image, and thus the
-   alpha mask have dimensions 'cols' by 'rows'.
-
-   transparent_index == -1 means there are no transparent pixels.
------------------------------------------------------------------------------*/
-
-    if (alpha_file) {
-        bit *alpha_row;  /* malloc'ed */
-        xel transparent_color;
-
-        if (transparent_index != -1) 
-            PPM_ASSIGN(transparent_color, 
-                       cmap[CM_RED][transparent_index],
-                       cmap[CM_GRN][transparent_index],
-                       cmap[CM_BLU][transparent_index]
-                );
-        
-        alpha_row = pbm_allocrow(cols);
-
-        pbm_writepbminit(alpha_file, cols, rows, FALSE);
-
-        {
-            int row;
-            for (row = 0; row < rows; row++) {
-                int col;
-                for (col = 0; col < cols; col++) {
-                    if (transparent_index != -1 && 
-                        PNM_EQUAL(xels[row][col], transparent_color))
-                        alpha_row[col] = PBM_BLACK;
-                    else 
-                        alpha_row[col] = PBM_WHITE;
-                }
-                pbm_writepbmrow(alpha_file, alpha_row, cols, FALSE);
-            }
-        }
-        pbm_freerow(alpha_row);
-    }
-}
-
-
-
 static void
 readGifHeader(FILE * const gifFile, struct gifScreen * const gifScreenP) {
 /*----------------------------------------------------------------------------
@@ -1084,7 +1167,7 @@ readGifHeader(FILE * const gifFile, struct gifScreen * const gifScreenP) {
     if (verbose)
         pm_message("GIF format version is '%s'", version);
     
-    if ((strcmp(version, "87a") != 0) && (strcmp(version, "89a") != 0))
+    if ((!STREQ(version, "87a")) && (!STREQ(version, "89a")))
         pm_error("bad version number, not '87a' or '89a'" );
     
     if (! ReadOK(gifFile,buf,7))
@@ -1092,8 +1175,8 @@ readGifHeader(FILE * const gifFile, struct gifScreen * const gifScreenP) {
     
     gifScreenP->Width           = LM_to_uint(buf[0],buf[1]);
     gifScreenP->Height          = LM_to_uint(buf[2],buf[3]);
-    gifScreenP->ColorMapSize    = 2<<(buf[4]&0x07);
-    gifScreenP->ColorResolution = (((buf[4]&0x70)>>3)+1);
+    gifScreenP->ColorMapSize    = 1 << ((buf[4] & 0x07) + 1);
+    gifScreenP->ColorResolution = (buf[4] & 0x70 >> 3) + 1;
     gifScreenP->Background      = buf[5];
     gifScreenP->AspectRatio     = buf[6];
 
@@ -1177,6 +1260,24 @@ readExtensions(FILE*          const ifP,
 
 
 
+static void
+reportImageInfo(unsigned int const cols,
+                unsigned int const rows,
+                bool         const useGlobalColormap,
+                unsigned int const localColorMapSize,
+                bool         const interlaced) {
+
+
+    pm_message("reading %u by %u%s GIF image",
+               cols, rows, interlaced ? " interlaced" : "" );
+    if (useGlobalColormap)
+        pm_message("  Uses global colormap");
+    else
+        pm_message("  Uses local colormap of %u colors", localColorMapSize);
+}
+
+
+
 static void
 convertImage(FILE *           const ifP, 
              bool             const skipIt, 
@@ -1194,50 +1295,68 @@ convertImage(FILE *           const ifP,
     unsigned char buf[16];
     bool useGlobalColormap;
     xel **xels;  /* The image raster, in libpnm format */
-    int cols, rows;  /* Dimensions of the image */
+    bit **alphabits;  
+        /* The image alpha mask, in libpbm format.  NULL if we aren't computing
+           an alpha mask.
+        */
+    unsigned int cols, rows;  /* Dimensions of the image */
     gifColorMap localColorMap;
-    int localColorMapSize;
+    unsigned int localColorMapSize;
+    bool interlaced;
 
     if (! ReadOK(ifP,buf,9))
         pm_error("couldn't read left/top/width/height");
 
     useGlobalColormap = ! BitSet(buf[8], LOCALCOLORMAP);
-        
-    localColorMapSize = 1<<((buf[8]&0x07)+1);
+    localColorMapSize = 1u << ((buf[8] & 0x07) + 1);
+    cols = LM_to_uint(buf[4], buf[5]);
+    rows = LM_to_uint(buf[6], buf[7]);
+    interlaced = !!BitSet(buf[8], INTERLACE);
 
-    cols = LM_to_uint(buf[4],buf[5]);
-    rows = LM_to_uint(buf[6],buf[7]);
+    if (verbose)
+        reportImageInfo(cols, rows, useGlobalColormap, localColorMapSize,
+                        interlaced);
         
     xels = pnm_allocarray(cols, rows);
     if (!xels)
         pm_error("couldn't alloc space for image" );
 
-    if (! useGlobalColormap) {
+    if (alphafile) {
+        alphabits = pbm_allocarray(cols, rows);
+        if (!alphabits)
+            pm_error("couldn't alloc space for alpha image" );
+    } else
+        alphabits = NULL;
+    
+    if (!useGlobalColormap) {
         int hasGray, hasColor;
 
         readColorMap(ifP, localColorMapSize, localColorMap, 
                      &hasGray, &hasColor);
         transparencyMessage(gif89.transparent, localColorMap);
-        readImageData(ifP, xels, cols, rows, localColorMap, 
-                      BitSet(buf[8], INTERLACE));
+        readImageData(ifP, xels, cols, rows, localColorMap, localColorMapSize,
+                      interlaced, gif89.transparent, alphabits);
         if (!skipIt) {
             writePnm(imageout_file, xels, cols, rows,
                      hasGray, hasColor);
-            outputAlpha(alphafile, xels, cols, rows, 
-                        gif89.transparent, localColorMap);
         }
     } else {
         transparencyMessage(gif89.transparent, gifScreen.ColorMap);
-        readImageData(ifP, xels, cols, rows, gifScreen.ColorMap, 
-                      BitSet(buf[8], INTERLACE));
+        readImageData(ifP, xels, cols, rows, 
+                      gifScreen.ColorMap, gifScreen.ColorMapSize,
+                      interlaced, gif89.transparent, alphabits);
         if (!skipIt) {
             writePnm(imageout_file, xels, cols, rows,
                      gifScreen.hasGray, gifScreen.hasColor);
-            outputAlpha(alphafile, xels, cols, rows, 
-                        gif89.transparent, gifScreen.ColorMap);
         }
     }
+
+    if (!skipIt && alphafile && alphabits)
+        pbm_writepbm(alphafile, alphabits, cols, rows, FALSE);
+
     pnm_freearray(xels, rows);
+    if (alphabits)
+        pbm_freearray(alphabits, rows);
 }
 
 
@@ -1246,6 +1365,7 @@ static void
 convertImages(FILE * const ifP, 
               bool   const allImages,
               int    const requestedImageSeq, 
+              bool   const drainStream,
               FILE * const imageout_file, 
               FILE * const alphafile) {
 /*----------------------------------------------------------------------------
@@ -1258,6 +1378,12 @@ convertImages(FILE * const ifP,
    'requestedImageSeq' is meaningful only when 'allImages' is FALSE.  It 
    is the sequence number of the one image Caller wants from the stream,
    with the first image being 0.
+
+   'drainInput' means to read the entire GIF stream, even after
+   reading the image Caller asked for.  We read the stream, not just
+   the file it's in, so we still recognize certain errors in the GIF
+   format in the tail of the stream and there may yet be more stuff in
+   the file when we return.
 -----------------------------------------------------------------------------*/
     int imageSeq;
         /* Sequence within GIF stream of image we are currently processing.
@@ -1272,8 +1398,10 @@ convertImages(FILE * const ifP,
 
     readGifHeader(ifP, &gifScreen);
 
-    eod = FALSE;  /* initial value */
-    for (imageSeq=0; !eod; ++imageSeq) {
+    for (imageSeq = 0, eod = FALSE;
+         !eod && (imageSeq <= requestedImageSeq || allImages || drainStream);
+         ++imageSeq) {
+
         readExtensions(ifP, &gif89, &eod);
 
         if (eod) {
@@ -1284,7 +1412,8 @@ convertImages(FILE * const ifP,
                          requestedImageSeq+1,
                          imageSeq, imageSeq>1?"s":"" );
         } else {
-            pm_message("Reading Image Sequence %d", imageSeq);
+            if (verbose)
+                pm_message("Reading Image Sequence %d", imageSeq);
             convertImage(ifP, !allImages && (imageSeq != requestedImageSeq), 
                          imageout_file, alphafile, gifScreen, gif89);
         }
@@ -1296,13 +1425,13 @@ convertImages(FILE * const ifP,
 int
 main(int argc, char **argv) {
 
-    struct cmdline_info cmdline;
+    struct cmdlineInfo cmdline;
     FILE *ifP;
     FILE *alpha_file, *imageout_file;
 
     pnm_init(&argc, argv);
 
-    parse_command_line(argc, argv, &cmdline);
+    parseCommandLine(argc, argv, &cmdline);
     verbose = cmdline.verbose;
     showComment = cmdline.comments;
    
@@ -1313,13 +1442,13 @@ main(int argc, char **argv) {
     else
         alpha_file = pm_openw(cmdline.alpha_filename);
 
-    if (alpha_file && strcmp(cmdline.alpha_filename, "-") == 0)
+    if (alpha_file && STREQ(cmdline.alpha_filename, "-"))
         imageout_file = NULL;
     else
         imageout_file = stdout;
 
     convertImages(ifP, cmdline.all_images, cmdline.image_no, 
-                  imageout_file, alpha_file);
+                  !cmdline.quitearly, imageout_file, alpha_file);
 
     pm_close(ifP);
     if (imageout_file != NULL) 
@@ -1327,5 +1456,5 @@ main(int argc, char **argv) {
     if (alpha_file != NULL)
         pm_close( alpha_file );
 
-    exit(0);
+    return 0;
 }
