@@ -123,6 +123,7 @@
 * Includes.
 \******************************************************************************/
 
+#include "pm_c_util.h"
 #include <jasper/jas_stream.h>
 #include <jasper/jas_seq.h>
 
@@ -340,7 +341,7 @@ jas_image_t *jas_image_create(uint_fast16_t numcmpts,
   jas_image_cmptparm_t *cmptparms, int colorspace);
 
 /* Create an "empty" image. */
-jas_image_t *jas_image_create0();
+jas_image_t *jas_image_create0(void);
 
 /* Clone an image. */
 jas_image_t *jas_image_copy(jas_image_t *image);
@@ -500,7 +501,7 @@ int jas_image_getcmptbytype(jas_image_t *image, int ctype);
 \******************************************************************************/
 
 /* Clear the table of image formats. */
-void jas_image_clearfmts();
+void jas_image_clearfmts(void);
 
 /* Add entry to table of image formats. */
 int jas_image_addfmt(int id, const char *name, const char *ext, 
