@@ -730,7 +730,7 @@ typedef struct {
 \******************************************************************************/
 
 /* Create a decoder segment object. */
-jpc_dec_seg_t *jpc_seg_alloc();
+jpc_dec_seg_t *jpc_seg_alloc(void);
 
 /* Destroy a decoder segment object. */
 void jpc_seg_destroy(jpc_dec_seg_t *seg);
