@@ -134,7 +134,7 @@
 *
 \******************************************************************************/
 
-static jpc_qmfb1d_t *jpc_qmfb1d_create();
+static jpc_qmfb1d_t *jpc_qmfb1d_create(void);
 
 static int jpc_ft_getnumchans(jpc_qmfb1d_t *qmfb);
 static int jpc_ft_getanalfilters(jpc_qmfb1d_t *qmfb, int len, jas_seq2d_t **filters);
