@@ -9,25 +9,17 @@
 *****************************************************************************/
 
 #define _BSD_SOURCE 1    /* Make sure strdup() is in string.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
 #include <string.h>
 
 #include "pam.h"
 #include "shhopt.h"
 #include "nstring.h"
-
-/* libjasper uses long longs, which are not ANSI.  So we define stuff here
-   to keep jas_types.h from declaring things long long.  libjasper will
-   use int when it thinks it's using a 64 bit integer.
-*/
-typedef int int_fast64_t;
-#define INT_FAST64_MIN	INT_MIN
-#define INT_FAST64_MAX	INT_MAX
-
-typedef uint uint_fast64_t;
-#define UINT_FAST64_MIN	UINT_MIN
-#define UINT_FAST64_MAX	UINT_MAX
+#include "mallocvar.h"
 
 #include <jasper/jasper.h>
+#include "libjasper_compat.h"
+
 
 enum compmode {COMPMODE_INTEGER, COMPMODE_REAL};
 
@@ -49,7 +41,7 @@ struct cmdlineInfo {
     unsigned int cblkwidth;
     unsigned int cblkheight;
     enum compmode compmode;
-    double       compressionRatio;
+    float        compressionRatio;
     char *       ilyrrates;
     enum progression progression;
     unsigned int numrlvls;
@@ -77,7 +69,7 @@ parseCommandLine(int argc, char ** argv,
    sometimes, one of these strings is actually just a suffix of an entry
    in argv!
 -----------------------------------------------------------------------------*/
-    optEntry *option_def = malloc(100*sizeof(optEntry));
+    optEntry *option_def;
         /* Instructions to OptParseOptions3 on how to parse our options.
          */
     optStruct3 opt;
@@ -95,6 +87,8 @@ parseCommandLine(int argc, char ** argv,
     char * modeOpt;
 
     unsigned int option_def_index;
+    
+    MALLOCARRAY_NOFAIL(option_def, 100);
 
     option_def_index = 0;   /* incremented by OPTENTRY */
     OPTENT3(0, "imgareatlx",   OPT_UINT,   &cmdlineP->imgareatlx,
@@ -225,7 +219,7 @@ parseCommandLine(int argc, char ** argv,
         cmdlineP->inputFilename = strdup(argv[1]);
     else 
         pm_error("Too many arguments.  The only argument accepted\n"
-                 "is the input file specificaton");
+                 "is the input file specification");
 
 }
 
@@ -243,9 +237,7 @@ createJasperRaster(struct pam *  const inpamP,
     bool oddMaxval;
     sample jasperMaxval;
 
-    matrix = malloc(inpamP->depth * sizeof(jas_matrix_t *));
-    if (matrix == NULL)
-        pm_error("Out of memory");
+    MALLOCARRAY_NOFAIL(matrix, inpamP->depth);
 
     for (plane = 0; plane < inpamP->depth; ++plane) {
         matrix[plane] = jas_matrix_create(1, inpamP->width);
@@ -309,10 +301,7 @@ createJasperImage(struct pam *   const inpamP,
 	jas_image_cmptparm_t * cmptparms;
     unsigned int plane;
 
-    cmptparms = malloc(sizeof(jas_image_cmptparm_t) * inpamP->depth);
-
-    if (cmptparms == NULL)
-        pm_error("Unable to allocate storage for cmptparms");
+    MALLOCARRAY_NOFAIL(cmptparms, inpamP->depth);
 
     for (plane = 0; plane < inpamP->depth; ++plane) {
         cmptparms[plane].tlx = 0;
@@ -325,7 +314,7 @@ createJasperImage(struct pam *   const inpamP,
         cmptparms[plane].sgnd = 0;
     }
     *jasperPP = 
-        jas_image_create(inpamP->depth, cmptparms, JAS_IMAGE_CS_UNKNOWN);
+        jas_image_create(inpamP->depth, cmptparms, JAS_CLRSPC_UNKNOWN);
     if (*jasperPP == NULL)
         pm_error("Unable to create jasper image structure.  "
                  "jas_image_create() failed.");
@@ -348,7 +337,7 @@ convertToJasperImage(struct pam *   const inpamP,
             pm_error("Input tuple type is RGB*, but depth is only %d.  "
                      "It should be at least 3.", inpamP->depth);
         else {
-            jas_image_setcolorspace(jasperP, JAS_IMAGE_CS_RGB);
+            jas_image_setclrspc(jasperP, JAS_CLRSPC_GENRGB);
             jas_image_setcmpttype(jasperP, 0,
                                   JAS_IMAGE_CT_COLOR(JAS_IMAGE_CT_RGB_R));
             jas_image_setcmpttype(jasperP, 1,
@@ -359,7 +348,7 @@ convertToJasperImage(struct pam *   const inpamP,
     } else {
         if (strncmp(inpamP->tuple_type, "GRAYSCALE", 9 == 0) ||
             strncmp(inpamP->tuple_type, "BLACKANDWHITE", 13) == 0) {
-            jas_image_setcolorspace(jasperP, JAS_IMAGE_CS_GRAY);
+            jas_image_setclrspc(jasperP, JAS_CLRSPC_GENGRAY);
             jas_image_setcmpttype(jasperP, 0,
                                   JAS_IMAGE_CT_COLOR(JAS_IMAGE_CT_GRAY_Y));
         }
@@ -374,7 +363,8 @@ convertToJasperImage(struct pam *   const inpamP,
 
 static void
 writeJpc(jas_image_t *      const jasperP, 
-         struct cmdlineInfo const cmdline) {
+         struct cmdlineInfo const cmdline,
+         FILE *             const ofP) {
 
     jas_stream_t * outStreamP;
     const char * options;
@@ -382,11 +372,11 @@ writeJpc(jas_image_t *      const jasperP,
     const char * prgValue;
     char rateOpt[20+1];
 
-/* Note: ilyrrates is a hack because we're too lazy to properly parse
-   command line options to get the information and then compose
-   a proper input to Jasper.  So the user can screw things up by 
-   specifying garbage for the -ilyrrates option 
-*/
+    /* Note: ilyrrates is a hack because we're too lazy to properly parse
+       command line options to get the information and then compose
+       a proper input to Jasper.  So the user can screw things up by 
+       specifying garbage for the -ilyrrates option 
+    */
     if (strlen(cmdline.ilyrrates) > 0)
         asprintfN(&ilyrratesOpt, "ilyrrates=%s", cmdline.ilyrrates);
     else
@@ -453,11 +443,10 @@ writeJpc(jas_image_t *      const jasperP,
 
 
     /* Open the output image file (Standard Output) */
-    outStreamP = jas_stream_fdopen(fileno(stdout), "w+b");
+    outStreamP = jas_stream_fdopen(fileno(ofP), "w+b");
     if (outStreamP == NULL)
         pm_error("Unable to open output stream.  jas_stream_fdopen() "
-                 "of stdout failed");
-
+                 "failed");
 
     {
         int rc;
@@ -517,11 +506,11 @@ main(int argc, char **argv)
     
     ifP = pm_openr(cmdline.inputFilename);
     
-    pnm_readpaminit(ifP, &inpam, sizeof(inpam));
+    pnm_readpaminit(ifP, &inpam, PAM_STRUCT_SIZE(tuple_type));
     
     convertToJasperImage(&inpam, &jasperP);
     
-    writeJpc(jasperP, cmdline);
+    writeJpc(jasperP, cmdline, stdout);
     
 	jas_image_destroy(jasperP);
 
@@ -529,5 +518,5 @@ main(int argc, char **argv)
 
     pm_close(stdout);
     
-    exit(0);
+    return 0;
 }
