@@ -16,14 +16,15 @@
 
 *****************************************************************************/
 
-#define _BSD_SOURCE 1
-    /* This makes sure strdup() is in string.h */
+#define _BSD_SOURCE 1      /* Make sure strdup() is in string.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
 
 #include <ctype.h>		/* to declare isdigit(), etc. */
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <stdio.h>
+#include <assert.h>
 /* Note: jpeglib.h prerequires stdlib.h and ctype.h.  It should include them
    itself, but doesn't.
 */
@@ -31,11 +32,10 @@
 #include "pnm.h"
 #include "shhopt.h"
 #include "mallocvar.h"
+#include "nstring.h"
 
 #define EXIT_WARNING 2   /* Goes with EXIT_SUCCESS, EXIT_FAILURE in stdlib.h */
 
-static const char * progname;	/* program name for error messages */
-
 enum restart_unit {RESTART_MCU, RESTART_ROW, RESTART_NONE};
 enum density_unit {DEN_UNSPECIFIED, DEN_DOTS_PER_INCH, DEN_DOTS_PER_CM};
 
@@ -48,7 +48,7 @@ struct density {
         /* Same as 'horiz', but vertical */
 };
 
-struct cmdline_info {
+struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
      */
@@ -79,7 +79,7 @@ struct cmdline_info {
            is undefined.
         */
     struct density density;
-} cmdline;
+};
 
 static void 
 interpret_maxmemory (const char * const maxmemory, 
@@ -178,11 +178,10 @@ interpret_density(const char *        const densityString,
                 if (matched < 3) 
                     densityP->unit = DEN_UNSPECIFIED;
                 else {
-                    if (strcmp(unitName, "dpi") == 0 || 
-                        strcmp(unitName, "DPI") == 0)
+                    if (streq(unitName, "dpi") || streq(unitName, "DPI"))
                         densityP->unit = DEN_DOTS_PER_INCH;
-                    else if (strcmp(unitName, "dpcm") == 0 || 
-                        strcmp(unitName, "DPCM") == 0)
+                    else if (streq(unitName, "dpcm") ||
+                             streq(unitName, "DPCM"))
                         densityP->unit = DEN_DOTS_PER_CM;
                     else
                         pm_error("Unrecognized unit '%s' in the density value "
@@ -198,11 +197,11 @@ interpret_density(const char *        const densityString,
 
 
 static void
-parse_command_line(const int argc, char ** argv,
-                   struct cmdline_info *cmdline_p) {
+parseCommandLine(const int argc, char ** argv,
+                   struct cmdlineInfo *cmdlineP) {
 /*----------------------------------------------------------------------------
    Note that many of the strings that this function returns in the
-   *cmdline_p structure are actually in the supplied argv array.  And
+   *cmdlineP structure are actually in the supplied argv array.  And
    sometimes, one of these strings is actually just a suffix of an entry
    in argv!
 
@@ -232,43 +231,43 @@ parse_command_line(const int argc, char ** argv,
     MALLOCARRAY(argv_parse, argc);
 
     option_def_index = 0;   /* incremented by OPTENTRY */
-    OPTENT3(0, "verbose",     OPT_FLAG,   NULL, &cmdline_p->verbose,        0);
-    OPTENT3(0, "quality",     OPT_UINT,   &cmdline_p->quality, 
+    OPTENT3(0, "verbose",     OPT_FLAG,   NULL, &cmdlineP->verbose,        0);
+    OPTENT3(0, "quality",     OPT_UINT,   &cmdlineP->quality, 
             &qualitySpec,        0);
-    OPTENT3(0, "baseline",    OPT_FLAG,   NULL, &cmdline_p->force_baseline, 0);
-    OPTENT3(0, "progressive", OPT_FLAG,   NULL, &cmdline_p->progressive,    0);
-    OPTENT3(0, "arithmetic",  OPT_FLAG,   NULL, &cmdline_p->arith_code,     0);
+    OPTENT3(0, "baseline",    OPT_FLAG,   NULL, &cmdlineP->force_baseline, 0);
+    OPTENT3(0, "progressive", OPT_FLAG,   NULL, &cmdlineP->progressive,    0);
+    OPTENT3(0, "arithmetic",  OPT_FLAG,   NULL, &cmdlineP->arith_code,     0);
     OPTENT3(0, "dct",         OPT_STRING, &dctval, NULL,                    0);
-    OPTENT3(0, "grayscale",   OPT_FLAG,   NULL, &cmdline_p->grayscale,      0);
-    OPTENT3(0, "greyscale",   OPT_FLAG,   NULL, &cmdline_p->grayscale,      0);
-    OPTENT3(0, "rgb",         OPT_FLAG,   NULL, &cmdline_p->rgb,            0);
+    OPTENT3(0, "grayscale",   OPT_FLAG,   NULL, &cmdlineP->grayscale,      0);
+    OPTENT3(0, "greyscale",   OPT_FLAG,   NULL, &cmdlineP->grayscale,      0);
+    OPTENT3(0, "rgb",         OPT_FLAG,   NULL, &cmdlineP->rgb,            0);
     OPTENT3(0, "maxmemory",   OPT_STRING, &maxmemory, NULL,                 0);
-    OPTENT3(0, "tracelevel",  OPT_UINT,   &cmdline_p->trace_level, NULL,    0);
-    OPTENT3(0, "qslots",      OPT_STRING, &cmdline_p->qslots,      NULL,    0);
-    OPTENT3(0, "qtables",     OPT_STRING, &cmdline_p->qtablefile,  NULL,    0);
-    OPTENT3(0, "sample",      OPT_STRING, &cmdline_p->sample,      NULL,    0);
-    OPTENT3(0, "scans",       OPT_STRING, &cmdline_p->scans,       NULL,    0);
-    OPTENT3(0, "smooth",      OPT_UINT,   &cmdline_p->smoothing_factor, 
+    OPTENT3(0, "tracelevel",  OPT_UINT,   &cmdlineP->trace_level, NULL,    0);
+    OPTENT3(0, "qslots",      OPT_STRING, &cmdlineP->qslots,      NULL,    0);
+    OPTENT3(0, "qtables",     OPT_STRING, &cmdlineP->qtablefile,  NULL,    0);
+    OPTENT3(0, "sample",      OPT_STRING, &cmdlineP->sample,      NULL,    0);
+    OPTENT3(0, "scans",       OPT_STRING, &cmdlineP->scans,       NULL,    0);
+    OPTENT3(0, "smooth",      OPT_UINT,   &cmdlineP->smoothing_factor, 
             &smoothSpec,  0);
-    OPTENT3(0, "optimize",    OPT_FLAG,   NULL, &cmdline_p->optimize,       0);
-    OPTENT3(0, "optimise",    OPT_FLAG,   NULL, &cmdline_p->optimize,       0);
+    OPTENT3(0, "optimize",    OPT_FLAG,   NULL, &cmdlineP->optimize,       0);
+    OPTENT3(0, "optimise",    OPT_FLAG,   NULL, &cmdlineP->optimize,       0);
     OPTENT3(0, "restart",     OPT_STRING, &restart, NULL,                   0);
-    OPTENT3(0, "comment",     OPT_STRING, &cmdline_p->comment, NULL,        0);
-    OPTENT3(0, "exif",        OPT_STRING, &cmdline_p->exif_filespec, NULL,  0);
+    OPTENT3(0, "comment",     OPT_STRING, &cmdlineP->comment, NULL,        0);
+    OPTENT3(0, "exif",        OPT_STRING, &cmdlineP->exif_filespec, NULL,  0);
     OPTENT3(0, "density",     OPT_STRING, &density, 
-            &cmdline_p->density_spec, 0);
+            &cmdlineP->density_spec, 0);
 
     /* Set the defaults */
     dctval = NULL;
     maxmemory = NULL;
-    cmdline_p->trace_level = 0;
-    cmdline_p->qslots = NULL;
-    cmdline_p->qtablefile = NULL;
-    cmdline_p->sample = NULL;
-    cmdline_p->scans = NULL;
+    cmdlineP->trace_level = 0;
+    cmdlineP->qslots = NULL;
+    cmdlineP->qtablefile = NULL;
+    cmdlineP->sample = NULL;
+    cmdlineP->scans = NULL;
     restart = NULL;
-    cmdline_p->comment = NULL;
-    cmdline_p->exif_filespec = NULL;
+    cmdlineP->comment = NULL;
+    cmdlineP->exif_filespec = NULL;
 
     /* Make private copy of arguments for optParseOptions to corrupt */
     argc_parse = argc;
@@ -281,47 +280,47 @@ parse_command_line(const int argc, char ** argv,
     optParseOptions3(&argc_parse, argv_parse, opt, sizeof(opt), 0);
 
     if (!qualitySpec)
-        cmdline_p->quality = -1;  /* unspecified */
+        cmdlineP->quality = -1;  /* unspecified */
 
     if (!smoothSpec)
-        cmdline_p->smoothing_factor = -1;
+        cmdlineP->smoothing_factor = -1;
 
-    if (cmdline_p->rgb && cmdline_p->grayscale)
+    if (cmdlineP->rgb && cmdlineP->grayscale)
         pm_error("You can't specify both -rgb and -grayscale");
 
     if (argc_parse - 1 == 0)
-        cmdline_p->input_filespec = strdup("-");  /* he wants stdin */
+        cmdlineP->input_filespec = strdup("-");  /* he wants stdin */
     else if (argc_parse - 1 == 1)
-        cmdline_p->input_filespec = strdup(argv_parse[1]);
+        cmdlineP->input_filespec = strdup(argv_parse[1]);
     else 
-        pm_error("Too many arguments.  The only argument accepted"
+        pm_error("Too many arguments.  The only argument accepted "
                  "is the input file specification.");
     if (dctval == NULL)
-        cmdline_p->dct_method = JDCT_DEFAULT;
+        cmdlineP->dct_method = JDCT_DEFAULT;
     else {
-        if (strcmp(dctval, "int") == 0)
-            cmdline_p->dct_method = JDCT_ISLOW;
-        else if (strcmp(dctval, "fast") == 0)
-            cmdline_p->dct_method = JDCT_IFAST;
-        else if (strcmp(dctval, "float") == 0)
-            cmdline_p->dct_method = JDCT_FLOAT;
+        if (streq(dctval, "int"))
+            cmdlineP->dct_method = JDCT_ISLOW;
+        else if (streq(dctval, "fast"))
+            cmdlineP->dct_method = JDCT_IFAST;
+        else if (streq(dctval, "float"))
+            cmdlineP->dct_method = JDCT_FLOAT;
         else
             pm_error("Invalid value for the --dct option: '%s'.", dctval);
     }
 
-    interpret_maxmemory(maxmemory, &cmdline_p->max_memory_to_use);
-    interpret_restart(restart, &cmdline_p->restart_value,
-                      &cmdline_p->restart_unit);
-    if (cmdline_p->density_spec) 
-        interpret_density(density, &cmdline_p->density);
+    interpret_maxmemory(maxmemory, &cmdlineP->max_memory_to_use);
+    interpret_restart(restart, &cmdlineP->restart_value,
+                      &cmdlineP->restart_unit);
+    if (cmdlineP->density_spec) 
+        interpret_density(density, &cmdlineP->density);
     
-    if (cmdline_p->smoothing_factor > 100)
+    if (cmdlineP->smoothing_factor > 100)
         pm_error("Smoothing factor %d is greater than 100 (%%).",
-                 cmdline_p->smoothing_factor);
+                 cmdlineP->smoothing_factor);
 
-    if (strcmp(cmdline_p->input_filespec, "=") == 0 &&
-        cmdline_p->exif_filespec && 
-        strcmp(cmdline_p->exif_filespec, "-") == 0)
+    if (streq(cmdlineP->input_filespec, "=") &&
+        cmdlineP->exif_filespec && 
+        streq(cmdlineP->exif_filespec, "-"))
 
         pm_error("Cannot have both input image and exif header be from "
                  "Standard Input.");
@@ -335,7 +334,7 @@ static void
 compute_rescaling_array(JSAMPLE ** const rescale_p, const pixval maxval,
                         const struct jpeg_compress_struct cinfo);
 static void
-convert_scanlines(struct jpeg_compress_struct *cinfo_p, FILE *input_file,
+convert_scanlines(struct jpeg_compress_struct * const cinfo_p, FILE * const input_file,
                   const pixval maxval, const int input_fmt,
                   JSAMPLE xlate_table[]);
 
@@ -427,11 +426,12 @@ setup_jpeg_density(struct jpeg_compress_struct * const cinfoP,
 static void
 setup_jpeg(struct jpeg_compress_struct * const cinfoP,
            struct jpeg_error_mgr       * const jerrP,
-           struct cmdline_info           const cmdline, 
-           int const width, int const height, pixval const maxval,
-           int const input_fmt,
-           FILE * const output_file
-    ) {
+           struct cmdlineInfo            const cmdline, 
+           int                           const width,
+           int                           const height,
+           pixval                        const maxval,
+           int                           const input_fmt,
+           FILE *                        const output_file) {
   
     int quality;
     int q_scale_factor;
@@ -545,10 +545,15 @@ write_exif_header(struct jpeg_compress_struct * const cinfoP,
 
     if (length == 0) {
         /* Special value meaning "no header" */
-    } else {
+    } else if (length < 3)
+        pm_error("Invalid length %u at start of exif file", length);
+    else {
         unsigned char * exif_data;
         int rc;
-        int const data_length = length - 2;  /* Subtract 2 byte length field*/
+        size_t const data_length = length - 2;  
+            /* Subtract 2 byte length field*/
+
+        assert(data_length > 0);
 
         exif_data = malloc(data_length);
         if (exif_data == NULL)
@@ -638,16 +643,18 @@ translate_row(const pixel pnm_buffer[],
 
 
 static void
-convert_scanlines(struct jpeg_compress_struct *cinfo_p, FILE *input_file,
-                  const pixval maxval, const int input_fmt,
-                  JSAMPLE xlate_table[]){
+convert_scanlines(struct jpeg_compress_struct * const cinfo_p,
+                  FILE *                        const input_file,
+                  pixval                        const maxval,
+                  int                           const input_fmt,
+                  JSAMPLE                             xlate_table[]){
 /*----------------------------------------------------------------------------
    Read scan lines from the input file, which is already opened in the 
    netpbm library sense and ready for reading, and write them to the 
    output JPEG object.  Translate the pnm sample values to JPEG sample
    values through the thable xlate_table[].
 -----------------------------------------------------------------------------*/
-  xel* pnm_buffer;  
+  xel * pnm_buffer;  
     /* contains the row of the input image currently being processed,
        in pnm_readpnmrow format
     */
@@ -675,13 +682,13 @@ convert_scanlines(struct jpeg_compress_struct *cinfo_p, FILE *input_file,
                   xlate_table);
     jpeg_write_scanlines(cinfo_p, buffer, 1);
     if (cinfo_p->err->trace_level > 1) 
-      pm_message("Done.");
+        pm_message("Done.");
   }
 
   pnm_freerow(pnm_buffer);
   /* Dont' worry about the compressor input buffer; it gets freed 
      automatically
-     */
+  */
 
 }
 
@@ -1022,70 +1029,70 @@ set_sample_factors (j_compress_ptr cinfo, char *arg)
 
 int
 main(int argc, char ** argv) {
-  struct jpeg_compress_struct cinfo;
-  struct jpeg_error_mgr jerr;
-  FILE *input_file;
-  FILE * output_file;
-  int height;  
-    /* height of the input image in rows, as specified by its header */
-  int width;   
-    /* width of the input image in columns, as specified by its header */
-  pixval maxval;  
-    /* maximum value of an input pixel component, as specified by header */
-  int input_fmt;
-    /* The input format, as determined by its header.  */
-  JSAMPLE *rescale;         /* => maxval-remapping array, or NULL */
-    /* This is an array that maps each possible pixval in the input to
-       a new value such that while the range of the input values is
-       0 .. maxval, the range of the output values is 0 .. MAXJSAMPLE.
-       */
 
-  pnm_init(&argc, argv);
+    struct cmdlineInfo cmdline;
+    struct jpeg_compress_struct cinfo;
+    struct jpeg_error_mgr jerr;
+    FILE *input_file;
+    FILE * output_file;
+    int height;  
+        /* height of the input image in rows, as specified by its header */
+    int width;   
+        /* width of the input image in columns, as specified by its header */
+    pixval maxval;  
+        /* maximum value of an input pixel component, as specified by header */
+    int input_fmt;
+        /* The input format, as determined by its header.  */
+    JSAMPLE *rescale;         /* => maxval-remapping array, or NULL */
+        /* This is an array that maps each possible pixval in the input to
+           a new value such that while the range of the input values is
+           0 .. maxval, the range of the output values is 0 .. MAXJSAMPLE.
+        */
 
-  progname = argv[0];
+    pnm_init(&argc, argv);
 
-  parse_command_line(argc, argv, &cmdline);
+    parseCommandLine(argc, argv, &cmdline);
 
-  input_file = pm_openr(cmdline.input_filespec);
-  free(cmdline.input_filespec);
+    input_file = pm_openr(cmdline.input_filespec);
+    free(cmdline.input_filespec);
 
-  output_file = stdout;
+    output_file = stdout;
 
-  /* Open the pnm input */
-  pnm_readpnminit(input_file, &width, &height, &maxval, &input_fmt);
-  if (cmdline.verbose) {
-    pm_message("Input file has format %c%c.\n"
-               "It has %d rows of %d columns of pixels "
-               "with max sample value of %d.", 
-               (char) (input_fmt/256), (char) (input_fmt % 256),
-               height, width, maxval);
-  }
+    /* Open the pnm input */
+    pnm_readpnminit(input_file, &width, &height, &maxval, &input_fmt);
+    if (cmdline.verbose) {
+        pm_message("Input file has format %c%c.\n"
+                   "It has %d rows of %d columns of pixels "
+                   "with max sample value of %d.", 
+                   (char) (input_fmt/256), (char) (input_fmt % 256),
+                   height, width, maxval);
+    }
 
-  setup_jpeg(&cinfo, &jerr, cmdline, width, height, maxval, input_fmt,
-             output_file);
+    setup_jpeg(&cinfo, &jerr, cmdline, width, height, maxval, input_fmt,
+               output_file);
 
-  compute_rescaling_array(&rescale, maxval, cinfo);
+    compute_rescaling_array(&rescale, maxval, cinfo);
 
-  if (cmdline.comment) 
-      jpeg_write_marker(&cinfo, JPEG_COM, (const JOCTET *) cmdline.comment, 
-                        strlen(cmdline.comment));
+    if (cmdline.comment) 
+        jpeg_write_marker(&cinfo, JPEG_COM, (const JOCTET *) cmdline.comment, 
+                          strlen(cmdline.comment));
 
-  if (cmdline.exif_filespec)
-      write_exif_header(&cinfo, cmdline.exif_filespec);
+    if (cmdline.exif_filespec)
+        write_exif_header(&cinfo, cmdline.exif_filespec);
+    
+    /* Translate and copy over the actual scanlines */
+    convert_scanlines(&cinfo, input_file, maxval, input_fmt, rescale);
 
-  /* Translate and copy over the actual scanlines */
-  convert_scanlines(&cinfo, input_file, maxval, input_fmt, rescale);
+    /* Finish compression and release memory */
+    jpeg_finish_compress(&cinfo);
+    jpeg_destroy_compress(&cinfo);
 
-  /* Finish compression and release memory */
-  jpeg_finish_compress(&cinfo);
-  jpeg_destroy_compress(&cinfo);
+    /* Close files, if we opened them */
+    if (input_file != stdin)
+        fclose(input_file);
 
-  /* Close files, if we opened them */
-  if (input_file != stdin)
-      fclose(input_file);
-
-  /* Program may have exited with non-zero completion code via various function
-     calls above.
-     */
-  exit((jerr.num_warnings > 0) ? EXIT_WARNING : EXIT_SUCCESS);
+    /* Program may have exited with non-zero completion code via
+       various function calls above. 
+    */
+    return jerr.num_warnings > 0 ? EXIT_WARNING : EXIT_SUCCESS;
 }
