@@ -55,7 +55,9 @@ Software copyrights will soon need family trees... :-)
 
 */
 
-#include "pnm.h"
+#define _XOPEN_SOURCE
+
+#include <string.h>
 #include <math.h>
 /* float.h used to be included only if __osf__ was defined.  On some
    platforms, float.h is also included automatically by math.h.  But at
@@ -64,8 +66,13 @@ Software copyrights will soon need family trees... :-)
 #include <float.h>
 #include <limits.h>
 #include <fcntl.h>
+/* See warning about tiffio.h in pamtotiff.c */
 #include <tiffio.h>
 
+#include "pm_c_util.h"
+#include "nstring.h"
+#include "pnm.h"
+
 /* first release after testing */
 /* second release after testing - small mods to netpbm package */
 #define VERSION 1.01
@@ -657,7 +664,7 @@ typedef struct tagConv {
    removal of cmy under k */
 
 typedef struct {
-  int initialised ;
+  int initialized ;
   float theta ;
   float gamma ;
   float gammap ;
@@ -722,21 +729,21 @@ standardConvert( Conv *conv, Root *rt, float r, float g, float b,
                  int *c, int *m, int *y, int *k ) {
 
   float c0, m0, y0, k0 ; /* CMYK before colour removal */
-  float grey, white, colour ;
+  float gray, white, colour ;
   double phi ;
 
   Standard *s ; /* private conversion data */
 
   s = (Standard*)(conv->private) ;
 
-  if ( ! s->initialised ) {
+  if ( ! s->initialized ) {
     s->theta *= 2.0 * PI / 360.0 ; /* to radians */
     /* if gammap never specified in options, set to gamma now */
     if ( s->remove == UNSET ) {
       s->remove = 1 ;
       s->gammap = s->gamma ;
     }
-    s->initialised = 1 ;
+    s->initialized = 1 ;
   }
 
   /* rotate in rgb */
@@ -755,13 +762,13 @@ standardConvert( Conv *conv, Root *rt, float r, float g, float b,
 
   /* remove colour under black and convert to integer range 0 - rt->maxOut */
   if ( s->remove ) {
-    grey = pow( k0, s->gammap ) ;
+    gray = pow( k0, s->gammap ) ;
   } else {
-    grey = 0.0 ;
+    gray = 0.0 ;
   }
-  *c = TOMAX( c0 - grey ) ;
-  *m = TOMAX( m0 - grey ) ;
-  *y = TOMAX( y0 - grey ) ;
+  *c = TOMAX( c0 - gray ) ;
+  *m = TOMAX( m0 - gray ) ;
+  *y = TOMAX( y0 - gray ) ;
 
   return ERR_OK ;
 }
@@ -787,7 +794,7 @@ standardOpt( Conv *conv, Root *r, int *argn, int argc, char **argv ) {
   }
   /* handle the special case of -1 (no removal) */
   if ( oldn == *argn && pm_keymatch( argv[*argn], "-gammap", 7 ) &&
-       *argn + 1 < argc && ! strcmp( argv[*argn + 1], "-1" ) ) {
+       *argn + 1 < argc && STREQ(argv[*argn + 1], "-1") ) {
     p->remove = 0 ;
     *argn = (*argn) + 2 ;
   } 
@@ -819,7 +826,7 @@ newStandardMap( Conv **conv ) {
 
   if ( ! (*conv = (Conv*)calloc( 1, sizeof( Conv ) )) ) goto error ;
   if ( ! (s = (Standard*)calloc( 1, sizeof( Standard ) )) ) goto error ;
-  s->initialised = 0 ;
+  s->initialized = 0 ;
   s->remove = UNSET ; /* gammap unset */
   s->gamma = 1.0 ;
   (*conv)->private = s ;
