@@ -46,13 +46,14 @@
    give the user the -byrow option to order (2) only.
 */
 
-#define _BSD_SOURCE 1
-    /* Make sure strdup() is in string.h */
+#define _BSD_SOURCE 1      /* Make sure strdup() is in string.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
 
 #include <string.h>
 #include "pnm.h"
 #include "shhopt.h"
 #include "mallocvar.h"
+#include "nstring.h"
 
 #ifdef VMS
 #ifdef SYSV
@@ -60,6 +61,7 @@
 #endif
 #include <tiffioP.h>
 #endif
+/* See warning about tiffio.h in pamtotiff.c */
 #include <tiffio.h>
 
 /* The following are in current tiff.h, but so that we can compile against
@@ -82,19 +84,20 @@ struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
-    char *inputFilename;
+    char * inputFilename;
     unsigned int headerdump;
-    char *alphaFilename;
-    int alphaStdout;
+    char * alphaFilename;
+    bool alphaStdout;
     unsigned int respectfillorder;   /* -respectfillorder option */
     unsigned int byrow;
-} cmdline;
+    unsigned int verbose;
+};
 
 
 
 static void
 parseCommandLine(int argc, char ** argv,
-                 struct cmdlineInfo *cmdlineP) {
+                 struct cmdlineInfo * const cmdlineP) {
 /*----------------------------------------------------------------------------
    Note that many of the strings that this function returns in the
    *cmdlineP structure are actually in the supplied argv array.  And
@@ -102,43 +105,51 @@ parseCommandLine(int argc, char ** argv,
    in argv!
 -----------------------------------------------------------------------------*/
     optStruct3 opt;
-    optEntry *option_def = malloc(100*sizeof(optEntry));
+    optEntry *option_def;
     unsigned int option_def_index;
-    unsigned int alpha_spec;
+    unsigned int alphaSpec;
+
+    MALLOCARRAY_NOFAIL(option_def, 100);
 
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;
     opt.allowNegNum = FALSE;
 
-    option_def_index = 0;   /* incremented by OPTENTRY */
+    option_def_index = 0;   /* incremented by OPTENT3 */
+    OPTENT3(0, "verbose", 
+            OPT_FLAG,   NULL, &cmdlineP->verbose,              0);
     OPTENT3(0, "respectfillorder", 
-            OPT_FLAG,   NULL, &cmdlineP->respectfillorder, 0);
+            OPT_FLAG,   NULL, &cmdlineP->respectfillorder,     0);
     OPTENT3(0,   "byrow",   
-            OPT_FLAG,   NULL, &cmdlineP->byrow, 0);
+            OPT_FLAG,   NULL, &cmdlineP->byrow,                0);
     OPTENT3('h', "headerdump", 
-            OPT_FLAG,   NULL, &cmdlineP->headerdump, 0);
+            OPT_FLAG,   NULL, &cmdlineP->headerdump,           0);
     OPTENT3(0,   "alphaout",   
-            OPT_STRING, &cmdlineP->alphaFilename, &alpha_spec, 0);
+            OPT_STRING, &cmdlineP->alphaFilename, &alphaSpec,  0);
 
     optParseOptions3(&argc, argv, opt, sizeof(opt), 0);
 
     if (argc - 1 == 0)
-        cmdlineP->inputFilename = NULL;  /* he wants stdin */
+        cmdlineP->inputFilename = strdup("-");  /* he wants stdin */
     else if (argc - 1 == 1)
         cmdlineP->inputFilename = strdup(argv[1]);
     else 
-        pm_error("Too many arguments.  The only argument accepted\n"
-                 "is the input file specificaton");
+        pm_error("Too many arguments.  The only argument accepted "
+                 "is the input file name");
 
-    if (alpha_spec && strcmp(cmdlineP->alphaFilename, "-") == 0 )
-        cmdlineP->alphaStdout = 1;
-    else 
-        cmdlineP->alphaStdout = 0;
+    if (alphaSpec) {
+        if (STREQ(cmdlineP->alphaFilename, "-"))
+            cmdlineP->alphaStdout = TRUE;
+        else
+            cmdlineP->alphaStdout = FALSE;
+    } else {
+        cmdlineP->alphaFilename = NULL;
+        cmdlineP->alphaStdout = FALSE;
+    }
 }
 
 
 
-
 static void 
 read_directory(TIFF * const tif,
                unsigned short * const bps_p,
@@ -197,19 +208,6 @@ read_directory(TIFF * const tif,
         *planarconfig_p = PLANARCONFIG_CONTIG;
     }
 
-    switch (*spp_p) {
-    case 1:
-    case 3:
-    case 4:
-        break;
-    default:
-        pm_error(
-            "This program can only handle 1-channel gray scale "
-            "or 1- or 3-channel color.  The input Tiff file has %d channels "
-            "(samples per pixel)",
-            *spp_p);
-    }
-
     switch(*planarconfig_p) {
     case PLANARCONFIG_CONTIG:
         break;
@@ -346,21 +344,36 @@ static void
 pick_cmyk_pixel(const unsigned int samplebuf[], const int sample_cursor,
                 xelval * const r_p, xelval * const b_p, xelval * const g_p) {
 
-    unsigned char c, y, m, k;
-
-    c = samplebuf[sample_cursor+0];
-    y = samplebuf[sample_cursor+1];
-    m = samplebuf[sample_cursor+2];
-    k = samplebuf[sample_cursor+3];
-
-    /* I don't know why this works.  It seems to me that yellow pigment should
-       translate into both red and green light, not just green as you see here.
-       But this is the formula used by TIFFRGBAImageGet() in the Tiff library.
+    /* Note that the TIFF spec does not say which of the 4 samples is
+       which, but common sense says they're in order C,M,Y,K.  Before
+       Netpbm 10.21 (March 2004), we assumed C,Y,M,K for some reason.
+       But combined with a compensating error in the CMYK->RGB
+       calculation, it had the same effect as C,M,Y,K.
     */
-    
-    *r_p = ((255-k)*(255-c))/255;
-    *g_p = ((255-k)*(255-y))/255;
-    *b_p = ((255-k)*(255-m))/255;
+    unsigned int const c = samplebuf[sample_cursor+0];
+    unsigned int const m = samplebuf[sample_cursor+1];
+    unsigned int const y = samplebuf[sample_cursor+2];
+    unsigned int const k = samplebuf[sample_cursor+3];
+
+    /* The CMYK->RGB formula used by TIFFRGBAImageGet() in the TIFF 
+       library is the following, (with some apparent confusion with
+       the names of the yellow and magenta pigments being reversed).
+
+       R = (1-K)*(1-C)     (with C,Y,M,K normalized to 0..1)
+       G = (1-K)*(1-M)
+       B = (1-K)*(1-Y)
+
+       We used that too before Netpbm 10.21 (March 2004).
+
+       Now we use the inverse of what Pnmtotiffcmyk has always used, which
+       makes sense as follows:  A microliter of black ink is simply a 
+       substitute for a microliter each of cyan, magenta, and yellow ink.
+       Yellow ink removes blue light from what the white paper reflects.  
+    */
+
+    *r_p = 255 - MIN(255, c + k);
+    *g_p = 255 - MIN(255, m + k);
+    *b_p = 255 - MIN(255, y + k);
 }
 
 
@@ -391,25 +404,33 @@ computeFillorder(unsigned short   const fillorderTag,
 
 
 static void
-analyze_image_type(TIFF * const tif, unsigned short const bps, 
-                   unsigned short const spp, unsigned short const photomet,
-                   xelval * const maxvalP, int * const formatP, 
-                   xel colormap[], bool const headerdump) {
+analyzeImageType(TIFF *             const tif, 
+                 unsigned short     const bps, 
+                 unsigned short     const spp, 
+                 unsigned short     const photomet,
+                 xelval *           const maxvalP, 
+                 int *              const formatP, 
+                 xel                      colormap[],
+                 bool               const headerdump,
+                 struct cmdlineInfo const cmdline) {
 
     bool grayscale; 
 
-    if ( bps == 1 && spp == 1 ) {
-        if ( cmdline.headerdump )
-            pm_message("bilevel" );
+    if (bps == 1 && spp == 1) {
+        if (cmdline.headerdump)
+            pm_message("bilevel");
         grayscale = TRUE;
         *maxvalP = 1;
     } else {
         /* How come we don't deal with the photometric for the monochrome 
            case (make sure it's one we know)?  -Bryan 00.03.04
-           */
-        switch ( photomet ) {
+        */
+        switch (photomet) {
         case PHOTOMETRIC_MINISBLACK:
         case PHOTOMETRIC_MINISWHITE:
+            if (spp != 1)
+                pm_error("This grayscale image has %d samples per pixel.  "
+                         "We understand only 1.", spp);
             grayscale = TRUE;
             *maxvalP = pm_bitstomaxval(MIN(bps,16));
             if (headerdump)
@@ -430,6 +451,10 @@ analyze_image_type(TIFF * const tif, unsigned short const bps,
             if (headerdump)
                 pm_message("colormapped");
 
+            if (spp != 1)
+                pm_error("This paletted image has %d samples per pixel.  "
+                         "We understand only 1.", spp);
+
             if (!TIFFGetField(tif, TIFFTAG_COLORMAP, 
                               &redcolormap, &greencolormap, &bluecolormap))
                 pm_error("error getting colormaps");
@@ -473,6 +498,11 @@ analyze_image_type(TIFF * const tif, unsigned short const bps,
             if (headerdump)
                 pm_message("RGB truecolor");
             grayscale = FALSE;
+
+            if (spp != 3 && spp != 4)
+                pm_error("This RGB image has %d samples per pixel.  "
+                         "We understand only 3 or 4.", spp);
+
             *maxvalP = (1 << bps) - 1;
             break;
 
@@ -857,6 +887,9 @@ convertRasterInMemory(FILE *         const imageoutFile,
         } else {
             uint32* raster ;
 
+            /* Note that TIFFRGBAImageGet() converts any bits per sample
+               to 8.  Maxval of the raster it returns is always 255.
+            */
             MALLOCARRAY(raster, cols * rows);
             if (raster == NULL) {
                 pm_message("Unable to allocate space for a raster of %u "
@@ -892,12 +925,11 @@ convertRasterInMemory(FILE *         const imageoutFile,
 
 
 
-
 static void
-convertIt(TIFF*              const tif,
-          FILE *             const alphaFile, 
-          FILE *             const imageoutFile,
-          struct cmdlineInfo const cmdline) {
+convertImage(TIFF *             const tifP,
+             FILE *             const alphaFile, 
+             FILE *             const imageoutFile,
+             struct cmdlineInfo const cmdline) {
 
     unsigned int cols, rows;
     int format;
@@ -908,15 +940,15 @@ convertIt(TIFF*              const tif,
     unsigned short photomet, planarconfig, fillorderTag;
     unsigned short fillorder;
 
-    read_directory(tif, &bps, &spp, &photomet, &planarconfig, &fillorderTag,
+    read_directory(tifP, &bps, &spp, &photomet, &planarconfig, &fillorderTag,
                    &cols, &rows, 
                    cmdline.headerdump);
 
 
     computeFillorder(fillorderTag, &fillorder, cmdline.respectfillorder);
 
-    analyze_image_type(tif, bps, spp, photomet, 
-                       &maxval, &format, colormap, cmdline.headerdump);
+    analyzeImageType(tifP, bps, spp, photomet, 
+                     &maxval, &format, colormap, cmdline.headerdump, cmdline);
 
     if (imageoutFile != NULL) 
         pnm_writepnminit( imageoutFile, 
@@ -931,18 +963,50 @@ convertIt(TIFF*              const tif,
         else {
             convertRasterInMemory(
                 imageoutFile, alphaFile, cols, rows, maxval, format, 
-                tif, photomet, planarconfig, bps, spp, fillorder,
+                tifP, photomet, planarconfig, bps, spp, fillorder,
                 colormap, &status);
         }
-        if (status != CONV_DONE) {
+        if (status == CONV_DONE) {
+            if (bps > 8)
+                pm_message("actual resolution has been reduced to 24 bits "
+                           "per pixel in the conversion.  You can get the "
+                           "full %u bits that are in the TIFF with the "
+                           "-byrow option.", bps);
+        } else {
             if (status != CONV_NOTATTEMPTED)
                 pm_message("In-memory conversion failed; "
                            "using more primitive row-by-row conversion.");
             
             convertRasterByRows(
                 imageoutFile, alphaFile, cols, rows, maxval, format, 
-                tif, photomet, planarconfig, bps, spp, fillorder, colormap);
-        }            
+                tifP, photomet, planarconfig, bps, spp, fillorder, colormap);
+        }
+    }
+}
+
+
+
+static void
+convertIt(TIFF *             const tifP,
+          FILE *             const alphaFile, 
+          FILE *             const imageoutFile,
+          struct cmdlineInfo const cmdline) {
+
+    unsigned int imageSeq;
+    bool eof;
+
+    imageSeq = 0;
+    eof = FALSE;
+
+    while (!eof) {
+        bool success;
+
+        if (cmdline.verbose)
+            pm_message("Converting Image %u", imageSeq);
+        convertImage(tifP, alphaFile, imageoutFile, cmdline);
+        success = TIFFReadDirectory(tifP);
+        eof = !success;
+        ++imageSeq;
     }
 }
 
@@ -951,30 +1015,31 @@ convertIt(TIFF*              const tif,
 int
 main(int argc, char * argv[]) {
 
-    TIFF* tif;
-    FILE *alphaFile, *imageoutFile;
+    struct cmdlineInfo cmdline;
+    TIFF * tif;
+    FILE * alphaFile;
+    FILE * imageoutFile;
 
     pnm_init( &argc, argv );
 
     parseCommandLine(argc, argv, &cmdline);
 
-    if ( cmdline.inputFilename != NULL ) {
-        tif = TIFFOpen( cmdline.inputFilename, "r" );
-        if ( tif == NULL )
-            pm_error( "error opening TIFF file %s", cmdline.inputFilename );
+    if (!STREQ(cmdline.inputFilename, "-")) {
+        tif = TIFFOpen(cmdline.inputFilename, "r");
+        if (tif == NULL)
+            pm_error("error opening TIFF file %s", cmdline.inputFilename);
     } else {
-        tif = TIFFFdOpen( 0, "Standard Input", "r" );
-        if ( tif == NULL )
-            pm_error( "error opening standard input as TIFF file" );
+        tif = TIFFFdOpen(0, "Standard Input", "r");
+        if (tif == NULL)
+            pm_error("error opening standard input as TIFF file");
     }
 
     if (cmdline.alphaStdout)
         alphaFile = stdout;
     else if (cmdline.alphaFilename == NULL) 
         alphaFile = NULL;
-    else {
+    else
         alphaFile = pm_openw(cmdline.alphaFilename);
-    }
 
     if (cmdline.alphaStdout) 
         imageoutFile = NULL;
@@ -988,9 +1053,10 @@ main(int argc, char * argv[]) {
     if (alphaFile != NULL)
         pm_close( alphaFile );
 
+    strfree(cmdline.inputFilename);
+
     /* If the program failed, it previously aborted with nonzero completion
        code, via various function calls.
     */
     return 0;
 }
-
