@@ -40,5 +40,5 @@ main(int argc, char *argv[]) {
 
     pm_close(stdin);
 
-    exit(0);
+    return 0;
 }
