@@ -21,7 +21,7 @@
  *  - "-stdpalette" option to enforce this
  *  - row-by-row operation (PPM output)
  *  11/Dec/94:
- *  - support for 24bit and 32bit (24bit + 8bit intensity) images
+ *  - capability for 24bit and 32bit (24bit + 8bit intensity) images
  *  - row-by-row operation (PCX input, for 16-color and truecolor images)
  *  - some code restructuring
  *  15/Feb/95:
@@ -31,8 +31,9 @@
  *  - moved check of 16-color-palette into pcx_16col_to_ppm(),
  *    now checks if it contains only a single color
  */
-#include "ppm.h"
+#include "mallocvar.h"
 #include "shhopt.h"
+#include "ppm.h"
 
 #define PCX_MAGIC       0x0a            /* PCX magic number             */
 #define PCX_HDR_SIZE    128             /* size of PCX header           */
@@ -40,16 +41,6 @@
 
 #define PCX_MAXVAL      (pixval)255
 
-/* prototypes */
-static void pcx_16col_to_ppm ARGS((FILE *ifp, int cols, int rows,
-                int BytesPerLine, int BitsPerPixel, int Planes, pixel *cmap));
-static void pcx_256col_to_ppm ARGS((FILE *ifp, int cols, int rows, int BytesPerLine));
-static void pcx_truecol_to_ppm ARGS((FILE *ifp, int cols, int rows, int BytesPerLine, int Planes));
-static void GetPCXRow ARGS((FILE *ifp, unsigned char *pcxrow, int bytesperline));
-static void pcx_planes_to_pixels ARGS(( unsigned char *pixels, unsigned char *bitplanes, int bytesperline, int planes, int bitsperpixel ));
-static void pcx_unpack_pixels ARGS(( unsigned char *pixels, unsigned char *bitplanes, int bytesperline, int planes, int bitsperpixel ));
-static int GetByte ARGS(( FILE *fp ));
-
 /* standard palette */
 static unsigned char const StdRed[]   = { 0, 255,   0,   0, 170, 170, 170, 170, 85,  85,  85,  85, 255, 255, 255, 255 };
 static unsigned char const StdGreen[] = { 0, 255, 170, 170,   0,   0, 170, 170, 85,  85, 255, 255,  85,  85, 255, 255 };
@@ -164,6 +155,19 @@ readInt(FILE * const ifP, short * const retvalP) {
 
 
 
+static int
+GetByte(FILE * const fp) {
+
+    int    c;
+
+    if ((c = fgetc(fp)) == EOF)
+        pm_error("unexpected end of file" );
+
+    return c;
+}
+
+
+
 static void
 readPcxHeader(FILE *             const ifP, 
               struct pcxHeader * const pcxHeaderP) {
@@ -203,10 +207,9 @@ readPcxHeader(FILE *             const ifP,
          * get the 16-color color map
          */
         for (i = 0; i < 16; i++) {
-            int r, g, b;
-            r = GetByte(ifP);
-            g = GetByte(ifP);
-            b = GetByte(ifP);
+            unsigned int const r = GetByte(ifP);
+            unsigned int const g = GetByte(ifP);
+            unsigned int const b = GetByte(ifP);
             PPM_ASSIGN(pcxHeaderP->cmap16[i], r, g, b);
         }
     }
@@ -240,339 +243,83 @@ reportPcxHeader(struct pcxHeader const pcxHeader) {
     pm_message("Planes: %d    BytesPerLine: %d    PaletteInfo: %d", 
                pcxHeader.Planes, pcxHeader.BytesPerLine, 
                pcxHeader.PaletteInfo);
-}
-
-
-
-int
-main(int argc, char *argv[]) {
-
-    FILE *ifP;
-    struct cmdlineInfo cmdline;
-    struct pcxHeader pcxHeader;
-    int Width, Height;
-    pixel *cmap16;
-
-    ppm_init( &argc, argv );
-
-    generateStdPalette();
-
-    parseCommandLine(argc, argv, &cmdline);
-
-    ifP = pm_openr(cmdline.inputFilespec);
-
-    readPcxHeader(ifP, &pcxHeader);
-
-    if (cmdline.verbose)
-        reportPcxHeader(pcxHeader);
-
-    Width  = (pcxHeader.Xmax - pcxHeader.Xmin) + 1;
-    Height = (pcxHeader.Ymax - pcxHeader.Ymin) + 1;
-
-    if (cmdline.stdpalette) 
-        cmap16 = stdPalette;
-    else
-        cmap16 = pcxHeader.cmap16;
-
-    ppm_writeppminit(stdout, Width, Height, PCX_MAXVAL, 0);
-    switch (pcxHeader.BitsPerPixel) {
-    case 1:
-        if(pcxHeader.Planes >= 1 && pcxHeader.Planes <= 4)
-            pcx_16col_to_ppm(ifP, Width, Height, pcxHeader.BytesPerLine, 
-                             pcxHeader.BitsPerPixel, pcxHeader.Planes, cmap16);
-        else
-            goto fail;
-        break;
-    case 2:
-    case 4:
-        if (pcxHeader.Planes == 1)
-            pcx_16col_to_ppm(ifP, Width, Height, pcxHeader.BytesPerLine, 
-                             pcxHeader.BitsPerPixel, pcxHeader.Planes, cmap16);
-        else
-            goto fail;
-        break;
-    case 8:
-        switch(pcxHeader.Planes) {
-        case 1:
-            pcx_256col_to_ppm(ifP, Width, Height, pcxHeader.BytesPerLine);
-            break;
-        case 3:
-        case 4:
-            pcx_truecol_to_ppm(ifP, Width, Height, 
-                               pcxHeader.BytesPerLine, pcxHeader.Planes);
-            break;
-        default:
-            goto fail;
-        }
-        break;
-    default:
-    fail:
-        pm_error("can't handle %d bits per pixel image with %d planes",
-                 pcxHeader.BitsPerPixel, pcxHeader.Planes);
-    }
-    pm_close(ifP);
-    exit(0);
-}
-
-
-static void
-pcx_16col_to_ppm(ifp, cols, rows, BytesPerLine, BitsPerPixel, Planes, cmap)
-    FILE *ifp;
-    int cols, rows;
-    int BytesPerLine, BitsPerPixel, Planes;
-    pixel *cmap;    /* colormap from header */
-{
-    int row, col, rawcols, colors;
-    unsigned char *pcxrow, *rawrow;
-    pixel *ppmrow;
-    short palette_ok = 0;
-
-#ifdef DEBUG
-    pm_message("16 color -> ppm");
-#endif
-
-    /* check if palette is ok  */
-    colors = (1 << BitsPerPixel) * (1 << Planes);
-    for( col = 0; col < colors-1; col++ ) {
-        if( !PPM_EQUAL(cmap[col], cmap[col+1]) ) {
-            palette_ok = 1;
-            break;
-        }
-    }
-    if( !palette_ok ) {
-        pm_message("warning - useless header palette, "
-                   "using builtin standard palette");
-        for( col = 0; col < colors; col++ )
-            PPM_ASSIGN(cmap[col], StdRed[col], StdGreen[col], StdBlue[col]);
-    }
-
-    /*  BytesPerLine should be >= BitsPerPixel * cols / 8  */
-    rawcols = BytesPerLine * 8 / BitsPerPixel;
-    if( cols > rawcols ) {
-        pm_message("warning - BytesPerLine = %d, "
-                   "truncating image to %d pixels",
-                   BytesPerLine, rawcols);
-        cols = rawcols;
-    }
-    pcxrow = (unsigned char *)
-        pm_allocrow(Planes * BytesPerLine, sizeof(unsigned char));
-    rawrow = (unsigned char *)pm_allocrow(rawcols, sizeof(unsigned char));
-    ppmrow = ppm_allocrow(cols);
-
-    for( row = 0; row < rows; row++ ) {
-        GetPCXRow(ifp, pcxrow, Planes * BytesPerLine);
-
-        if (Planes == 1)
-            pcx_unpack_pixels(rawrow, pcxrow, BytesPerLine, 
-                              Planes, BitsPerPixel);
-        else
-            pcx_planes_to_pixels(rawrow, pcxrow, BytesPerLine, 
-                                 Planes, BitsPerPixel);
-
-        for( col = 0; col < cols; col++ )
-            ppmrow[col] = cmap[rawrow[col]];
-        ppm_writeppmrow(stdout, ppmrow, cols, PCX_MAXVAL, 0);
-    }
-
-#ifdef DEBUG
-    pm_message("done!");
-#endif
-    ppm_freerow(ppmrow);
-    pm_freerow((char*)rawrow);
-    pm_freerow((void*)pcxrow);
-}
-
-
-
-static void
-pcx_256col_to_ppm(ifp, cols, rows, BytesPerLine)
-    FILE *ifp;
-    int cols, rows;
-    int BytesPerLine;
-{
-    pixel colormap[256];
-    pixel *ppmrow;
-    unsigned char **image;
-    unsigned char colormapSignature;
-    int row, col;
-
-#ifdef DEBUG
-    pm_message("256 color -> ppm, reading index array...");
-#endif
-
-    if( cols > BytesPerLine ) {
-        pm_message("warning - BytesPerLine = %d, "
-                   "truncating image to %d pixels",
-                   BytesPerLine,  BytesPerLine);
-        cols = BytesPerLine;
-    }
-
-    image = (unsigned char **)pm_allocarray(BytesPerLine, rows, 
-                                            sizeof(unsigned char));
-    for( row = 0; row < rows; row++ )
-        GetPCXRow(ifp, image[row], BytesPerLine);
-
-#ifdef DEBUG
-    pm_message("ok, now reading colormap...");
-#endif
-
-    /*
-     * 256 color images have their color map at the end of the file
-     * preceeded by a magic byte
-     */
-    colormapSignature = GetByte(ifp);
-    if (colormapSignature != PCX_256_COLORS)
-        pm_error("bad color map signature.  In a 1-plane PCX image "
-                 "such as this, we expect a magic number of %u in the byte "
-                 "following the raster, to introduce the color map.  "
-                 "Instead, this image has %u.", 
-                 PCX_256_COLORS, colormapSignature);
-    else {
+    pm_message("Color map in image:  (index: r/g/b)");
+    if (pcxHeader.BitsPerPixel < 8) {
         unsigned int colorIndex;
-
-        for(colorIndex = 0; colorIndex < 256; ++colorIndex) {
-            int r, g, b;
-            r = GetByte(ifp);
-            g = GetByte(ifp);
-            b = GetByte(ifp);
-            PPM_ASSIGN(colormap[colorIndex], r, g, b);
+        for (colorIndex = 0; colorIndex < 16; ++colorIndex) {
+            pixel const p = pcxHeader.cmap16[colorIndex];
+            pm_message("  %u: %u/%u/%u", colorIndex, 
+                       PPM_GETR(p), PPM_GETG(p), PPM_GETB(p));
         }
     }
-#ifdef DEBUG
-    pm_message("ok, converting...");
-#endif
-
-    ppmrow = ppm_allocrow(cols);
-    for( row = 0; row < rows; row++ ) {
-        for( col = 0; col < cols; col++ )
-            ppmrow[col] = colormap[image[row][col]];
-        ppm_writeppmrow(stdout, ppmrow, cols, PCX_MAXVAL, 0);
-    }
-#ifdef DEBUG
-    pm_message("done!");
-#endif
-
-    ppm_freerow(ppmrow);
-    pm_freearray((void *)image, rows);
 }
 
 
 
-static void
-pcx_truecol_to_ppm(ifp, cols, rows, BytesPerLine, Planes)
-    FILE *ifp;
-    int cols, rows;
-    int BytesPerLine, Planes;
-{
-    unsigned char *redrow, *greenrow, *bluerow, *intensityrow;
-    pixel *ppmrow;
-    int row, col;
-    int r, g, b, i;
+static bool
+allBlackPalette(pixel cmap16[]) {
 
-#ifdef DEBUG
-    pm_message("%dbit truecolor -> ppm", Planes * 8);
-#endif
+    unsigned int colorIndex;
+    bool allBlack;
 
-    if( cols > BytesPerLine ) {
-        pm_message("warning - BytesPerLine = %d, "
-                   "truncating image to %d pixels",
-                   BytesPerLine,  BytesPerLine);
-        cols = BytesPerLine;
+    allBlack = TRUE;  /* initial assumption */
+    for (colorIndex = 0; colorIndex < 16; ++colorIndex) {
+        pixel const p = cmap16[colorIndex];
+
+        if (PPM_GETR(p) != 0 ||
+            PPM_GETG(p) != 0 ||
+            PPM_GETB(p) != 0)
+
+            allBlack = FALSE;
     }
-
-    redrow   = (unsigned char *)
-        pm_allocrow(BytesPerLine, sizeof(unsigned char*));
-    greenrow = (unsigned char *)
-        pm_allocrow(BytesPerLine, sizeof(unsigned char*));
-    bluerow  = (unsigned char *)
-        pm_allocrow(BytesPerLine, sizeof(unsigned char*));
-    if( Planes == 4 )
-        intensityrow = (unsigned char *)
-            pm_allocrow(BytesPerLine, sizeof(unsigned char*));
-    else
-        intensityrow = (unsigned char *)NULL;
-
-    ppmrow = ppm_allocrow(cols);
-    for( row = 0; row < rows; row++ ) {
-        GetPCXRow(ifp, redrow, BytesPerLine);
-        GetPCXRow(ifp, greenrow, BytesPerLine);
-        GetPCXRow(ifp, bluerow, BytesPerLine);
-        if( intensityrow )
-            GetPCXRow(ifp, intensityrow, BytesPerLine);
-        for( col = 0; col < cols; col++ ) {
-            r = redrow[col];
-            g = greenrow[col];
-            b = bluerow[col];
-            if( intensityrow ) {
-                i = intensityrow[col];
-                r = r * i / 256;
-                g = g * i / 256;
-                b = b * i / 256;
-            }
-            PPM_ASSIGN(ppmrow[col], r, g, b);
-        }
-        ppm_writeppmrow(stdout, ppmrow, cols, PCX_MAXVAL, 0);
-    }
-#ifdef DEBUG
-    pm_message("done!");
-#endif
-
-    ppm_freerow(ppmrow);
-    if( intensityrow )
-        pm_freerow((void*)intensityrow);
-    pm_freerow((void*)bluerow);
-    pm_freerow((void*)greenrow);
-    pm_freerow((void*)redrow);
+    return allBlack;
 }
 
 
+
 /*
- * convert multi-plane format into 1 pixel per byte
+ *  read a single row encoded row, handles encoding across rows
  */
 static void
-pcx_planes_to_pixels(pixels, bitplanes, bytesperline, planes, bitsperpixel)
-    unsigned char   *pixels;
-    unsigned char   *bitplanes;
-    int             bytesperline;
-    int             planes;
-    int             bitsperpixel;
-{
-    int  i, j;
-    int  npixels;
-    unsigned char    *p;
+GetPCXRow(FILE * const ifP, unsigned char * const pcxrow, 
+          int const bytesperline) {
+/*----------------------------------------------------------------------------
+   Read one row from the PCX raster.
+   
+   The PCX raster is run length encoded as follows:  If the upper two
+   bits of a byte are 11, the lower 6 bits are a repetition count for the
+   raster byte that follows.  If the upper two bits are not 11, the byte 
+   _is_ a raster byte, with repetition count 1.
 
-    if (planes > 4)
-        pm_error("can't handle more than 4 planes" );
-    if (bitsperpixel != 1)
-        pm_error("can't handle more than 1 bit per pixel" );
+   A run can't span rows, but it can span planes within a row.  That's
+   why 'repetitionsLeft' and 'c' are static variables in this
+   subroutine.
+-----------------------------------------------------------------------------*/
+    static int repetitionsLeft = 0;
+    static int c;
+    int bytesGenerated;
 
-    /*
-     * clear the pixel buffer
-     */
-    npixels = (bytesperline * 8) / bitsperpixel;
-    p    = pixels;
-    while (--npixels >= 0)
-        *p++ = 0;
-
-    /*
-     * do the format conversion
-     */
-    for (i = 0; i < planes; i++)
-    {
-        int pixbit, bits, mask;
-
-        p    = pixels;
-        pixbit    = (1 << i);
-        for (j = 0; j < bytesperline; j++)
-        {
-            bits = *bitplanes++;
-            for (mask = 0x80; mask != 0; mask >>= 1, p++)
-                if (bits & mask)
-                    *p |= pixbit;
+    bytesGenerated = 0;
+    while( bytesGenerated < bytesperline ) {
+        if(repetitionsLeft > 0) {
+            pcxrow[bytesGenerated++] = c;
+            --repetitionsLeft;
+        } else {
+            c = GetByte(ifP);
+            if ((c & 0xc0) != 0xc0)
+                /* This is a 1-shot byte, not a repetition count */
+                pcxrow[bytesGenerated++] = c;
+            else {
+                /* This is a repetition count for the following byte */
+                repetitionsLeft = c & 0x3f;
+                c = GetByte(ifP);
+            }
         }
     }
 }
 
+
+
 /*
  * convert packed pixel format into 1 pixel per byte
  */
@@ -637,64 +384,327 @@ pcx_unpack_pixels(pixels, bitplanes, bytesperline, planes, bitsperpixel)
 }
 
 
-static int rowsRead=0;
-
-static int
-GetByte(FILE * const fp) {
-
-    int    c;
-
-    if ((c = fgetc(fp)) == EOF)
-        pm_error("unexpected end of file" );
-
-    return c;
-}
-
-
 
 /*
- *  read a single row encoded row, handles encoding across rows
+ * convert multi-plane format into 1 pixel per byte
  */
 static void
-GetPCXRow(FILE * const ifP, unsigned char * const pcxrow, 
-          int const bytesperline) {
-/*----------------------------------------------------------------------------
-   Read one row from the PCX raster.
-   
-   The PCX raster is run length encoded as follows:  If the upper two
-   bits of a byte are 11, the lower 6 bits are a repetition count for the
-   raster byte that follows.  If the upper two bits are not 11, the byte 
-   _is_ a raster byte, with repetition count 1.
+pcx_planes_to_pixels(pixels, bitplanes, bytesperline, planes, bitsperpixel)
+    unsigned char   *pixels;
+    unsigned char   *bitplanes;
+    int             bytesperline;
+    int             planes;
+    int             bitsperpixel;
+{
+    int  i, j;
+    int  npixels;
+    unsigned char    *p;
 
-   A run can't span rows, but it can span planes within a row.  That's
-   why 'repetitionsLeft' and 'c' are static variables in this
-   subroutine.
------------------------------------------------------------------------------*/
-    static int repetitionsLeft = 0;
-    static int c;
-    int bytesGenerated;
+    if (planes > 4)
+        pm_error("can't handle more than 4 planes" );
+    if (bitsperpixel != 1)
+        pm_error("can't handle more than 1 bit per pixel" );
 
-    bytesGenerated = 0;
-    while( bytesGenerated < bytesperline ) {
-        if(repetitionsLeft > 0) {
-            pcxrow[bytesGenerated++] = c;
-            --repetitionsLeft;
-        } else {
- if (rowsRead==241)
-     pm_message("Reading Byte %ld", ftell(ifP));
-            c = GetByte(ifP);
-            if ((c & 0xc0) != 0xc0)
-                /* This is a 1-shot byte, not a repetition count */
-                pcxrow[bytesGenerated++] = c;
-            else {
-                /* This is a repetition count for the following byte */
-                repetitionsLeft = c & 0x3f;
-if (rowsRead == 241)
-pm_message("doing run of %d of Byte %ld in Row 241", repetitionsLeft, ftell(ifP));
-                c = GetByte(ifP);
-            }
+    /*
+     * clear the pixel buffer
+     */
+    npixels = (bytesperline * 8) / bitsperpixel;
+    p    = pixels;
+    while (--npixels >= 0)
+        *p++ = 0;
+
+    /*
+     * do the format conversion
+     */
+    for (i = 0; i < planes; i++)
+    {
+        int pixbit, bits, mask;
+
+        p    = pixels;
+        pixbit    = (1 << i);
+        for (j = 0; j < bytesperline; j++)
+        {
+            bits = *bitplanes++;
+            for (mask = 0x80; mask != 0; mask >>= 1, p++)
+                if (bits & mask)
+                    *p |= pixbit;
         }
     }
-rowsRead++;
 }
 
+
+
+static void
+pcx_16col_to_ppm(FILE *       const ifP,
+                 unsigned int const headerCols,
+                 unsigned int const rows,
+                 unsigned int const BytesPerLine,
+                 unsigned int const BitsPerPixel,
+                 unsigned int const Planes,
+                 pixel *      const cmap) {
+
+    unsigned int cols;
+    int row, col, rawcols, colors;
+    unsigned char * pcxrow;
+    unsigned char * rawrow;
+    pixel * ppmrow;
+    bool paletteOk;
+
+    paletteOk = FALSE;
+
+    /* check if palette is ok  */
+    colors = (1 << BitsPerPixel) * (1 << Planes);
+    for (col = 0; col < colors - 1; ++col) {
+        if (!PPM_EQUAL(cmap[col], cmap[col+1])) {
+            paletteOk = TRUE;
+            break;
+        }
+    }
+    if (!paletteOk) {
+        unsigned int col;
+        pm_message("warning - useless header palette, "
+                   "using builtin standard palette");
+        for (col = 0; col < colors; ++col)
+            PPM_ASSIGN(cmap[col], StdRed[col], StdGreen[col], StdBlue[col]);
+    }
+
+    /*  BytesPerLine should be >= BitsPerPixel * cols / 8  */
+    rawcols = BytesPerLine * 8 / BitsPerPixel;
+    if (headerCols > rawcols) {
+        pm_message("warning - BytesPerLine = %d, "
+                   "truncating image to %d pixels",
+                   BytesPerLine, rawcols);
+        cols = rawcols;
+    } else
+        cols = headerCols;
+
+    MALLOCARRAY(pcxrow, Planes * BytesPerLine);
+    if (pcxrow == NULL)
+        pm_error("Out of memory");
+    MALLOCARRAY(rawrow, rawcols);
+    if (rawrow == NULL)
+        pm_error("Out of memory");
+
+    ppmrow = ppm_allocrow(cols);
+
+    for (row = 0; row < rows; ++row) {
+        unsigned int col;
+
+        GetPCXRow(ifP, pcxrow, Planes * BytesPerLine);
+
+        if (Planes == 1)
+            pcx_unpack_pixels(rawrow, pcxrow, BytesPerLine, 
+                              Planes, BitsPerPixel);
+        else
+            pcx_planes_to_pixels(rawrow, pcxrow, BytesPerLine, 
+                                 Planes, BitsPerPixel);
+
+        for (col = 0; col < cols; ++col)
+            ppmrow[col] = cmap[rawrow[col]];
+        ppm_writeppmrow(stdout, ppmrow, cols, PCX_MAXVAL, 0);
+    }
+    ppm_freerow(ppmrow);
+    free(rawrow);
+    free(pcxrow);
+}
+
+
+
+static void
+pcx_256col_to_ppm(FILE *       const ifP,
+                  unsigned int const headerCols,
+                  unsigned int const rows,
+                  unsigned int const BytesPerLine) {
+
+    unsigned int cols;
+    pixel colormap[256];
+    pixel * ppmrow;
+    unsigned char ** image;
+    unsigned char colormapSignature;
+    unsigned int row;
+
+    if (headerCols > BytesPerLine) {
+        pm_message("warning - BytesPerLine = %u, "
+                   "truncating image to %u pixels",
+                   BytesPerLine,  BytesPerLine);
+        cols = BytesPerLine;
+    } else
+        cols = headerCols;
+
+    image = (unsigned char **)pm_allocarray(BytesPerLine, rows, 
+                                            sizeof(unsigned char));
+    for (row = 0; row < rows; ++row)
+        GetPCXRow(ifP, image[row], BytesPerLine);
+
+    /*
+     * 256 color images have their color map at the end of the file
+     * preceeded by a magic byte
+     */
+    colormapSignature = GetByte(ifP);
+    if (colormapSignature != PCX_256_COLORS)
+        pm_error("bad color map signature.  In a 1-plane PCX image "
+                 "such as this, we expect a magic number of %u in the byte "
+                 "following the raster, to introduce the color map.  "
+                 "Instead, this image has %u.", 
+                 PCX_256_COLORS, colormapSignature);
+    else {
+        unsigned int colorIndex;
+        
+        for (colorIndex = 0; colorIndex < 256; ++colorIndex) {
+            pixval const r = GetByte(ifP);
+            pixval const g = GetByte(ifP);
+            pixval const b = GetByte(ifP);
+            PPM_ASSIGN(colormap[colorIndex], r, g, b);
+        }
+    }
+
+    ppmrow = ppm_allocrow(cols);
+    for (row = 0; row < rows; ++row) {
+        unsigned int col;
+        for (col = 0; col < cols; ++col)
+            ppmrow[col] = colormap[image[row][col]];
+        ppm_writeppmrow(stdout, ppmrow, cols, PCX_MAXVAL, 0);
+    }
+
+    ppm_freerow(ppmrow);
+    pm_freearray((void *)image, rows);
+}
+
+
+
+static void
+pcx_truecol_to_ppm(FILE *       const ifP,
+                   unsigned int const headerCols,
+                   unsigned int const rows,
+                   unsigned int const BytesPerLine,
+                   unsigned int const Planes) {
+
+    unsigned int cols;
+    unsigned char *redrow, *greenrow, *bluerow, *intensityrow;
+    pixel * ppmrow;
+    unsigned int row;
+    
+    if (headerCols > BytesPerLine) {
+        pm_message("warning - BytesPerLine = %u, "
+                   "truncating image to %u pixels",
+                   BytesPerLine,  BytesPerLine);
+        cols = BytesPerLine;
+    } else
+        cols = headerCols;
+
+    MALLOCARRAY(redrow, BytesPerLine);
+    MALLOCARRAY(greenrow, BytesPerLine);
+    MALLOCARRAY(bluerow, BytesPerLine);
+
+    if (redrow == NULL || greenrow == NULL || bluerow == NULL)
+        pm_error("out of memory");
+
+    if (Planes == 4) {
+        MALLOCARRAY(intensityrow, BytesPerLine);
+        if (intensityrow == NULL)
+            pm_error("out of memory");
+    } else
+        intensityrow = NULL;
+
+    ppmrow = ppm_allocrow(cols);
+    for (row = 0; row < rows; ++row) {
+        unsigned int col;
+        GetPCXRow(ifP, redrow, BytesPerLine);
+        GetPCXRow(ifP, greenrow, BytesPerLine);
+        GetPCXRow(ifP, bluerow, BytesPerLine);
+        if( intensityrow )
+            GetPCXRow(ifP, intensityrow, BytesPerLine);
+        for (col = 0; col < cols; ++col) {
+            unsigned int const r = redrow[col];
+            unsigned int const g = greenrow[col];
+            unsigned int const b = bluerow[col];
+            unsigned int const i = intensityrow ? intensityrow[col] : 256;
+
+            PPM_ASSIGN(ppmrow[col],
+                       r * i / 256, g * i / 256, b * i / 256);
+        }
+        ppm_writeppmrow(stdout, ppmrow, cols, PCX_MAXVAL, 0);
+    }
+
+    ppm_freerow(ppmrow);
+    if (intensityrow)
+        free(intensityrow);
+    free(bluerow);
+    free(greenrow);
+    free(redrow);
+}
+
+
+
+int
+main(int argc, char *argv[]) {
+
+    FILE * ifP;
+    struct cmdlineInfo cmdline;
+    struct pcxHeader pcxHeader;
+    unsigned int Width, Height;
+    pixel * cmap16;
+
+    ppm_init(&argc, argv);
+
+    generateStdPalette();
+
+    parseCommandLine(argc, argv, &cmdline);
+
+    ifP = pm_openr(cmdline.inputFilespec);
+
+    readPcxHeader(ifP, &pcxHeader);
+
+    if (cmdline.verbose)
+        reportPcxHeader(pcxHeader);
+
+    Width  = (pcxHeader.Xmax - pcxHeader.Xmin) + 1;
+    Height = (pcxHeader.Ymax - pcxHeader.Ymin) + 1;
+
+    if (cmdline.stdpalette || allBlackPalette(pcxHeader.cmap16))
+        cmap16 = stdPalette;
+    else
+        cmap16 = pcxHeader.cmap16;
+
+    ppm_writeppminit(stdout, Width, Height, PCX_MAXVAL, 0);
+    switch (pcxHeader.BitsPerPixel) {
+    case 1:
+        if(pcxHeader.Planes >= 1 && pcxHeader.Planes <= 4)
+            pcx_16col_to_ppm(ifP, Width, Height, pcxHeader.BytesPerLine, 
+                             pcxHeader.BitsPerPixel, pcxHeader.Planes, cmap16);
+        else
+            goto fail;
+        break;
+    case 2:
+    case 4:
+        if (pcxHeader.Planes == 1)
+            pcx_16col_to_ppm(ifP, Width, Height, pcxHeader.BytesPerLine, 
+                             pcxHeader.BitsPerPixel, pcxHeader.Planes, cmap16);
+        else
+            goto fail;
+        break;
+    case 8:
+        switch(pcxHeader.Planes) {
+        case 1:
+            pcx_256col_to_ppm(ifP, Width, Height, pcxHeader.BytesPerLine);
+            break;
+        case 3:
+        case 4:
+            pcx_truecol_to_ppm(ifP, Width, Height, 
+                               pcxHeader.BytesPerLine, pcxHeader.Planes);
+            break;
+        default:
+            goto fail;
+        }
+        break;
+    default:
+    fail:
+        pm_error("can't handle %d bits per pixel image with %d planes",
+                 pcxHeader.BitsPerPixel, pcxHeader.Planes);
+    }
+    pm_close(ifP);
+    
+    return 0;
+}
+
+
