@@ -51,10 +51,8 @@ int psearchAlg;
 
 /*===========================================================================*
  *
- * PMotionSearch
- *
- *  compute the best P-frame motion vector we can.  If it's better than
- *  (*motionXP, *motionYP), update *motionXP and *motionYP to it.
+ *  Compute the best P-frame motion vector we can.  If it's better than
+ *  *motionP, update *motionP to it.
  *
  * PRECONDITIONS:   The relevant block in 'current' is valid (it has not
  *          been dct'd).  Thus, the data in 'current' can be
@@ -79,37 +77,36 @@ int psearchAlg;
  *
  *===========================================================================*/
 void
-PMotionSearch(LumBlock currentBlock, 
-              MpegFrame * const prev, 
-              int const by, int const bx, 
-              int * const motionYP, int * const motionXP) {
+PMotionSearch(const LumBlock * const currentBlockP, 
+              MpegFrame *      const prev, 
+              int              const by,
+              int              const bx, 
+              vector *         const motionP) {
 
     /* CALL SEARCH PROCEDURE */
 
     switch(psearchAlg) {
     case PSEARCH_SUBSAMPLE:
-        PSubSampleSearch(currentBlock, prev, by, bx, 
-                         motionYP, motionXP, searchRangeP);
+        PSubSampleSearch(currentBlockP, prev, by, bx, motionP, searchRangeP);
         break;
     case PSEARCH_EXHAUSTIVE:
-        PLocalSearch(currentBlock, prev, by, bx, 
-                     motionYP, motionXP, INT_MAX, searchRangeP);
+        PLocalSearch(currentBlockP, prev, by, bx, 
+                     motionP, INT_MAX, searchRangeP);
         break;
     case PSEARCH_LOGARITHMIC:
-        PLogarithmicSearch(currentBlock, prev, by, bx, 
-                           motionYP, motionXP, searchRangeP);
+        PLogarithmicSearch(currentBlockP, prev, by, bx, motionP, searchRangeP);
         break;
     case PSEARCH_TWOLEVEL:
-        PTwoLevelSearch(currentBlock, prev, by, bx, 
-                        motionYP, motionXP, INT_MAX, searchRangeP);
+        PTwoLevelSearch(currentBlockP, prev, by, bx, 
+                        motionP, INT_MAX, searchRangeP);
         break;
     default:
-        fprintf(stderr, "ILLEGAL PSEARCH ALG:  %d\n", psearchAlg);
-        exit(1);
+        pm_error("IMPOSSIBLE PSEARCH ALG:  %d", psearchAlg);
     }
 }
 
 
+
 /*===========================================================================*
  *
  * SetPixelSearch
@@ -248,9 +245,7 @@ SetSearchRange(int const pixelsP, int const pixelsB) {
  *
  *===========================================================================*/
 void
-MotionSearchPreComputation(frame)
-    MpegFrame *frame;
-{
+MotionSearchPreComputation(MpegFrame * const frameP) {
     /* do nothing */
 }
 
@@ -271,17 +266,20 @@ MotionSearchPreComputation(frame)
  *
  *===========================================================================*/
 int
-PSubSampleSearch(LumBlock currentBlock, MpegFrame *prev, 
-                 int by, int bx, int * motionYP, int * motionXP, 
-                 int searchRange)
-{
-    register int mx, my;
-    int diff, bestBestDiff;
-    int     stepSize;
-    register int x;
-    int     bestMY[4], bestMX[4], bestDiff[4];
-    int     leftMY, leftMX;
-    int     rightMY, rightMX;
+PSubSampleSearch(const LumBlock * const currentBlockP,
+                 MpegFrame *      const prev, 
+                 int              const by,
+                 int              const bx,
+                 vector *         const motionP,
+                 int              const searchRange) {
+    
+    int my, mx;
+    int bestBestDiff;
+    int stepSize;
+    int x;
+    int bestMY[4], bestMX[4], bestDiff[4];
+    int leftMY, leftMX;
+    int rightMY, rightMX;
 
     stepSize = (pixelFullSearch ? 2 : 1);
 
@@ -302,159 +300,159 @@ PSubSampleSearch(LumBlock currentBlock, MpegFrame *prev,
     }
 
     /* do A pattern */
-    for ( my = -searchRange; my < rightMY; my += 2*stepSize ) {
-        if ( my < leftMY ) {
-            continue;
-        }
-
-        for ( mx = -searchRange; mx < rightMX; mx += 2*stepSize ) {
-            if ( mx < leftMX ) {
-                continue;
-            }
-
-            diff = LumMotionErrorA(currentBlock, prev, by, bx, my, mx, 
-                                   bestDiff[0]);
-
-            if ( diff < bestDiff[0] ) {
-                bestMY[0] = my;
-                bestMX[0] = mx;
-                bestDiff[0] = diff;
+    for (my = -searchRange; my < rightMY; my += 2*stepSize) {
+        if (my >= leftMY) {
+            for ( mx = -searchRange; mx < rightMX; mx += 2*stepSize ) {
+                if (mx >= leftMX) {
+                    int diff;
+                    vector m;
+                    m.y = my; m.x = mx;
+                    diff = LumMotionErrorA(currentBlockP, prev, by, bx, m,
+                                           bestDiff[0]);
+                    
+                    if (diff < bestDiff[0]) {
+                        bestMY[0] = my;
+                        bestMX[0] = mx;
+                        bestDiff[0] = diff;
+                    }
+                }
             }
         }
     }
 
     /* do B pattern */
-    for ( my = stepSize-searchRange; my < rightMY; my += 2*stepSize ) {
-        if ( my < leftMY ) {
-            continue;
-        }
-
-        for ( mx = -searchRange; mx < rightMX; mx += 2*stepSize ) {
-            if ( mx < leftMX ) {
-                continue;
-            }
-
-            diff = LumMotionErrorB(currentBlock, prev, by, bx, my, mx, 
-                                   bestDiff[1]);
-
-            if ( diff < bestDiff[1] ) {
-                bestMY[1] = my;
-                bestMX[1] = mx;
-                bestDiff[1] = diff;
+    for (my = stepSize-searchRange; my < rightMY; my += 2*stepSize) {
+        if (my >= leftMY) {
+            for (mx = -searchRange; mx < rightMX; mx += 2*stepSize) {
+                if (mx >= leftMX) {
+                    int diff;
+                    vector m;
+                    m.y = my; m.x = mx;
+                    diff = LumMotionErrorB(currentBlockP, prev, by, bx, m, 
+                                           bestDiff[1]);
+                    
+                    if (diff < bestDiff[1]) {
+                        bestMY[1] = my;
+                        bestMX[1] = mx;
+                        bestDiff[1] = diff;
+                    }
+                }
             }
         }
     }
 
     /* do C pattern */
-    for ( my = stepSize-searchRange; my < rightMY; my += 2*stepSize ) {
-        if ( my < leftMY ) {
-            continue;
-        }
-
-        for ( mx = stepSize-searchRange; mx < rightMX; mx += 2*stepSize ) {
-            if ( mx < leftMX ) {
-                continue;
-            }
-
-            diff = LumMotionErrorC(currentBlock, prev, by, bx, my, mx, 
-                                   bestDiff[2]);
-
-            if ( diff < bestDiff[2] ) {
-                bestMY[2] = my;
-                bestMX[2] = mx;
-                bestDiff[2] = diff;
+    for (my = stepSize-searchRange; my < rightMY; my += 2*stepSize) {
+        if (my >= leftMY) {
+            for ( mx = stepSize-searchRange; mx < rightMX; mx += 2*stepSize ) {
+                if (mx >= leftMX) {
+                    int diff;
+                    vector m;
+                    m.y = my; m.x = mx;
+                    diff = LumMotionErrorC(currentBlockP, prev, by, bx, m,
+                                           bestDiff[2]);
+                    
+                    if (diff < bestDiff[2]) {
+                        bestMY[2] = my;
+                        bestMX[2] = mx;
+                        bestDiff[2] = diff;
+                    }
+                }
             }
         }
     }
 
     /* do D pattern */
-    for ( my = -searchRange; my < rightMY; my += 2*stepSize ) {
-        if ( my < leftMY ) {
-            continue;
-        }
-
-        for ( mx = stepSize-searchRange; mx < rightMX; mx += 2*stepSize ) {
-            if ( mx < leftMX ) {
-                continue;
-            }
-
-            diff = LumMotionErrorD(currentBlock, prev, by, bx, my, mx, 
-                                   bestDiff[3]);
-
-            if ( diff < bestDiff[3] ) {
-                bestMY[3] = my;
-                bestMX[3] = mx;
-                bestDiff[3] = diff;
+    for (my = -searchRange; my < rightMY; my += 2*stepSize) {
+        if (my >= leftMY) {
+            for (mx = stepSize-searchRange; mx < rightMX; mx += 2*stepSize) {
+                if (mx >= leftMX) {
+                    int diff;
+                    vector m;
+                    m.y = my; m.x = mx;
+                    diff = LumMotionErrorD(currentBlockP, prev, by, bx, m,
+                                           bestDiff[3]);
+                    
+                    if (diff < bestDiff[3]) {
+                        bestMY[3] = my;
+                        bestMX[3] = mx;
+                        bestDiff[3] = diff;
+                    }
+                }
             }
         }
     }
 
     /* first check old motion */
-    if ( (*motionYP >= leftMY) && (*motionYP < rightMY) &&
-         (*motionXP >= leftMX) && (*motionXP < rightMX) ) {
-        bestBestDiff = LumMotionError(currentBlock, prev, by, bx, 
-                                      *motionYP, *motionXP, INT_MAX);
-    } else {
+    if ((motionP->y >= leftMY) && (motionP->y < rightMY) &&
+        (motionP->x >= leftMX) && (motionP->x < rightMX)) {
+        bestBestDiff = LumMotionError(currentBlockP, prev, by, bx, 
+                                      *motionP, INT_MAX);
+    } else
         bestBestDiff = INT_MAX;
-    }
 
     /* look at Error of 4 different motion vectors */
-    for ( x = 0; x < 4; x++ ) {
-        bestDiff[x] = LumMotionError(currentBlock, prev, by, bx,
-                                     bestMY[x], bestMX[x], bestBestDiff);
+    for (x = 0; x < 4; ++x) {
+        vector m;
+        m.y = bestMY[x];
+        m.x = bestMX[x];
+        bestDiff[x] = LumMotionError(currentBlockP, prev, by, bx, m,
+                                     bestBestDiff);
 
-        if ( bestDiff[x] < bestBestDiff ) {
+        if (bestDiff[x] < bestBestDiff) {
             bestBestDiff = bestDiff[x];
-            *motionYP = bestMY[x];
-            *motionXP = bestMX[x];
+            *motionP     = m;
         }
     }
-
     return bestBestDiff;
 }
 
 
 
 static void
-findBestSpaced(int const minBoundY, int const minBoundX, 
-               int const maxBoundY, int const maxBoundX,
-               int const spacing,
-               LumBlock currentBlock, 
-               MpegFrame * const prev,
-               int const by, int const bx,
-               int * const bestDiffP, 
-               int * const centerYP, int * const centerXP) {
+findBestSpaced(int              const minMY,
+               int              const minMX,
+               int              const maxMY,
+               int              const maxMX,
+               int              const spacing,
+               const LumBlock * const currentBlockP, 
+               MpegFrame *      const prev,
+               int              const by,
+               int              const bx,
+               int *            const bestDiffP, 
+               vector *         const centerP) {
 /*----------------------------------------------------------------------------
    Examine every 'spacing'th half-pixel within the rectangle 
    ('minBoundX', 'minBoundY', 'maxBoundX', 'maxBoundY'), 
 
    If one of the half-pixels examined has a lower "LumMotionError" value
    than *bestDiffP, update *bestDiffP to that value and update
-   *centerYP and *centerXP to the location of that half-pixel.
+   *centerP to the location of that half-pixel.
 -----------------------------------------------------------------------------*/
+    int const minBoundY = MAX(minMY, centerP->y - spacing);
+    int const minBoundX = MAX(minMX, centerP->x - spacing);
+    int const maxBoundY = MIN(maxMY, centerP->y + spacing + 1);
+    int const maxBoundX = MIN(maxMX, centerP->x + spacing + 1);
+
     int my;
-    int bestDiff;
-    
-    bestDiff = *bestDiffP; /* initial value */
 
     for (my = minBoundY; my < maxBoundY; my += spacing) {
         int mx;
 
         for (mx = minBoundX; mx < maxBoundX; mx += spacing) {
             int diff;
+            vector m;
+
+            m.y = my; m.x = mx;
             
-            diff = LumMotionError(currentBlock, prev, by, bx, my, mx, 
-                                  bestDiff);
+            diff = LumMotionError(currentBlockP, prev, by, bx, m, *bestDiffP);
             
-            if ( diff < bestDiff ) {
-                *centerYP = my;
-                *centerXP = mx;
-                
-                bestDiff = diff;
+            if (diff < *bestDiffP) {
+                *centerP   = m;
+                *bestDiffP = diff;
             }
         }
     }
-    *bestDiffP = bestDiff;
 }
 
 
@@ -473,26 +471,27 @@ findBestSpaced(int const minBoundY, int const minBoundX,
  *
  *===========================================================================*/
 int
-PLogarithmicSearch(LumBlock currentBlock, 
-                   MpegFrame * const prev, 
-                   int const by, int const bx, 
-                   int * const motionYP, int * const motionXP, 
-                   int const searchRange) {
+PLogarithmicSearch(const LumBlock * const currentBlockP,
+                   MpegFrame *      const prev, 
+                   int              const by,
+                   int              const bx, 
+                   vector *         const motionP,
+                   int              const searchRange) {
 
     int const stepSize = (pixelFullSearch ? 2 : 1);
 
     int minMY, minMX, maxMY, maxMX;
     int spacing;  /* grid spacing */
-    int motionX, motionY;
+    vector motion;
         /* Distance from (bx,by) (in half-pixels) of the block that is most
            like the current block among those that we have examined so far.
            (0,0) means we haven't examined any.
         */
-    int     bestDiff;
+    int bestDiff;
         /* The difference between the current block and the block offset
-           (motionX, motionY) from it.
+           'motion' from it.
         */
-
+    
     COMPUTE_MOTION_BOUNDARY(by, bx, stepSize, minMY, minMX, maxMY, maxMX);
     minMX = max(minMX, - searchRange);
     minMY = max(minMY, - searchRange);
@@ -504,49 +503,43 @@ PLogarithmicSearch(LumBlock currentBlock,
        pixels away, but with rounding error, it can.
     */
 
-    motionX = 0;
-    motionY = 0;
+    motion.x = motion.y = 0;
     bestDiff = INT_MAX;
 
-    for( spacing = searchRange; spacing >= stepSize; ) {
-        if ( stepSize == 2 ) {  /* make sure spacing is even */
-            if ( spacing == 2 ) 
+    for (spacing = searchRange; spacing >= stepSize;) {
+        if (stepSize == 2) {  /* make sure spacing is even */
+            if (spacing == 2) 
                 spacing = 0;
             else {
                 spacing = (spacing+1)/2;
-                if ( (spacing % 2) != 0 ) 
-                    spacing--;
+                if (spacing % 2 != 0) 
+                    --spacing;
             }
         } else {
-            if ( spacing == 1 ) {
+            if (spacing == 1) {
                 spacing = 0;
             } else 
-                spacing = (spacing+1)/2;
+                spacing = (spacing + 1) / 2;
         }
         if (spacing >= stepSize) 
-            findBestSpaced(max(minMY, motionY - spacing),
-                           max(minMX, motionX - spacing),
-                           min(maxMY, motionY + spacing + 1),
-                           min(maxMX, motionX + spacing + 1),
-                           spacing,
-                           currentBlock, prev, by, bx,
-                           &bestDiff, &motionY, &motionX);
+            findBestSpaced(minMY, minMX, maxMY, maxMX,
+                           spacing, currentBlockP, prev, by, bx,
+                           &bestDiff, &motion);
     }
 
     {
-        int     diff;
+        int diff;
         /* check old motion -- see if it's better */
-        if ( (*motionYP >= minMY) && (*motionYP < maxMY) &&
-             (*motionXP >= minMX) && (*motionXP < maxMX) ) {
-            diff = LumMotionError(currentBlock, prev, by, bx, 
-                                  *motionYP, *motionXP, bestDiff);
+        if ((motionP->y >= minMY) && (motionP->y < maxMY) &&
+            (motionP->x >= minMX) && (motionP->x < maxMX)) {
+            diff = LumMotionError(currentBlockP, prev, by, bx, 
+                                  *motionP, bestDiff);
         } else 
             diff = INT_MAX;
         
-        if ( bestDiff < diff ) {
-            *motionYP = motionY;
-            *motionXP = motionX;
-        } else 
+        if (bestDiff < diff)
+            *motionP = motion;
+        else 
             bestDiff = diff;
     }
 
@@ -554,6 +547,7 @@ PLogarithmicSearch(LumBlock currentBlock,
 }
 
 
+
 /*===========================================================================*
  *
  * PLocalSearch
@@ -566,97 +560,89 @@ PLogarithmicSearch(LumBlock currentBlock,
  *
  *===========================================================================*/
 int
-PLocalSearch(LumBlock currentBlock, MpegFrame * prev, 
-             int by, int bx, int * motionYP, int * motionXP, 
-             int bestSoFar, int searchRange)
-{
-    register int mx, my;
-    int     diff, bestDiff;
-    int     stepSize;
-    int     leftMY, leftMX;
-    int     rightMY, rightMX;
-    int     distance;
-    int     tempRightMY, tempRightMX;
+PLocalSearch(const LumBlock * const currentBlockP,
+             MpegFrame *      const prev, 
+             int              const by,
+             int              const bx,
+             vector *         const motionP,
+             int              const bestSoFar,
+             int              const searchRange) {
+
+    int mx, my;
+    int bestDiff;
+    int stepSize;
+    int leftMY, leftMX;
+    int rightMY, rightMX;
+    int distance;
+    int tempRightMY, tempRightMX;
 
     stepSize = (pixelFullSearch ? 2 : 1);
 
     COMPUTE_MOTION_BOUNDARY(by,bx,stepSize,leftMY,leftMX,rightMY,rightMX);
 
     /* try old motion vector first */
-    if ( VALID_MOTION(*motionYP, *motionXP) ) {
-        bestDiff = LumMotionError(currentBlock, prev, by, bx, 
-                                  *motionYP, *motionXP, bestSoFar);
+    if (VALID_MOTION(*motionP)) {
+        bestDiff = LumMotionError(currentBlockP, prev, by, bx, 
+                                  *motionP, bestSoFar);
 
-        if ( bestSoFar < bestDiff ) {
+        if (bestSoFar < bestDiff)
             bestDiff = bestSoFar;
-        }
     } else {
-        *motionYP = 0;
-        *motionXP = 0;
-
+        motionP->y = motionP->x = 0;
         bestDiff = bestSoFar;
     }
 
     /* try a spiral pattern */    
-    for ( distance = stepSize; distance <= searchRange;
-          distance += stepSize ) {
-        tempRightMY = rightMY;
-        if ( distance < tempRightMY ) {
-            tempRightMY = distance;
-        }
-        tempRightMX = rightMX;
-        if ( distance < tempRightMX ) {
-            tempRightMX = distance;
-        }
+    for (distance = stepSize; distance <= searchRange; distance += stepSize) {
+        tempRightMY = MIN(distance, rightMY);
+        tempRightMX = MIN(distance, rightMX);
 
         /* do top, bottom */
-        for ( my = -distance; my < tempRightMY;
-              my += max(tempRightMY+distance-stepSize, stepSize) ) {
-            if ( my < leftMY ) {
-                continue;
-            }
-
-            for ( mx = -distance; mx < tempRightMX; mx += stepSize ) {
-                if ( mx < leftMX ) {
-                    continue;
-                }
-
-                diff = LumMotionError(currentBlock, prev, by, bx, my, mx, 
-                                      bestDiff);
-
-                if ( diff < bestDiff ) {
-                    *motionYP = my;
-                    *motionXP = mx;
-                    bestDiff = diff;
+        for (my = -distance; my < tempRightMY;
+             my += max(tempRightMY+distance-stepSize, stepSize)) {
+            if (my >= leftMY) {
+                for ( mx = -distance; mx < tempRightMX; mx += stepSize ) {
+                    if (mx >= leftMX) {
+                        int diff;
+                        vector m;
+                        
+                        m.y = my; m.x = mx;
+                        diff = LumMotionError(currentBlockP, prev, by, bx, m, 
+                                              bestDiff);
+                        
+                        if (diff < bestDiff) {
+                            *motionP = m;
+                            bestDiff = diff;
+                        }
+                    }
                 }
             }
         }
 
         /* do left, right */
-        for ( mx = -distance; mx < tempRightMX;
-              mx += max(tempRightMX+distance-stepSize, stepSize) ) {
-            if ( mx < leftMX ) {
-                continue;
-            }
+        for (mx = -distance; mx < tempRightMX;
+             mx += max(tempRightMX+distance-stepSize, stepSize)) {
 
-            for ( my = -distance+stepSize; my < tempRightMY-stepSize;
-                  my += stepSize ) {
-                if ( my < leftMY ) {
-                    continue;
-                }
+            if (mx >= leftMX) {
+                for (my = -distance+stepSize; my < tempRightMY-stepSize;
+                     my += stepSize) {
+                    if (my >= leftMY) {
+                        int diff;
+                        vector m;
 
-                diff = LumMotionError(currentBlock, prev, by, bx, my, mx, 
-                                      bestDiff);
-
-                if ( diff < bestDiff ) {
-                    *motionYP = my;
-                    *motionXP = mx;
-                    bestDiff = diff;
+                        m.y = my; m.x = mx;
+                        diff = LumMotionError(currentBlockP, prev, by, bx, m,
+                                              bestDiff);
+                        
+                        if (diff < bestDiff) {
+                            *motionP = m;
+                            bestDiff = diff;
+                        }
+                    }
                 }
             }
         }
     }
-
     return bestDiff;
 }
 
@@ -675,19 +661,21 @@ PLocalSearch(LumBlock currentBlock, MpegFrame * prev,
  *
  *===========================================================================*/
 int
-PTwoLevelSearch(LumBlock currentBlock, MpegFrame * prev, 
-                int by, int bx, 
-                int * motionYP, int * motionXP, 
-                int bestSoFar, int searchRange)
-{
-    register int mx, my;
-    register int   loopInc;
+PTwoLevelSearch(const LumBlock * const currentBlockP,
+                MpegFrame *      const prev, 
+                int              const by,
+                int              const bx, 
+                vector *         const motionP,
+                int              const bestSoFar,
+                int              const searchRange) {
+    int mx, my;
+    int loopInc;
     int diff, bestDiff;
-    int     leftMY, leftMX;
-    int     rightMY, rightMX;
-    int     distance;
-    int     tempRightMY, tempRightMX;
-    int     xOffset, yOffset;
+    int leftMY, leftMX;
+    int rightMY, rightMX;
+    int distance;
+    int tempRightMY, tempRightMX;
+    int xOffset, yOffset;
 
     /* exhaustive full-pixel search first */
 
@@ -697,125 +685,111 @@ PTwoLevelSearch(LumBlock currentBlock, MpegFrame * prev,
     rightMX--;
 
     /* convert vector into full-pixel vector */
-    if ( *motionYP > 0 ) {
-        if ( ((*motionYP) % 2) == 1 ) {
-            (*motionYP)--;
+    if (motionP->y > 0) {
+        if ((motionP->y % 2) == 1) {
+            --motionP->y;
         }
-    } else if ( ((-(*motionYP)) % 2) == 1 ) {
-        (*motionYP)++;
-    }
+    } else if (((-motionP->y) % 2) == 1)
+        ++motionP->y;
 
-    if ( *motionXP > 0 ) {
-        if ( ((*motionXP) % 2) == 1 ) {
-            (*motionXP)--;
-        }
-    } else if ( ((-(*motionXP)) % 2) == 1 ) {
-        (*motionXP)++;
-    }
+    if (motionP->x > 0) {
+        if ((motionP->x % 2) == 1)
+            --motionP->x;
+    } else if ((-motionP->x % 2) == 1)
+        ++motionP->x;
 
     /* try old motion vector first */
-    if ( VALID_MOTION(*motionYP, *motionXP) ) {
-        bestDiff = LumMotionError(currentBlock, prev, by, bx, 
-                                  *motionYP, *motionXP, bestSoFar);
+    if (VALID_MOTION(*motionP)) {
+        bestDiff = LumMotionError(currentBlockP, prev, by, bx, 
+                                  *motionP, bestSoFar);
 
         if ( bestSoFar < bestDiff ) {
             bestDiff = bestSoFar;
         }
     } else {
-        *motionYP = 0;
-        *motionXP = 0;
-
+        motionP->y = motionP->x = 0;
         bestDiff = bestSoFar;
     }
 
-    rightMY++;
-    rightMX++;
+    ++rightMY;
+    ++rightMX;
 
     /* try a spiral pattern */    
     for ( distance = 2; distance <= searchRange; distance += 2 ) {
-        tempRightMY = rightMY;
-        if ( distance < tempRightMY ) {
-            tempRightMY = distance;
-        }
-        tempRightMX = rightMX;
-        if ( distance < tempRightMX ) {
-            tempRightMX = distance;
-        }
+        tempRightMY = MIN(distance, rightMY);
+        tempRightMX = MIN(distance, rightMX);
 
         /* do top, bottom */
-        loopInc = max(tempRightMY+distance-2, 2);
-        for ( my = -distance; my < tempRightMY; my += loopInc ) {
-            if ( my < leftMY ) {
-                continue;
-            }
-
-            for ( mx = -distance; mx < tempRightMX; mx += 2 ) {
-                if ( mx < leftMX ) {
-                    continue;
-                }
-
-                diff = LumMotionError(currentBlock, prev, by, bx, my, mx, 
-                                      bestDiff);
-
-                if ( diff < bestDiff ) {
-                    *motionYP = my;
-                    *motionXP = mx;
-                    bestDiff = diff;
+        loopInc = max(tempRightMY + distance - 2, 2);
+        for (my = -distance; my < tempRightMY; my += loopInc) {
+            if (my >= leftMY) {
+                for (mx = -distance; mx < tempRightMX; mx += 2) {
+                    if (mx >= leftMX) {
+                        vector m;
+                        m.y = my; m.x = mx;
+                        diff = LumMotionError(currentBlockP, prev, by, bx, m, 
+                                              bestDiff);
+                        
+                        if (diff < bestDiff) {
+                            *motionP = m;
+                            bestDiff = diff;
+                        }
+                    }
                 }
             }
         }
 
         /* do left, right */
         loopInc = max(tempRightMX+distance-2, 2);
-        for ( mx = -distance; mx < tempRightMX; mx += loopInc ) {
-            if ( mx < leftMX ) {
-                continue;
-            }
-
-            for ( my = -distance+2; my < tempRightMY-2; my += 2 ) {
-                if ( my < leftMY ) {
-                    continue;
-                }
-
-                diff = LumMotionError(currentBlock, prev, by, bx, my, mx, 
-                                      bestDiff);
-
-                if ( diff < bestDiff ) {
-                    *motionYP = my;
-                    *motionXP = mx;
-                    bestDiff = diff;
+        for (mx = -distance; mx < tempRightMX; mx += loopInc) {
+            if (mx >= leftMX) {
+                for ( my = -distance+2; my < tempRightMY-2; my += 2 ) {
+                    if (my >= leftMY) {
+                        int diff;
+                        vector m;
+                        m.y = my; m.x = mx;
+                        diff = LumMotionError(currentBlockP, prev, by, bx, m, 
+                                              bestDiff);
+                        
+                        if ( diff < bestDiff ) {
+                            *motionP = m;
+                            bestDiff = diff;
+                        }
+                    }
                 }
             }
         }
     }
 
     /* now look at neighboring half-pixels */
-    my = *motionYP;
-    mx = *motionXP;
+    my = motionP->y;
+    mx = motionP->x;
 
-    rightMY--;
-    rightMX--;
+    --rightMY;
+    --rightMX;
 
-    for ( yOffset = -1; yOffset <= 1; yOffset++ ) {
-        for ( xOffset = -1; xOffset <= 1; xOffset++ ) {
-            if ( (yOffset == 0) && (xOffset == 0) )
-                continue;
-
-            if ( VALID_MOTION(my+yOffset, mx+xOffset) &&
-                 ((diff = LumMotionError(currentBlock, prev, by, bx,
-                                         my+yOffset, mx+xOffset, bestDiff)) 
-                  < bestDiff) ) {
-                *motionYP = my+yOffset;
-                *motionXP = mx+xOffset;
-                bestDiff = diff;
+    for (yOffset = -1; yOffset <= 1; ++yOffset) {
+        for (xOffset = -1; xOffset <= 1; ++xOffset) {
+            if ((yOffset != 0) || (xOffset != 0)) {
+                vector m;
+                m.y = my+yOffset; m.x = mx+xOffset;
+                if (VALID_MOTION(m)) {
+                    int diff;
+                    diff = LumMotionError(currentBlockP, prev, by, bx,
+                                          m, bestDiff);
+                    if (diff < bestDiff) {
+                        *motionP = m;
+                        bestDiff = diff;
+                    }
+                }
             }
         }
     }
-
     return bestDiff;
 }
 
 
+
 void
 ShowPMVHistogram(fpointer)
     FILE *fpointer;
