@@ -25,16 +25,21 @@
 #include <ctype.h>
 #include <string.h>
 #include <unistd.h>
+#include <assert.h>
+#include "ppm.h"
+#include "nstring.h"
+
 #include "mtypes.h"
 #include "frames.h"
 #include "prototypes.h"
 #include "parallel.h"
 #include "param.h"
-#include "readframe.h"
+#include "input.h"
 #include "fsize.h"
 #include "rgbtoycc.h"
 #include "jpeg.h"
 #include "opts.h"
+#include "readframe.h"
 
 
 /*==================*
@@ -43,10 +48,10 @@
 
 static int  fileType = BASE_FILE_TYPE;
 struct YuvLine {
-	uint8	data[3072];
-	uint8	y[1024];
-	int8	cr[1024];
-	int8	cb[1024];
+    uint8   data[3072];
+    uint8   y[1024];
+    int8    cr[1024];
+    int8    cb[1024];
 };
 
 
@@ -66,23 +71,22 @@ extern boolean GammaCorrection;
 extern float GammaValue;
 extern int outputWidth,outputHeight;
 boolean resizeFrame;
-char *CurrFile;
+const char *CurrFile;
 
 /*===============================*
  * INTERNAL PROCEDURE prototypes *
  *===============================*/
 
-static void ReadPNM _ANSI_ARGS_((FILE * fp, MpegFrame * mf));
 static void ReadEYUV _ANSI_ARGS_((MpegFrame * mf, FILE *fpointer,
-				 int width, int height));
+                 int width, int height));
 static void ReadAYUV _ANSI_ARGS_((MpegFrame * mf, FILE *fpointer,
-				 int width, int height));
+                 int width, int height));
 static void SeparateLine _ANSI_ARGS_((FILE *fpointer, struct YuvLine *lineptr,
-				     int width));
+                     int width));
 static void ReadY _ANSI_ARGS_((MpegFrame * mf, FILE *fpointer,
-				 int width, int height));
+                 int width, int height));
 static void ReadSub4 _ANSI_ARGS_((MpegFrame * mf, FILE *fpointer,
-				  int width, int height));
+                  int width, int height));
 static void DoGamma  _ANSI_ARGS_((MpegFrame *mf, int width, int height));
 
 static void DoKillDim _ANSI_ARGS_((MpegFrame *mf, int w, int h));
@@ -98,210 +102,274 @@ static void DoKillDim _ANSI_ARGS_((MpegFrame *mf, int w, int h));
 
 
 
-void	SetResize(set)
-    boolean	set;
-{
+void    
+SetResize(bool const set) {
     resizeFrame = set;
 }
 
 
 
-/*===========================================================================*
- *
- * ReadFrame
- *
- *	reads the given frame, performing conversion as necessary
- *	if addPath = TRUE, then must add the current path before the
- *	file name
- *
- * RETURNS:	frame modified
- *
- * SIDE EFFECTS:    none
- *
- *===========================================================================*/
-void
-ReadFrame(MpegFrame *  const frame, 
-          const char * const fileName, 
-          const char * const conversion, 
-          boolean      const addPath)
-{
-    FILE    *ifp;
-    char    command[1024];
-    char    fullFileName[1024];
-    MpegFrame    tempFrame;
-    MpegFrame    *framePtr;
-#ifdef BLEAH
-    static int32    readDiskTime = 0;
-    int32    diskStartTime, diskEndTime;
+static void
+ReadPNM(MpegFrame * const mpegFrameP,
+        FILE *      const fp) {
+/*----------------------------------------------------------------------------
+   Read a PPM file containing one movie frame.
+-----------------------------------------------------------------------------*/
+    int pnmCols, pnmRows;
+    xelval maxval;
+    xel ** xels;
 
-time(&diskStartTime);
-#endif
+    xels = ppm_readppm(fp, &pnmCols, &pnmRows, &maxval);
+    ERRCHK(mpegFrameP, "ppm_readppm");
 
-    if ( resizeFrame ) {
-      tempFrame.inUse = FALSE;
-      tempFrame.ppm_data = NULL;
-      tempFrame.rgb_data = NULL;
-      tempFrame.orig_y = NULL;
-      tempFrame.y_blocks = NULL;
-      tempFrame.decoded_y = NULL;
-      tempFrame.halfX = NULL;
-      framePtr = &tempFrame;
-    } else {
-      framePtr = frame;
-    }
+    /*
+     * if this is the first frame read, set the global frame size
+     */
+    Fsize_Note(mpegFrameP->id, pnmCols, pnmRows);
 
-    if ( addPath ) {
-      sprintf(fullFileName, "%s/%s", currentPath, fileName);
-    } else {
-      sprintf(fullFileName, "%s", fileName);
-    }
+    PNMtoYUV(mpegFrameP, xels, Fsize_x, Fsize_y, maxval);
+    ppm_freearray(xels, pnmRows);
+}
 
-    CurrFile = fullFileName;
 
-#ifdef BLEAH
-    if ( ! childProcess ) {
-    fprintf(stdout, "+++++READING Frame %d  (type %d):  %s\n", framePtr->id,
-            framePtr->type, fullFileName);
-    }
-#endif
 
-    if ( fileType == ANY_FILE_TYPE ) {
-      const char *convertPtr;
-      char *commandPtr, *charPtr;
+static void
+openFile(struct inputSource * const inputSourceP,
+         unsigned int         const frameNumber,
+         const char *         const conversion, 
+         FILE **              const ifPP) {
 
-      if ( stdinUsed ) {
-        fprintf(stderr, "ERROR : You cannot use a converter on frames when "
+    if (inputSourceP->stdinUsed) {
+        if (fileType == ANY_FILE_TYPE)
+            pm_error(
+                "ERROR : You cannot use a converter on frames when "
                 "you supply them as Standard Input.  Either specify "
                 "INPUT_CONVERTER * in the parameter file or supply the "
                 "frames in files by specifying a directory with "
-                "INPUT_DIRECTORY in the parameter file.\n");
-        exit(1);
-      }
-
-      /* replace every occurrence of '*' with fullFileName */
-      convertPtr = conversion;
-      commandPtr = command;
-      while ( *convertPtr != '\0' ) {
-        while ( (*convertPtr != '\0') && (*convertPtr != '*') ) {
-          *commandPtr = *convertPtr;
-          commandPtr++;
-          convertPtr++;
-        }
-
-        if ( *convertPtr == '*' ) {
-          /* copy fullFileName */
-          charPtr = fullFileName;
-          while ( *charPtr != '\0' ) {
-            *commandPtr = *charPtr;
-            commandPtr++;
-            charPtr++;
-          }
-
-          convertPtr++;   /* go past '*' */
-        }
-      }
-      *commandPtr = '\0';
-
-      if ( (ifp = popen(command, "r")) == NULL ) {
-        fprintf(stderr, "ERROR:  Couldn't execute input conversion command:\n");
-        fprintf(stderr, "\t%s\n", command);
-        fprintf(stderr, "errno = %d\n", errno);
-        if ( ioServer ) {
-          fprintf(stderr, "IO SERVER:  EXITING!!!\n");
+                "INPUT_DIRECTORY in the parameter file.");
+        
+        *ifPP = stdin;
+    } else {
+        const char * fileName;
+        const char * fullFileName;
+        
+        GetNthInputFileName(inputSourceP, frameNumber, &fileName);
+        
+        asprintfN(&fullFileName, "%s/%s", currentPath, fileName);
+        
+        CurrFile = fullFileName;
+        
+        if (fileType == ANY_FILE_TYPE) {
+            char command[1024];
+            const char * convertPtr;
+            char * commandPtr;
+            const char * charPtr;
+            
+            /* replace every occurrence of '*' with fullFileName */
+            convertPtr = conversion;
+            commandPtr = command;
+            while (*convertPtr != '\0') {
+                while ((*convertPtr != '\0') && (*convertPtr != '*')) {
+                    *commandPtr = *convertPtr;
+                    ++commandPtr;
+                    ++convertPtr;
+                }
+                
+                if (*convertPtr == '*') {
+                    /* copy fullFileName */
+                    charPtr = fullFileName;
+                    while (*charPtr != '\0') {
+                        *commandPtr = *charPtr;
+                        ++commandPtr;
+                        ++charPtr;
+                    }
+                    ++convertPtr;   /* go past '*' */
+                }
+            }
+            *commandPtr = '\0';
+            
+            *ifPP = popen(command, "r");
+            if (*ifPP == NULL) {
+                pm_message(
+                    "ERROR:  Couldn't execute input conversion command "
+                    "'%s'.  errno=%d (%s)",
+                    command, errno, strerror(errno));
+                if (ioServer)
+                    pm_error("IO SERVER:  EXITING!!!");
+                else
+                    pm_error("SLAVE EXITING!!!");
+            }
         } else {
-          fprintf(stderr, "SLAVE EXITING!!!\n");
+            *ifPP = fopen(fullFileName, "rb");
+            if (*ifPP == NULL)
+                pm_error("Couldn't open input file '%s'", fullFileName);
+            
+            if (baseFormat == JMOVIE_FILE_TYPE)
+                unlink(fullFileName);
         }
-        exit(1);
-      }
-    } else if (stdinUsed) {
-      ifp = stdin;
-    } else if ( (ifp = fopen(fullFileName, "rb")) == NULL ) {
-      fprintf(stderr, "ERROR:  Couldn't open input file %s\n",
-              fullFileName);
-      exit(1);
+        strfree(fullFileName);
+        strfree(fileName);
     }
-
-    switch(baseFormat) {
-    case YUV_FILE_TYPE:
-
-        /* Encoder YUV */
-        if ((strncmp (yuvConversion, "EYUV", 4) == 0) ||
-            (strncmp (yuvConversion, "UCB", 3) == 0) ) 
-        {
-            ReadEYUV(framePtr, ifp, realWidth, realHeight);
-        }
-
-        /* Abekas-type (interlaced) YUV */
-        else {
-            ReadAYUV(framePtr, ifp, realWidth, realHeight);
-        }
-
-        break;
-    case Y_FILE_TYPE:
-        ReadY(framePtr, ifp, realWidth, realHeight);
-        break;
-    case PNM_FILE_TYPE:
-        ReadPNM(ifp, framePtr);
-        PNMtoYUV(framePtr);
-        break;
-    case SUB4_FILE_TYPE:
-        ReadSub4(framePtr, ifp, yuvWidth, yuvHeight);
-        break;
-    case JPEG_FILE_TYPE:
-    case JMOVIE_FILE_TYPE:
-        ReadJPEG(framePtr, ifp);
-        break;
-    default:
-        break;
-    }
-
-    if (! stdinUsed) {
-      if ( fileType == ANY_FILE_TYPE ) {
-	int errorcode;
-	if ( (errorcode = pclose(ifp)) != 0) {
-	  fprintf(stderr, "WARNING:  Pclose reported error (%d)\n", errorcode);
-	}
-      } else {
-        fclose(ifp);
-      }
-    }
-    
-    if ( baseFormat == JMOVIE_FILE_TYPE ) {
-      remove(fullFileName);
-    }
-
-    if ( resizeFrame ) {
-      Frame_Resize(frame, &tempFrame, Fsize_x, Fsize_y, outputWidth, outputHeight);
-    }
-
-#ifdef BLEAH
-time(&diskEndTime);
-
-readDiskTime += (diskEndTime-diskStartTime);
-
-fprintf(stdout, "cumulative disk read time:  %d seconds\n", readDiskTime);
-#endif
-
-    if ( GammaCorrection ) {
-      DoGamma(frame, Fsize_x, Fsize_y);
-    }
-
-    if ( kill_dim ) {
-      DoKillDim(frame, Fsize_x, Fsize_y);
-    }
-
-    MotionSearchPreComputation(frame);
 }
 
 
+
+static void
+closeFile(struct inputSource * const inputSourceP,
+          FILE *               const ifP) {
+
+    if (!inputSourceP->stdinUsed) {
+        if (fileType == ANY_FILE_TYPE) {
+            int rc;
+            rc = pclose(ifP);
+            if (rc != 0)
+                pm_message("WARNING:  pclose() failed with errno %d (%s)",
+                           errno, strerror(errno));
+        } else
+            fclose(ifP);
+    }
+}
+
+
+
+static bool
+fileIsAtEnd(FILE * const ifP) {
+
+    int c;
+    bool eof;
+
+    c = getc(ifP);
+    if (c == EOF) {
+        if (feof(ifP)) 
+            eof = TRUE;
+        else
+            pm_error("File error on getc() to position to image");
+    } else {
+        int rc;
+
+        eof = FALSE;
+
+        rc = ungetc(c, ifP);
+        if (rc == EOF) 
+            pm_error("File error doing ungetc() to position to image.");
+    }
+    return eof;
+}
+
+
+
+void
+ReadFrameFile(MpegFrame *  const frameP,
+              FILE *       const ifP,
+              const char * const conversion,
+              bool *       const eofP) {
+/*----------------------------------------------------------------------------
+   Read a frame from the file 'ifP'.
+
+   Return *eofP == TRUE iff we encounter EOF before we can get the
+   frame.
+-----------------------------------------------------------------------------*/
+    MpegFrame   tempFrame;
+    MpegFrame * framePtr;
+
+    /* To make this code fit Netpbm properly, we should remove handling
+       of all types except PNM and use pm_nextimage() to handle sensing
+       of end of stream.
+    */
+
+    if (fileIsAtEnd(ifP))
+        *eofP = TRUE;
+    else {
+        *eofP = FALSE;
+
+        if (resizeFrame) {
+            tempFrame.inUse = FALSE;
+            tempFrame.orig_y = NULL;
+            tempFrame.y_blocks = NULL;
+            tempFrame.decoded_y = NULL;
+            tempFrame.halfX = NULL;
+            framePtr = &tempFrame;
+        } else
+            framePtr = frameP;
+
+        switch(baseFormat) {
+        case YUV_FILE_TYPE:
+
+            /* Encoder YUV */
+            if ((strncmp (yuvConversion, "EYUV", 4) == 0) ||
+                (strncmp (yuvConversion, "UCB", 3) == 0)) 
+
+                ReadEYUV(framePtr, ifP, realWidth, realHeight);
+
+            else
+                /* Abekas-type (interlaced) YUV */
+                ReadAYUV(framePtr, ifP, realWidth, realHeight);
+
+            break;
+        case Y_FILE_TYPE:
+            ReadY(framePtr, ifP, realWidth, realHeight);
+            break;
+        case PNM_FILE_TYPE:
+            ReadPNM(framePtr, ifP);
+            break;
+        case SUB4_FILE_TYPE:
+            ReadSub4(framePtr, ifP, yuvWidth, yuvHeight);
+            break;
+        case JPEG_FILE_TYPE:
+        case JMOVIE_FILE_TYPE:
+            ReadJPEG(framePtr, ifP);
+            break;
+        default:
+            break;
+        }
+
+        if (resizeFrame)
+            Frame_Resize(frameP, &tempFrame, Fsize_x, Fsize_y, 
+                         outputWidth, outputHeight);
+    
+        if (GammaCorrection)
+            DoGamma(frameP, Fsize_x, Fsize_y);
+
+        if (kill_dim)
+            DoKillDim(frameP, Fsize_x, Fsize_y);
+
+        MotionSearchPreComputation(frameP);
+    }
+}
+
+
+
+void
+ReadFrame(MpegFrame *          const frameP, 
+          struct inputSource * const inputSourceP,
+          unsigned int         const frameNumber,
+          const char *         const conversion,
+          bool *               const endOfStreamP) {
+/*----------------------------------------------------------------------------
+   Read the given frame, performing conversion as necessary.
+-----------------------------------------------------------------------------*/
+    FILE * ifP;
+
+    openFile(inputSourceP, frameNumber, conversion, &ifP);
+
+    ReadFrameFile(frameP, ifP, conversion, endOfStreamP);
+
+    if (*endOfStreamP && !inputSourceP->stdinUsed)
+        pm_error("Premature EOF on file containing Frame %u", frameNumber);
+
+    closeFile(inputSourceP, ifP);
+}
+
+
+
 /*===========================================================================*
  *
  * SetFileType
  *
- *	set the file type to be either a base type (no conversion), or
- *	any type (conversion required)
+ *  set the file type to be either a base type (no conversion), or
+ *  any type (conversion required)
  *
- * RETURNS:	nothing
+ * RETURNS: nothing
  *
  * SIDE EFFECTS:    fileType
  *
@@ -310,9 +378,9 @@ void
 SetFileType(const char * const conversion)
 {
     if ( strcmp(conversion, "*") == 0 ) {
-	fileType = BASE_FILE_TYPE;
+    fileType = BASE_FILE_TYPE;
     } else {
-	fileType = ANY_FILE_TYPE;
+    fileType = ANY_FILE_TYPE;
     }
 }
 
@@ -321,9 +389,9 @@ SetFileType(const char * const conversion)
  *
  * SetFileFormat
  *
- *	set the file format (PNM, YUV, JPEG)
+ *  set the file format (PNM, YUV, JPEG)
  *
- * RETURNS:	nothing
+ * RETURNS: nothing
  *
  * SIDE EFFECTS:    baseFormat
  *
@@ -332,133 +400,92 @@ void
 SetFileFormat(const char * const format)
 {
     if ( strcmp(format, "PPM") == 0 ) {
-	baseFormat = PNM_FILE_TYPE;
+    baseFormat = PNM_FILE_TYPE;
     } else if ( strcmp(format, "YUV") == 0 ) {
-	baseFormat = YUV_FILE_TYPE;
+    baseFormat = YUV_FILE_TYPE;
     } else if ( strcmp(format, "Y") == 0 ) {
-	baseFormat = Y_FILE_TYPE;
+    baseFormat = Y_FILE_TYPE;
     } else if ( strcmp(format, "PNM") == 0 ) {
-	baseFormat = PNM_FILE_TYPE;
+    baseFormat = PNM_FILE_TYPE;
     } else if (( strcmp(format, "JPEG") == 0 ) || ( strcmp(format, "JPG") == 0 )) {
-	baseFormat = JPEG_FILE_TYPE;
+    baseFormat = JPEG_FILE_TYPE;
     } else if ( strcmp(format, "JMOVIE") == 0 ) {
-	baseFormat = JMOVIE_FILE_TYPE;
+    baseFormat = JMOVIE_FILE_TYPE;
     } else if ( strcmp(format, "SUB4") == 0 ) {
-	baseFormat = SUB4_FILE_TYPE;
+    baseFormat = SUB4_FILE_TYPE;
     } else {
-	fprintf(stderr, "ERROR:  Invalid file format:  %s\n", format);
-	exit(1);
+    fprintf(stderr, "ERROR:  Invalid file format:  %s\n", format);
+    exit(1);
     }
 }
 
 
-/*===========================================================================*
- *
- * ReadPNM
- *
- *	read a PPM file
- *
- * RETURNS:	mf modified
- *
- * SIDE EFFECTS:    none
- *
- *===========================================================================*/
-static void
-ReadPNM(fp, mf)
-    FILE *fp;
-    MpegFrame *mf;
-{
-    int x, y;
-    pixval maxval;
 
-    if (mf->rgb_data) {
-	ppm_freearray(mf->rgb_data, Fsize_y);
-    }
-    mf->rgb_data = ppm_readppm(fp, &x, &y, &maxval);
-    ERRCHK(mf, "ppm_readppm");
-
-    /*
-     * if this is the first frame read, set the global frame size
-     */
-    Fsize_Note(mf->id, x, y);
-
-    mf->rgb_maxval = maxval;
-    mf->rgb_format = PPM_FORMAT;
-}
-
-
-
-/*===========================================================================*
- *
- * ReadIOConvert
- *
- *	do conversion; return a pointer to the appropriate file
- *
- * RETURNS:	pointer to the appropriate file
- *
- * SIDE EFFECTS:    none
- *
- *===========================================================================*/
 FILE *
-ReadIOConvert(const char * const fileName)
-{
-    FILE	*ifp;
-    char	command[1024];
-    char	fullFileName[1024];
-    char *convertPtr, *commandPtr, *charPtr;
+ReadIOConvert(struct inputSource * const inputSourceP,
+              unsigned int         const frameNumber) {
+/*----------------------------------------------------------------------------
+   Do conversion; return handle to the appropriate file.
+-----------------------------------------------------------------------------*/
+    FILE    *ifp;
+    char    command[1024];
+    const char * fullFileName;
+    char *convertPtr, *commandPtr;
+    const char * fileName;
 
-    sprintf(fullFileName, "%s/%s", currentPath, fileName);
+    GetNthInputFileName(inputSourceP, frameNumber, &fileName);
 
-#ifdef BLEAH
-    if ( ! childProcess ) {
-	fprintf(stdout, "+++++READING (IO CONVERT) Frame %d  (type %d):  %s\n", frame->id,
-		frame->type, fullFileName); }
-#endif
+    asprintfN(&fullFileName, "%s/%s", currentPath, fileName);
 
     if ( strcmp(ioConversion, "*") == 0 ) {
-      char buff[1024];
-      ifp = fopen(fullFileName, "rb");
-      sprintf(buff,"fopen \"%s\"",fullFileName);
-      ERRCHK(ifp, buff);
-      return ifp;
+        char buff[1024];
+        ifp = fopen(fullFileName, "rb");
+        sprintf(buff,"fopen \"%s\"",fullFileName);
+        ERRCHK(ifp, buff);
+        return ifp;
     }
 
     /* replace every occurrence of '*' with fullFileName */
     convertPtr = ioConversion;
     commandPtr = command;
     while ( *convertPtr != '\0' ) {
-	while ( (*convertPtr != '\0') && (*convertPtr != '*') ) {
-	    *commandPtr = *convertPtr;
-	    commandPtr++;
-	    convertPtr++;
-	}
+        while ( (*convertPtr != '\0') && (*convertPtr != '*') ) {
+            *commandPtr = *convertPtr;
+            commandPtr++;
+            convertPtr++;
+        }
 
-	if ( *convertPtr == '*' ) {
-	    /* copy fullFileName */
-	    charPtr = fullFileName;
-	    while ( *charPtr != '\0' ) {
-		*commandPtr = *charPtr;
-		commandPtr++;
-		charPtr++;
-	    }
+        if ( *convertPtr == '*' ) {
+            /* copy fullFileName */
+            const char * charPtr;
+            charPtr = fullFileName;
+            while ( *charPtr != '\0' ) {
+                *commandPtr = *charPtr;
+                commandPtr++;
+                charPtr++;
+            }
 
-	    convertPtr++;   /* go past '*' */
-	}
+            convertPtr++;   /* go past '*' */
+        }
     }
     *commandPtr = '\0';
 
     if ( (ifp = popen(command, "r")) == NULL ) {
-	fprintf(stderr, "ERROR:  Couldn't execute input conversion command:\n");
-	fprintf(stderr, "\t%s\n", command);
-	fprintf(stderr, "errno = %d\n", errno);
-	if ( ioServer ) {
-	    fprintf(stderr, "IO SERVER:  EXITING!!!\n");
-	} else {
-	    fprintf(stderr, "SLAVE EXITING!!!\n");
-	}
-	exit(1);
+        fprintf(stderr, "ERROR:  "
+                "Couldn't execute input conversion command:\n");
+        fprintf(stderr, "\t%s\n", command);
+        fprintf(stderr, "errno = %d\n", errno);
+        if ( ioServer ) {
+            fprintf(stderr, "IO SERVER:  EXITING!!!\n");
+        } else {
+            fprintf(stderr, "SLAVE EXITING!!!\n");
+        }
+        exit(1);
     }
 
+    strfree(fullFileName);
+    strfree(fileName);
+
     return ifp;
 }
 
@@ -468,9 +495,9 @@ ReadIOConvert(const char * const fileName)
  *
  * ReadEYUV
  *
- *	read a Encoder-YUV file (concatenated Y, U, and V)
+ *  read a Encoder-YUV file (concatenated Y, U, and V)
  *
- * RETURNS:	mf modified
+ * RETURNS: mf modified
  *
  * SIDE EFFECTS:    none
  *
@@ -490,41 +517,41 @@ ReadEYUV(mf, fpointer, width, height)
 
     Frame_AllocYCC(mf);
 
-    for (y = 0; y < Fsize_y; y++) {			/* Y */
-	safe_fread(mf->orig_y[y], 1, Fsize_x, fpointer);
+    for (y = 0; y < Fsize_y; y++) {         /* Y */
+    safe_fread(mf->orig_y[y], 1, Fsize_x, fpointer);
 
-	/* read the leftover stuff on the right side */
-	if ( width != Fsize_x ) {
-	    safe_fread(junk, 1, width-Fsize_x, fpointer);
-	}
+    /* read the leftover stuff on the right side */
+    if ( width != Fsize_x ) {
+        safe_fread(junk, 1, width-Fsize_x, fpointer);
+    }
     }
 
     /* read the leftover stuff on the bottom */
     for (y = Fsize_y; y < height; y++) {
-	safe_fread(junk, 1, width, fpointer);
+    safe_fread(junk, 1, width, fpointer);
     }
 
-    for (y = 0; y < (Fsize_y >> 1); y++) {			/* U */
-	safe_fread(mf->orig_cb[y], 1, Fsize_x >> 1, fpointer);
+    for (y = 0; y < (Fsize_y >> 1); y++) {          /* U */
+    safe_fread(mf->orig_cb[y], 1, Fsize_x >> 1, fpointer);
 
-	/* read the leftover stuff on the right side */
-	if ( width != Fsize_x ) {
-	    safe_fread(junk, 1, (width-Fsize_x)>>1, fpointer);
-	}
+    /* read the leftover stuff on the right side */
+    if ( width != Fsize_x ) {
+        safe_fread(junk, 1, (width-Fsize_x)>>1, fpointer);
+    }
     }
 
     /* read the leftover stuff on the bottom */
     for (y = (Fsize_y >> 1); y < (height >> 1); y++) {
-	safe_fread(junk, 1, width>>1, fpointer);
+    safe_fread(junk, 1, width>>1, fpointer);
     }
 
-    for (y = 0; y < (Fsize_y >> 1); y++) {			/* V */
-	safe_fread(mf->orig_cr[y], 1, Fsize_x >> 1, fpointer);
+    for (y = 0; y < (Fsize_y >> 1); y++) {          /* V */
+    safe_fread(mf->orig_cr[y], 1, Fsize_x >> 1, fpointer);
 
-	/* read the leftover stuff on the right side */
-	if ( width != Fsize_x ) {
-	    safe_fread(junk, 1, (width-Fsize_x)>>1, fpointer);
-	}
+    /* read the leftover stuff on the right side */
+    if ( width != Fsize_x ) {
+        safe_fread(junk, 1, (width-Fsize_x)>>1, fpointer);
+    }
     }
 
     /* ignore leftover stuff on the bottom */
@@ -534,9 +561,9 @@ ReadEYUV(mf, fpointer, width, height)
  *
  * ReadAYUV
  *
- *	read an Abekas-YUV file
+ *  read an Abekas-YUV file
  *
- * RETURNS:	mf modified
+ * RETURNS: mf modified
  *
  * SIDE EFFECTS:    none
  *
@@ -559,31 +586,31 @@ ReadAYUV(mf, fpointer, width, height)
     Frame_AllocYCC(mf);
 
     for (y = 0; y < Fsize_y; y += 2) {
-	SeparateLine(fpointer, &line1, width);
-	SeparateLine(fpointer, &line2, width);
+    SeparateLine(fpointer, &line1, width);
+    SeparateLine(fpointer, &line2, width);
 
-	/* Copy the Y values for each line to the frame */
-	for (x = 0; x < Fsize_x; x++) {
-	    mf->orig_y[y][x]   = line1.y[x];
-	    mf->orig_y[y+1][x] = line2.y[x];
-	}
+    /* Copy the Y values for each line to the frame */
+    for (x = 0; x < Fsize_x; x++) {
+        mf->orig_y[y][x]   = line1.y[x];
+        mf->orig_y[y+1][x] = line2.y[x];
+    }
 
-	cbptr = &(mf->orig_cb[y>>1][0]);
-	crptr = &(mf->orig_cr[y>>1][0]);
+    cbptr = &(mf->orig_cb[y>>1][0]);
+    crptr = &(mf->orig_cr[y>>1][0]);
 
-	/* One U and one V for each two pixels horizontal as well */
-	/* Toss the second line of Cr/Cb info, averaging was worse,
-	   so just subsample */
-	for (x = 0; x < (Fsize_x >> 1); x ++) {
-	    cbptr[x] =  line1.cb[x];
-	    crptr[x] =  line1.cr[x];
+    /* One U and one V for each two pixels horizontal as well */
+    /* Toss the second line of Cr/Cb info, averaging was worse,
+       so just subsample */
+    for (x = 0; x < (Fsize_x >> 1); x ++) {
+        cbptr[x] =  line1.cb[x];
+        crptr[x] =  line1.cr[x];
 
-	}
+    }
     }
 
     /* read the leftover stuff on the bottom */
     for (y = Fsize_y; y < height; y++) {
-	safe_fread(junk, 1, width<<1, fpointer);
+    safe_fread(junk, 1, width<<1, fpointer);
     }
 
 }
@@ -592,9 +619,9 @@ ReadAYUV(mf, fpointer, width, height)
  *
  * SeparateLine
  *
- *	Separates one line of pixels into Y, U, and V components
+ *  Separates one line of pixels into Y, U, and V components
  *
- * RETURNS:	lineptr modified
+ * RETURNS: lineptr modified
  *
  * SIDE EFFECTS:    none
  *
@@ -614,20 +641,20 @@ SeparateLine(fpointer, lineptr, width)
 
     /* Sets the deinterlacing pattern */
 
-	/* shorthand for UYVY */
+    /* shorthand for UYVY */
     if (strncmp(yuvConversion, "ABEKAS", 6) == 0) {
-	strcpy(yuvConversion, "UYVY");
+    strcpy(yuvConversion, "UYVY");
 
-	/* shorthand for YUYV */
+    /* shorthand for YUYV */
     } else if (strncmp(yuvConversion, "PHILLIPS", 8) == 0) {
-	strcpy(yuvConversion, "YUYV");
+    strcpy(yuvConversion, "YUYV");
     }
 
     length = strlen (yuvConversion);
 
     if ((length % 2) != 0) {
-	fprintf (stderr, "ERROR : YUV_FORMAT must represent two pixels, hence must be even in length.\n");
-	exit(1);
+    fprintf (stderr, "ERROR : YUV_FORMAT must represent two pixels, hence must be even in length.\n");
+    exit(1);
     }
 
     /* each line in 4:2:2 chroma format takes 2X bytes to represent X pixels.
@@ -638,7 +665,7 @@ SeparateLine(fpointer, lineptr, width)
 
     /* read the leftover stuff on the right side */
     if ( width != Fsize_x ) {
-	safe_fread(junk, 1, (width-Fsize_x)*(length>>1), fpointer);
+    safe_fread(junk, 1, (width-Fsize_x)*(length>>1), fpointer);
     }
 
     crptr = &(lineptr->cr[0]);
@@ -646,29 +673,29 @@ SeparateLine(fpointer, lineptr, width)
     yptr = &(lineptr->y[0]);
 
     for (num = 0; num < (Fsize_x*(length>>1)); num++) {
-	switch (yuvConversion[num % length]) {
-	case 'U':
-	case 'u':
-	    *(cbptr++) = (lineptr->data[num]);
-	    break;
-	case 'V':
-	case 'v':
-	    *(crptr++) = (lineptr->data[num]);
-	    break;
-	case 'Y':
-	case 'y':
-	    *(yptr++) = (lineptr->data[num]);
-	    break;
-	default:
+    switch (yuvConversion[num % length]) {
+    case 'U':
+    case 'u':
+        *(cbptr++) = (lineptr->data[num]);
+        break;
+    case 'V':
+    case 'v':
+        *(crptr++) = (lineptr->data[num]);
+        break;
+    case 'Y':
+    case 'y':
+        *(yptr++) = (lineptr->data[num]);
+        break;
+    default:
             fprintf(stderr, "ERROR: YUV_FORMAT must be one of the following:\n");
             fprintf(stderr, "       ABEKAS\n");
             fprintf(stderr, "       EYUV\n");
             fprintf(stderr, "       PHILLIPS\n");
             fprintf(stderr, "       UCB\n");
-	    fprintf(stderr, "       or any even-length string consisting of the letters U, V, and Y.\n");
+        fprintf(stderr, "       or any even-length string consisting of the letters U, V, and Y.\n");
             exit(1);
         }
-	
+    
     }
 
 }
@@ -678,9 +705,9 @@ SeparateLine(fpointer, lineptr, width)
  *
  * ReadY
  *
- *	read a Y file
+ *  read a Y file
  *
- * RETURNS:	mf modified
+ * RETURNS: mf modified
  *
  * SIDE EFFECTS:    none
  *
@@ -700,18 +727,18 @@ ReadY(mf, fpointer, width, height)
 
     Frame_AllocYCC(mf);
 
-    for (y = 0; y < Fsize_y; y++) {			/* Y */
-	safe_fread(mf->orig_y[y], 1, Fsize_x, fpointer);
+    for (y = 0; y < Fsize_y; y++) {         /* Y */
+    safe_fread(mf->orig_y[y], 1, Fsize_x, fpointer);
 
-	/* read the leftover stuff on the right side */
-	if ( width != Fsize_x ) {
-	    safe_fread(junk, 1, width-Fsize_x, fpointer);
-	}
+    /* read the leftover stuff on the right side */
+    if ( width != Fsize_x ) {
+        safe_fread(junk, 1, width-Fsize_x, fpointer);
+    }
     }
 
     /* read the leftover stuff on the bottom */
     for (y = Fsize_y; y < height; y++) {
-	safe_fread(junk, 1, width, fpointer);
+    safe_fread(junk, 1, width, fpointer);
     }
     
     for (y = 0 ; y < (Fsize_y >> 1); y++) {
@@ -725,9 +752,9 @@ ReadY(mf, fpointer, width, height)
  *
  * ReadSub4
  *
- *	read a YUV file (subsampled even further by 4:1 ratio)
+ *  read a YUV file (subsampled even further by 4:1 ratio)
  *
- * RETURNS:	mf modified
+ * RETURNS: mf modified
  *
  * SIDE EFFECTS:    none
  *
@@ -748,34 +775,34 @@ ReadSub4(mf, fpointer, width, height)
 
     Frame_AllocYCC(mf);
 
-    for (y = 0; y < (height>>1); y++) {			/* Y */
-	safe_fread(buffer, 1, width>>1, fpointer);
-	for ( x = 0; x < (width>>1); x++ ) {
-	    mf->orig_y[2*y][2*x] = buffer[x];
-	    mf->orig_y[2*y][2*x+1] = buffer[x];
-	    mf->orig_y[2*y+1][2*x] = buffer[x];
-	    mf->orig_y[2*y+1][2*x+1] = buffer[x];
-	}
+    for (y = 0; y < (height>>1); y++) {         /* Y */
+    safe_fread(buffer, 1, width>>1, fpointer);
+    for ( x = 0; x < (width>>1); x++ ) {
+        mf->orig_y[2*y][2*x] = buffer[x];
+        mf->orig_y[2*y][2*x+1] = buffer[x];
+        mf->orig_y[2*y+1][2*x] = buffer[x];
+        mf->orig_y[2*y+1][2*x+1] = buffer[x];
+    }
     }
 
-    for (y = 0; y < (height >> 2); y++) {			/* U */
-	safe_fread(buffer, 1, width>>2, fpointer);
-	for ( x = 0; x < (width>>2); x++ ) {
-	    mf->orig_cb[2*y][2*x] = buffer[x];
-	    mf->orig_cb[2*y][2*x+1] = buffer[x];
-	    mf->orig_cb[2*y+1][2*x] = buffer[x];
-	    mf->orig_cb[2*y+1][2*x+1] = buffer[x];
-	}
+    for (y = 0; y < (height >> 2); y++) {           /* U */
+    safe_fread(buffer, 1, width>>2, fpointer);
+    for ( x = 0; x < (width>>2); x++ ) {
+        mf->orig_cb[2*y][2*x] = buffer[x];
+        mf->orig_cb[2*y][2*x+1] = buffer[x];
+        mf->orig_cb[2*y+1][2*x] = buffer[x];
+        mf->orig_cb[2*y+1][2*x+1] = buffer[x];
+    }
     }
 
-    for (y = 0; y < (height >> 2); y++) {			/* V */
-	safe_fread(buffer, 1, width>>2, fpointer);
-	for ( x = 0; x < (width>>2); x++ ) {
-	    mf->orig_cr[2*y][2*x] = buffer[x];
-	    mf->orig_cr[2*y][2*x+1] = buffer[x];
-	    mf->orig_cr[2*y+1][2*x] = buffer[x];
-	    mf->orig_cr[2*y+1][2*x+1] = buffer[x];
-	}
+    for (y = 0; y < (height >> 2); y++) {           /* V */
+    safe_fread(buffer, 1, width>>2, fpointer);
+    for ( x = 0; x < (width>>2); x++ ) {
+        mf->orig_cr[2*y][2*x] = buffer[x];
+        mf->orig_cr[2*y][2*x+1] = buffer[x];
+        mf->orig_cr[2*y+1][2*x] = buffer[x];
+        mf->orig_cr[2*y+1][2*x+1] = buffer[x];
+    }
     }
 }
 
@@ -788,9 +815,9 @@ ReadSub4(mf, fpointer, width, height)
  *
  * DoGamma
  *
- *	Gamma Correct the Lum values
+ *  Gamma Correct the Lum values
  *
- * RETURNS:	nothing
+ * RETURNS: nothing
  *
  * SIDE EFFECTS:    Raises Y values to power gamma.
  *
@@ -823,9 +850,9 @@ int w,h;
  *
  * DoKillDim
  *
- *	Applies an input filter to small Y values.
+ *  Applies an input filter to small Y values.
  *
- * RETURNS:	nothing
+ * RETURNS: nothing
  *
  * SIDE EFFECTS:    Changes Y values:
  *
@@ -963,7 +990,7 @@ int w,h;
  * Debugging of ReadAYUV
  *
  * Revision 1.8  1995/01/16  13:18:24  eyhung
- * Interlaced YUV format (e.g. Abekas) support added (slightly buggy)
+ * Interlaced YUV format (e.g. Abekas) capability added (slightly buggy)
  *
  * Revision 1.7  1995/01/16  06:58:23  eyhung
  * Added skeleton of ReadAYUV (for Abekas YUV files)
