@@ -11,9 +11,12 @@
 **
 ** Changes:
 ** 
-** 03/2003 - Added 24+32 bpp support.
+** 03/2003 - Added 24+32 bpp capability.
 */
 
+#define _BSD_SOURCE 1      /* Make sure strdup() is in string.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
+
 #include <math.h>
 #include <string.h>
 #include <assert.h>
@@ -392,33 +395,45 @@ read8Bitmap (int width, int height)
     return bitmap;
 }
 
+
+
 /*
  * Read a true color bitmap. (24/32 bits)
  * 
  * The output routine deplanarizes it for us, we keep it flat here.
  */
 static u1 *
-readXBitmap (int width, int height, int bpp)
-{
-    int tmp = 0;
-    int bytes = bpp >> 3;
+readXBitmap (int const width, 
+             int const height, 
+             int const bpp) {
+    int          const bytes  = bpp >> 3;
+    unsigned int const xBytes = width * bytes;
+
     u1 * bitmap;
-    /* remember - bmp (dib) stored upside down, so reverse */
-    u1 * bitcurptr = bitmap + (bytes * width * (height-1) ) * (sizeof (u1));
-    unsigned int xBytes = width * bytes;
+        /* remember - bmp (dib) stored upside down, so reverse */
 
     MALLOCARRAY(bitmap, bytes * width * height);
     if (bitmap == NULL)
-        pm_error("out of memory");
+        pm_error("out of memory allocating bitmap array");
 
-    for (tmp = 0;tmp < height; tmp++, bitcurptr -= xBytes) {
-        u1 * row = readU1String(xBytes);
-        memcpy(bitcurptr, row, xBytes);
-        free(row);
+    {
+        unsigned int i;
+        u1 * bitcurptr;
+
+        for (i = 0, bitcurptr = &bitmap[bytes * width * (height-1)];
+             i < height; 
+             ++i, bitcurptr -= xBytes) {
+
+            u1 * const row = readU1String(xBytes);
+            memcpy(bitcurptr, row, xBytes);
+            free(row);
+        }
     }
     return bitmap;
 }
 
+
+
 static MS_Ico 
 readIconFile (bool const verbose) {
     int iter,iter2;
@@ -557,7 +572,7 @@ trimOutputName(const char inputName[])
      * oh, for =~ ... :)
      */
     char * outFile = strdup(inputName);
-    if (!strcmp (outFile + (strlen (outFile) - 4), ".ppm")) {
+    if (STREQ(outFile + (strlen (outFile) - 4), ".ppm")) {
         *(outFile + (strlen (outFile) - 4)) = 0;
     }
     return outFile;
@@ -813,7 +828,7 @@ main(int argc, char *argv[]) {
         pm_message("-bestqual doesn't make sense with -allicons.  "
                    "Ignoring -bestqual.");
    
-    if (strcmp(cmdline.outputFilespec, "-") == 0)
+    if (STREQ(cmdline.outputFilespec, "-"))
         outputFileBase = NULL;
     else
         outputFileBase = trimOutputName(cmdline.outputFilespec);
