@@ -12,7 +12,7 @@
 **
 ** The paper recommends using a hexagon sampling region around each
 ** pixel being processed, allowing an effective sub pixel radius to be
-** specified. The hexagon values are sythesised by area sampling the
+** specified. The hexagon values are sythesized by area sampling the
 ** rectangular pixels with a hexagon grid. The seven hexagon values
 ** obtained from the 3x3 pixel grid are used to compute the alpha
 ** trimmed mean. Note that an alpha value of 0.0 gives a conventional
@@ -48,6 +48,8 @@
 */
 
 #include <math.h>
+
+#include "pm_c_util.h"
 #include "pnm.h"
 
 /* MXIVAL is the maximum input sample value we can handle.
@@ -103,6 +105,12 @@ static  xelval maxval;
 /* rounded un-scale */
 #define UNSCALE(x) ((x) >> SCALEB)
 
+static double
+sqr(double const arg) {
+    return arg * arg;
+}
+
+
 
 /* We restrict radius to the values: 0.333333 <= radius <= 1.0 */
 /* so that no fewer and no more than a 3x3 grid of pixels around */
@@ -297,131 +305,207 @@ hex_area(double sx, double sy, double hx, double hy, double d) {
 
 
 
-/* Table initialisation function - return alpha range */
-static int 
-atfilt_setup(double alpha, double radius, double maxscale) {
-    /* other function variables */
-    int alpharange;                 /* alpha range value 0 - 3 */
-    double meanscale;               /* scale for finding mean */
-    double mmeanscale;              /* scale for finding mean - midle hex */
-    double alphafraction;   
-    /* fraction of next largest/smallest to subtract from sum */
 
-    /* do setup */
-    
-    if (alpha >= 0.0 && alpha < 1.0) {
+static void
+setupAvediv(void) {
+
+    unsigned int i;
+
+    for (i=0; i < (7 * NOCSVAL); ++i) {
+        /* divide scaled value by 7 lookup */
+        AVEDIV[i] = CSCTOSC(i)/7;       /* scaled divide by 7 */
+    }
+
+}
+
+
+
+
+static void
+setupSquare(void) {
+
+    unsigned int i;
+
+    for (i=0; i < (2 * NOCSVAL); ++i) {
+        /* compute square and rescale by (val >> (2 * SCALEB + 2)) table */
+        int const val = CSCTOSC(i - NOCSVAL); 
+        /* NOCSVAL offset to cope with -ve input values */
+        SQUARE[i] = (val * val) >> (2 * SCALEB + 2);
+    }
+}
+
+
+
+
+static void
+setup1(double   const alpha,
+       double   const radius,
+       double   const maxscale,
+       int *    const alpharangeP,
+       double * const meanscaleP,
+       double * const mmeanscaleP,
+       double * const alphafractionP,
+       int *    const noisevarianceP) {
+
+
+    setupAvediv();
+    setupSquare();
+
+    if (alpha >= 0.0 && alpha <= 0.5) {
         /* alpha trimmed mean */
-        double noinmean;
+        double const noinmean =  ((0.5 - alpha) * 12.0) + 1.0;
             /* number of elements (out of a possible 7) used in the mean */
-        noinmean = ((0.5 - alpha) * 12.0) + 1.0;
-        mmeanscale = meanscale = maxscale/noinmean;
+
+        *mmeanscaleP = *meanscaleP = maxscale/noinmean;
         if (alpha == 0.0) {
             /* mean filter */ 
-            alpharange = 0;
-            alphafraction = 0.0;            /* not used */
+            *alpharangeP = 0;
+            *alphafractionP = 0.0;            /* not used */
         } else if (alpha < (1.0/6.0)) {
             /* mean of 5 to 7 middle values */
-            alpharange = 1;
-            alphafraction = (7.0 - noinmean)/2.0;
+            *alpharangeP = 1;
+            *alphafractionP = (7.0 - noinmean)/2.0;
         } else if (alpha < (1.0/3.0)) {
             /* mean of 3 to 5 middle values */
-            alpharange = 2;
-            alphafraction = (5.0 - noinmean)/2.0;
+            *alpharangeP = 2;
+            *alphafractionP = (5.0 - noinmean)/2.0;
         } else {
             /* mean of 1 to 3 middle values */
             /* alpha == 0.5 == median filter */
-            alpharange = 3;
-            alphafraction = (3.0 - noinmean)/2.0;
+            *alpharangeP = 3;
+            *alphafractionP = (3.0 - noinmean)/2.0;
         }
-    } else if (alpha > 0.5) {
+    } else if (alpha >= 1.0 && alpha <= 2.0) {
         /* optimal estimation - alpha controls noise variance threshold. */
-        int i;
-        double noinmean = 7.0;
-        alpharange = 5;                 /* edge enhancement function */
-        alpha -= 1.0;                   /* normalise it to 0.0 -> 1.0 */
-        mmeanscale = meanscale = maxscale;      /* compute scaled hex values */
-        alphafraction = 1.0/noinmean;   
+        double const alphaNormalized = alpha - 1.0;
+            /* normalize it to 0.0 -> 1.0 */
+        double const noinmean = 7.0;
+        *alpharangeP = 5;                 /* edge enhancement function */
+        *mmeanscaleP = *meanscaleP = maxscale;  /* compute scaled hex values */
+        *alphafractionP = 1.0/noinmean;   
             /* Set up 1:1 division lookup - not used */
-        noisevariance = alpha * (double)omaxval;
-        noisevariance = noisevariance * noisevariance / 8.0;    
+        *noisevarianceP = sqr(alphaNormalized * omaxval) / 8.0;    
             /* estimate of noise variance */
-        /* set yp optimal estimation specific stuff */
-        for (i=0; i < (7 * NOCSVAL); i++) {
-            /* divide scaled value by 7 lookup */
-            AVEDIV[i] = CSCTOSC(i)/7;       /* scaled divide by 7 */
-        }
-        for (i=0; i < (2 * NOCSVAL); i++) {
-            /* compute square and rescale by (val >> (2 * SCALEB + 2)) table */
-            int val;
-            val = CSCTOSC(i - NOCSVAL); 
-                /* NOCSVAL offset to cope with -ve input values */
-            SQUARE[i] = (val * val) >> (2 * SCALEB + 2);
-        }
-    } else {
+    } else if (alpha >= -0.9 && alpha <= -0.1) {
         /* edge enhancement function */
-        alpharange = 4;                 /* edge enhancement function */
-        alpha = -alpha;                 /* turn it the right way up */
-        meanscale = maxscale * (-alpha/((1.0 - alpha) * 7.0));
-            /* mean of 7 and scaled by -alpha/(1-alpha) */
-        mmeanscale = maxscale * (1.0/(1.0 - alpha) + meanscale);    
-            /* middle pixel has 1/(1-alpha) as well */
-        alphafraction = 0.0;    /* not used */
+        double const posAlpha = -alpha;
+            /* positive alpha value */
+        *alpharangeP = 4;                 /* edge enhancement function */
+        *meanscaleP = maxscale * (-posAlpha/((1.0 - posAlpha) * 7.0));
+            /* mean of 7 and scaled by -posAlpha/(1-posAlpha) */
+        *mmeanscaleP = maxscale * (1.0/(1.0 - posAlpha) + *meanscaleP);    
+            /* middle pixel has 1/(1-posAlpha) as well */
+        *alphafractionP = 0.0;    /* not used */
+    } else {
+        /* An entry condition on 'alpha' makes this impossible */
+        pm_error("INTERNAL ERROR: impossible alpha value: %f", alpha);
     }
+}
 
-    {
-        /* Setup pixel weighting tables - note we pre-compute mean
-           division here too. 
-        */
-        int i;
-        double hexhoff,hexvoff;
-        double tabscale,mtabscale;
-        double v0,v1,v2,v3,m0,m1,m2,h0,h1,h2,h3;
 
-        hexhoff = radius/2;      
-            /* horizontal offset of vertical hex centers */
-        hexvoff = 3.0 * radius/sqrt(12.0); 
-            /* vertical offset of vertical hex centers */
-        /* scale tables to normalise by hexagon area, and number of
-           hexes used in mean 
-        */
-        tabscale = meanscale / (radius * hexvoff);
-        mtabscale = mmeanscale / (radius * hexvoff);
-        v0 = hex_area(0.0,0.0,hexhoff,hexvoff,radius) * tabscale;
-        v1 = hex_area(0.0,1.0,hexhoff,hexvoff,radius) * tabscale;
-        v2 = hex_area(1.0,1.0,hexhoff,hexvoff,radius) * tabscale;
-        v3 = hex_area(1.0,0.0,hexhoff,hexvoff,radius) * tabscale;
-        m0 = hex_area(0.0,0.0,0.0,0.0,radius) * mtabscale;
-        m1 = hex_area(0.0,1.0,0.0,0.0,radius) * mtabscale;
-        m2 = hex_area(0.0,-1.0,0.0,0.0,radius) * mtabscale;
-        h0 = hex_area(0.0,0.0,radius,0.0,radius) * tabscale;
-        h1 = hex_area(1.0,1.0,radius,0.0,radius) * tabscale;
-        h2 = hex_area(1.0,0.0,radius,0.0,radius) * tabscale;
-        h3 = hex_area(1.0,-1.0,radius,0.0,radius) * tabscale;
 
-        for (i=0; i <= MXIVAL; i++) {
-            double fi;
-            fi = (double)i;
-            V0[i] = ROUNDSCALE(fi * v0);
-            V1[i] = ROUNDSCALE(fi * v1);
-            V2[i] = ROUNDSCALE(fi * v2);
-            V3[i] = ROUNDSCALE(fi * v3);
-            M0[i] = ROUNDSCALE(fi * m0);
-            M1[i] = ROUNDSCALE(fi * m1);
-            M2[i] = ROUNDSCALE(fi * m2);
-            H0[i] = ROUNDSCALE(fi * h0);
-            H1[i] = ROUNDSCALE(fi * h1);
-            H2[i] = ROUNDSCALE(fi * h2);
-            H3[i] = ROUNDSCALE(fi * h3);
-        }
-        /* set up alpha fraction lookup table used on big/small */
-        for (i=0; i < (NOIVAL * 8); i++) {
-            ALFRAC[i] = ROUNDSCALE((double)i * alphafraction);
-        }
+
+static void
+setupAlfrac(double const alphafraction) {
+    /* set up alpha fraction lookup table used on big/small */
+
+    unsigned int i;
+
+    for (i=0; i < (NOIVAL * 8); ++i) {
+        ALFRAC[i] = ROUNDSCALE(i * alphafraction);
     }
+}
+
+
+
+
+static void
+setupPixelWeightingTables(double const radius,
+                          double const meanscale,
+                          double const mmeanscale) {
+
+    /* Setup pixel weighting tables - note we pre-compute mean
+       division here too. 
+    */
+    double const hexhoff = radius/2;      
+        /* horizontal offset of vertical hex centers */
+    double const hexvoff = 3.0 * radius/sqrt(12.0); 
+        /* vertical offset of vertical hex centers */
+
+    double const tabscale  = meanscale  / (radius * hexvoff);
+    double const mtabscale = mmeanscale / (radius * hexvoff);
+
+    /* scale tables to normalize by hexagon area, and number of
+       hexes used in mean 
+    */
+    double const v0 =
+        hex_area(0.0,  0.0, hexhoff, hexvoff, radius) * tabscale;
+    double const v1 =
+        hex_area(0.0,  1.0, hexhoff, hexvoff, radius) * tabscale;
+    double const v2 =
+        hex_area(1.0,  1.0, hexhoff, hexvoff, radius) * tabscale;
+    double const v3 =
+        hex_area(1.0,  0.0, hexhoff, hexvoff, radius) * tabscale;
+    double const m0 =
+        hex_area(0.0,  0.0, 0.0,     0.0,     radius) * mtabscale;
+    double const m1 =
+        hex_area(0.0,  1.0, 0.0,     0.0,     radius) * mtabscale;
+    double const m2 =
+        hex_area(0.0, -1.0, 0.0,     0.0,     radius) * mtabscale;
+    double const h0 =
+        hex_area(0.0,  0.0, radius,  0.0,     radius) * tabscale;
+    double const h1 =
+        hex_area(1.0,  1.0, radius,  0.0,     radius) * tabscale;
+    double const h2 =
+        hex_area(1.0,  0.0, radius,  0.0,     radius) * tabscale;
+    double const h3 =
+        hex_area(1.0, -1.0, radius,  0.0,     radius) * tabscale;
+
+    unsigned int i;
+
+    for (i=0; i <= MXIVAL; ++i) {
+        V0[i] = ROUNDSCALE(i * v0);
+        V1[i] = ROUNDSCALE(i * v1);
+        V2[i] = ROUNDSCALE(i * v2);
+        V3[i] = ROUNDSCALE(i * v3);
+        M0[i] = ROUNDSCALE(i * m0);
+        M1[i] = ROUNDSCALE(i * m1);
+        M2[i] = ROUNDSCALE(i * m2);
+        H0[i] = ROUNDSCALE(i * h0);
+        H1[i] = ROUNDSCALE(i * h1);
+        H2[i] = ROUNDSCALE(i * h2);
+        H3[i] = ROUNDSCALE(i * h3);
+    }
+}
+
+
+
+
+/* Table initialization function - return alpha range */
+static int 
+atfilt_setup(double const alpha,
+             double const radius,
+             double const maxscale) {
+
+    int alpharange;                 /* alpha range value 0 - 5 */
+    double meanscale;               /* scale for finding mean */
+    double mmeanscale;              /* scale for finding mean - midle hex */
+    double alphafraction;   
+        /* fraction of next largest/smallest to subtract from sum */
+
+    setup1(alpha, radius, maxscale,
+           &alpharange, &meanscale, &mmeanscale, &alphafraction,
+           &noisevariance);
+
+    setupAlfrac(alphafraction);
+
+    setupPixelWeightingTables(radius, meanscale, mmeanscale);
+
     return alpharange;
 }
 
 
+
 static int 
 atfilt0(int * p) {
 /* Core pixel processing function - hand it 3x3 pixels and return result. */
@@ -843,9 +927,12 @@ verifySame(unsigned int const imageSeq,
 int (*atfuncs[6]) (int *) = {atfilt0,atfilt1,atfilt2,atfilt3,atfilt4,atfilt5};
 
 
+
+
 int
 main(int argc, char *argv[]) {
-    FILE* ifp;
+
+    FILE * ifp;
 	bool eof;  /* We've hit the end of the input stream */
     unsigned int imageSeq;  /* Sequence number of image, starting from 0 */
 
@@ -855,86 +942,87 @@ main(int argc, char *argv[]) {
         "-0.1 >= alpha >= -0.9 for edge enhancement\n"
         "0.3333 <= radius <= 1.0 specify effective radius\n";
 
-        pnm_init( &argc, argv );
+    pnm_init( &argc, argv );
 
-        if ( argc < 3 || argc > 4 )
-            pm_usage( usage );
+    if ( argc < 3 || argc > 4 )
+        pm_usage( usage );
 
-        if ( sscanf( argv[1], "%lf", &alpha ) != 1 )
-            pm_usage( usage );
-        if ( sscanf( argv[2], "%lf", &radius ) != 1 )
-            pm_usage( usage );
+    if ( sscanf( argv[1], "%lf", &alpha ) != 1 )
+        pm_usage( usage );
+    if ( sscanf( argv[2], "%lf", &radius ) != 1 )
+        pm_usage( usage );
         
-        if ((alpha > -0.1 && alpha < 0.0) || (alpha > 0.5 && alpha < 1.0))
-            pm_error( "Alpha must be in range 0.0 <= alpha <= 0.5 "
-                      "for alpha trimmed mean" );
-        if (alpha > 2.0)
-            pm_error( "Alpha must be in range 1.0 <= alpha <= 2.0 "
-                      "for optimal estimation" );
-        if (alpha < -0.9 || (alpha > -0.1 && alpha < 0.0))
-            pm_error( "Alpha must be in range -0.9 <= alpha <= -0.1 "
-                      "for edge enhancement" );
-        if (radius < 0.333 || radius > 1.0)
-            pm_error( "Radius must be in range 0.333333333 <= radius <= 1.0" );
+    if ((alpha > -0.1 && alpha < 0.0) || (alpha > 0.5 && alpha < 1.0))
+        pm_error( "Alpha must be in range 0.0 <= alpha <= 0.5 "
+                  "for alpha trimmed mean" );
+    if (alpha > 2.0)
+        pm_error( "Alpha must be in range 1.0 <= alpha <= 2.0 "
+                  "for optimal estimation" );
+    if (alpha < -0.9 || (alpha > -0.1 && alpha < 0.0))
+        pm_error( "Alpha must be in range -0.9 <= alpha <= -0.1 "
+                  "for edge enhancement" );
+    if (radius < 0.333 || radius > 1.0)
+        pm_error( "Radius must be in range 0.333333333 <= radius <= 1.0" );
 
-        if ( argc == 4 )
-            ifp = pm_openr( argv[3] );
-        else
-            ifp = stdin;
+    if ( argc == 4 )
+        ifp = pm_openr( argv[3] );
+    else
+        ifp = stdin;
         
-        pnm_readpnminit( ifp, &cols, &rows, &maxval, &format );
+    pnm_readpnminit( ifp, &cols, &rows, &maxval, &format );
         
-        if (maxval > MXIVAL) 
-            pm_error("The maxval of the input image (%d) is too large.\n"
-                     "This program's limit is %d.", 
-                     maxval, MXIVAL);
+    if (maxval > MXIVAL) 
+        pm_error("The maxval of the input image (%d) is too large.\n"
+                 "This program's limit is %d.", 
+                 maxval, MXIVAL);
         
-        oformat = PNM_FORMAT_TYPE(format);
-        /* force output to max precision without forcing new 2-byte format */
-        omaxval = MIN(maxval, PPM_MAXMAXVAL);
+    oformat = PNM_FORMAT_TYPE(format);
+    /* force output to max precision without forcing new 2-byte format */
+    omaxval = MIN(maxval, PPM_MAXMAXVAL);
         
-        atfunc = atfuncs[atfilt_setup(alpha,radius,
-                                      (double)omaxval/(double)maxval)];
+    atfunc = atfuncs[atfilt_setup(alpha, radius,
+                                  (double)omaxval/(double)maxval)];
 
-        if ( oformat < PGM_TYPE ) {
-            oformat = RPGM_FORMAT;
-            pm_message( "promoting file to PGM" );
+    if ( oformat < PGM_TYPE ) {
+        oformat = RPGM_FORMAT;
+        pm_message( "promoting file to PGM" );
+    }
+
+    orow = pnm_allocrow(cols);
+    irows[0] = pnm_allocrow(cols);
+    irows[1] = pnm_allocrow(cols);
+    irows[2] = pnm_allocrow(cols);
+    irow0 = irows[0];
+    irow1 = irows[1];
+    irow2 = irows[2];
+
+    eof = FALSE;  /* We're already in the middle of the first image */
+    imageSeq = 0;
+    while (!eof) {
+        do_one_frame(ifp);
+        pm_nextimage(ifp, &eof);
+        if (!eof) {
+            /* Read and validate header of next image */
+            int imageCols, imageRows;
+            xelval imageMaxval;
+            int imageFormat;
+
+            ++imageSeq;
+            pnm_readpnminit(ifp, &imageCols, &imageRows, 
+                            &imageMaxval, &imageFormat);
+            verifySame(imageSeq,
+                       imageCols, imageRows, imageMaxval, imageFormat,
+                       cols, rows, maxval, format);
         }
+    }
 
-        orow = pnm_allocrow( cols );
-        irows[0] = pnm_allocrow( cols );
-        irows[1] = pnm_allocrow( cols );
-        irows[2] = pnm_allocrow( cols );
-        irow0 = irows[0];
-        irow1 = irows[1];
-        irow2 = irows[2];
+    pnm_freerow(irow0);
+    pnm_freerow(irow1);
+    pnm_freerow(irow2);
+    pnm_freerow(orow);
+    pm_close(ifp);
 
-        eof = FALSE;  /* We're already in the middle of the first image */
-        imageSeq = 0;
-        while (!eof) {
-            do_one_frame(ifp);
-            pm_nextimage(ifp, &eof);
-            if (!eof) {
-                /* Read and validate header of next image */
-                int imageCols, imageRows;
-                xelval imageMaxval;
-                int imageFormat;
-
-                ++imageSeq;
-                pnm_readpnminit( ifp, &imageCols, &imageRows, 
-                                 &imageMaxval, &imageFormat );
-                verifySame(imageSeq,
-                           imageCols, imageRows, imageMaxval, imageFormat,
-                           cols, rows, maxval, format);
-            }
-	      }
-
-        pnm_freerow(irow0);
-        pnm_freerow(irow1);
-        pnm_freerow(irow2);
-        pnm_freerow(orow);
-        pm_close(ifp);
-        exit(0);
-	}
+    return 0;
+}
 
 
