@@ -27,11 +27,13 @@
   package.
 *****************************************************************************/
 
+#include <assert.h>
+
 #include "pnm.h"
 #include "shhopt.h"
 #include "mallocvar.h"
 
-enum brightMethod {BRIGHT_MAX, BRIGHT_LUMIN};
+enum brightMethod {BRIGHT_LUMINOSITY, BRIGHT_COLORVALUE, BRIGHT_SATURATION};
 
 struct cmdlineInfo {
     /* All the information the user supplied in the command line,
@@ -40,12 +42,19 @@ struct cmdlineInfo {
     const char *inputFilespec;  /* Filespec of input file */
     unsigned int bvalueSpec;
     xelval bvalue;
+    unsigned int bpercentSpec;
     float bpercent;
     unsigned int wvalueSpec;
     xelval wvalue;
+    unsigned int wpercentSpec;
     float wpercent;
     enum brightMethod brightMethod;
     unsigned int keephues;
+    float maxExpansion;
+        /* The maximum allowed expansion factor for expansion specified
+           by per centile.  This is a factor, not a per cent increase.
+           E.g. 50% increase means a factor of 1.50.
+        */
 };
 
 
@@ -63,29 +72,45 @@ parseCommandLine (int argc, char ** argv,
    Note that the strings we return are stored in the storage that
    was passed to us as the argv array.  We also trash *argv.
 -----------------------------------------------------------------------------*/
-    optEntry *option_def = malloc( 100*sizeof( optEntry ) );
+    optEntry *option_def;
         /* Instructions to optParseOptions3 on how to parse our options.
          */
     optStruct3 opt;
 
-    unsigned int bpercentSpec, wpercentSpec;
-    unsigned int brightmax;
-
+    unsigned int luminosity, colorvalue, saturation;
+    unsigned int maxexpandSpec;
+    float maxexpand;
+    
     unsigned int option_def_index;
 
+    MALLOCARRAY_NOFAIL(option_def, 100);
+
     option_def_index = 0;   /* incremented by OPTENT3 */
     OPTENT3(0,   "bpercent",      OPT_FLOAT,   
-            &cmdlineP->bpercent,   &bpercentSpec, 0);
+            &cmdlineP->bpercent,   &cmdlineP->bpercentSpec, 0);
     OPTENT3(0,   "wpercent",      OPT_FLOAT,   
-            &cmdlineP->wpercent,   &wpercentSpec, 0);
+            &cmdlineP->wpercent,   &cmdlineP->wpercentSpec, 0);
     OPTENT3(0,   "bvalue",        OPT_UINT,   
             &cmdlineP->bvalue,     &cmdlineP->bvalueSpec, 0);
     OPTENT3(0,   "wvalue",        OPT_UINT,   
             &cmdlineP->wvalue,     &cmdlineP->wvalueSpec, 0);
+    OPTENT3(0,   "maxexpand",     OPT_FLOAT,   
+            &maxexpand,            &maxexpandSpec, 0);
     OPTENT3(0,   "keephues",      OPT_FLAG,   
             NULL,                  &cmdlineP->keephues, 0);
-    OPTENT3(0,   "brightmax",      OPT_FLAG,   
-            NULL,                  &brightmax, 0);
+    OPTENT3(0,   "luminosity",    OPT_FLAG,   
+            NULL,                  &luminosity, 0);
+    OPTENT3(0,   "colorvalue",    OPT_FLAG,   
+            NULL,                  &colorvalue, 0);
+    OPTENT3(0,   "saturation",    OPT_FLAG,   
+            NULL,                  &saturation, 0);
+    OPTENT3(0,   "brightmax",     OPT_FLAG,   
+            NULL,                  &colorvalue, 0);
+
+    /* Note: -brightmax was documented and accepted long before it was
+       actually implemented.  By the time we implemented it, we
+       decided -colorvalue was a better name for it.
+    */
 
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
@@ -94,15 +119,9 @@ parseCommandLine (int argc, char ** argv,
     optParseOptions3( &argc, argv, opt, sizeof(opt), 0 );
         /* Uses and sets argc, argv, and some of *cmdline_p and others. */
 
-    if(wpercentSpec && cmdlineP->wvalueSpec)
-        pm_error("You may specify only one of -wpercent and -wvalue");
-
-    if(bpercentSpec && cmdlineP->bvalueSpec)
-        pm_error("You may specify only one of -bpercent and -bvalue");
-
-    if (!wpercentSpec)
+    if (!cmdlineP->wpercentSpec)
         cmdlineP->wpercent = 1.0;
-    if (!bpercentSpec)
+    if (!cmdlineP->bpercentSpec)
         cmdlineP->bpercent = 2.0;
 
     if (cmdlineP->wpercent < 0.0)
@@ -118,7 +137,25 @@ parseCommandLine (int argc, char ** argv,
         pm_error("You specified a per centage > 100 for bpercent: %f",
                  cmdlineP->bpercent);
 
-    cmdlineP->brightMethod = brightmax > 0 ? BRIGHT_MAX : BRIGHT_LUMIN;
+    if (luminosity + colorvalue + saturation > 1)
+        pm_error("You can specify only one of "
+                 "-luminosity, -colorvalue, and -saturation");
+    else {
+        if (colorvalue)
+            cmdlineP->brightMethod = BRIGHT_COLORVALUE;
+        else if (saturation)
+            cmdlineP->brightMethod = BRIGHT_SATURATION;
+        else
+            cmdlineP->brightMethod = BRIGHT_LUMINOSITY;
+    }
+    if (maxexpandSpec) {
+        if (maxexpand < 0)
+            pm_error("-maxexpand must be positive.  You specified %f",
+                     maxexpand);
+        else
+            cmdlineP->maxExpansion = 1 + (float)maxexpand/100;
+    } else
+        cmdlineP->maxExpansion = 1e6;  /* essentially infinite */
 
     if (argc-1 > 1)
         pm_error("Program takes at most one argument: the input file "
@@ -144,12 +181,13 @@ buildHistogram(FILE *   const ifp,
    Build the histogram of brightness values for the image that is in file
    'ifp', which is positioned just after the header (at the raster).
 
-   The histogram is the array hist[] such that hist[x] is the number of
-   xels in the image that have brightness x.  That brightness is either
-   the intensity of most intense component of the xel or it is the 
-   luminosity of the xel, depending on 'brightMethod'.  In either case,
-   it is based on the same maxval as the image, which is 'maxval'.  The
-   image is 'cols' columns wide by 'rows' rows high.
+   The histogram is the array hist[] such that hist[x] is the number
+   of xels in the image that have brightness x.  That brightness is
+   either the color value (intensity of most intense component) of the
+   xel or it is the luminosity of the xel, depending on
+   'brightMethod'.  In either case, it is based on the same maxval as
+   the image, which is 'maxval'.  The image is 'cols' columns wide by
+   'rows' rows high.
 
    Leave the file positioned arbitrarily.
 -----------------------------------------------------------------------------*/
@@ -170,11 +208,17 @@ buildHistogram(FILE *   const ifp,
             xelval brightness;
             xel const p = xelrow[col];
             if (PNM_FORMAT_TYPE(format) == PPM_TYPE) {
-                if (brightMethod == BRIGHT_MAX)
-                    brightness = MAX(PPM_GETR(p), 
-                                     MAX(PPM_GETR(p), PPM_GETB(p)));
-                else
+                switch(brightMethod) {
+                case BRIGHT_LUMINOSITY:
                     brightness = PPM_LUMIN(p);
+                    break;
+                case BRIGHT_COLORVALUE:
+                    brightness = ppm_colorvalue(p);
+                    break;
+                case BRIGHT_SATURATION:
+                    brightness = ppm_saturation(p, maxval);
+                    break;
+                }
             } else
                 brightness = PNM_GET1(p);
             ++hist[brightness];
@@ -246,14 +290,78 @@ computeTopPercentile(unsigned int         hist[],
 
 
 static void
-computeEndValues(FILE *   const ifp,
-                 int      const cols,
-                 int      const rows,
-                 xelval   const maxval,
-                 int      const format,
+computeAdjustmentForExpansionLimit(xelval   const maxval,
+                                   xelval   const unlBvalue,
+                                   xelval   const unlWvalue,
+                                   float    const maxExpansion,
+                                   xelval * const bLowerP,
+                                   xelval * const wRaiseP) {
+/*----------------------------------------------------------------------------
+   Assuming 'unlBvalue' and 'unlWvalue' are the appropriate bvalue and
+   wvalue to normalize the image to 0 .. maxval, compute the amount
+   by which the bvalue must be raised and the wvalue lowered from that
+   in order to cap the expansion factor at 'maxExpansion'.
+
+   E.g. if 'maxval' is 100, 'unlBvalue' is 20 and 'unlWvalue' is 70, that
+   implies an expansion factor of 100/50 (because the range goes from
+   70-20, which is 50, to 100 - 0, which is 100).  If 'maxEpansion' is
+   1.333, these values are unacceptable.  To get down to the desired 1.333
+   factor, we need the span of bvalue to wvalue to be 75, not 50.  So
+   we need to raise the bvalue and lower the wvalue by a total of 25.
+   We apportion that adjustment to bvalue and wvalue in proportion to
+   how close each is already to it's end (which we call the margin).
+   'unlBvalue' is 20 from its end, while 'unlWvalue' is 30 from its end,
+   so we want to lower the bvalue by 10 and raise the wvalue by 15.
+   Ergo we return *bLowerP = 10 and *wRaise = 15.
+-----------------------------------------------------------------------------*/
+    unsigned int const newRange = maxval - 0;
+        /* The range of sample values after normalization, if we used
+           the unlimited bvalue and wvalue
+        */
+    unsigned int const oldRange = unlWvalue - unlBvalue;
+        /* The range of sample values in the original image that normalize
+           to 0 .. maxval, if we used the unlimited bvalue and wvalue
+        */
+    float const unlExpansion = (float)newRange/oldRange;
+    
+    if (unlExpansion <= maxExpansion) {
+        /* No capping is necessary.  Unlimited values are already within
+           range.
+           */
+        *bLowerP = 0;
+        *wRaiseP = 0;
+    } else {
+        unsigned int const totalWidening = newRange/maxExpansion - oldRange;
+            /* Amount by which the (bvalue, wvalue) range must be widened
+               to limit expansion to 'maxExpansion'
+            */
+        unsigned int const bMargin = unlBvalue - 0;
+        unsigned int const wMargin = maxval - unlWvalue;
+
+        /* Apportion 'totalWidening' between the black and and the
+           white end
+        */
+        *bLowerP =
+            ROUNDU((float)bMargin / (bMargin + wMargin) * totalWidening);
+        *wRaiseP =
+            ROUNDU((float)wMargin / (bMargin + wMargin) * totalWidening);
+
+        pm_message("limiting expansion of %.1f%% to %.1f%%",
+                   (unlExpansion - 1) * 100, (maxExpansion -1) * 100);
+    }
+}
+
+
+
+static void
+computeEndValues(FILE *             const ifp,
+                 int                const cols,
+                 int                const rows,
+                 xelval             const maxval,
+                 int                const format,
                  struct cmdlineInfo const cmdline,
-                 xelval * const bvalueP,
-                 xelval * const wvalueP) {
+                 xelval *           const bvalueP,
+                 xelval *           const wvalueP) {
 /*----------------------------------------------------------------------------
    Figure out what original values will be translated to full white and
    full black -- thus defining to what all the other values get translated.
@@ -269,18 +377,43 @@ computeEndValues(FILE *   const ifp,
     if (hist == NULL)
         pm_error("Unable to allocate storage for intensity histogram.");
     else {
-        buildHistogram(ifp, cols, rows, maxval, format, hist, BRIGHT_LUMIN);
-        
-        if (cmdline.bvalueSpec)
-            *bvalueP = cmdline.bvalue;
-        else 
+        xelval unlimitedBvalue, unlimitedWvalue;
+        unsigned int bLower, wRaise;
+
+        buildHistogram(ifp, cols, rows, maxval, format, hist,
+                       cmdline.brightMethod);
+
+        if (cmdline.bvalueSpec && !cmdline.bpercentSpec) {
+            unlimitedBvalue = cmdline.bvalue;
+        } else {
+            xelval percentBvalue;
             computeBottomPercentile(hist, maxval, cols*rows, cmdline.bpercent, 
-                                bvalueP);
-        if (cmdline.wvalueSpec) 
-            *wvalueP = cmdline.wvalue;
-        else
+                                    &percentBvalue);
+            if (cmdline.bvalueSpec)
+                unlimitedBvalue = MIN(percentBvalue, cmdline.bvalue);
+            else
+                unlimitedBvalue = percentBvalue;
+        }
+
+        if (cmdline.wvalueSpec && !cmdline.wpercentSpec) {
+            unlimitedWvalue = cmdline.wvalue;
+        } else {
+            xelval percentWvalue;
             computeTopPercentile(hist, maxval, cols*rows, cmdline.wpercent, 
-                                 wvalueP);
+                                 &percentWvalue);
+            if (cmdline.wvalueSpec)
+                unlimitedWvalue = MIN(percentWvalue, cmdline.wvalue);
+            else
+                unlimitedWvalue = percentWvalue;
+        }
+
+        computeAdjustmentForExpansionLimit(
+            maxval, unlimitedBvalue, unlimitedWvalue, cmdline.maxExpansion,
+            &bLower, &wRaise);
+
+        *bvalueP = unlimitedBvalue - bLower;
+        *wvalueP = unlimitedWvalue + wRaise;
+
         free(hist);
     }
 }
@@ -288,55 +421,116 @@ computeEndValues(FILE *   const ifp,
 
 
 static void
-computeTransferFunction(xelval const bvalue, 
-                        xelval const wvalue,
-                        xelval const maxval,
-                        xelval newIntensity[]) {
+computeTransferFunction(xelval            const bvalue, 
+                        xelval            const wvalue,
+                        xelval            const maxval,
+                        xelval **         const newBrightnessP) {
 /*----------------------------------------------------------------------------
-   Compute the transfer function, i.e. the array newIntensity[] such that
-   newIntensity[x] is the intensity of the xel that should replace a
-   xel with intensity x.
+   Compute the transfer function, i.e. the array *newBrightnessP such that
+   (*newBrightnessP)[x] is the brightness of the xel that should replace a
+   xel with brightness x.  Brightness in this case means either luminosity
+   or color value (and it doesn't matter to us which).
+
+   'bvalue' is the highest brightness that should map to zero brightness;
+   'wvalue' is the lowest brightness that should map to full brightness.
+   brightnesses in between should be stretched linearly.  (That stretching
+   could conceivably result in more brightnesses mapping to zero and full
+   brightness, due to rounding).
 
    Define function only for values 0..maxval.
 -----------------------------------------------------------------------------*/
+    xelval * newBrightness;
     xelval i;
 
-    /* Map the lowest intensities to zero */
+    MALLOCARRAY(newBrightness, maxval+1);
+
+    if (newBrightness == NULL)
+        pm_error("Unable to allocate memory for transfer function.");
+
+    /* Clip the lowest brightnesses to zero */
     if (bvalue > 0) 
-        for (i = 0; i <= bvalue-1; ++i)
-            newIntensity[i] = 0;
+        for (i = 0; i < bvalue; ++i)
+            newBrightness[i] = 0;
+
+    /* Map the middle brightnesses linearly onto 0..maxval */
     {
-        /* Map the middle intensities to 0..maxval */
         unsigned int const range = wvalue - bvalue;
         unsigned int val;
         /* The following for loop is a hand optimization of this one:
            for (i = bvalue; i <= wvalue; ++i)
-           newIntensity[i] = (i-bvalue)*maxval/range);
+             newBrightness[i] = (i-bvalue)*maxval/range);
            (with proper rounding)
         */
-        for (i = bvalue, val = maxval/2; 
+        for (i = bvalue, val = range/2; 
              i <= wvalue; 
              ++i, val += maxval)
-            newIntensity[i] = MIN(val / range, maxval);
-        }
-    /* Map the highest intensities to maxval */
+            newBrightness[i] = MIN(val / range, maxval);
+
+        assert(newBrightness[bvalue] == 0);
+        assert(newBrightness[wvalue] == maxval);
+    }
+
+    /* Clip the highest brightnesses to maxval */
     for (i = wvalue+1; i <= maxval; ++i)
-        newIntensity[i] = maxval;
+        newBrightness[i] = maxval;
+
+    *newBrightnessP = newBrightness;
+}
+            
+
+
+static float
+brightScaler(xel               const p,
+             pixval            const maxval,
+             xelval            const newBrightness[],
+             enum brightMethod const brightMethod) {
+/*----------------------------------------------------------------------------
+  Return the multiple by which the brightness pixel of color 'p' (based
+  on maxval 'maxval') should be changed according to the transfer
+  function newBrightness[], using the 'brightMethod' measure of
+  brightness.
+
+  For example, if 'brightMethod' is BRIGHT_LUMINOSITY, p is has
+  luminosity 50, and newBrightness[50] is 75, we would return 1.5.
+-----------------------------------------------------------------------------*/
+    xelval oldBrightness;
+    float scaler;
+             
+    switch (brightMethod) {
+    case BRIGHT_LUMINOSITY:
+        oldBrightness = PPM_LUMIN(p);
+        break;
+    case BRIGHT_COLORVALUE:
+        oldBrightness = ppm_colorvalue(p);
+        break;
+    case BRIGHT_SATURATION:
+        oldBrightness = ppm_saturation(p, maxval);
+        break;
+    }
+    if (oldBrightness == 0) {
+        assert(newBrightness[oldBrightness] == 0);
+        /* Doesn't matter what we scale by.  zero times anything is zero. */
+        scaler = 1.0;
+    } else
+        scaler = (float)newBrightness[oldBrightness]/oldBrightness;
+
+    return scaler;
 }
             
 
 
 static void
-writeRowNormalized(xel *  const xelrow,
-                   int    const cols,
-                   xelval const maxval,
-                   int    const format,
-                   bool   const keephues,
-                   xelval const newIntensity[],
-                   xel *  const rowbuf) {
+writeRowNormalized(xel *             const xelrow,
+                   int               const cols,
+                   xelval            const maxval,
+                   int               const format,
+                   enum brightMethod const brightMethod,
+                   bool              const keephues,
+                   xelval            const newBrightness[],
+                   xel *             const rowbuf) {
 /*----------------------------------------------------------------------------
    Write to Standard Output a normalized version of the xel row 
-   'xelrow'.  Normalize it via the transfer function newIntensity[].
+   'xelrow'.  Normalize it via the transfer function newBrightness[].
 
    Use 'rowbuf' as a work buffer.  It is at least 'cols' columns wide.
 -----------------------------------------------------------------------------*/
@@ -348,21 +542,20 @@ writeRowNormalized(xel *  const xelrow,
 
         if (PPM_FORMAT_TYPE(format) == PPM_TYPE) {
             if (keephues) {
-                xelval const oldIntensity = PPM_LUMIN(p);
-                float const scaler = 
-                    (float)newIntensity[oldIntensity]/oldIntensity;
-            
+                float const scaler =
+                    brightScaler(p, maxval, newBrightness, brightMethod);
+
                 xelval const r = MIN((int)(PPM_GETR(p)*scaler+0.5), maxval);
                 xelval const g = MIN((int)(PPM_GETG(p)*scaler+0.5), maxval);
                 xelval const b = MIN((int)(PPM_GETB(p)*scaler+0.5), maxval);
                 PNM_ASSIGN(outrow[col], r, g, b);
             } else 
                 PNM_ASSIGN(outrow[col], 
-                           newIntensity[PPM_GETR(p)], 
-                           newIntensity[PPM_GETG(p)], 
-                           newIntensity[PPM_GETB(p)]);
+                           newBrightness[PPM_GETR(p)], 
+                           newBrightness[PPM_GETG(p)], 
+                           newBrightness[PPM_GETB(p)]);
         } else 
-            PNM_ASSIGN1(outrow[col], newIntensity[PNM_GET1(p)]);
+            PNM_ASSIGN1(outrow[col], newBrightness[PNM_GET1(p)]);
     }
     pnm_writepnmrow(stdout, outrow, cols, maxval, format, 0);
 }
@@ -373,7 +566,7 @@ int
 main(int argc, char *argv[]) {
 
     struct cmdlineInfo cmdline;
-    FILE *ifp;
+    FILE *ifP;
     pm_filepos imagePos;
     xelval maxval;
     int rows, cols, format;
@@ -383,20 +576,20 @@ main(int argc, char *argv[]) {
 
     parseCommandLine(argc, argv, &cmdline);
 
-    ifp = pm_openr_seekable(cmdline.inputFilespec);
+    ifP = pm_openr_seekable(cmdline.inputFilespec);
 
     /* Rescale so that bvalue maps to 0, wvalue maps to maxval. */
-    pnm_readpnminit(ifp, &cols, &rows, &maxval, &format);
-    pm_tell2(ifp, &imagePos, sizeof(imagePos));
+    pnm_readpnminit(ifP, &cols, &rows, &maxval, &format);
+    pm_tell2(ifP, &imagePos, sizeof(imagePos));
 
-    computeEndValues(ifp, cols, rows, maxval, format, cmdline, 
+    computeEndValues(ifP, cols, rows, maxval, format, cmdline, 
                      &bvalue, &wvalue);
         
-    if (wvalue < bvalue)
+    if (wvalue <= bvalue)
         pm_error("The colors which become black would overlap the "
                  "colors which become white.");
     else {
-        xelval newIntensity[PNM_OVERALLMAXVAL+1];
+        xelval * newBrightness;
         int row;
         xel * xelrow;
         xel * rowbuf;
@@ -405,21 +598,23 @@ main(int argc, char *argv[]) {
 
         pm_message("remapping %d..%d to %d..%d", bvalue, wvalue, 0, maxval);
 
-        computeTransferFunction(bvalue, wvalue, maxval, newIntensity);
+        computeTransferFunction(bvalue, wvalue, maxval, &newBrightness);
 
-        pm_seek2(ifp, &imagePos, sizeof(imagePos));
+        pm_seek2(ifP, &imagePos, sizeof(imagePos));
         pnm_writepnminit(stdout, cols, rows, maxval, format, 0);
 
         rowbuf = pnm_allocrow(cols);
 
         for (row = 0; row < rows; ++row) {
-            pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
+            pnm_readpnmrow(ifP, xelrow, cols, maxval, format);
             writeRowNormalized(xelrow, cols, maxval, format,
-                               cmdline.keephues, newIntensity, rowbuf);
+                               cmdline.keephues, cmdline.brightMethod,
+                               newBrightness, rowbuf);
         }
+        free(newBrightness);
         pnm_freerow(rowbuf);
         pnm_freerow(xelrow);
     }
-    pm_close(ifp);
-    exit(0);
+    pm_close(ifP);
+    return 0;
 }
