@@ -22,11 +22,15 @@
 ******************************************************************************/
 
 #include <limits.h>
+#include <math.h>
+#include <assert.h>
 
+#include "pm_c_util.h"
+#include "mallocvar.h"
+#include "nstring.h"
+#include "shhopt.h"
 #include "pam.h"
 #include "pammap.h"
-#include "shhopt.h"
-#include "mallocvar.h"
 
 #define MAXCOLORS 32767u
 
@@ -49,8 +53,8 @@ struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
-    const char *inputFilespec;  /* Filespec of input file */
-    char *mapFilespec;    /* Filespec of colormap file; NULL if none */
+    const char * inputFilespec;  /* Filespec of input file */
+    const char * mapFilespec;    /* Filespec of colormap file */
     unsigned int floyd;   /* Boolean: -floyd/-fs option */
     enum missingMethod missingMethod;
     char * missingcolor;      
@@ -60,12 +64,6 @@ struct cmdlineInfo {
 
 
 
-static int __inline__ sqr(const int x) {
-    return x*x;
-}
-
-
-
 static void
 parseCommandLine (int argc, char ** argv,
                   struct cmdlineInfo *cmdlineP) {
@@ -79,7 +77,7 @@ parseCommandLine (int argc, char ** argv,
    Note that the strings we return are stored in the storage that
    was passed to us as the argv array.  We also trash *argv.
 -----------------------------------------------------------------------------*/
-    optEntry *option_def = malloc( 100*sizeof( optEntry ) );
+    optEntry * option_def;
         /* Instructions to optParseOptions3 on how to parse our options.
          */
     optStruct3 opt;
@@ -88,6 +86,8 @@ parseCommandLine (int argc, char ** argv,
 
     unsigned int nofloyd, firstisdefault;
     unsigned int missingSpec, mapfileSpec;
+
+    MALLOCARRAY_NOFAIL(option_def, 100);
     
     option_def_index = 0;   /* incremented by OPTENT3 */
     OPTENT3(0,   "floyd",        OPT_FLAG,   
@@ -145,45 +145,95 @@ parseCommandLine (int argc, char ** argv,
 
 
 static void
-adjustDepth(struct pam * const pamP,
-            tuple *      const tupleRow,
-            unsigned int const newDepth) {
+rgbToDepth1(const struct pam * const pamP,
+            tuple *            const tupleRow) {
+    
+    unsigned int col;
+
+    for (col = 0; col < pamP->width; ++col) {
+        unsigned int plane;
+        double grayvalue;
+        grayvalue = 0.0;  /* initial value */
+        for (plane = 0; plane < pamP->depth; ++plane)
+            grayvalue += pnm_lumin_factor[plane] * tupleRow[col][plane];
+        tupleRow[col][0] = (sample) (grayvalue + 0.5);
+    }
+}
+
+
+
+static void
+grayscaleToDepth3(const struct pam * const pamP,
+                  tuple *            const tupleRow) {
+    
+    unsigned int col;
+
+    assert(pamP->allocation_depth >= 3);
+
+    for (col = 0; col < pamP->width; ++col) {
+        tupleRow[col][1] = tupleRow[col][0];
+        tupleRow[col][2] = tupleRow[col][0];
+    }
+}
+
+
+
+static void
+adjustDepth(const struct pam * const pamP,
+            tuple *            const tupleRow,
+            unsigned int       const newDepth) {
 /*----------------------------------------------------------------------------
    Change the depth of the raster row tupleRow[] of the image
-   described by 'pamP' to newDepth to match a user-supplied color map.
+   described by 'pamP' to newDepth.
 
-   The only depth change we allow is from an RGB image of depth 3 to
-   depth 1.  In that case, we change it to grayscale or black and white.
-   tuplerow[] remains allocated for the depth of 3, but only the first
-   plane is meaningful after this.
+   We don't change the memory allocation; tupleRow[] must already have
+   space allocated for at least 'newDepth' planes.  When we're done,
+   all but the first 'newDepth' planes are meaningless, but the space is
+   still there.
+
+   The only depth changes we know how to do are:
+
+     - from tuple type RGB, depth 3 to depth 1 
+
+       We change it to grayscale or black and white.
+
+     - from tuple type GRAYSCALE or BLACKANDWHITE depth 1 to depth 3.
+
+       We change it to RGB.
 
    For any other depth change request, we issue an error message and abort
-   the program.  (It is technically possible to go from grayscale to RGB,
-   but there seems to be little requirement for that, so we save the 
-   programming effort).
+   the program.
 -----------------------------------------------------------------------------*/
     if (newDepth != pamP->depth) {
 
-        if (!stripeq(pamP->tuple_type, "RGB"))
-            pm_error("Map file depth of %d is incompatible with image "
-                     "tuple type of '%*s'.  Only type RGB images can have "
-                     "their depths changed.", newDepth, 
-                     sizeof(pamP->tuple_type), pamP->tuple_type);
-        else if (newDepth != 1)
-            pm_error("Map file depth is %d, while image depth is %d.  "
-                     "The only depth change allowed is from 3 to 1.",
-                     newDepth, pamP->depth);
-        else {
-            int col;
-            for (col = 0; col < pamP->width; ++col) {
-                unsigned int plane;
-                double grayvalue;
-                grayvalue = 0.0;  /* initial value */
-                for (plane = 0; plane < pamP->depth; ++plane)
-                    grayvalue += 
-                        pnm_lumin_factor[plane] * tupleRow[col][plane];
-                tupleRow[col][0] = (int) (grayvalue + 0.5);
-            }
+        if (stripeq(pamP->tuple_type, "RGB")) {
+            if (newDepth != 1) {
+                pm_error("Map image depth of %u differs from input image "
+                         "depth of %u, and the tuple type is RGB.  "
+                         "The only depth to which I know how to convert "
+                         "an RGB tuple is 1.",
+                         newDepth, pamP->depth);
+            } else
+                rgbToDepth1(pamP, tupleRow);
+        } else if (stripeq(pamP->tuple_type, "GRAYSCALE") ||
+                   stripeq(pamP->tuple_type, "BLACKANDWHITE")) {
+            if (newDepth != 3) {
+                pm_error("Map image depth of %u differs from input image "
+                         "depth of %u, and the tuple type is GRAYSCALE "
+                         "or BLACKANDWHITE.  "
+                         "The only depth to which I know how to convert "
+                         "a GRAYSCALE or BLACKANDWHITE tuple is 3.",
+                         newDepth, pamP->depth);
+            } else
+                grayscaleToDepth3(pamP, tupleRow);
+        } else {
+            pm_error("Map image depth of %u differs from input image depth "
+                     "of %u, and the input image does not have a tuple type "
+                     "that I know how to convert to the map depth.  "
+                     "I can convert RGB, GRAYSCALE, and BLACKANDWHITE.  "
+                     "The input image is '%.*s'.",
+                     newDepth, pamP->depth, 
+                     (int)sizeof(pamP->tuple_type), pamP->tuple_type);
         }
     }
 }
@@ -220,7 +270,8 @@ computeColorMapFromMap(struct pam *   const mappamP,
 
 
 static void
-initFserr(struct pam * const pamP, struct fserr * const fserrP) {
+initFserr(struct pam *   const pamP,
+          struct fserr * const fserrP) {
 /*----------------------------------------------------------------------------
    Initialize the Floyd-Steinberg error vectors
 -----------------------------------------------------------------------------*/
@@ -230,18 +281,22 @@ initFserr(struct pam * const pamP, struct fserr * const fserrP) {
 
     MALLOCARRAY(fserrP->thiserr, pamP->depth);
     if (fserrP->thiserr == NULL)
-        pm_error("Out of memory allocating Floyd-Steinberg structures");
+        pm_error("Out of memory allocating Floyd-Steinberg structures "
+                 "for depth %u", pamP->depth);
     MALLOCARRAY(fserrP->nexterr, pamP->depth);
     if (fserrP->nexterr == NULL)
-        pm_error("Out of memory allocating Floyd-Steinberg structures");
+        pm_error("Out of memory allocating Floyd-Steinberg structures "
+                 "for depth %u", pamP->depth);
     
     for (plane = 0; plane < pamP->depth; ++plane) {
         MALLOCARRAY(fserrP->thiserr[plane], fserrSize);
         if (fserrP->thiserr[plane] == NULL)
-            pm_error("Out of memory allocating Floyd-Steinberg structures");
+            pm_error("Out of memory allocating Floyd-Steinberg structures "
+                     "for Plane %u, size %u", plane, fserrSize);
         MALLOCARRAY(fserrP->nexterr[plane], fserrSize);
         if (fserrP->nexterr[plane] == NULL)
-            pm_error("Out of memory allocating Floyd-Steinberg structures");
+            pm_error("Out of memory allocating Floyd-Steinberg structures "
+                     "for Plane %u, size %u", plane, fserrSize);
     }
 
     srand((int)(time(0) ^ getpid()));
@@ -296,31 +351,24 @@ floydAdjustColor(struct pam *   const pamP,
 
 
 static void
-floydPropagateErr(struct pam *   const inpamP, 
+floydPropagateErr(struct pam *   const pamP, 
                   struct fserr * const fserrP, 
                   int            const col, 
                   tuple          const oldtuple, 
-                  struct pam *   const outpamP,
                   tuple          const newtuple) {
 /*----------------------------------------------------------------------------
   Propagate Floyd-Steinberg error terms.
 
-  The error is due to substituting the tuple value 'newtuple'
-  (described by *inpamP) for the tuple value 'oldtuple' (described by
-  *oldpamP).  The error terms are meant to be used to introduce a
-  compensating error into the future selection of tuples nearby in the
-  image.
-
-  We assume *inpamP and *outpamP specify the same tuple depth.
+  The error is due to substituting the tuple value 'newtuple' for the
+  tuple value 'oldtuple' (both described by *pamP).  The error terms
+  are meant to be used to introduce a compensating error into the
+  future selection of tuples nearby in the image.
 -----------------------------------------------------------------------------*/
-    float const maxvalNormalizer = inpamP->maxval / outpamP->maxval;
-    
     unsigned int plane;
-    for (plane = 0; plane < inpamP->depth; ++plane) {
-        long const newSampleNormalized = (long)
-            (maxvalNormalizer*newtuple[plane] + 0.5);
+    for (plane = 0; plane < pamP->depth; ++plane) {
+        long const newSample = newtuple[plane];
         long const oldSample = oldtuple[plane];
-        long const err = (oldSample - newSampleNormalized) * FS_SCALE;
+        long const err = (oldSample - newSample) * FS_SCALE;
             
         if (fserrP->fsForward) {
             fserrP->thiserr[plane][col + 2] += ( err * 7 ) / 16;
@@ -353,93 +401,82 @@ floydSwitchDir(struct pam * const pamP, struct fserr * const fserrP) {
 
 
 
-static void
-searchColormapCloseSimple(struct pam * const pamP,
-                          tupletable   const colormap, 
-                          int          const colors,
-                          tuple        const tuple,
-                          int *        const colormapIndexP) {
+struct colormapFinder {
 /*----------------------------------------------------------------------------
-   Same as searchColorMap(), except works only where the maxval of the
-   color map is the same as that of the search tuple and less than 256.
+   This is an object that finds a color in a colormap.  The methods
+   'searchColormapClose' and 'searchColormapExact' belong to it.
+
+   This object ought to encompass the hash table as well some day and
+   possibly encapsulate the color map altogether and just be an object
+   that opaquely maps input colors to output colors.
 -----------------------------------------------------------------------------*/
-    unsigned int i;
-    unsigned long dist;
-    dist = ULONG_MAX;
-    for (i = 0; i < colors; ++i) {
-        unsigned long newdist;  /* candidate for new 'dist' value */
-        unsigned int plane;
-
-        newdist = 0;
-        for (plane=0; plane < pamP->depth; ++plane) 
-            newdist += sqr(tuple[plane] - colormap[i]->tuple[plane]);
-        if (newdist < dist) {
-            *colormapIndexP = i;
-            dist = newdist;
-        }
-    }
-}
-
-
-
-static void
-searchColormapCloseComplex(struct pam * const pamP,
-                           tupletable   const colormap, 
-                           int          const colors,
-                           sample       const colormapMaxval,
-                           tuple        const tuple,
-                           int *        const colormapIndexP) {
-/*----------------------------------------------------------------------------
-   Same as searchColorMap().  Not as fast as searchColorMapSimple().
------------------------------------------------------------------------------*/
-    unsigned int i;
-    unsigned int const divider = colormapMaxval < 256 ? 1 : 2;
-        /* We divide our sum by this to make sure it fits in a "long" */
-    unsigned long dist;
-        /* The closest distance we've found so far between the value of
-           tuple 'tuple' and a tuple in the colormap.  This is measured as
-           the square of the cartesian distance between the tuples, except
-           that it's divided by 'divider' (above) to make sure it will fit
-           in a 'long' (minimum: 32 bits).  Note that each sample can be
-           up to 65535 (PPM_OVERALLMAXVAL).  
+    tupletable colormap;
+    unsigned int colors;
+        /* Number of colors in 'colormap'.  At least 1 */
+    unsigned int distanceDivider;
+        /* The value by which our intermediate distance calculations
+           have to be divided to make sure we don't overflow our
+           unsigned int data structure.
+           
+           To the extent 'distanceDivider' is greater than 1, closest
+           color results will be approximate -- there could
+           conceivably be a closer one that we miss.
         */
-    float const maxvalNormalizer = (float) colormapMaxval/pamP->maxval;
+};
 
-    dist = ULONG_MAX;  /* initial value */
 
-    for (i = 0; i < colors; ++i) {
-        unsigned long newdist;  /* candidate for new 'dist' value */
-        unsigned int plane;
 
-        newdist = 0;
+static void
+createColormapFinder(struct pam *             const pamP,
+                     tupletable               const colormap,
+                     unsigned int             const colors,
+                     struct colormapFinder ** const colormapFinderPP) {
 
-        for (plane=0; plane < pamP->depth; ++plane) {
-            sample const normalizedSample = (sample)
-                (tuple[plane] * maxvalNormalizer) + 0.5;
-            
-            newdist += sqr(normalizedSample - colormap[i]->tuple[plane])
-                / divider;
-        }
-        if (newdist < dist) {
-            *colormapIndexP = i;
-            dist = newdist;
-        }
+    struct colormapFinder * colormapFinderP;
+
+    MALLOCVAR_NOFAIL(colormapFinderP);
+
+    colormapFinderP->colormap = colormap;
+    colormapFinderP->colors = colors;
+
+    {
+        unsigned int const maxHandleableSqrDiff = 
+            (unsigned int)UINT_MAX / pamP->depth;
+        
+        if (SQR(pamP->maxval) > maxHandleableSqrDiff)
+            colormapFinderP->distanceDivider = (unsigned int)
+                (SQR(pamP->maxval) / maxHandleableSqrDiff + 0.1 + 1.0);
+                /* The 0.1 is a fudge factor to keep us out of rounding 
+                   trouble.  The 1.0 effects a round-up.
+                */
+        else
+            colormapFinderP->distanceDivider = 1;
     }
+    *colormapFinderPP = colormapFinderP;
 }
 
 
 
 static void
-searchColormapClose(struct pam * const pamP,
-                    tupletable   const colormap, 
-                    int          const colors,
-                    sample       const colormapMaxval,
-                    tuple        const tuple,
-                    int *        const colormapIndexP) {
+destroyColormapFinder(struct colormapFinder * const colormapFinderP) {
+
+    free(colormapFinderP);
+}
+
+
+
+static void
+searchColormapClose(struct pam *            const pamP,
+                    tuple                   const tuple,
+                    struct colormapFinder * const colorFinderP,
+                    int *                   const colormapIndexP) {
 /*----------------------------------------------------------------------------
-   Search colormap 'colormap', which contains 'colors' colors (at
-   least 1) for the color closest to that of tuple 'tuple'.  Return
-   its index as *colormapIndexP.
+   Search the colormap indicated by *colorFinderP for the color closest to
+   that of tuple 'tuple'.  Return its index as *colormapIndexP.
+
+   *pamP describes the tuple 'tuple' and *colorFinderP has to be
+   compatible with it (i.e. the tuples in the color map must also be
+   described by *pamP).
 
    We compute distance between colors simply as the cartesian distance
    between them in the RGB space.  An alternative would be to look at
@@ -453,48 +490,68 @@ searchColormapClose(struct pam * const pamP,
    relationship between adjacent pixels.  If you replace a pixel with
    one that has the same chromaticity as the original, but much
    darker, it may stand out among its neighbors in a way the original
-   pixel did not.  In fact, on an image with blurred edges, we say
+   pixel did not.  In fact, on an image with blurred edges, we saw
    ugly effects at the edges when we substituted colors using a
    chromaticity-first color closeness formula.
 -----------------------------------------------------------------------------*/
-    if (pamP->maxval == colormapMaxval && colormapMaxval < 256) 
-        /* Do a fast computation for this usual case */
-        searchColormapCloseSimple(pamP, colormap, colors, 
-                                  tuple, colormapIndexP);
-    else 
-        /* Do a general computation that works for all maxvals */
-        searchColormapCloseComplex(pamP, colormap, colors, colormapMaxval,
-                                   tuple, colormapIndexP);
+    unsigned int i;
+    unsigned int dist;
+        /* The closest distance we've found so far between the value of
+           tuple 'tuple' and a tuple in the colormap.  This is measured as
+           the square of the cartesian distance between the tuples, except
+           that it's divided by 'distanceDivider' to make sure it will fit
+           in an unsigned int.
+        */
+
+    dist = UINT_MAX;  /* initial value */
+
+    assert(colorFinderP->colors > 0);
+
+    for (i = 0; i < colorFinderP->colors; ++i) {
+        unsigned int newdist;  /* candidate for new 'dist' value */
+        unsigned int plane;
+
+        newdist = 0;
+
+        for (plane=0; plane < pamP->depth; ++plane) {
+            newdist += 
+                SQR(tuple[plane] - colorFinderP->colormap[i]->tuple[plane]) 
+                / colorFinderP->distanceDivider;
+        }
+        if (newdist < dist) {
+            *colormapIndexP = i;
+            dist = newdist;
+        }
+    }
 }
 
 
 
 static void
-searchColormapExact(struct pam * const pamP,
-                    tupletable   const colormap, 
-                    int          const colormapSize,
-                    tuple        const tuple,
-                    int *        const colormapIndexP,
-                    bool *       const foundP) {
+searchColormapExact(struct pam *            const pamP,
+                    struct colormapFinder * const colorFinderP,
+                    tuple                   const tuple,
+                    int *                   const colormapIndexP,
+                    bool *                  const foundP) {
 /*----------------------------------------------------------------------------
-   Search colormap 'colormap', which contains 'colors' colors (at
-   least 1) for the color of tuple 'tuple'.  If it's in the map, return its
-   index as *colormapIndexP and return *foundP == TRUE.  Otherwise,
-   return *foundP = FALSE.
+   Search the colormap indicated by *colorFinderP for the color of
+   tuple 'tuple'.  If it's in the map, return its index as
+   *colormapIndexP and return *foundP == TRUE.  Otherwise, return
+   *foundP = FALSE.
 
-   We assume the maxval of the colormap and of 'tuple' are the same,
-   to wit pamP->maxval.  It doesn't make a lot of sense to call this 
-   subroutine otherwise, because an exact match would be difficult.
+   *pamP describes the tuple 'tuple' and *colorFinderP has to be
+   compatible with it (i.e. the tuples in the color map must also be
+   described by *pamP).
 -----------------------------------------------------------------------------*/
     unsigned int i;
     bool found;
     
     found = FALSE;  /* initial value */
-    for (i = 0; i < colormapSize && !found; ++i) {
+    for (i = 0; i < colorFinderP->colors && !found; ++i) {
         unsigned int plane;
         found = TRUE;  /* initial assumption */
         for (plane=0; plane < pamP->depth; ++plane) 
-            if (tuple[plane] != colormap[i]->tuple[plane]) 
+            if (tuple[plane] != colorFinderP->colormap[i]->tuple[plane]) 
                 found = FALSE;
         if (found) 
             *colormapIndexP = i;
@@ -505,22 +562,20 @@ searchColormapExact(struct pam * const pamP,
 
 
 static void
-lookupThroughHash(struct pam * const pamP, 
-                  tuple        const tuple, 
-                  bool         const needExactMatch,
-                  tupletable   const colormap, 
-                  unsigned int const colormapSize,
-                  sample       const colormapMaxval,
-                  tuplehash    const colorhash,       
-                  int *        const colormapIndexP,
-                  bool *       const usehashP) {
+lookupThroughHash(struct pam *            const pamP, 
+                  tuple                   const tuple, 
+                  bool                    const needExactMatch,
+                  struct colormapFinder * const colorFinderP,
+                  tuplehash               const colorhash,       
+                  int *                   const colormapIndexP,
+                  bool *                  const usehashP) {
 /*----------------------------------------------------------------------------
-   Look up the color of tuple 'tuple' in the color map 'colormap' and, if
-   it's in there, return its index as *colormapIndexP.  If not, return
-   *colormapIndexP == -1.
+   Look up the color of tuple 'tuple' in the color map indicated by
+   'colorFinderP' and, if it's in there, return its index as
+   *colormapIndexP.  If not, return *colormapIndexP == -1.
 
-   If 'needExactMatch' is true, we assume colormapMaxval == pamP->maxval,
-   i.e. the colormap and the input image have the same maxval.
+   Both the tuple 'tuple' and the colors in color map 'colormap' are
+   described by *pamP.
 
    If 'needExactMatch' isn't true, we find the closest color in the color map,
    and never return *colormapIndex == -1.
@@ -539,13 +594,12 @@ lookupThroughHash(struct pam * const pamP,
         if (needExactMatch) {
             bool found;
 
-            searchColormapExact(pamP, colormap, colormapSize, tuple,
+            searchColormapExact(pamP, colorFinderP, tuple,
                                 colormapIndexP, &found);
             if (!found)
                 *colormapIndexP = -1;
         } else 
-            searchColormapClose(pamP, colormap, colormapSize, 
-                                colormapMaxval, tuple, colormapIndexP);
+            searchColormapClose(pamP, tuple, colorFinderP, colormapIndexP);
         if (*usehashP) {
             bool fits;
             pnm_addtotuplehash(pamP, colorhash, tuple, *colormapIndexP, 
@@ -562,20 +616,28 @@ lookupThroughHash(struct pam * const pamP,
 
 
 static void
-convertRow(struct pam * const inpamP, tuple tuplerow[],
-           struct pam * const outpamP,
-           tupletable const colormap, unsigned int newcolors,
-           tuplehash const colorhash, bool *usehashP,
-           bool const floyd, enum missingMethod const missingMethod, 
-           tuple const defaultColor, struct fserr * const fserrP,
-           unsigned int * const missingCountP) {
+convertRow(struct pam *            const pamP, 
+           tuple                         tuplerow[],
+           tupletable              const colormap,
+           struct colormapFinder * const colorFinderP,
+           tuplehash               const colorhash, 
+           bool *                  const usehashP,
+           bool                    const floyd, 
+           enum missingMethod      const missingMethod, 
+           tuple                   const defaultColor,
+           struct fserr *          const fserrP,
+           unsigned int *          const missingCountP) {
 /*----------------------------------------------------------------------------
-  Replace the colors in row tuplerow[] with the new colors.
+  Replace the colors in row tuplerow[] (described by *pamP) with the
+  new colors.
 
   Use and update the Floyd-Steinberg state *fserrP.
 
   Return the number of pixels that were not matched in the color map as
   *missingCountP.
+
+  *colorFinderP is a color finder based on 'colormap' -- it tells us what
+  index of 'colormap' corresponds to a certain color.
 -----------------------------------------------------------------------------*/
     int col;
     int limitcol;
@@ -585,15 +647,15 @@ convertRow(struct pam * const inpamP, tuple tuplerow[],
         */
     
     if (floyd)
-        floydInitRow(inpamP, fserrP);
+        floydInitRow(pamP, fserrP);
 
     *missingCountP = 0;  /* initial value */
     
     if ((!floyd) || fserrP->fsForward) {
         col = 0;
-        limitcol = inpamP->width;
+        limitcol = pamP->width;
     } else {
-        col = inpamP->width - 1;
+        col = pamP->width - 1;
         limitcol = -1;
     }
     do {
@@ -603,30 +665,29 @@ convertRow(struct pam * const inpamP, tuple tuplerow[],
             */
 
         if (floyd) 
-            floydAdjustColor(inpamP, tuplerow[col], fserrP, col);
+            floydAdjustColor(pamP, tuplerow[col], fserrP, col);
 
-        lookupThroughHash(inpamP, tuplerow[col], 
-                          missingMethod != MISSING_CLOSE, colormap, 
-                          newcolors, outpamP->maxval,
+        lookupThroughHash(pamP, tuplerow[col], 
+                          missingMethod != MISSING_CLOSE, colorFinderP, 
                           colorhash, &colormapIndex, usehashP);
         if (floyd)
-            floydPropagateErr(inpamP, fserrP, col, tuplerow[col], 
-                              outpamP, colormap[colormapIndex]->tuple);
+            floydPropagateErr(pamP, fserrP, col, tuplerow[col], 
+                              colormap[colormapIndex]->tuple);
 
         if (colormapIndex == -1) {
             ++*missingCountP;
             switch (missingMethod) {
             case MISSING_SPECIFIED:
-                pnm_assigntuple(outpamP, tuplerow[col], defaultColor);
+                pnm_assigntuple(pamP, tuplerow[col], defaultColor);
                 break;
             case MISSING_FIRST:
-                pnm_assigntuple(outpamP, tuplerow[col], colormap[0]->tuple);
+                pnm_assigntuple(pamP, tuplerow[col], colormap[0]->tuple);
                 break;
             default:
                 pm_error("Internal error: invalid value of missingMethod");
             }
         } else 
-            pnm_assigntuple(outpamP, tuplerow[col], 
+            pnm_assigntuple(pamP, tuplerow[col], 
                             colormap[colormapIndex]->tuple);
 
         if (floyd && !fserrP->fsForward) 
@@ -636,7 +697,7 @@ convertRow(struct pam * const inpamP, tuple tuplerow[],
     } while (col != limitcol);
 
     if (floyd)
-        floydSwitchDir(inpamP, fserrP);
+        floydSwitchDir(pamP, fserrP);
 }
 
 
@@ -652,20 +713,23 @@ copyRaster(struct pam *   const inpamP,
            unsigned int * const missingCountP) {
 
     tuplehash const colorhash = pnm_createtuplehash();
+    struct colormapFinder * colorFinderP;
     bool usehash;
     struct fserr fserr;
     tuple * tuplerow = pnm_allocpamrow(inpamP);
     int row;
-    
+
     if (outpamP->maxval != inpamP->maxval && missingMethod != MISSING_CLOSE)
         pm_error("The maxval of the colormap (%u) is not equal to the "
-                 "maxval of the input image (%u).  This allowable only "
+                 "maxval of the input image (%u).  This is allowable only "
                  "if you are doing an approximate mapping (i.e. you don't "
                  "specify -firstisdefault or -missingcolor)",
                  (unsigned int)outpamP->maxval, (unsigned int)inpamP->maxval);
 
     usehash = TRUE;
 
+    createColormapFinder(outpamP, colormap, colormapSize, &colorFinderP);
+
     if (floyd)
         initFserr(inpamP, &fserr);
 
@@ -680,18 +744,20 @@ copyRaster(struct pam *   const inpamP,
            *outpamP instead of *inpamP.
         */
         adjustDepth(inpamP, tuplerow, outpamP->depth); 
+        pnm_scaletuplerow(inpamP, tuplerow, tuplerow, outpamP->maxval);
 
         /* The following both consults and adds to 'colorhash' and
            its associated 'usehash'.  It modifies 'tuplerow' too.
         */
-        convertRow(inpamP, tuplerow, outpamP, colormap, colormapSize,
-                   colorhash, &usehash,
+        convertRow(outpamP, tuplerow, colormap, colorFinderP, colorhash, 
+                   &usehash,
                    floyd, missingMethod, defaultColor, &fserr, &missingCount);
-            
+        
         *missingCountP += missingCount;
         
         pnm_writepamrow(outpamP, tuplerow);
     }
+    destroyColormapFinder(colorFinderP);
     pnm_freepamrow(tuplerow);
     pnm_destroytuplehash(colorhash);
 }
@@ -699,70 +765,108 @@ copyRaster(struct pam *   const inpamP,
 
 
 
+static void
+remap(FILE * const ifP,
+      const struct pam * const outpamCommonP,
+      tupletable         const colormap, 
+      unsigned int       const colormapSize,
+      bool               const floyd,
+      enum missingMethod const missingMethod,
+      tuple              const defaultColor,
+      bool               const verbose) {
+
+    bool eof;
+    eof = FALSE;
+    while (!eof) {
+        struct pam inpam, outpam;
+        unsigned int missingCount;
+            /* Number of pixels that were not matched in the color map (where
+               missingMethod is MISSING_CLOSE, this is always zero).
+            */
+
+        pnm_readpaminit(ifP, &inpam, PAM_STRUCT_SIZE(allocation_depth));
+    
+        outpam = *outpamCommonP;
+        outpam.width  = inpam.width;
+        outpam.height = inpam.height;
+
+        pnm_writepaminit(&outpam);
+
+        /* Set up so input buffers have extra space as needed to
+           convert the input to the output depth.
+        */
+        pnm_setminallocationdepth(&inpam, outpam.depth);
+    
+        copyRaster(&inpam, &outpam, colormap, colormapSize, floyd,
+                   missingMethod, defaultColor, &missingCount);
+        
+        if (verbose)
+            pm_message("%d pixels not matched in color map", missingCount);
+        
+        pnm_nextimage(ifP, &eof);
+    }
+}
+
+
+
+
 int
 main(int argc, char * argv[] ) {
 
     struct cmdlineInfo cmdline;
-    FILE* ifp;
-    struct pam inpam, outpam;
+    FILE * ifP;
+    struct pam outpamCommon;
+        /* Describes the output images.  Width and height fields are
+           not meaningful, because different output images might have
+           different dimensions.  The rest of the information is common
+           across all output images.
+        */
     tupletable colormap;
     unsigned int colormapSize;
     tuple defaultColor;
         /* A tuple of the color that should replace any input color that is 
            not in the colormap, if we're doing MISSING_SPECIFIED.
         */
-    unsigned int missingCount;
-        /* Number of pixels that were not matched in the color map (where
-           missingMethod is MISSING_CLOSE, this is always zero).
-        */
 
     pnm_init(&argc, argv);
 
     parseCommandLine(argc, argv, &cmdline);
 
-    ifp = pm_openr(cmdline.inputFilespec);
-
-    pnm_readpaminit(ifp, &inpam, sizeof(inpam));
-    
+    ifP = pm_openr(cmdline.inputFilespec);
     {
         FILE * mapfile;
         struct pam mappam;
         tuple ** maptuples;
 
         mapfile = pm_openr(cmdline.mapFilespec);
-        maptuples = pnm_readpam(mapfile, &mappam, sizeof(mappam));
+        maptuples = pnm_readpam(mapfile, &mappam, PAM_STRUCT_SIZE(tuple_type));
         pm_close(mapfile);
 
         computeColorMapFromMap(&mappam, maptuples, &colormap, &colormapSize);
         pnm_freepamarray(maptuples, &mappam);
 
-        outpam = mappam; 
-        outpam.file = stdout;
-        outpam.width = inpam.width;
-        outpam.height = inpam.height;
+        outpamCommon = mappam; 
+        outpamCommon.file = stdout;
     }
 
-    defaultColor = pnm_allocpamtuple(&outpam);
-    if (cmdline.missingcolor && outpam.depth == 3) {
+    defaultColor = pnm_allocpamtuple(&outpamCommon);
+    if (cmdline.missingcolor && outpamCommon.depth == 3) {
         pixel const color = 
-            ppm_parsecolor(cmdline.missingcolor, outpam.maxval);
+            ppm_parsecolor(cmdline.missingcolor, outpamCommon.maxval);
         defaultColor[PAM_RED_PLANE] = PPM_GETR(color);
         defaultColor[PAM_GRN_PLANE] = PPM_GETG(color);
         defaultColor[PAM_BLU_PLANE] = PPM_GETB(color);
     }
-    pnm_writepaminit(&outpam);
-    
-    copyRaster(&inpam, &outpam, colormap, colormapSize, cmdline.floyd,
-               cmdline.missingMethod, defaultColor, &missingCount);
-    
-    if (cmdline.verbose)
-        pm_message("%d pixels not matched in color map", missingCount);
+
+    remap(ifP, &outpamCommon, colormap, colormapSize, 
+          cmdline.floyd, cmdline.missingMethod, defaultColor,
+          cmdline.verbose);
 
     pnm_freepamtuple(defaultColor);
 
     pm_close(stdout);
 
-    pm_close(ifp);
+    pm_close(ifP);
 
-    exit(0);
+    return 0;
 }
