@@ -10,16 +10,15 @@
 ** implied warranty.
 */
 
+#define _XOPEN_SOURCE   /* get M_PI in math.h */
+
 #include <math.h>
+#include <assert.h>
 
 #include "pnm.h"
 #include "shhopt.h"
 #include "mallocvar.h"
 
-#ifndef M_PI
-#define M_PI    3.14159265358979323846
-#endif /*M_PI*/
-
 #define SCALE 4096
 #define HALFSCALE 2048
 
@@ -27,12 +26,12 @@ struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
-    const char *inputFilespec;  /* Filespecs of input file */
+    const char * inputFilespec;  /* Filespecs of input file */
     float angle;                /* Angle to rotate, in radians */
     unsigned int noantialias;
-    const char *background;  /* NULL if none */
-    unsigned int debugStage1;
-    unsigned int debugStage2;
+    const char * background;  /* NULL if none */
+    unsigned int keeptemp;  /* For debugging */
+    unsigned int verbose;
 };
 
 
@@ -44,7 +43,8 @@ struct shearParm {
     */
     long fracnew0;
     long omfracnew0;
-    int intnew0;
+    unsigned int shiftWhole;
+    unsigned int shiftUnits;
 };
 
 
@@ -66,13 +66,13 @@ parseCommandLine(int argc, char ** const argv,
 
     option_def_index = 0;   /* incremented by OPTENTRY */
     OPTENT3(0, "background",  OPT_STRING, &cmdlineP->background, 
-            &backgroundSpec,      0);
+            &backgroundSpec,        0);
     OPTENT3(0, "noantialias", OPT_FLAG,   NULL, 
             &cmdlineP->noantialias, 0);
-    OPTENT3(0, "debug_stage1", OPT_FLAG,   NULL, 
-            &cmdlineP->debugStage1, 0);
-    OPTENT3(0, "debug_stage2", OPT_FLAG,   NULL, 
-            &cmdlineP->debugStage2, 0);
+    OPTENT3(0, "keeptemp",    OPT_FLAG,   NULL, 
+            &cmdlineP->keeptemp,    0);
+    OPTENT3(0, "verbose",     OPT_FLAG,   NULL, 
+            &cmdlineP->verbose,     0);
 
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
@@ -121,15 +121,36 @@ parseCommandLine(int argc, char ** const argv,
 
 
 static void
-computeNewFormat(bool const antialias, 
-                 int const format, xelval const maxval,
-                 int * const newformatP, xelval * const newmaxvalP) {
+storeImage(const char * const fileName,
+           xel **       const xels,
+           unsigned int const cols,
+           unsigned int const rows,
+           xelval       const maxval,
+           int          const format) {
+
+    FILE * ofP;
+
+    ofP = pm_openw(fileName);
+
+    pnm_writepnm(ofP, xels, cols, rows, maxval, format, 0);
+
+    pm_close(ofP);
+}
+
+  
+
+static void
+computeNewFormat(bool     const antialias, 
+                 int      const format,
+                 xelval   const maxval,
+                 int *    const newformatP,
+                 xelval * const newmaxvalP) {
 
     if (antialias && PNM_FORMAT_TYPE(format) == PBM_TYPE) {
         *newformatP = PGM_TYPE;
         *newmaxvalP = PGM_MAXMAXVAL;
-        pm_message( "promoting from PBM to PGM - "
-                    "use -noantialias to avoid this" );
+        pm_message("promoting from PBM to PGM - "
+                   "use -noantialias to avoid this");
     } else {
         *newformatP = format;
         *newmaxvalP = maxval;
@@ -138,6 +159,27 @@ computeNewFormat(bool const antialias,
 
 
 
+static bool
+isWhite(xel    const color,
+        xelval const maxval) {
+
+    return (PPM_GETR(color) == maxval &&
+            PPM_GETG(color) == maxval &&
+            PPM_GETB(color) == maxval);
+}
+
+
+
+static bool
+isBlack(xel const color) {
+
+    return (PPM_GETR(color) == 0 &&
+            PPM_GETG(color) == 0 &&
+            PPM_GETB(color) == 0);
+}
+
+
+
 static xel
 backgroundColor(const char * const backgroundColorName,
                 xel *        const topRow,
@@ -159,16 +201,14 @@ backgroundColor(const char * const backgroundColorName,
 
             break;
         case PBM_TYPE:
-            if (!PNM_EQUAL(retval, pnm_whitexel(maxval, format)) &&
-                !PNM_EQUAL(retval, pnm_blackxel(maxval, format)))
+            if (!isWhite(retval, maxval) && !isBlack(retval))
                 pm_error("Image is PBM (black and white), "
                          "but you specified '%s', which is neither black "
                          "nor white, as background color", 
                          backgroundColorName);
             break;
         }
-    }
-    else 
+    } else 
         retval = pnm_backgroundxelrow(topRow, cols, maxval, format);
 
     return retval;
@@ -176,6 +216,15 @@ backgroundColor(const char * const backgroundColorName,
 
 
 
+static void
+reportBackground(xel const bgColor) {
+
+    pm_message("Background color %u/%u/%u",
+               PPM_GETR(bgColor), PPM_GETG(bgColor), PPM_GETB(bgColor));
+}
+
+
+
 static void
 shearX(xel * const inRow, 
        xel * const outRow, 
@@ -183,87 +232,185 @@ shearX(xel * const inRow,
        int   const format,
        xel   const bgxel,
        bool  const antialias,
-       float const new0,
-       int   const tempcols) {
+       float const shiftAmount,
+       int   const newcols) {
+/*----------------------------------------------------------------------------
+   Shift a the row inRow[] right by 'shiftAmount' pixels and return the
+   result as outRow[].
 
-    int const intnew0 = (int) new0;
+   The input row is 'cols' columns wide, whereas the output row is
+   'newcols'.
 
-    if ( antialias ) {
-        long const fracnew0 = ( new0 - intnew0 ) * SCALE;
-        long const omfracnew0 = SCALE - fracnew0;
+   The format of the input row is 'format'.
 
-        int col;
+   We shift the row on a background of color 'bgxel'.
+
+   The output row has the same format and maxval as the input.
+   
+   'shiftAmount' may not be negative.
+   
+   'shiftAmount' can be fractional, so we either just go by the
+   nearest integer value or mix pixels to achieve the shift, depending
+   on 'antialias'.
+-----------------------------------------------------------------------------*/
+    assert(shiftAmount >= 0.0);
+
+    if (antialias) {
+        unsigned int const shiftWhole = (unsigned int) shiftAmount;
+        long const fracShift = (shiftAmount - shiftWhole) * SCALE;
+        long const omfracShift = SCALE - fracShift;
+
+        unsigned int col;
         xel * nxP;
-        xel * xP;
         xel prevxel;
 
-        for (col = 0; col < tempcols; ++col)
+        for (col = 0; col < newcols; ++col)
             outRow[col] = bgxel;
             
         prevxel = bgxel;
-        for (col = 0, nxP = &(outRow[intnew0]), xP = inRow;
-             col < cols; ++col, ++nxP, ++xP) {
+        for (col = 0, nxP = &(outRow[shiftWhole]);
+             col < cols; ++col, ++nxP) {
+
+            xel const p = inRow[col];
+
             switch (PNM_FORMAT_TYPE(format)) {
             case PPM_TYPE:
                 PPM_ASSIGN(*nxP,
-                           (fracnew0 * PPM_GETR(prevxel) 
-                            + omfracnew0 * PPM_GETR(*xP) 
+                           (fracShift * PPM_GETR(prevxel) 
+                            + omfracShift * PPM_GETR(p) 
                             + HALFSCALE) / SCALE,
-                           (fracnew0 * PPM_GETG(prevxel) 
-                            + omfracnew0 * PPM_GETG(*xP) 
+                           (fracShift * PPM_GETG(prevxel) 
+                            + omfracShift * PPM_GETG(p) 
                             + HALFSCALE) / SCALE,
-                           (fracnew0 * PPM_GETB(prevxel) 
-                            + omfracnew0 * PPM_GETB(*xP) 
+                           (fracShift * PPM_GETB(prevxel) 
+                            + omfracShift * PPM_GETB(p) 
                             + HALFSCALE) / SCALE );
                 break;
                 
             default:
                 PNM_ASSIGN1(*nxP,
-                            (fracnew0 * PNM_GET1(prevxel) 
-                             + omfracnew0 * PNM_GET1(*xP) 
+                            (fracShift * PNM_GET1(prevxel) 
+                             + omfracShift * PNM_GET1(p) 
                              + HALFSCALE) / SCALE );
                 break;
             }
-            prevxel = *xP;
+            prevxel = p;
         }
-        if (fracnew0 > 0 && intnew0 + cols < tempcols) {
+        if (fracShift> 0 && shiftWhole + cols < newcols) {
             switch (PNM_FORMAT_TYPE(format)) {
             case PPM_TYPE:
                 PPM_ASSIGN(*nxP,
-                           (fracnew0 * PPM_GETR(prevxel) 
-                            + omfracnew0 * PPM_GETR(bgxel) 
+                           (fracShift * PPM_GETR(prevxel) 
+                            + omfracShift * PPM_GETR(bgxel) 
                             + HALFSCALE) / SCALE,
-                           (fracnew0 * PPM_GETG(prevxel) 
-                            + omfracnew0 * PPM_GETG(bgxel) 
+                           (fracShift * PPM_GETG(prevxel) 
+                            + omfracShift * PPM_GETG(bgxel) 
                             + HALFSCALE) / SCALE,
-                           (fracnew0 * PPM_GETB(prevxel) 
-                            + omfracnew0 * PPM_GETB(bgxel) 
+                           (fracShift * PPM_GETB(prevxel) 
+                            + omfracShift * PPM_GETB(bgxel) 
                             + HALFSCALE) / SCALE );
                 break;
                     
             default:
                 PNM_ASSIGN1(*nxP,
-                            (fracnew0 * PNM_GET1(prevxel) 
-                             + omfracnew0 * PNM_GET1(bgxel) 
+                            (fracShift * PNM_GET1(prevxel) 
+                             + omfracShift * PNM_GET1(bgxel) 
                              + HALFSCALE) / SCALE );
                 break;
             }
         }
     } else {
-        int col;
-        xel * nxP;
+        unsigned int const shiftCols = (unsigned int) (shiftAmount + 0.5);
+        unsigned int col;
+        unsigned int outcol;
+
+        outcol = 0;  /* initial value */
         
-        for (col = 0, nxP = outRow; col < intnew0; ++col, ++nxP )
-            *nxP = bgxel;
-        for (col = 0; col < cols; ++col, ++nxP)
-            *nxP = inRow[col];
-        for (col = intnew0 + cols; col < tempcols; ++col, ++nxP)
-            *nxP = bgxel;
+        for (col = 0; col < shiftCols; ++col)
+            outRow[outcol++] = bgxel;
+        for (col = 0; col < cols; ++col)
+            outRow[outcol++] = inRow[col];
+        for (col = shiftCols + cols; col < newcols; ++col)
+            outRow[outcol++] = bgxel;
+        
+        assert(outcol == newcols);
     }
 }
 
 
 
+static void
+shearXFromInputFile(FILE *                 const ifP,
+                    unsigned int           const cols,
+                    unsigned int           const rows,
+                    xelval                 const maxval,
+                    int                    const format,
+                    enum rotationDirection const direction,
+                    float                  const xshearfac,
+                    xelval                 const newmaxval,
+                    int                    const newformat,
+                    bool                   const antialias,
+                    const char *           const background,
+                    xel ***                const shearedXelsP,
+                    unsigned int *         const newcolsP,
+                    xel *                  const bgColorP) {
+/*----------------------------------------------------------------------------
+   Shear X from input file into newly malloced xel array.  Return that
+   array as *shearedColsP, and its width as *tempColsP.  Everything else
+   about the sheared image is the same as for the input image.
+
+   The input image on file 'ifP' is described by 'cols', 'rows',
+   'maxval', and 'format'.
+
+   Along the way, figure out what the background color of the output should
+   be based on the contents of the file and the user's directive
+   'background' and return that as *bgColorP.
+-----------------------------------------------------------------------------*/
+    unsigned int const maxShear = (rows - 0.5) * xshearfac + 0.5;
+    unsigned int const newcols = cols + maxShear;
+    
+    xel ** shearedXels;
+    xel * xelrow;
+    xel bgColor;
+    unsigned int row;
+
+    shearedXels = pnm_allocarray(newcols, rows);
+
+    xelrow = pnm_allocrow(cols);
+
+    for (row = 0; row < rows; ++row) {
+        /* The shear factor is designed to shear over the entire width
+           from the left edge of of the left pixel to the right edge of
+           the right pixel.  We use the distance of the center of this
+           pixel from the relevant edge to compute shift amount:
+        */
+        float const xDistance = 
+            (direction == COUNTERCLOCKWISE ? row + 0.5 : (rows-0.5 - row));
+        float const shiftAmount = xshearfac * xDistance;
+
+        pnm_readpnmrow(ifP, xelrow, cols, maxval, format);
+
+        pnm_promoteformatrow(xelrow, cols, maxval, format, 
+                             newmaxval, newformat);
+
+        if (row == 0)
+            bgColor =
+                backgroundColor(background, xelrow, cols, newmaxval, format);
+
+        shearX(xelrow, shearedXels[row], cols, newformat, bgColor,
+               antialias, shiftAmount, newcols);
+    }
+    pnm_freerow(xelrow);
+
+    *shearedXelsP = shearedXels;
+    *newcolsP = newcols;
+
+    assert(rows >= 1);  /* Ergo, bgColor is defined */
+    *bgColorP = bgColor;
+}
+
+
+
 static void 
 shearYNoAntialias(xel **           const inxels,
                   xel **           const outxels,
@@ -271,48 +418,40 @@ shearYNoAntialias(xel **           const inxels,
                   int              const inrows,
                   int              const outrows,
                   int              const format,
-                  xel              const bgxel,
+                  xel              const bgColor,
                   struct shearParm const shearParm[]) {
 /*----------------------------------------------------------------------------
    Shear the image in 'inxels' ('cols' x 'inrows') vertically into
    'outxels' ('cols' x 'outrows'), both format 'format'.  shearParm[X]
-   tells how much to shear pixels in Column X and 'bgxel' is what to
-   use for background where there is none of the input in the output.
+   tells how much to shear pixels in Column X (clipped to Rows 0
+   through 'outrow' -1) and 'bgColor' is what to use for background
+   where there is none of the input in the output.
 
    We do not do any antialiasing.  We simply move whole pixels.
 
    We go row by row instead of column by column to save real memory.  Going
-   row by row, the working set is only a few pages, whereas going row by
-   row, it would be one page per output row plus one page per input row.
+   row by row, the working set is only a few pages, whereas going column by
+   column, it would be one page per output row plus one page per input row.
 -----------------------------------------------------------------------------*/
-    int inrow;
+    unsigned int inrow;
+    unsigned int outrow;
 
+    /* Fill the output with background */
+    for (outrow = 0; outrow < outrows; ++outrow) {
+        unsigned int col;
+        for (col = 0; col < cols; ++col)
+            outxels[outrow][col] = bgColor;
+    }
+
+    /* Overlay that background with sheared image */
     for (inrow = 0; inrow < inrows; ++inrow) {
-        int col;
+        unsigned int col;
         for (col = 0; col < cols; ++col) {
-            int const outrow = inrow + shearParm[col].intnew0;
+            int const outrow = inrow + shearParm[col].shiftUnits;
             if (outrow >= 0 && outrow < outrows)
                 outxels[outrow][col] = inxels[inrow][col];
         }
     }
-    {
-        /* Now fill in the pixels that we didn't fill above, with background */
-        int col;
-        for (col = 0; col < cols; ++col) {
-            int startBgRow, endBgRow;
-            int outrow;
-            
-            if (shearParm[col].intnew0 > 0 ) {
-                startBgRow = 0;
-                endBgRow   = shearParm[col].intnew0;
-            } else {
-                startBgRow = outrows + shearParm[col].intnew0;
-                endBgRow   = outrows;
-            }
-            for (outrow = startBgRow; outrow < endBgRow; ++outrow)
-                outxels[outrow][col] = bgxel;
-        }
-    }
 }
 
 
@@ -331,7 +470,7 @@ shearYColAntialias(xel ** const inxels,
 -----------------------------------------------------------------------------*/
     long const fracnew0   = shearParm[col].fracnew0;
     long const omfracnew0 = shearParm[col].omfracnew0;
-    int  const intnew0    = shearParm[col].intnew0;
+    int  const shiftWhole = shearParm[col].shiftWhole;
         
     int outrow;
 
@@ -344,7 +483,7 @@ shearYColAntialias(xel ** const inxels,
 
     prevxel = bgxel;
     for (inrow = 0; inrow < inrows; ++inrow) {
-        int const outrow = inrow + intnew0;
+        int const outrow = inrow + shiftWhole;
 
         if (outrow >= 0 && outrow < outrows) {
             xel * const nxP = &(outxels[outrow][col]);
@@ -373,8 +512,8 @@ shearYColAntialias(xel ** const inxels,
             prevxel = x;
         }
     }
-    if (fracnew0 > 0 && intnew0 + inrows < outrows) {
-        xel * const nxP = &(outxels[intnew0 + inrows][col]);
+    if (fracnew0 > 0 && shiftWhole + inrows < outrows) {
+        xel * const nxP = &(outxels[shiftWhole + inrows][col]);
         switch (PNM_FORMAT_TYPE(format)) {
         case PPM_TYPE:
             PPM_ASSIGN(*nxP,
@@ -401,6 +540,66 @@ shearYColAntialias(xel ** const inxels,
 
 
 
+static void
+shearImageY(xel **                 const inxels,
+            int                    const cols,
+            int                    const inrows,
+            int                    const format,
+            xel                    const bgxel,
+            bool                   const antialias,
+            enum rotationDirection const direction,
+            float                  const yshearfac,
+            int                    const yshearjunk,
+            xel ***                const outxelsP,
+            unsigned int *         const outrowsP) {
+    
+    unsigned int const maxShear = (cols - 0.5) * yshearfac + 0.5;
+    unsigned int const outrows = inrows + maxShear - 2 * yshearjunk;
+
+    struct shearParm * shearParm;  /* malloc'ed */
+    int col;
+    xel ** outxels;
+    
+    outxels = pnm_allocarray(cols, outrows);
+
+    MALLOCARRAY(shearParm, cols);
+    if (shearParm == NULL)
+        pm_error("Unable to allocate memory for shearParm");
+
+    for (col = 0; col < cols; ++col) {
+        /* The shear factor is designed to shear over the entire height
+           from the top edge of of the top pixel to the bottom edge of
+           the bottom pixel.  We use the distance of the center of this
+           pixel from the relevant edge to compute shift amount:
+        */
+        float const yDistance = 
+            (direction == CLOCKWISE ? col + 0.5 : (cols-0.5 - col));
+        float const shiftAmount = yshearfac * yDistance;
+
+        shearParm[col].fracnew0   = (shiftAmount - (int)shiftAmount) * SCALE;
+        shearParm[col].omfracnew0 = SCALE - shearParm[col].fracnew0;
+        shearParm[col].shiftWhole = (int)shiftAmount - yshearjunk;
+        shearParm[col].shiftUnits = (int)(shiftAmount + 0.5) - yshearjunk;
+    }
+    if (!antialias)
+        shearYNoAntialias(inxels, outxels, cols, inrows, outrows, format,
+                          bgxel, shearParm);
+    else {
+        /* TODO: do this row-by-row, same as for noantialias, to save
+           real memory.
+        */
+        for (col = 0; col < cols; ++col) 
+            shearYColAntialias(inxels, outxels, col, inrows, outrows, format, 
+                               bgxel, shearParm);
+    }
+    free(shearParm);
+    
+    *outxelsP = outxels;
+    *outrowsP = outrows;
+}
+
+
+
 static void
 shearFinal(xel * const inRow, 
            xel * const outRow, 
@@ -409,25 +608,29 @@ shearFinal(xel * const inRow,
            int   const format,
            xel   const bgxel,
            bool  const antialias,
-           float const new0,
+           float const shiftAmount,
            int   const x2shearjunk) {
 
-    long const fracnew0 = ( new0 - (int) new0 ) * SCALE; 
-    long const omfracnew0 = SCALE - fracnew0; 
-    int  const intnew0 = (int) new0 - x2shearjunk;
 
-    int col;
+    assert(shiftAmount >= 0.0);
+
+    {
+        unsigned int col;
+        for (col = 0; col < outcols; ++col)
+            outRow[col] = bgxel;
+    }
 
-    for (col = 0; col < outcols; ++col)
-        outRow[col] = bgxel;
-        
     if (antialias) {
+        long const fracnew0   = (shiftAmount - (int) shiftAmount) * SCALE; 
+        long const omfracnew0 = SCALE - fracnew0; 
+        unsigned int const shiftWhole = (int)shiftAmount - x2shearjunk;
+
         xel prevxel;
-        int col;
+        unsigned int col;
 
         prevxel = bgxel;
         for (col = 0; col < incols; ++col) {
-            int const new = intnew0 + col;
+            int const new = shiftWhole + col;
             if (new >= 0 && new < outcols) {
                 xel * const nxP = &(outRow[new]);
                 xel const x = inRow[col];
@@ -455,8 +658,8 @@ shearFinal(xel * const inRow,
                 prevxel = x;
             }
         }
-        if (fracnew0 > 0 && intnew0 + incols < outcols) {
-            xel * const nxP = &(outRow[intnew0 + incols]);
+        if (fracnew0 > 0 && shiftWhole + incols < outcols) {
+            xel * const nxP = &(outRow[shiftWhole + incols]);
             switch (PNM_FORMAT_TYPE(format)) {
             case PPM_TYPE:
                 PPM_ASSIGN(*nxP,
@@ -480,11 +683,14 @@ shearFinal(xel * const inRow,
             }
         }
     } else {
-        int col;
+        unsigned int const shiftCols =
+            (unsigned int)(shiftAmount + 0.5) - x2shearjunk;
+
+        unsigned int col;
         for (col = 0; col < incols; ++col) {
-            int const new = intnew0 + col;
-            if (new >= 0 && new < outcols)
-                outRow[new] = inRow[col];
+            unsigned int const outcol = shiftCols + col;
+            if (outcol >= 0 && outcol < outcols)
+                outRow[outcol] = inRow[col];
         }
     }
 }
@@ -492,148 +698,131 @@ shearFinal(xel * const inRow,
 
 
 static void
-shearImageY(xel ** const inxels,
-            xel ** const outxels,
-            int    const cols,
-            int    const inrows,
-            int    const outrows,
-            int    const format,
-            xel    const bgxel,
-            bool   const antialias,
-            enum rotationDirection const direction,
-            float  const yshearfac,
-            int    const yshearjunk) {
+shearXToOutputFile(FILE *                 const ofP,
+                   xel **                 const xels,
+                   unsigned int           const cols, 
+                   unsigned int           const rows,
+                   xelval                 const maxval,
+                   int                    const format,
+                   enum rotationDirection const direction,
+                   float                  const xshearfac,
+                   int                    const x2shearjunk,
+                   xel                    const bgColor,
+                   bool                   const antialias) {
+/*----------------------------------------------------------------------------
+   Shear horizontally the image in 'xels' and write the result to file
+   'ofP'.  'cols', 'rows', 'maxval', and 'format' describe the image in
+   'xels'.  They also describe the output image, except that it will be
+   wider as dictated by the shearing parameters.
+
+   Shear over background color 'bgColor'.
+
+   Do a smooth pixel-mixing shear iff 'antialias' is true.
+-----------------------------------------------------------------------------*/
+    unsigned int const maxShear = (rows - 0.5) * xshearfac + 0.5;
+    unsigned int const newcols = cols + maxShear - 2 * x2shearjunk;
+
+    unsigned int row;
+    xel * xelrow;
     
-    struct shearParm * shearParm;  /* malloc'ed */
-    int col;
+    pnm_writepnminit(stdout, newcols, rows, maxval, format, 0);
 
-    MALLOCARRAY(shearParm, cols);
-    if (shearParm == NULL)
-        pm_error("Unable to allocate memory for shearParm");
+    xelrow = pnm_allocrow(newcols);
 
-    for ( col = 0; col < cols; ++col ) {
-        float const new0 = yshearfac * 
-            (direction == CLOCKWISE ? col : cols - col);
-
-        shearParm[col].fracnew0   = ( new0 - (int) new0 ) * SCALE;
-        shearParm[col].omfracnew0 = SCALE - shearParm[col].fracnew0;
-        shearParm[col].intnew0    = (int) new0 - yshearjunk;
-    }
-    if (!antialias)
-        shearYNoAntialias(inxels, outxels, cols, inrows, outrows, format,
-                          bgxel, shearParm);
-    else {
-        /* TODO: do this row-by-row, same as for noantialias, to save
-           real memory.
+    for (row = 0; row < rows; ++row) {
+        /* The shear factor is designed to shear over the entire width
+           from the left edge of of the left pixel to the right edge of
+           the right pixel.  We use the distance of the center of this
+           pixel from the relevant edge to compute shift amount:
         */
-        for ( col = 0; col < cols; ++col) 
-            shearYColAntialias(inxels, outxels, col, inrows, outrows, format, 
-                               bgxel, shearParm);
+        float const xDistance = 
+            (direction == COUNTERCLOCKWISE ? row + 0.5 : (rows-0.5 - row));
+        float const shiftAmount = xshearfac * xDistance;
+
+        shearFinal(xels[row], xelrow, cols, newcols, format, 
+                   bgColor, antialias, shiftAmount, x2shearjunk);
+
+        pnm_writepnmrow(stdout, xelrow, newcols, maxval, format, 0);
     }
-    free(shearParm);
+    pnm_freerow(xelrow);
 }
 
 
 
 int
 main(int argc, char *argv[]) { 
+
     struct cmdlineInfo cmdline;
-    FILE* ifP;
-    xel* xelrow;
-    xel** temp1xels;
-    xel** temp2xels;
-    xel* newxelrow;
-    xel bgxel;
+    FILE * ifP;
+    xel ** shear1xels;
+    xel ** shear2xels;
+    xel bgColor;
     int rows, cols, format;
-    int newformat, newrows;
-    int tempcols, newcols;
+    int newformat;
+    unsigned int newrows;
+    int newRowsWithJunk;
+    unsigned int shear1Cols;
     int yshearjunk, x2shearjunk;
-    int row;
     xelval maxval, newmaxval;
     float xshearfac, yshearfac;
     enum rotationDirection direction;
 
-    pnm_init( &argc, argv );
+    pnm_init(&argc, argv);
 
     parseCommandLine(argc, argv, &cmdline);
 
     ifP = pm_openr(cmdline.inputFilespec);
 
-    pnm_readpnminit( ifP, &cols, &rows, &maxval, &format );
-    xelrow = pnm_allocrow( cols );
-
+    pnm_readpnminit(ifP, &cols, &rows, &maxval, &format);
+    
     computeNewFormat(!cmdline.noantialias, format, maxval, 
                      &newformat, &newmaxval);
 
-    xshearfac = tan( cmdline.angle / 2.0 );
-    if ( xshearfac < 0.0 )
-        xshearfac = -xshearfac;
-    yshearfac = sin( cmdline.angle );
-    if ( yshearfac < 0.0 )
-        yshearfac = -yshearfac;
-    tempcols = rows * xshearfac + cols + 0.999999;
-    yshearjunk = ( tempcols - cols ) * yshearfac;
-    newrows = tempcols * yshearfac + rows + 0.999999;
-    x2shearjunk = ( newrows - rows - yshearjunk ) * xshearfac;
-    newrows -= 2 * yshearjunk;
-    newcols = newrows * xshearfac + tempcols + 0.999999 - 2 * x2shearjunk;
+    xshearfac = fabs(tan(cmdline.angle / 2.0));
+    yshearfac = fabs(sin(cmdline.angle));
     direction = cmdline.angle > 0 ? COUNTERCLOCKWISE : CLOCKWISE;
 
-    /* First shear X from input file into temp1xels. */
-    temp1xels = pnm_allocarray( tempcols, rows );
-    for ( row = 0; row < rows; ++row ) {
-        float const new0 = xshearfac * (cmdline.angle > 0 ? row : (rows-row));
+    /* The algorithm we use, for maximum speed, is 3 simple shears:
+       A horizontal, a vertical, and another horizontal.
+    */
 
-        pnm_readpnmrow( ifP, xelrow, cols, maxval, format );
-
-        pnm_promoteformatrow(xelrow, cols, maxval, format, 
-                             newmaxval, newformat);
-
-        if (row == 0)
-            bgxel = backgroundColor(cmdline.background, 
-                                    xelrow, cols, newmaxval, format);
-
-        shearX(xelrow, temp1xels[row], cols, newformat, bgxel, 
-               !cmdline.noantialias, new0, tempcols);
-    }
-    pm_close( ifP );
-    pnm_freerow( xelrow );
-
-    if (cmdline.debugStage1) {
-        pnm_writepnm(stdout, temp1xels, tempcols, rows, 
-                     newmaxval, newformat, 0);
-        exit(1);
-    }
-  
-    /* Now inverse shear Y from temp1xels into temp2xels. */
-    temp2xels = pnm_allocarray( tempcols, newrows );
-    shearImageY(temp1xels, temp2xels, tempcols, rows, newrows, newformat,
-                bgxel, !cmdline.noantialias, direction, yshearfac, yshearjunk);
-
-    pnm_freearray( temp1xels, rows );
-
-    if (cmdline.debugStage2) {
-        pnm_writepnm(stdout, temp2xels, tempcols, newrows, 
-                     newmaxval, newformat, 0);
-        exit(1);
-    }
-  
-    /* Finally, shear X from temp2xels to output file. */
-    pnm_writepnminit( stdout, newcols, newrows, newmaxval, newformat, 0 );
-    newxelrow = pnm_allocrow( newcols );
-    for ( row = 0; row < newrows; ++row ) {
-        float const new0 = xshearfac * (cmdline.angle > 0 ? row : newrows-row);
-
-        shearFinal(temp2xels[row], newxelrow, tempcols, newcols, newformat, 
-                   bgxel, !cmdline.noantialias, new0, x2shearjunk);
-
-        pnm_writepnmrow( stdout, newxelrow, newcols, newmaxval, newformat, 0 );
-    }
-
-    pnm_freerow( newxelrow );
-    pnm_freearray( temp2xels, newrows );
-    pm_close( stdout );
+    shearXFromInputFile(ifP, cols, rows, maxval, format,
+                        direction, xshearfac,
+                        newmaxval, newformat,
+                        !cmdline.noantialias, cmdline.background,
+                        &shear1xels, &shear1Cols, &bgColor);
     
-    exit( 0 );
-}
+    pm_close(ifP);
 
+    if (cmdline.verbose)
+        reportBackground(bgColor);
+
+    if (cmdline.keeptemp)
+        storeImage("pnmrotate_stage1.pnm", shear1xels, shear1Cols, rows,
+                   newmaxval, newformat);
+
+    yshearjunk = (shear1Cols - cols) * yshearfac;
+    newRowsWithJunk = (shear1Cols - 1) * yshearfac + rows + 0.999999;
+    x2shearjunk = (newRowsWithJunk - rows - yshearjunk - 1) * xshearfac;
+
+    shearImageY(shear1xels, shear1Cols, rows, newformat,
+                bgColor, !cmdline.noantialias, direction,
+                yshearfac, yshearjunk,
+                &shear2xels, &newrows);
+
+    pnm_freearray(shear1xels, rows);
+
+    if (cmdline.keeptemp)
+        storeImage("pnmrotate_stage2.pnm", shear2xels, shear1Cols, newrows, 
+                   newmaxval, newformat);
+
+    shearXToOutputFile(stdout, shear2xels, shear1Cols, newrows,
+                       newmaxval, newformat,
+                       direction, xshearfac, x2shearjunk, 
+                       bgColor, !cmdline.noantialias);
+
+    pnm_freearray(shear2xels, newrows);
+    pm_close(stdout);
+    
+    return 0;
+}
