@@ -83,16 +83,19 @@
  */
 
 #define _BSD_SOURCE 1   /* Make sure strdup() is in string.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <limits.h>
 #include <math.h>
 
-#include "pam.h"
+#include "pm_c_util.h"
 #include "shhopt.h"
 #include "nstring.h"
 #include "mallocvar.h"
+#include "pam.h"
 
 /*
  *  Structures
@@ -398,9 +401,7 @@ openWithPossibleExtension(const char *  const baseName,
 
     ifP = NULL;   /* initial value -- no file opened yet */
 
-    for (extIndex = 0; 
-         extIndex < sizeof(extlist)/sizeof(extlist[0]) && !ifP;
-         ++extIndex) {
+    for (extIndex = 0; extIndex < ARRAY_SIZE(extlist) && !ifP; ++extIndex) {
         
         const char * trialName;
         
@@ -444,9 +445,8 @@ readinit(const char * const name)
         } else {
             openWithPossibleExtension(name, &ifP, &image->name);
         }
-        image->tuple = pnm_readpam(ifP,
-                                   &(image->pam), 
-                                   (unsigned)sizeof(image->pam));
+        image->tuple = pnm_readpam(ifP, &(image->pam), 
+                                   PAM_STRUCT_SIZE(tuple_type));
         fclose (ifP);
         image->pam.file = NULL;
 
@@ -589,10 +589,9 @@ update_best(Best * best, unsigned long total, int x, int y)
  *  Print helper for the Best structure.
  */
 static void
-pr_best(Best * best)
+pr_best(Best * const best)
 {
     int i;
-    fprintf (stderr,"%p", best);
     if (best != (Best *)NULL)
     for (i = 0; i < NUM_BEST; ++i) {
         fprintf (stderr, " (%d,%d)%lu",
