@@ -10,6 +10,9 @@
 ** implied warranty.
 */
 
+#define _BSD_SOURCE 1      /* Make sure strdup() is in string.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
+
 #include <string.h>
 
 #include "pbm.h"
@@ -27,7 +30,7 @@ struct cmdline_info {
     unsigned int dump;   
         /* undocumented dump option for installing a new built-in font */
     float space;   /* -space option value or default */
-    unsigned int width;     /* future -width option value or zero */
+    unsigned int width;     /* -width option value or zero */
     int lspace;    /* lspace option value or default */
     unsigned int nomargins;     /* -nomargins */
 };
@@ -103,6 +106,43 @@ parse_command_line(int argc, char ** argv,
 }
 
 
+struct text {
+    char **      textArray;  /* malloc'ed */
+    unsigned int allocatedLineCount;
+    unsigned int lineCount;
+};
+
+
+static void
+allocTextArray(struct text * const textP,
+               unsigned int  const maxLineCount,
+               unsigned int  const maxColumnCount) {
+
+    unsigned int line;
+
+    textP->allocatedLineCount = maxLineCount;
+
+    MALLOCARRAY_NOFAIL(textP->textArray, maxLineCount);
+    
+    for (line = 0; line < maxLineCount; ++line)
+        MALLOCARRAY_NOFAIL(textP->textArray[line], maxColumnCount+1);
+
+    textP->lineCount = 0;
+}
+
+
+static void
+freeTextArray(struct text const text) {
+
+    unsigned int line;
+
+    for (line = 0; line < text.allocatedLineCount; ++line)
+        free((char **)text.textArray[line]);
+
+    free(text.textArray);
+}
+
+
 
 static void
 fix_control_chars(char * const buf, struct font * const fn) {
@@ -164,7 +204,6 @@ get_line_dimensions(const char line[], const struct font * const font_p,
    as *backup_space_needed_p.
 -----------------------------------------------------------------------------*/
     int cursor;  /* cursor into the line of text */
-    unsigned char lastch;  /* line[cursor] */
     float accumulated_ics;
         /* accumulated intercharacter space so far in the line we are 
            stepping through.  Because the intercharacter space might not be
@@ -176,20 +215,27 @@ get_line_dimensions(const char line[], const struct font * const font_p,
         /* logical: we haven't seen any renderable characters yet in 
            the line.
         */
+    struct glyph * lastGlyphP;
+        /* Glyph of last character processed so far.  Undefined if
+           'no_chars_yet'.
+        */
+
     no_chars_yet = TRUE;   /* initial value */
     *bwid_p = 0;  /* initial value */
     accumulated_ics = 0.0;  /* initial value */
 
     for (cursor = 0; line[cursor] != '\0'; cursor++) {
-        lastch = line[cursor];
-        if (font_p->glyph[(unsigned char)lastch]) {
+        struct glyph * const glyphP = 
+            font_p->glyph[(unsigned char)line[cursor]];
+
+        if (glyphP) {
             if (no_chars_yet) {
                 no_chars_yet = FALSE;
-                if (font_p->glyph[lastch]->x < 0) 
-                    *backup_space_needed_p = -font_p->glyph[lastch]->x;
+                if (glyphP->x < 0) 
+                    *backup_space_needed_p = -glyphP->x;
                 else {
                     *backup_space_needed_p = 0;
-                    *bwid_p += font_p->glyph[lastch]->x;
+                    *bwid_p += glyphP->x;
                 }
             } else {
                 /* handle extra intercharacter space (-space option) */
@@ -201,7 +247,8 @@ get_line_dimensions(const char line[], const struct font * const font_p,
                     accumulated_ics -= full_pixels;
                 }
             }
-            *bwid_p += font_p->glyph[lastch]->xadd;
+            lastGlyphP = glyphP;
+            *bwid_p += glyphP->xadd;
         }
     }
     if (no_chars_yet)
@@ -213,8 +260,8 @@ get_line_dimensions(const char line[], const struct font * const font_p,
            right at the right edge of the glyph (no extra space to
            anticipate another character).
         */
-        *bwid_p -= font_p->glyph[lastch]->xadd;
-        *bwid_p += font_p->glyph[lastch]->width + font_p->glyph[lastch]->x;
+        *bwid_p -= lastGlyphP->xadd;
+        *bwid_p += lastGlyphP->width + lastGlyphP->x;
     }
 }
 
@@ -222,8 +269,9 @@ get_line_dimensions(const char line[], const struct font * const font_p,
 
 static void
 insert_character(const struct glyph * const glyph, 
-                 const int toprow, const int leftcol,
-                 bit ** const bits) {
+                 int                  const toprow, 
+                 int                  const leftcol,
+                 bit **               const bits) {
 /*----------------------------------------------------------------------------
    Insert one character (whose glyph is 'glyph') into the image bits[].
    Its top left corner shall be row 'toprow', column 'leftcol'.
@@ -243,18 +291,21 @@ insert_character(const struct glyph * const glyph,
 
 
 static void
-insert_characters(bit ** const bits, const char ** const lp, const int lines,
-                  struct font *font_p, 
-                  const int topmargin, const int leftmargin,
-                  const float intercharacter_space, const int lspace) {
+insert_characters(bit **        const bits, 
+                  struct text   const lp,
+                  struct font * const fontP, 
+                  int           const topmargin, 
+                  int           const leftmargin,
+                  float         const intercharacter_space,
+                  int           const lspace) {
 /*----------------------------------------------------------------------------
-   Put the lines of text 'lp' (array of 'lines' strings) into the image 'bits'
-   using font *font_p and putting 'intercharacter_space' pixels between
-   characters and 'lspace' pixels between the lines.
+   Render the text 'lp' into the image 'bits' using font *fontP and
+   putting 'intercharacter_space' pixels between characters and
+   'lspace' pixels between the lines.
 -----------------------------------------------------------------------------*/
     int line;  /* Line number in input text */
 
-    for ( line = 0; line < lines; ++line ) {
+    for (line = 0; line < lp.lineCount; ++line) {
         int row;  /* row in image of top of current typeline */
         int leftcol;  /* Column in image of left edge of current glyph */
         int cursor;  /* cursor into a line of input text */
@@ -265,16 +316,18 @@ insert_characters(bit ** const bits, const char ** const lp, const int lines,
                full pixels whenever we have more than one pixel. 
             */
 
-        row = topmargin + line * (font_p->maxheight + lspace);
+        row = topmargin + line * (fontP->maxheight + lspace);
         leftcol = leftmargin;
         accumulated_ics = 0.0;  /* initial value */
     
-        for ( cursor = 0; lp[line][cursor] != '\0'; ++cursor ) {
+        for (cursor = 0; lp.textArray[line][cursor] != '\0'; ++cursor) {
+            unsigned int const glyphIndex = 
+                (unsigned char)lp.textArray[line][cursor];
             struct glyph* glyph;   /* the glyph for this character */
 
-            glyph = font_p->glyph[(unsigned char)lp[line][cursor]];
+            glyph = fontP->glyph[glyphIndex];
             if (glyph != NULL) {
-                const int toprow = row + font_p->maxheight + font_p->y 
+                const int toprow = row + fontP->maxheight + fontP->y 
                     - glyph->height - glyph->y;
                     /* row number in image of top row in glyph */
                 
@@ -297,141 +350,293 @@ insert_characters(bit ** const bits, const char ** const lp, const int lines,
 }
 
 
+struct outputTextCursor {
+    struct text text;
+        /* The output text.  The lineCount field of this represents
+           the number of lines we have completed.  The line after that
+           is the one we are currently filling.
+        */
+    unsigned int maxWidth;
+        /* A line of output can't be wider than this many pixels */
+    float intercharacterSpace;
+        /* The amount of extra space, in characters, that should be added
+           between every two characters (Pbmtext -space option)
+        */
+    unsigned int columnNo;
+        /* The column Number (starting at 0) in the current line that we are
+           filling where the next character goes.
+        */
+    bool noCharsYet;
+        /* We haven't put any renderable characters yet in the
+           output line. 
+        */
+    unsigned int widthSoFar;
+        /* The accumulated width, in pixels, of all the characters now
+           in the current output line 
+        */
+    float accumulatedIcs;
+        /* accumulated intercharacter space so far in the line we
+           are stepping through.  Because the intercharacter space
+           might not be an integer, we accumulate it here and
+           realize full pixels whenever we have more than one
+           pixel.  
+        */
+};
+
+
+
 static void
-flow_text(const char **lp[], const char input_text[], int *linesP,
-          const int width, 
-          struct font * const font_p, const float intercharacter_space) {
-/* under construction */
-    *lp[0] = "not implemented";
-    *linesP = 1;
+initializeFlowedOutputLine(struct outputTextCursor * const cursorP) {
+
+    cursorP->columnNo = 0;
+    cursorP->noCharsYet = TRUE;
+    cursorP->widthSoFar = 0.0;
+    cursorP->accumulatedIcs = 0.0;
 }
 
 
-static void
-truncate_text(const char **lp[], const char *input_text[], const int lines,
-              const int width, 
-              struct font * const font_p, const float intercharacter_space) {
-/* under construction */
 
+static void
+initializeFlowedOutput(struct outputTextCursor * const cursorP,
+                       unsigned int              const maxLines,
+                       unsigned int              const maxWidth,
+                       float                     const intercharacterSpace) {
+    
+    allocTextArray(&cursorP->text, maxLines, maxWidth);
+    cursorP->maxWidth = maxWidth;
+    cursorP->intercharacterSpace = intercharacterSpace;
+    initializeFlowedOutputLine(cursorP);
+}
+
+
+
+static void
+finishOutputLine(struct outputTextCursor * const cursorP) {
+
+    if (cursorP->text.lineCount < cursorP->text.allocatedLineCount) {
+        char * const currentLine = 
+            cursorP->text.textArray[cursorP->text.lineCount];
+        currentLine[cursorP->columnNo++] = '\0';
+        ++cursorP->text.lineCount;
+    }
+}
+
+
+
+static void
+placeCharacterInOutput(char                      const lastch,
+                       struct font *             const fontP, 
+                       struct outputTextCursor * const cursorP) {
+/*----------------------------------------------------------------------------
+   Place a character of text in the text array at the position indicated
+   by *cursorP, keeping track of what space this character will occupy
+   when this text array is ultimately rendered using font *fontP.
+
+   Note that while we compute how much space the character will take when
+   rendered, we don't render it.
+-----------------------------------------------------------------------------*/
+    if (cursorP->text.lineCount < cursorP->text.allocatedLineCount) {
+        unsigned int const glyphIndex = (unsigned char)lastch;
+        if (fontP->glyph[glyphIndex]) {
+            if (cursorP->noCharsYet) {
+                cursorP->noCharsYet = FALSE;
+                if (fontP->glyph[glyphIndex]->x > 0) 
+                    cursorP->widthSoFar += fontP->glyph[glyphIndex]->x;
+            } else {
+                /* handle extra intercharacter space (-space option) */
+                unsigned int fullPixels;  /* integer part of accumulatedIcs */
+                cursorP->accumulatedIcs += cursorP->intercharacterSpace;
+                fullPixels = (int) cursorP->accumulatedIcs;
+                if (fullPixels > 0) {
+                    cursorP->widthSoFar += fullPixels;
+                    cursorP->accumulatedIcs -= fullPixels;
+                }
+            }
+            cursorP->widthSoFar += fontP->glyph[glyphIndex]->xadd;
+        }
+        if (cursorP->widthSoFar < cursorP->maxWidth) {
+            char * const currentLine = 
+                cursorP->text.textArray[cursorP->text.lineCount];
+            currentLine[cursorP->columnNo++] = lastch;
+        } else {
+            /* Line is full; finish it off, start the next one, and
+               place the character there.
+            */
+            /* TODO: We really should back up to the previous white space
+               character and move the rest of the line to the next line
+            */
+            finishOutputLine(cursorP);
+            initializeFlowedOutputLine(cursorP);
+            placeCharacterInOutput(lastch, fontP, cursorP);
+        }
+    }
+}
+
+
+
+static void
+flowText(struct text    const inputText,
+         int            const width, 
+         struct font *  const fontP, 
+         float          const intercharacterSpace,
+         struct text *  const outputTextP) {
+    
+    unsigned int const maxLineCount = 50;
+
+    unsigned int inputLine;  
+        /* Input line number on which we are currently working */
+    struct outputTextCursor outputCursor;
+
+    for (inputLine = 0; inputLine < inputText.lineCount; ++inputLine) {
+        unsigned int incursor;   /* cursor into the line we are reading */
+
+        initializeFlowedOutput(&outputCursor, maxLineCount,
+                               width, intercharacterSpace);
+        
+        for (incursor = 0; 
+             inputText.textArray[inputLine][incursor] != '\0'; 
+             ++incursor)
+            placeCharacterInOutput(inputText.textArray[inputLine][incursor],
+                                   fontP, &outputCursor);
+        finishOutputLine(&outputCursor);
+    }
+    *outputTextP = outputCursor.text;
+}
+
+
+
+static void
+truncateText(struct text   const inputText, 
+             unsigned int  const width, 
+             struct font * const fontP, 
+             float         const intercharacterSpace,
+             struct text * const outputTextP) {
+
+    struct text truncatedText;
     int line;  /* Line number on which we are currently working */
 
-    *lp[0] = "not implemented";
+    allocTextArray(&truncatedText, inputText.lineCount, width);
 
-    for (line = 0; line < lines; line++){
-            
+    for (line = 0; line < inputText.lineCount; ++line){
         int cursor;  /* cursor into the line of text */
         unsigned char lastch;  /* line[cursor] */
-        int width_so_far;
+        int widthSoFar;
             /* How long the line we've built, in pixels, is so far */
-        float accumulated_ics;
+        float accumulatedIcs;
         /* accumulated intercharacter space so far in the line we are 
            stepping through.  Because the intercharacter space might not be
            an integer, we accumulate it here and realize full pixels whenever
            we have more than one pixel.
            */
 
-        int no_chars_yet; 
+        int noCharsYet; 
         /* logical: we haven't seen any renderable characters yet in 
            the line.
         */
-        no_chars_yet = TRUE;   /* initial value */
-        width_so_far = 0;  /* initial value */
-        accumulated_ics = 0.0;  /* initial value */
+        noCharsYet = TRUE;   /* initial value */
+        widthSoFar = 0;  /* initial value */
+        accumulatedIcs = 0.0;  /* initial value */
+
+        truncatedText.textArray[line][0] = '\0';  /* Start with empty line */
 
         for (cursor = 0; 
-             input_text[line][cursor] != '\0' && width_so_far < width; 
+             inputText.textArray[line][cursor] != '\0' && widthSoFar < width; 
              cursor++) {
-            lastch = input_text[line][cursor];
-            if (font_p->glyph[(unsigned char)lastch]) {
-                if (no_chars_yet) {
-                    no_chars_yet = FALSE;
-                    if (font_p->glyph[lastch]->x > 0) 
-                        width_so_far += font_p->glyph[lastch]->x;
+            lastch = inputText.textArray[line][cursor];
+            if (fontP->glyph[(unsigned char)lastch]) {
+                if (noCharsYet) {
+                    noCharsYet = FALSE;
+                    if (fontP->glyph[lastch]->x > 0) 
+                        widthSoFar += fontP->glyph[lastch]->x;
                 } else {
                     /* handle extra intercharacter space (-space option) */
-                    int full_pixels;  /* integer part of accumulated_ics */
-                    accumulated_ics += intercharacter_space;
-                    full_pixels = (int) intercharacter_space;
-                    if (full_pixels > 0) {
-                        width_so_far += full_pixels;
-                        accumulated_ics -= full_pixels;
+                    int fullPixels;  /* integer part of accumulatedIcs */
+                    accumulatedIcs += intercharacterSpace;
+                    fullPixels = (int) intercharacterSpace;
+                    if (fullPixels > 0) {
+                        widthSoFar += fullPixels;
+                        accumulatedIcs -= fullPixels;
                     }
                 }
-                width_so_far += font_p->glyph[lastch]->xadd;
+                widthSoFar += fontP->glyph[lastch]->xadd;
             }
-            if (width_so_far < width) {
-/* not implemented yet
-                lp[line][cursor] = input_text[line][cursor];
-                lp[line][cursor+1] = '\0';
-*/
+            if (widthSoFar < width) {
+                truncatedText.textArray[line][cursor] = 
+                    inputText.textArray[line][cursor];
+                truncatedText.textArray[line][cursor+1] = '\0';
             }
         }
     }
+    truncatedText.lineCount = inputText.lineCount;
+    *outputTextP = truncatedText;
 }
 
 
 
 static void
-get_text(const char cmdline_text[], struct font * const fn, 
-         const char *** const input_textP, int * linesP) {
+getText(const char          cmdline_text[], 
+        struct font * const fn, 
+        struct text * const input_textP) {
 
-    int maxlines;  
-        /* Maximum number of lines for which we presently have space
-           in the text array 
-        */
-
-    char ** input_text;
-
-    maxlines = 50;  /* initial value */
-    MALLOCARRAY(input_text, maxlines);
-    if (input_text == NULL)
-        pm_error("out of memory");
+    struct text input_text;
 
     if (cmdline_text) {
-        input_text[0] = malloc(strlen(cmdline_text)+1);
-        if (input_text[0] == NULL)
-            pm_error("Out of memory.");
-        strcpy(input_text[0], cmdline_text);
-        fix_control_chars(input_text[0], fn);
-        *linesP = 1;
+        allocTextArray(&input_text, 1, strlen(cmdline_text));
+        strcpy(input_text.textArray[0], cmdline_text);
+        fix_control_chars(input_text.textArray[0], fn);
+        input_text.lineCount = 1;
     } else {
         /* Read text from stdin. */
-        char buf[5000];
 
-        *linesP = 0;  /* initial value */
+        unsigned int maxlines;  
+            /* Maximum number of lines for which we presently have space
+               in the text array 
+            */
+        char buf[5000];
+        char ** text_array;
+        unsigned int lineCount;
+
+        maxlines = 50;  /* initial value */
+        MALLOCARRAY_NOFAIL(text_array, maxlines);
+        
+        lineCount = 0;  /* initial value */
         while (fgets(buf, sizeof(buf), stdin) != NULL) {
             fix_control_chars(buf, fn);
-            if (*linesP >= maxlines) {
+            if (lineCount >= maxlines) {
                 maxlines *= 2;
-                input_text = (char**) realloc((char*) input_text, 
-                                                maxlines * sizeof(char*));
-                if(input_text == NULL)
-                    pm_error( "out of memory" );
+                text_array = (char**) realloc((char*) text_array, 
+                                              maxlines * sizeof(char*));
+                if (text_array == NULL)
+                    pm_error("out of memory");
             }
-            input_text[*linesP] = strdup(buf);
-            if (input_text[*linesP] == NULL)
-                pm_error( "out of memory" );
-            ++(*linesP);
+            text_array[lineCount] = strdup(buf);
+            if (text_array[lineCount] == NULL)
+                pm_error("out of memory");
+            ++lineCount;
         }
+        input_text.textArray = text_array;
+        input_text.lineCount = lineCount;
+        input_text.allocatedLineCount = lineCount;
     }
-    *input_textP = (const char **) input_text;
+    *input_textP = input_text;
 }
 
 
 
 static void
-compute_image_width(const char ** const lp, const int lines,
+compute_image_width(struct text         const lp, 
                     const struct font * const fn,
-                    const float intercharacter_space,
-                    int * const maxwidthP, int * maxleftbP) {
+                    float               const intercharacter_space,
+                    int *               const maxwidthP, 
+                    int *               const maxleftbP) {
     int line;
     
     *maxwidthP = 0;  /* initial value */
     *maxleftbP = 0;  /* initial value */
-    for (line = 0; line < lines; ++line) {
+    for (line = 0; line < lp.lineCount; ++line) {
         int bwid, backup_space_needed;
         
-        get_line_dimensions(lp[line], fn, intercharacter_space,
+        get_line_dimensions(lp.textArray[line], fn, intercharacter_space,
                             &bwid, &backup_space_needed);
         
         *maxwidthP = MAX(*maxwidthP, bwid);
@@ -447,11 +652,10 @@ main(int argc, char *argv[]) {
     struct cmdline_info cmdline;
     bit** bits;
     int rows, cols;
-    struct font* fn;
+    struct font* fontP;
     int vmargin, hmargin;
-    const char** input_text;
-    const char** lp;
-    int lines;
+    struct text inputText;
+    struct text formattedText;
     int maxwidth;
         /* width in pixels of the longest line of text in the image */
     int maxleftb;
@@ -461,49 +665,52 @@ main(int argc, char *argv[]) {
     parse_command_line(argc, argv, &cmdline);
     
     if (cmdline.font)
-        fn = pbm_loadfont(cmdline.font);
+        fontP = pbm_loadfont(cmdline.font);
     else {
         if (cmdline.builtin)
-            fn = pbm_defaultfont(cmdline.builtin);
+            fontP = pbm_defaultfont(cmdline.builtin);
         else
-            fn = pbm_defaultfont("bdf");
+            fontP = pbm_defaultfont("bdf");
     }
 
     if (cmdline.dump) {
-        pbm_dumpfont(fn);
+        pbm_dumpfont(fontP);
         exit(0);
     }
 
-    get_text(cmdline.text, fn, &input_text, &lines);
+    getText(cmdline.text, fontP, &inputText);
     
     if (cmdline.nomargins) {
         vmargin = 0;
         hmargin = 0;
     } else {
-        if (lines == 1) {
-            vmargin = fn->maxheight / 2;
-            hmargin = fn->maxwidth;
+        if (inputText.lineCount == 1) {
+            vmargin = fontP->maxheight / 2;
+            hmargin = fontP->maxwidth;
         } else {
-            vmargin = fn->maxheight;
-            hmargin = 2 * fn->maxwidth;
+            vmargin = fontP->maxheight;
+            hmargin = 2 * fontP->maxwidth;
         }
     }
 
     if (cmdline.width > 0) {
         /* Flow or truncate lines to meet user's width request */
-        /* Not actually implemented yet. */
-        if (lines == 1) 
-            flow_text(&lp, input_text[0], &lines, cmdline.width, 
-                      fn, cmdline.space);
-        else 
-            truncate_text(&lp, (const char **) input_text, lines, 
-                          cmdline.width, fn, cmdline.space);
+        if (inputText.lineCount == 1) 
+            flowText(inputText, cmdline.width, fontP, cmdline.space,
+                     &formattedText);
+        else
+            truncateText(inputText, cmdline.width, fontP, cmdline.space,
+                         &formattedText);
+        freeTextArray(inputText);
     } else
-        lp = input_text;
+        formattedText = inputText;
     
-    rows = 2 * vmargin + lines * fn->maxheight + (lines-1) * cmdline.lspace;
+    rows = 2 * vmargin + 
+        formattedText.lineCount * fontP->maxheight + 
+        (formattedText.lineCount-1) * cmdline.lspace;
 
-    compute_image_width(lp, lines, fn, cmdline.space, &maxwidth, &maxleftb);
+    compute_image_width(formattedText, fontP, cmdline.space,
+                        &maxwidth, &maxleftb);
 
     cols = 2 * hmargin + maxwidth;
     bits = pbm_allocarray(cols, rows);
@@ -512,12 +719,14 @@ main(int argc, char *argv[]) {
     fill_rect(bits, 0, 0, rows, cols, PBM_WHITE);
 
     /* Put the text in  */
-    insert_characters(bits, (const char **) lp, lines, fn, 
-               vmargin, hmargin + maxleftb, cmdline.space, cmdline.lspace);
+    insert_characters(bits, formattedText, fontP, vmargin, hmargin + maxleftb, 
+                      cmdline.space, cmdline.lspace);
 
     /* All done. */
     pbm_writepbm(stdout, bits, cols, rows, 0);
+
+    freeTextArray(formattedText);
     pm_close(stdout);
 
-    exit(0);
+    return 0;
 }
