@@ -90,7 +90,7 @@ validateComputableSize(struct pam * const pamP) {
 
     if (depth > INT_MAX/sizeof(sample))
         pm_error("image depth (%u) too large to be processed", depth);
-    else if (depth * sizeof(sample) > INT_MAX/pamP->width)
+    else if (pamP->width > 0 && depth * sizeof(sample) > INT_MAX/pamP->width)
         pm_error("image width and depth (%u, %u) too large "
                  "to be processed.", pamP->width, depth);
     else if (pamP->width * (depth * sizeof(sample)) >
@@ -679,9 +679,9 @@ pnm_readpaminitrestaspnm(FILE * const fileP,
 -----------------------------------------------------------------------------*/
     struct pam pam;
 
-    pam.size        = PAM_STRUCT_SIZE(tuple_type);
+    pam.size        = sizeof(struct pam);
     pam.file        = fileP;
-    pam.len         = sizeof(struct pam);
+    pam.len         = PAM_STRUCT_SIZE(tuple_type);
     pam.format      = PAM_FORMAT;
 
     readpaminitrest(&pam);
@@ -1074,7 +1074,7 @@ pm_feed_from_pamtuples(int    const pipeToFeedFd,
     outpam.file = fdopen(pipeToFeedFd, "w");
 
     /* The following signals (and normally kills) the process with
-       SIGPIPE if the pipe does not take all 'inputLength' bytes.
+       SIGPIPE if the pipe does not take all the data.
     */
     pnm_writepam(&outpam, *inputTuplesP->tuplesP);
 
@@ -1089,10 +1089,11 @@ pm_accept_to_pamtuples(int    const pipeToSuckFd,
 
     struct pamtuples * const outputTuplesP = accepterParm;
 
-    struct pam inpam;
+    struct pam * const inpamP = outputTuplesP->pamP;
 
     *outputTuplesP->tuplesP =
-        pnm_readpam(fdopen(pipeToSuckFd, "r"), &inpam, sizeof(inpam));
+        pnm_readpam(fdopen(pipeToSuckFd, "r"), inpamP,
+                    PAM_STRUCT_SIZE(tuple_type));
 
-    pm_close(inpam.file);
+    pm_close(inpamP->file);
 }
