@@ -13,6 +13,10 @@
 
 ******************************************************************************/
 
+#include <assert.h>
+
+#include "pm_c_util.h"
+#include "mallocvar.h"
 #include "pam.h"
 #include "pammap.h"
 
@@ -45,10 +49,15 @@ pnm_createtuplehash(void) {
 /*----------------------------------------------------------------------------
    Create an empty tuple hash -- i.e. a hash table of zero length hash chains.
 -----------------------------------------------------------------------------*/
-
-    tuplehash const retval = malloc(HASH_SIZE * sizeof(tupleint_list));
+    tuplehash retval;
     unsigned int i;
 
+    MALLOCARRAY(retval, HASH_SIZE);
+
+    if (retval == NULL)
+        pm_error("Out of memory allocating tuple hash of size %u",
+                 HASH_SIZE);
+
     for (i = 0; i < HASH_SIZE; ++i) 
         retval[i] = NULL;
 
@@ -60,7 +69,6 @@ pnm_createtuplehash(void) {
 void
 pnm_destroytuplehash(tuplehash const tuplehash) {
 
-
     int i;
 
     /* Free the chains */
@@ -94,9 +102,11 @@ allocTupleIntListItem(struct pam * const pamP) {
     */
     struct tupleint_list_item * retval;
 
-    retval = (struct tupleint_list_item *)
-        malloc(sizeof(*retval) - sizeof(retval->tupleint.tuple) 
-               + pamP->depth * sizeof(sample));
+    unsigned int const size = 
+        sizeof(*retval) - sizeof(retval->tupleint.tuple) 
+        + pamP->depth * sizeof(sample);
+
+    retval = (struct tupleint_list_item *) malloc(size);
 
     return retval;
 }
@@ -135,9 +145,11 @@ pnm_addtotuplehash(struct pam *   const pamP,
 
 
 void
-pnm_lookuptuple(struct pam * const pamP, const tuplehash tuplehash, 
-                const tuple searchval, 
-                int * const foundP, int * const retvalP) {
+pnm_lookuptuple(struct pam *    const pamP, 
+                const tuplehash       tuplehash, 
+                const tuple           searchval, 
+                int *           const foundP, 
+                int *           const retvalP) {
     
     unsigned int const hashvalue = pnm_hashtuple(pamP, searchval);
     struct tupleint_list_item * p;
@@ -159,12 +171,12 @@ pnm_lookuptuple(struct pam * const pamP, const tuplehash tuplehash,
 
 
 static void
-addColorToHash(tuple          const color, 
-               tuplehash      const tuplefreqhash,
-               struct pam *   const pamP,
-               unsigned int   const maxsize,
-               unsigned int * const sizeP,
-               bool *         const fullP) {
+addColorOccurrenceToHash(tuple          const color, 
+                         tuplehash      const tuplefreqhash,
+                         struct pam *   const pamP,
+                         unsigned int   const maxsize,
+                         unsigned int * const sizeP,
+                         bool *         const fullP) {
                
     unsigned int const hashvalue = pnm_hashtuple(pamP, color);
             
@@ -175,7 +187,8 @@ addColorToHash(tuple          const color,
          p = p->next);
 
     if (p) {
-        p->tupleint.value++;
+        /* It's in the hash; just tally one more occurence */
+        ++p->tupleint.value;
         *fullP = FALSE;
     } else {
         /* It's not in the hash yet, so add it (if allowed) */
@@ -197,6 +210,104 @@ addColorToHash(tuple          const color,
 
 
 
+void
+pnm_addtuplefreqoccurrence(struct pam *   const pamP,
+                           tuple          const value,
+                           tuplehash      const tuplefreqhash,
+                           int *          const firstOccurrenceP) {
+
+    unsigned int const hashvalue = pnm_hashtuple(pamP, value);
+            
+    struct tupleint_list_item * p;
+
+    for (p = tuplefreqhash[hashvalue]; 
+         p && !pnm_tupleequal(pamP, p->tupleint.tuple, value);
+         p = p->next);
+
+    if (p) {
+        /* It's in the hash; just tally one more occurence */
+        ++p->tupleint.value;
+        *firstOccurrenceP = FALSE;
+    } else {
+        struct tupleint_list_item * p;
+
+        /* It's not in the hash yet, so add it */
+        *firstOccurrenceP = TRUE;
+
+        p = allocTupleIntListItem(pamP);
+        if (p == NULL)
+            pm_error("out of memory computing hash table");
+
+        pnm_assigntuple(pamP, p->tupleint.tuple, value);
+        p->tupleint.value = 1;
+        p->next = tuplefreqhash[hashvalue];
+        tuplefreqhash[hashvalue] = p;
+    }
+}
+
+
+
+static void
+computehashrecoverable(struct pam *   const pamP,
+                       tuple **       const tupleArray, 
+                       unsigned int   const maxsize, 
+                       sample         const newMaxval,
+                       unsigned int * const sizeP,
+                       tuplehash *    const tuplefreqhashP,
+                       tuple **       const rowbufferP,
+                       tuple *        const colorP) {
+/*----------------------------------------------------------------------------
+   This is computetuplefreqhash(), only it leaves a trail so that if it
+   happens to longjmp out because of a failed memory allocation, the
+   setjmp'er can cleanup whatever it had done so far.
+-----------------------------------------------------------------------------*/
+    unsigned int row;
+    struct pam freqPam;
+    bool full;
+
+    freqPam = *pamP;
+    freqPam.maxval = newMaxval;
+
+    *tuplefreqhashP = pnm_createtuplehash();
+    *sizeP = 0;   /* initial value */
+    
+    *rowbufferP = pnm_allocpamrow(pamP);
+    
+    *colorP = pnm_allocpamtuple(&freqPam);
+    
+    full = FALSE;  /* initial value */
+    
+    /* Go through the entire raster, building a hash table of
+       tuple values. 
+    */
+    for (row = 0; row < pamP->height && !full; ++row) {
+        int col;
+        const tuple * tuplerow;  /* The row of tuples we are processing */
+        
+        if (tupleArray)
+            tuplerow = tupleArray[row];
+        else {
+            pnm_readpamrow(pamP, *rowbufferP);
+            tuplerow = *rowbufferP;
+        }
+        for (col = 0; col < pamP->width && !full; ++col) {
+            pnm_scaletuple(pamP, *colorP, tuplerow[col], freqPam.maxval);
+            addColorOccurrenceToHash(
+                *colorP, *tuplefreqhashP, &freqPam, maxsize, sizeP, &full);
+        }
+    }
+
+    pnm_freepamtuple(*colorP); *colorP = NULL;
+    pnm_freepamrow(*rowbufferP); *rowbufferP = NULL;
+
+    if (full) {
+        pnm_destroytuplehash(*tuplefreqhashP);
+        *tuplefreqhashP = NULL;
+    }
+}
+
+
+
 static tuplehash
 computetuplefreqhash(struct pam *   const pamP,
                      tuple **       const tupleArray, 
@@ -223,55 +334,41 @@ computetuplefreqhash(struct pam *   const pamP,
   However, if the number of unique tuple values is greater than 'maxsize', 
   return a null return value and *sizeP undefined.
 
+  The tuple values that index the hash are scaled to a new maxval of
+  'newMaxval'.  E.g.  if the input has maxval 100 and 'newMaxval' is
+  50, and a particular tuple has sample value 50, it would be counted
+  as sample value 25 in the hash.
 -----------------------------------------------------------------------------*/
     tuplehash tuplefreqhash;
-    int row;
     tuple * rowbuffer;  /* malloc'ed */
         /* Buffer for a row read from the input file; undefined (but still
            allocated) if input is not from a file.
         */
     tuple color;  
-        /* The color being current being added, scaled to the new maxval */
-    struct pam freqPam;
-    bool full;
+        /* The color currently being added, scaled to the new maxval */
+    jmp_buf jmpbuf;
+    jmp_buf * origJmpbufP;
     
-    freqPam = *pamP;
-    freqPam.maxval = newMaxval;
+    /* Initialize to "none" for purposes of error recovery */
+    tuplefreqhash = NULL;
+    rowbuffer = NULL;
+    color = NULL;
 
-    tuplefreqhash = pnm_createtuplehash();
-    *sizeP = 0;   /* initial value */
-
-    rowbuffer = pnm_allocpamrow(pamP);
-
-    color = pnm_allocpamtuple(&freqPam);
-
-    full = FALSE;  /* initial value */
-
-    /* Go through the entire raster, building a hash table of tuple values. */
-    for (row = 0; row < pamP->height && !full; ++row) {
-        int col;
-        const tuple * tuplerow;  /* The row of tuples we are processing */
-
-        if (tupleArray)
-            tuplerow = tupleArray[row];
-        else {
-            pnm_readpamrow(pamP, rowbuffer);
-            tuplerow = rowbuffer;
-        }
-        for (col = 0; col < pamP->width && !full; ++col) {
-            pnm_scaletuple(pamP, color, tuplerow[col], freqPam.maxval);
-            addColorToHash(color, tuplefreqhash, &freqPam, maxsize, 
-                           sizeP, &full);
-        }
+    if (setjmp(jmpbuf) == 0) {
+        pm_setjmpbufsave(&jmpbuf, &origJmpbufP);
+        computehashrecoverable(pamP, tupleArray, maxsize, newMaxval, sizeP,
+                               &tuplefreqhash, &rowbuffer, &color);
+        pm_setjmpbuf(origJmpbufP);
+    } else {
+        if (color) 
+            pnm_freepamtuple(color);
+        if (rowbuffer)
+            pnm_freepamrow(rowbuffer);
+        if (tuplefreqhash)
+            pnm_destroytuplehash(tuplefreqhash);
+        pm_longjmp();
     }
-    pnm_freepamtuple(color);
-    pnm_freepamrow(rowbuffer);
-
-    if (full) {
-        pnm_destroytuplehash(tuplefreqhash);
-        return NULL;
-    } else
-        return tuplefreqhash;
+    return tuplefreqhash;
 }
 
 
@@ -290,38 +387,47 @@ pnm_computetuplefreqhash(struct pam *   const pamP,
 
 
 
-
 tupletable
-pnm_alloctupletable(const struct pam * const pamP, unsigned int const size) {
+pnm_alloctupletable(const struct pam * const pamP, 
+                    unsigned int       const size) {
     
-    unsigned int const tupleIntSize = 
-        sizeof(struct tupleint) - sizeof(sample) 
-        + pamP->depth * sizeof(sample);
-    unsigned int const mainTableSize = size * sizeof(struct tupleint *);
-    void * pool;
     tupletable retval;
-    unsigned int i;
-    
-    /* To save the enormous amount of time it could take to allocate
-       each individual tuple, we do a trick here and allocate everything
-       as a single malloc block and suballocate internally.
-    */
-    
-    pool = malloc(mainTableSize + size * tupleIntSize);
-    
-    retval = (tupletable) pool;
 
-    for (i = 0; i < size; ++i)
-        retval[i] = (struct tupleint *)
-            ((char*)pool + mainTableSize + i * tupleIntSize);
+    if (UINT_MAX / sizeof(struct tupleint) < size)
+        pm_error("size %u is too big for arithmetic", size);
+    else {
+        unsigned int const mainTableSize = size * sizeof(struct tupleint *);
+        unsigned int const tupleIntSize = 
+            sizeof(struct tupleint) - sizeof(sample) 
+            + pamP->depth * sizeof(sample);
+
+        /* To save the enormous amount of time it could take to allocate
+           each individual tuple, we do a trick here and allocate everything
+           as a single malloc block and suballocate internally.
+        */
+        if ((UINT_MAX - mainTableSize) / tupleIntSize < size)
+            pm_error("size %u is too big for arithmetic", size);
+        else {
+            void * pool;
+            unsigned int i;
     
+            pool = malloc(mainTableSize + size * tupleIntSize);
+    
+            retval = (tupletable) pool;
+
+            for (i = 0; i < size; ++i)
+                retval[i] = (struct tupleint *)
+                    ((char*)pool + mainTableSize + i * tupleIntSize);
+        }
+    }
     return retval;
 }
 
 
 
 void
-pnm_freetupletable(struct pam * const pamP, tupletable const tupletable) {
+pnm_freetupletable(struct pam * const pamP,
+                   tupletable   const tupletable) {
 
     /* Note that the address 'tupletable' is, to the operating system, 
        the address of a larger block of memory that contains not only 
@@ -334,38 +440,67 @@ pnm_freetupletable(struct pam * const pamP, tupletable const tupletable) {
 
 
 
-tupletable
-pnm_tuplehashtotable(const struct pam * const pamP,
-                     tuplehash          const tuplehash,
-                     unsigned int       const maxsize) {
+void
+pnm_freetupletable2(struct pam * const pamP,
+                    tupletable2  const tupletable) {
+
+    pnm_freetupletable(pamP, tupletable.table);
+}
+
+
+
+static tupletable
+tuplehashtotable(const struct pam * const pamP,
+                 tuplehash          const tuplehash,
+                 unsigned int       const allocsize) {
 /*----------------------------------------------------------------------------
    Create a tuple table containing the info from a tuple hash.  Allocate
-   space in the table for 'maxsize' elements even if there aren't that
+   space in the table for 'allocsize' elements even if there aren't that
    many tuple values in the input hash.  That's so the caller has room
    for expansion.
 
+   Caller must ensure that 'allocsize' is at least as many tuple values
+   as there are in the input hash.
+
    We allocate new space for all the table contents; there are no pointers
    in the table to tuples or anything else in existing space.
 -----------------------------------------------------------------------------*/
     tupletable tupletable;
-    int i, j;
 
-    tupletable = pnm_alloctupletable(pamP, maxsize);
+    tupletable = pnm_alloctupletable(pamP, allocsize);
+
+    if (tupletable != NULL) {
+        unsigned int i, j;
+        /* Loop through the hash table. */
+        j = 0;
+        for (i = 0; i < HASH_SIZE; ++i) {
+            /* Walk this hash chain */
+            struct tupleint_list_item * p;
+            for (p = tuplehash[i]; p; p = p->next) {
+                assert(j < allocsize);
+                tupletable[j]->value = p->tupleint.value;
+                pnm_assigntuple(pamP, tupletable[j]->tuple, p->tupleint.tuple);
+                ++j;
+            }
+        }
+    }
+    return tupletable;
+}
+
+
+
+tupletable
+pnm_tuplehashtotable(const struct pam * const pamP,
+                     tuplehash          const tuplehash,
+                     unsigned int       const allocsize) {
+
+    tupletable tupletable;
+
+    tupletable = tuplehashtotable(pamP, tuplehash, allocsize);
 
     if (tupletable == NULL)
         pm_error("out of memory generating tuple table");
 
-    /* Loop through the hash table. */
-    j = 0;
-    for (i = 0; i < HASH_SIZE; ++i) {
-        /* Walk this hash chain */
-        struct tupleint_list_item * p;
-        for (p = tuplehash[i]; p; p = p->next) {
-            tupletable[j]->value = p->tupleint.value;
-            pnm_assigntuple(pamP, tupletable[j]->tuple, p->tupleint.tuple);
-            ++j;
-        }
-    }
     return tupletable;
 }
 
@@ -388,16 +523,18 @@ pnm_computetupletablehash(struct pam * const pamP,
 -----------------------------------------------------------------------------*/
     tuplehash tupletablehash;
     unsigned int i;
+    bool fits;
     
     tupletablehash = pnm_createtuplehash();
 
-    for (i = 0; i < tupletableSize; ++i) {
-        bool fits;
-        
+    fits = TRUE;  /* initial assumption */
+    for (i = 0; i < tupletableSize && fits; ++i) {
         pnm_addtotuplehash(pamP, tupletablehash, 
                            tupletable[i]->tuple, i, &fits);
-        if (!fits)
-            pm_error("Out of memory computing tuple hash from tuple table");
+    }
+    if (!fits) {
+        pnm_destroytuplehash(tupletablehash);
+        pm_error("Out of memory computing tuple hash from tuple table");
     }
     return tupletablehash;
 }
@@ -409,38 +546,56 @@ pnm_computetuplefreqtable2(struct pam *   const pamP,
                            tuple **       const tupleArray,
                            unsigned int   const maxsize,
                            sample         const newMaxval,
-                           unsigned int * const sizeP) {
+                           unsigned int * const countP) {
 /*----------------------------------------------------------------------------
-   Compute the a tuple frequency table from a PAM image.  This is an
+   Compute a tuple frequency table from a PAM image.  This is an
    array that tells how many times each tuple value occurs in the
    image.
 
    Except for the format of the output, this function is the same as
    computetuplefreqhash().
 
-   Return the array in newly malloc'ed storage.  Allocate space for 'maxsize'
-   entries even if there aren't that many colors in tupleArray[].  That's
-   so the caller has room for expansion.
-
-   Return the number of unique tuple values in tupleArray[][], which
-   is the size of the output array, as *sizeP.
-
    If there are more than 'maxsize' unique tuple values in tupleArray[][],
    give up.
 
+   Return the array in newly malloc'ed storage.  Allocate space for
+   'maxsize' entries even if there aren't that many distinct tuple
+   values in tupleArray[].  That's so the caller has room for
+   expansion.
+
+   If 'maxsize' is zero, allocate exactly as much space as there are
+   distinct tuple values in tupleArray[], and don't give up no matter
+   how many tuple values we find (except, of course, we abort if we
+   can't get enough memory).
+
+   Return the number of unique tuple values in tupleArray[][] as
+   *countP.
+
+
+   Scale the tuple values to a new maxval of 'newMaxval' before
+   processing them.  E.g. if the input has maxval 100 and 'newMaxval'
+   is 50, and a particular tuple has sample value 50, it would be
+   listed as sample value 25 in the output table.  This makes the
+   output table smaller and the processing time less.
 -----------------------------------------------------------------------------*/
     tuplehash tuplefreqhash;
     tupletable tuplefreqtable;
+    unsigned int uniqueCount;
 
     tuplefreqhash = computetuplefreqhash(pamP, tupleArray, maxsize, 
-                                         newMaxval, sizeP);
-
+                                         newMaxval, &uniqueCount);
     if (tuplefreqhash == NULL)
         tuplefreqtable = NULL;
     else {
-        tuplefreqtable = pnm_tuplehashtotable(pamP, tuplefreqhash, maxsize);
+        unsigned int tableSize = (maxsize == 0 ? uniqueCount : maxsize);
+        assert(tableSize >= uniqueCount);
+        tuplefreqtable = tuplehashtotable(pamP, tuplefreqhash, tableSize);
         pnm_destroytuplehash(tuplefreqhash);
+        if (tuplefreqtable == NULL)
+            pm_error("Out of memory generating tuple table");
     }
+    *countP = uniqueCount;
+
     return tuplefreqtable;
 }
 
