@@ -12,6 +12,11 @@
   (in addition to being the library for handling the PBM format).
 
 **************************************************************************/
+#define _SVID_SOURCE
+    /* Make sure P_tmpdir is defined in GNU libc 2.0.7 (_XOPEN_SOURCE 500
+       does it in other libc's).  pm_config.h defines TMPDIR as P_tmpdir
+       in some environments.
+    */
 #define _XOPEN_SOURCE 500    /* Make sure ftello, fseeko are defined */
 #define _LARGEFILE_SOURCE 1  /* Make sure ftello, fseeko are defined */
 #define _LARGEFILE64_SOURCE 1 
@@ -31,19 +36,21 @@
     /* This makes the the x64() functions available on AIX */
 
 #include <stdio.h>
-#include "version.h"
-#include "compile.h"
-#include "nstring.h"
-#include "shhopt.h"
-#include "pm.h"
 #include <stdarg.h>
 #include <string.h>
 #include <errno.h>
+#include <setjmp.h>
 #ifdef __DJGPP__
   #include <io.h>
 #endif
 
+#include "pm_c_util.h"
+#include "version.h"
+#include "compile.h"
+#include "nstring.h"
+#include "shhopt.h"
 #include "mallocvar.h"
+#include "pm.h"
 
 /* The following are set by pm_init(), then used by subsequent calls to other
    pm_xxx() functions.
@@ -54,76 +61,127 @@ static bool pm_showmessages;
        specify the --quiet option).
     */
 
-/* Programs should produce output in plain format */
 int pm_plain_output;
+    /* Boolean: programs should produce output in plain format */
+
+static jmp_buf * pm_jmpbufP = NULL;
+    /* A description of the point to which the program should hyperjump
+       if a libnetpbm function encounters an error (libnetpbm functions
+       don't normally return in that case).
+
+       User sets this to something in his own extra-library context.
+       Libnetpbm routines that have something that needs to be cleaned up
+       preempt it.
+
+       NULL, which is the default value, means when a libnetpbm function
+       encounters an error, it causes the process to exit.
+    */
+
+
 
 void
-pm_usage( const char usage[] ) {
-    fprintf( stderr, "usage:  %s %s\n", pm_progname, usage );
-    exit( 1 );
-    }
+pm_setjmpbuf(jmp_buf * const jmpbufP) {
+    pm_jmpbufP = jmpbufP;
+}
+
+
+
+void
+pm_setjmpbufsave(jmp_buf *  const jmpbufP,
+                 jmp_buf ** const oldJmpbufPP) {
+
+    *oldJmpbufPP = pm_jmpbufP;
+    pm_jmpbufP = jmpbufP;
+}
+
+
+
+void
+pm_longjmp(void) {
+
+    if (pm_jmpbufP)
+        longjmp(*pm_jmpbufP, 1);
+    else
+        exit(1);
+}
+
+
+
+void
+pm_usage(const char usage[]) {
+    pm_error("usage:  %s %s", pm_progname, usage);
+}
+
+
 
 void
 pm_perror(const char reason[] ) {
-    const char* e;
 
-#ifdef A_STRERROR
-    e = strerror(errno);
-#else /* A_STRERROR */
-    e = sys_errlist[errno];
-#endif /* A_STRERROR */
-
-    if ( reason != 0 && reason[0] != '\0' )
-        pm_error( "%s - %s", reason, e );
+    if (reason != NULL && strlen(reason) != 0)
+        pm_error("%s - errno=%d (%s)", reason, errno, strerror(errno));
     else
-        pm_error( "%s", e );
-    }
+        pm_error("Something failed with errno=%d (%s)", 
+                 errno, strerror(errno));
+}
 
-void GNU_PRINTF_ATTR
-pm_message(const char format[], ... ) {
+
+
+void PM_GNU_PRINTF_ATTR(1,2)
+pm_message(const char format[], ...) {
 
     va_list args;
 
-    va_start( args, format );
+    va_start(args, format);
 
-    if ( pm_showmessages )
-        {
-        fprintf( stderr, "%s: ", pm_progname );
-        (void) vfprintf( stderr, format, args );
-        fputc( '\n', stderr );
-        }
-    va_end( args );
+    if (pm_showmessages) {
+        fprintf(stderr, "%s: ", pm_progname);
+        vfprintf(stderr, format, args);
+        fputc('\n', stderr);
     }
+    va_end(args);
+}
 
-void GNU_PRINTF_ATTR
-pm_error(const char format[], ... ) {
+
+
+void PM_GNU_PRINTF_ATTR(1,2)
+pm_error(const char format[], ...) {
     va_list args;
 
-    va_start( args, format );
+    va_start(args, format);
 
-    fprintf( stderr, "%s: ", pm_progname );
-    (void) vfprintf( stderr, format, args );
-    fputc( '\n', stderr );
-    va_end( args );
-    exit( 1 );
-    }
+    fprintf(stderr, "%s: ", pm_progname);
+    vfprintf(stderr, format, args);
+    fputc('\n', stderr);
+    va_end(args);
+
+    pm_longjmp();
+}
 
 
 /* Variable-sized arrays. */
 
-char*
-pm_allocrow(int const cols, int const size) {
+char *
+pm_allocrow(unsigned int const cols,
+            unsigned int const size) {
+
     char * itrow;
 
-    itrow = malloc( cols * size );
-    if ( itrow == NULL )
-        pm_error( "out of memory allocating a row" );
+    if (UINT_MAX / cols < size)
+        pm_error("Arithmetic overflow multiplying %u by %u to get the "
+                 "size of a row to allocate.", cols, size);
+
+    itrow = malloc(cols * size);
+    if (itrow == NULL)
+        pm_error("out of memory allocating a row");
+
     return itrow;
-    }
+}
+
+
 
 void
 pm_freerow(char * const itrow) {
-    free( itrow );
+    free(itrow);
 }
 
 
@@ -139,7 +197,7 @@ pm_allocarray(int const cols, int const rows, int const size )  {
 
    We have two ways of allocating the space: fragmented and
    unfragmented.  In both, the row index (plus the extra element) is
-   in one block of memory.  In the fragemented format, each row is
+   in one block of memory.  In the fragmented format, each row is
    also in an independent memory block, and the extra row pointer is
    NULL.  In the unfragmented format, all the rows are in a single
    block of memory called the row heap and the extra row pointer is
@@ -152,11 +210,11 @@ pm_allocarray(int const cols, int const rows, int const size )  {
     char * rowheap;
 
     MALLOCARRAY(rowIndex, rows + 1);
-    if ( rowIndex == NULL )
+    if (rowIndex == NULL)
         pm_error("out of memory allocating row index (%u rows) for an array",
                  rows);
-    rowheap = malloc( rows * cols * size );
-    if ( rowheap == NULL ) {
+    rowheap = malloc(rows * cols * size);
+    if (rowheap == NULL) {
         /* We couldn't get the whole heap in one block, so try fragmented
            format.
         */
@@ -182,6 +240,8 @@ pm_allocarray(int const cols, int const rows, int const size )  {
     return rowIndex;
 }
 
+
+
 void
 pm_freearray(char ** const rowIndex, 
              int     const rows) {
@@ -282,11 +342,11 @@ pm_bitstomaxval(int const bits) {
 }
 
 
-unsigned int 
-pm_lcm(const unsigned int x, 
-       const unsigned int y, 
-       const unsigned int z, 
-       const unsigned int limit) {
+unsigned int PURE_FN_ATTR
+pm_lcm(unsigned int const x, 
+       unsigned int const y,
+       unsigned int const z,
+       unsigned int const limit) {
 /*----------------------------------------------------------------------------
   Compute the least common multiple of 'x', 'y', and 'z'.  If it's bigger than
   'limit', though, just return 'limit'.
@@ -358,7 +418,7 @@ pm_init(const char * const progname, unsigned int const flags) {
     pm_setMessage(FALSE, NULL);
 
     pm_progname = progname;
-    
+
 #ifdef O_BINARY
 #ifdef HAVE_SETMODE
     /* Set the stdin and stdout mode to binary.  This means nothing on Unix,
@@ -377,7 +437,6 @@ pm_init(const char * const progname, unsigned int const flags) {
     if (!isatty(1)) setmode(1,O_BINARY);  /* Standard Output */
 #endif /* HAVE_SETMODE */
 #endif /* O_BINARY */
-
 }
 
 
@@ -458,8 +517,7 @@ showNetpbmHelp(const char progname[]) {
             char line[80+1];
             fgets(line, sizeof(line), netpbmConfigFile);
             if (line[0] != '#') {
-                int rc;
-                rc = sscanf(line, "docurl=%s", docurl);
+                sscanf(line, "docurl=%s", docurl);
             }
         }
         if (docurl == NULL)
@@ -651,6 +709,171 @@ pm_openw(const char * const name) {
 
 
 
+static const char *
+tmpDir(void) {
+/*----------------------------------------------------------------------------
+   Return the name of the directory in which we should create temporary
+   files.
+
+   The name is a constant in static storage.
+-----------------------------------------------------------------------------*/
+    const char * tmpdir;
+        /* running approximation of the result */
+
+    tmpdir = getenv("TMPDIR");   /* Unix convention */
+
+    if (!tmpdir || strlen(tmpdir) == 0)
+        tmpdir = getenv("TMP");  /* Windows convention */
+
+    if (!tmpdir || strlen(tmpdir) == 0)
+        tmpdir = getenv("TEMP"); /* Windows convention */
+
+    if (!tmpdir || strlen(tmpdir) == 0)
+        tmpdir = TMPDIR;
+
+    return tmpdir;
+}
+
+
+
+static int
+mkstempx(char * const filenameBuffer) {
+/*----------------------------------------------------------------------------
+  This is meant to be equivalent to POSIX mkstemp().
+
+  On some old systems, mktemp() is a security hazard that allows a hacker
+  to read or write our temporary file or cause us to read or write some
+  unintended file.  On other systems, mkstemp() does not exist.
+
+  A Windows/mingw environment is one which doesn't have mkstemp()
+  (2006.06.15).
+
+  We assume that if a system doesn't have mkstemp() that its mktemp()
+  is safe, or that the total situation is such that the problems of
+  mktemp() are not a problem for the user.
+-----------------------------------------------------------------------------*/
+    int retval;
+    int fd;
+    unsigned int attempts;
+    bool gotFile;
+    bool error;
+
+    for (attempts = 0, gotFile = FALSE, error = FALSE;
+         !gotFile && !error && attempts < 100;
+         ++attempts) {
+
+        char * rc;
+        rc = mktemp(filenameBuffer);
+
+        if (rc == NULL)
+            error = TRUE;
+        else {
+            int rc;
+
+            rc = open(filenameBuffer, O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
+
+            if (rc == 0) {
+                fd = rc;
+                gotFile = TRUE;
+            } else {
+                if (errno == EEXIST) {
+                    /* We'll just have to keep trying */
+                } else 
+                    error = TRUE;
+            }
+        }
+    }    
+    if (gotFile)
+        retval = fd;
+    else
+        retval = -1;
+
+    return retval;
+}
+
+
+
+static int
+mkstemp2(char * const filenameBuffer) {
+
+#if HAVE_MKSTEMP
+    if (0)
+        mkstempx(NULL);  /* defeat compiler unused function warning */
+    return mkstemp(filenameBuffer);
+#else
+    return mkstempx(filenameBuffer);
+#endif
+}
+
+
+
+void
+pm_make_tmpfile(FILE **       const filePP,
+                const char ** const filenameP) {
+
+    int fd;
+    FILE * fileP;
+    const char * filenameTemplate;
+    char * filenameBuffer;  /* malloc'ed */
+    unsigned int fnamelen;
+    const char * tmpdir;
+    const char * dirseparator;
+
+    fnamelen = strlen (pm_progname) + 10; /* "/" + "_XXXXXX\0" */
+
+    tmpdir = tmpDir();
+
+    if (tmpdir[strlen(tmpdir) - 1] == '/')
+        dirseparator = "";
+    else
+        dirseparator = "/";
+    
+    asprintfN(&filenameTemplate, "%s%s%s%s", 
+              tmpdir, dirseparator, pm_progname, "_XXXXXX");
+
+    if (filenameTemplate == NULL)
+        pm_error("Unable to allocate storage for temporary file name");
+
+    filenameBuffer = strdup(filenameTemplate);
+
+    fd = mkstemp2(filenameBuffer);
+
+    if (fd < 0)
+        pm_error("Unable to create temporary file according to name "
+                 "pattern '%s'.  mkstemp() failed with "
+                 "errno %d (%s)", filenameTemplate, errno, strerror(errno));
+    else {
+        fileP = fdopen(fd, "w+b");
+
+        if (fileP == NULL)
+            pm_error("Unable to create temporary file.  fdopen() failed "
+                     "with errno %d (%s)", errno, strerror(errno));
+    }
+    strfree(filenameTemplate);
+
+    *filenameP = filenameBuffer;
+    *filePP = fileP;
+}
+
+
+
+FILE * 
+pm_tmpfile(void) {
+
+    FILE * fileP;
+    const char * tmpfile;
+
+    pm_make_tmpfile(&fileP, &tmpfile);
+
+    unlink(tmpfile);
+
+    strfree(tmpfile);
+
+    return fileP;
+}
+
+
+
 FILE *
 pm_openr_seekable(const char name[]) {
 /*----------------------------------------------------------------------------
@@ -662,7 +885,7 @@ pm_openr_seekable(const char name[]) {
   and return that file's stream descriptor.
 
   We use a file that the operating system recognizes as temporary, so
-  it picks the filename and deletes the file when we close it.
+  it picks the filename and deletes the file when Caller closes it.
 -----------------------------------------------------------------------------*/
     int stat_rc;
     int seekable;  /* logical: file is seekable */
@@ -693,8 +916,8 @@ pm_openr_seekable(const char name[]) {
     if (seekable) {
         seekable_file = original_file;
     } else {
-        seekable_file = tmpfile();
-
+        seekable_file = pm_tmpfile();
+        
         /* Copy the input into the temporary seekable file */
         while (!feof(original_file) && !ferror(original_file) 
                && !ferror(seekable_file)) {
@@ -756,107 +979,218 @@ pm_closew(FILE * const f) {
 
 
 /* Endian I/O.
+
+   Before Netpbm 10.27 (March 2005), these would return failure on EOF
+   or I/O failure.  For backward compatibility, they still have the return
+   code, but it is always zero and the routines abort the program in case
+   of EOF or I/O failure.  A program that wants to handle failure differently
+   must use lower level (C library) interfaces.  But that level of detail
+   is uncharacteristic of a Netpbm program; the ease of programming that
+   comes with not checking a return code is more Netpbm.
+
+   It is also for historical reasons that these return signed values,
+   when clearly unsigned would make more sense.
 */
 
-int
-pm_readbigshort(FILE * const in, short * const sP) {
+
+
+static void
+abortWithReadError(FILE * const ifP) {
+
+    if (feof(ifP))
+        pm_error("Unexpected end of input file");
+    else
+        pm_error("Error (not EOF) reading file.");
+}
+
+
+
+void
+pm_readchar(FILE * const ifP,
+            char * const cP) {
+    
     int c;
 
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *sP = ( c & 0xff ) << 8;
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *sP |= c & 0xff;
-    return 0;
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+
+    *cP = c;
 }
 
-int
-pm_writebigshort(FILE* const out, short const s) {
-    (void) putc( ( s >> 8 ) & 0xff, out );
-    (void) putc( s & 0xff, out );
-    return 0;
+
+
+void
+pm_writechar(FILE * const ofP,
+             char   const c) {
+
+    putc(c, ofP);
 }
 
+
+
 int
-pm_readbiglong(FILE * const in, long * const lP) {
+pm_readbigshort(FILE *  const ifP, 
+                short * const sP) {
     int c;
 
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *lP = ( c & 0xff ) << 24;
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *lP |= ( c & 0xff ) << 16;
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *lP |= ( c & 0xff ) << 8;
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *lP |= c & 0xff;
+    unsigned short s;
+
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    s = (c & 0xff) << 8;
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    s |= c & 0xff;
+
+    *sP = s;
+
     return 0;
 }
 
+
+
 int
-pm_writebiglong(FILE * const out, long const l) {
-    (void) putc( ( l >> 24 ) & 0xff, out );
-    (void) putc( ( l >> 16 ) & 0xff, out );
-    (void) putc( ( l >> 8 ) & 0xff, out );
-    (void) putc( l & 0xff, out );
+pm_writebigshort(FILE * const ofP, 
+                 short  const s) {
+
+    putc((s >> 8) & 0xff, ofP);
+    putc(s & 0xff, ofP);
+
     return 0;
 }
 
+
+
 int
-pm_readlittleshort(FILE * const in, short * const sP) {
+pm_readbiglong(FILE * const ifP, 
+               long * const lP) {
+
     int c;
+    unsigned long l;
+
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    l = c << 24;
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    l |= c << 16;
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    l |= c << 8;
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    l |= c;
+
+    *lP = l;
 
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *sP = c & 0xff;
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *sP |= ( c & 0xff ) << 8;
     return 0;
 }
 
+
+
 int
-pm_writelittleshort(FILE* const out, short const s) {
-    (void) putc( s & 0xff, out );
-    (void) putc( ( s >> 8 ) & 0xff, out );
+pm_writebiglong(FILE * const ofP, 
+                long   const l) {
+
+    putc((l >> 24) & 0xff, ofP);
+    putc((l >> 16) & 0xff, ofP);
+    putc((l >>  8) & 0xff, ofP);
+    putc((l >>  0) & 0xff, ofP);
+
     return 0;
 }
 
+
+
 int
-pm_readlittlelong(FILE * const in, long * const lP) {
+pm_readlittleshort(FILE *  const ifP, 
+                   short * const sP) {
     int c;
+    unsigned short s;
+
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    s = c & 0xff;
+
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    s |= (c & 0xff) << 8;
+
+    *sP = s;
 
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *lP = c & 0xff;
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *lP |= ( c & 0xff ) << 8;
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *lP |= ( c & 0xff ) << 16;
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *lP |= ( c & 0xff ) << 24;
     return 0;
 }
 
+
+
 int
-pm_writelittlelong(FILE * const out, long const l) {
-    (void) putc( l & 0xff, out );
-    (void) putc( ( l >> 8 ) & 0xff, out );
-    (void) putc( ( l >> 16 ) & 0xff, out );
-    (void) putc( ( l >> 24 ) & 0xff, out );
+pm_writelittleshort(FILE * const ofP, 
+                    short  const s) {
+
+    putc((s >> 0) & 0xff, ofP);
+    putc((s >> 8) & 0xff, ofP);
+
     return 0;
 }
 
 
+
+int
+pm_readlittlelong(FILE * const ifP, 
+                  long * const lP) {
+    int c;
+    unsigned long l;
+
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    l = c;
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    l |= c << 8;
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    l |= c << 16;
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    l |= c << 24;
+
+    *lP = l;
+
+    return 0;
+}
+
+
+
+int
+pm_writelittlelong(FILE * const ofP, 
+                   long   const l) {
+
+    putc((l >>  0) & 0xff, ofP);
+    putc((l >>  8) & 0xff, ofP);
+    putc((l >> 16) & 0xff, ofP);
+    putc((l >> 24) & 0xff, ofP);
+
+    return 0;
+}
+
+
+
 int 
 pm_readmagicnumber(FILE * const ifP) {
+
     int ich1, ich2;
 
     ich1 = getc(ifP);
@@ -869,6 +1203,8 @@ pm_readmagicnumber(FILE * const ifP) {
     return ich1 * 256 + ich2;
 }
 
+
+
 /* Read a file of unknown size to a buffer. Return the number of bytes
    read. Allocate more memory as we need it. The calling routine has
    to free() the buffer.
@@ -880,38 +1216,83 @@ pm_readmagicnumber(FILE * const ifP) {
 #define PM_MAX_BUF_INC 65536   /* Don't allocate more memory in larger blocks
                                    than this. */
 
-char *pm_read_unknown_size(FILE * const file, long * const nread) {
+char *
+pm_read_unknown_size(FILE * const file, 
+                     long * const nread) {
     long nalloc;
-    register int val;
-    char* buf;
+    char * buf;
+    bool eof;
 
     *nread = 0;
-    if ((buf=malloc(PM_BUF_SIZE)) == NULL)
-        pm_error("Cannot allocate memory");
     nalloc = PM_BUF_SIZE;
+    MALLOCARRAY(buf, nalloc);
+
+    eof = FALSE;  /* initial value */
+
+    while(!eof) {
+        int val;
 
-    while(1) {
         if (*nread >= nalloc) { /* We need a larger buffer */
             if (nalloc > PM_MAX_BUF_INC)
                 nalloc += PM_MAX_BUF_INC;
             else
                 nalloc += nalloc;
-            if ((buf=realloc(buf, nalloc)) == NULL)
-                pm_error("Cannot allocate %ld bytes of memory", nalloc);
+            REALLOCARRAY_NOFAIL(buf, nalloc);
         }
 
-        if ((val = getc(file)) == EOF)
-            return (buf);
-
-        buf[(*nread)++] = val;
+        val = getc(file);
+        if (val == EOF)
+            eof = TRUE;
+        else 
+            buf[(*nread)++] = val;
     }
+    return buf;
+}
+
+
+
+union cheat {
+    uint32n l;
+    short s;
+    unsigned char c[4];
+};
+
+
+
+short
+pm_bs_short(short const s) {
+    union cheat u;
+    unsigned char t;
+
+    u.s = s;
+    t = u.c[0];
+    u.c[0] = u.c[1];
+    u.c[1] = t;
+    return u.s;
+}
+
+
+
+long
+pm_bs_long(long const l) {
+    union cheat u;
+    unsigned char t;
+
+    u.l = l;
+    t = u.c[0];
+    u.c[0] = u.c[3];
+    u.c[3] = t;
+    t = u.c[1];
+    u.c[1] = u.c[2];
+    u.c[2] = t;
+    return u.l;
 }
 
 
 
 void
 pm_tell2(FILE *       const fileP, 
-         pm_filepos * const fileposP,
+         void *       const fileposP,
          unsigned int const fileposSize) {
 /*----------------------------------------------------------------------------
    Return the current file position as *filePosP, which is a buffer
@@ -928,15 +1309,18 @@ pm_tell2(FILE *       const fileP,
         pm_error("ftello() to get current file position failed.  "
                  "Errno = %s (%d)\n", strerror(errno), errno);
 
-    if (fileposSize == sizeof(pm_filepos))
-        *fileposP = filepos;
-    else if (fileposSize == sizeof(long)) {
+    if (fileposSize == sizeof(pm_filepos)) {
+        pm_filepos * const fileposP_filepos = fileposP;
+        *fileposP_filepos = filepos;
+    } else if (fileposSize == sizeof(long)) {
         if (sizeof(pm_filepos) > sizeof(long) &&
             filepos >= (pm_filepos) 1 << (sizeof(long)*8))
             pm_error("File size is too large to represent in the %u bytes "
                      "that were provided to pm_tell2()", fileposSize);
-        else 
-            *(long*)fileposP = (long)filepos;
+        else {
+            long * const fileposP_long = fileposP;
+            *fileposP_long = (long)filepos;
+        }
     } else
         pm_error("File position size passed to pm_tell() is invalid: %u.  "
                  "Valid sizes are %u and %u", 
@@ -950,7 +1334,7 @@ pm_tell(FILE * const fileP) {
     
     long filepos;
 
-    pm_tell2(fileP, (pm_filepos *)&filepos, sizeof(filepos));
+    pm_tell2(fileP, &filepos, sizeof(filepos));
 
     return filepos;
 }
