@@ -1,4 +1,3 @@
-/* $Id: shhopt.c,v 1.5 1999/09/10 19:19:48 sverrehu Exp $ */
 /*------------------------------------------------------------------------
  |  FILE            shhopt.c
  |
@@ -28,7 +27,9 @@
 #include <ctype.h>
 #include <limits.h>
 #include <errno.h>
+#include <assert.h>
 
+#include "mallocvar.h"
 #include "nstring.h"
 #include "shhopt.h"
 
@@ -62,6 +63,7 @@ optFatalFunc(const char *format, ...)
     va_start(ap, format);
     vfprintf(stderr, format, ap);
     va_end(ap);
+    fprintf(stderr, "\n");
     exit(99);
 }
 
@@ -226,6 +228,7 @@ optNeedsArgument(const optEntry opt)
 	|| opt.type == OPT_LONG
 	|| opt.type == OPT_ULONG
     || opt.type == OPT_FLOAT
+    || opt.type == OPT_NAMELIST
         ;
 }
 
@@ -251,6 +254,119 @@ argvRemove(int *argc, char *argv[], int i)
     --*argc;
 }
 
+
+
+static void
+getToken(const char *  const tokenStart,
+         char          const delimiter,
+         const char ** const tokenP,
+         const char ** const nextP) {
+/*----------------------------------------------------------------------------
+   Find the token starting at 'tokenStart' up to but not including
+   the first 'delimiter' character or end of string.  Return it in newly
+   malloced memory as *tokenP, NUL-terminated.
+
+   Make *nextP point just past the token, i.e. to the delimiter or
+   end of string NUL character.
+
+   Note that if the string is empty, or starts with the delimiter,
+   we return an empty string and *nextP == tokenStart, i.e. *nextP
+   doesn't necessarily advance.
+-----------------------------------------------------------------------------*/
+    char * token;
+    const char * cursor;
+    unsigned int charCount;
+
+    /* Run through the token, counting characters */
+
+    charCount = 0;
+    cursor = tokenStart;
+
+    while (*cursor != delimiter && *cursor != '\0') {
+        if (*cursor == '\\') {
+            ++cursor;
+            if (*cursor == '\0')
+                optFatal("string ends with an escape character (\\)");
+        }
+        ++cursor;
+        ++charCount;
+    }
+    
+    token = malloc(charCount + 1);
+    if (token == NULL)
+        optFatal("Could not allocate %u bytes of memory to parse a string",
+                 charCount + 1);
+
+    /* Go back and do it again, this time copying the characters */
+    charCount = 0;
+    cursor = tokenStart;
+
+    while (*cursor != delimiter && *cursor != '\0') {
+        if (*cursor == '\\')
+            ++cursor;
+
+        assert(*cursor != '\0');
+
+        token[charCount++] = *cursor++;
+    }
+    token[charCount] = '\0';
+
+    *tokenP = token;
+    *nextP = cursor;
+}
+
+
+
+static void
+parseNameList(const char *           const listText,
+              struct optNameValue ** const listP) {
+
+    unsigned int const maxOptionCount = 100;
+
+    const char * cursor;
+    unsigned int optionCount;
+    struct optNameValue * list;
+
+    MALLOCARRAY_NOFAIL(list, maxOptionCount+1);
+
+    cursor = &listText[0];  /* initial value */
+
+    optionCount = 0;  /* initial value */
+
+    while (optionCount < maxOptionCount && *cursor != '\0') {
+        const char * next;
+        struct optNameValue pair;
+
+        getToken(cursor, '=', &pair.name, &next);
+
+        cursor = next;
+
+        if (*cursor == '\0')
+            optFatal("name=value option value ends prematurely.  An equal "
+                     "sign was expected following name '%s'", pair.name);
+
+        assert(*cursor == '=');
+        ++cursor;
+
+        getToken(cursor, ',', &pair.value, &next);
+
+        cursor = next;
+
+        list[optionCount++] = pair;
+
+        if (*cursor != '\0') {
+            assert(*cursor == ',');
+            ++cursor;
+        }
+    }
+    list[optionCount].name  = NULL;
+    list[optionCount].value = NULL;
+
+    *listP = list;
+}
+
+
+
 /*------------------------------------------------------------------------
  |  NAME          optExecute
  |
@@ -271,16 +387,12 @@ optExecute(optEntry  const opt, char *arg, int lng)
 
     switch (opt.type) {
     case OPT_FLAG:
-        if (opt.flags & OPT_CALLFUNC)
-            ((void (*)(void)) opt.arg)();
-        else if (opt.arg)
+        if (opt.arg)
             *((int *) opt.arg) = 1;
         break;
 
     case OPT_STRING:
-        if (opt.flags & OPT_CALLFUNC)
-            ((void (*)(char *)) opt.arg)(arg);
-        else if (opt.arg)
+        if (opt.arg)
             *((char **) opt.arg) = arg;
         break;
 
@@ -294,24 +406,18 @@ optExecute(optEntry  const opt, char *arg, int lng)
                      "'%s'", optString(opt, lng));
         tmp = strtol(arg, &e, 10);
         if (*e)
-            optFatal("invalid number `%s'\n", arg);
+            optFatal("invalid number `%s'", arg);
         if (errno == ERANGE
             || (opt.type == OPT_INT && (tmp > INT_MAX || tmp < INT_MIN)))
-            optFatal("number `%s' to `%s' out of range\n",
+            optFatal("number `%s' to `%s' out of range",
                      arg, optString(opt, lng));
         if (opt.type == OPT_INT) {
-            if (opt.flags & OPT_CALLFUNC)
-                ((void (*)(int)) opt.arg)((int) tmp);
-            else
-                *((int *) opt.arg) = (int) tmp;
+            *((int *) opt.arg) = (int) tmp;
         } else /* OPT_LONG */ {
-            if (opt.flags & OPT_CALLFUNC)
-                ((void (*)(long)) opt.arg)(tmp);
-            else if (opt.arg)
+            if (opt.arg)
                 *((long *) opt.arg) = tmp;
         }
-        break;
-    }
+    } break;
 	
     case OPT_UINT:
     case OPT_ULONG: {
@@ -323,24 +429,19 @@ optExecute(optEntry  const opt, char *arg, int lng)
                      "'%s'", optString(opt, lng));
         tmp = strtoul(arg, &e, 10);
         if (*e)
-            optFatal("invalid number `%s'\n", arg);
+            optFatal("invalid number `%s'", arg);
         if (errno == ERANGE
             || (opt.type == OPT_UINT && tmp > UINT_MAX))
-            optFatal("number `%s' to `%s' out of range\n",
+            optFatal("number `%s' to `%s' out of range",
                      arg, optString(opt, lng));
         if (opt.type == OPT_UINT) {
-            if (opt.flags & OPT_CALLFUNC)
-                ((void (*)(unsigned)) opt.arg)((unsigned) tmp);
-            else if (opt.arg)
-                *((unsigned *) opt.arg) = (unsigned) tmp;
+           if (opt.arg)
+               *((unsigned *) opt.arg) = (unsigned) tmp;
         } else /* OPT_ULONG */ {
-            if (opt.flags & OPT_CALLFUNC)
-                ((void (*)(unsigned long)) opt.arg)(tmp);
-            else if (opt.arg)
+            if (opt.arg)
                 *((unsigned long *) opt.arg) = tmp;
         }
-        break;
-    }
+    } break;
     case OPT_FLOAT: {
         float tmp;
         char *e;
@@ -350,17 +451,22 @@ optExecute(optEntry  const opt, char *arg, int lng)
                      "'%s'", optString(opt, lng));
         tmp = strtod(arg, &e);
         if (*e)
-            optFatal("invalid floating point number `%s'\n", arg);
+            optFatal("invalid floating point number `%s'", arg);
         if (errno == ERANGE)
-            optFatal("floating point number `%s' to `%s' out of range\n",
+            optFatal("floating point number `%s' to `%s' out of range",
                      arg, optString(opt, lng));
-        if (opt.flags & OPT_CALLFUNC)
-            ((void (*)(float)) opt.arg)(tmp);
-        else if (opt.arg)
+        if (opt.arg)
             *((float *) opt.arg) = tmp;
-        break;
-    }
-    
+    } break;
+    case OPT_NAMELIST: {
+        if (arg == NULL)
+            optFatal("internal error: optExecute() called with NULL argument "
+                     "'%s'", optString(opt, lng));
+
+        if (opt.arg)
+            parseNameList(arg, (struct optNameValue **)opt.arg);
+
+    } break;
     default:
         break;
     }
@@ -436,95 +542,98 @@ optParseOptions(int *argc, char *argv[], optStruct opt[], int allowNegNum)
     optEntry *opt_table;  /* malloc'ed array */
 
     opt_table = optStructTblToEntryTbl(opt);
+    if (opt_table == NULL)
+        optFatal("Memory allocation failed (trying to allocate space for "
+                 "new-format option table)");
 
     /*
      *  Loop through all arguments.
      */
     for (ai = 0; ai < *argc; ) {
-	/*
-	 *  "--" indicates that the rest of the argv-array does not
-	 *  contain options.
-	 */
-	if (strcmp(argv[ai], "--") == 0) {
+        /*
+         *  "--" indicates that the rest of the argv-array does not
+         *  contain options.
+         */
+        if (strcmp(argv[ai], "--") == 0) {
             argvRemove(argc, argv, ai);
-	    break;
-	}
+            break;
+        }
 
-	if (allowNegNum && argv[ai][0] == '-' && ISDIGIT(argv[ai][1])) {
-	    ++ai;
-	    continue;
-	} else if (strncmp(argv[ai], "--", 2) == 0) {
-	    /* long option */
-	    /* find matching option */
-	    if ((mi = optMatch(opt_table, argv[ai] + 2, 1)) < 0)
-		optFatal("unrecognized option `%s'\n", argv[ai]);
+        if (allowNegNum && argv[ai][0] == '-' && ISDIGIT(argv[ai][1])) {
+            ++ai;
+            continue;
+        } else if (strncmp(argv[ai], "--", 2) == 0) {
+            /* long option */
+            /* find matching option */
+            if ((mi = optMatch(opt_table, argv[ai] + 2, 1)) < 0)
+                optFatal("unrecognized option `%s'", argv[ai]);
 
-	    /* possibly locate the argument to this option. */
-	    arg = NULL;
-	    if ((p = strchr(argv[ai], '=')) != NULL)
-		arg = p + 1;
+            /* possibly locate the argument to this option. */
+            arg = NULL;
+            if ((p = strchr(argv[ai], '=')) != NULL)
+                arg = p + 1;
 	    
-	    /* does this option take an argument? */
-	    optarg = -1;
-	    if (optNeedsArgument(opt_table[mi])) {
-		/* option needs an argument. find it. */
-		if (!arg) {
-		    if ((optarg = ai + 1) == *argc)
-			optFatal("option `%s' requires an argument\n",
-				 optString(opt_table[mi], 1));
-		    arg = argv[optarg];
-		}
-	    } else {
-		if (arg)
-		    optFatal("option `%s' doesn't allow an argument\n",
-			     optString(opt_table[mi], 1));
-	    }
-        optExecute(opt_table[mi], arg, 1);
-        /* remove option and any argument from the argv-array. */
-        if (optarg >= 0)
-            argvRemove(argc, argv, ai);
-        argvRemove(argc, argv, ai);
-	} else if (*argv[ai] == '-') {
-	    /* A dash by itself is not considered an option. */
-	    if (argv[ai][1] == '\0') {
-		++ai;
-		continue;
-	    }
-	    /* Short option(s) following */
-	    o = argv[ai] + 1;
-	    done = 0;
-	    optarg = -1;
-	    while (*o && !done) {
-		/* find matching option */
-		if ((mi = optMatch(opt_table, o, 0)) < 0)
-		    optFatal("unrecognized option `-%c'\n", *o);
-
-		/* does this option take an argument? */
-		optarg = -1;
-		arg = NULL;
-		if (optNeedsArgument(opt_table[mi])) {
-		    /* option needs an argument. find it. */
-		    arg = o + 1;
-		    if (!*arg) {
-			if ((optarg = ai + 1) == *argc)
-			    optFatal("option `%s' requires an argument\n",
-				     optString(opt_table[mi], 0));
-			arg = argv[optarg];
-		    }
-		    done = 1;
-		}
-        /* perform the action of this option. */
-        optExecute(opt_table[mi], arg, 0);
-		++o;
-	    }
-	    /* remove option and any argument from the argv-array. */
+            /* does this option take an argument? */
+            optarg = -1;
+            if (optNeedsArgument(opt_table[mi])) {
+                /* option needs an argument. find it. */
+                if (!arg) {
+                    if ((optarg = ai + 1) == *argc)
+                        optFatal("option `%s' requires an argument",
+                                 optString(opt_table[mi], 1));
+                    arg = argv[optarg];
+                }
+            } else {
+                if (arg)
+                    optFatal("option `%s' doesn't allow an argument",
+                             optString(opt_table[mi], 1));
+            }
+            optExecute(opt_table[mi], arg, 1);
+            /* remove option and any argument from the argv-array. */
             if (optarg >= 0)
                 argvRemove(argc, argv, ai);
             argvRemove(argc, argv, ai);
-	} else {
-	    /* a non-option argument */
-	    ++ai;
-	}
+        } else if (*argv[ai] == '-') {
+            /* A dash by itself is not considered an option. */
+            if (argv[ai][1] == '\0') {
+                ++ai;
+                continue;
+            }
+            /* Short option(s) following */
+            o = argv[ai] + 1;
+            done = 0;
+            optarg = -1;
+            while (*o && !done) {
+                /* find matching option */
+                if ((mi = optMatch(opt_table, o, 0)) < 0)
+                    optFatal("unrecognized option `-%c'", *o);
+
+                /* does this option take an argument? */
+                optarg = -1;
+                arg = NULL;
+                if (optNeedsArgument(opt_table[mi])) {
+                    /* option needs an argument. find it. */
+                    arg = o + 1;
+                    if (!*arg) {
+                        if ((optarg = ai + 1) == *argc)
+                            optFatal("option `%s' requires an argument",
+                                     optString(opt_table[mi], 0));
+                        arg = argv[optarg];
+                    }
+                    done = 1;
+                }
+                /* perform the action of this option. */
+                optExecute(opt_table[mi], arg, 0);
+                ++o;
+            }
+            /* remove option and any argument from the argv-array. */
+            if (optarg >= 0)
+                argvRemove(argc, argv, ai);
+            argvRemove(argc, argv, ai);
+        } else {
+            /* a non-option argument */
+            ++ai;
+        }
     }
     free(opt_table);
 }
@@ -551,7 +660,7 @@ parse_short_option_token(char *argv[], const int argc, const int ai,
     while (*o && !processed_arg) {
 		/* find matching option */
 		if ((mi = optMatch(opt_table, o, 0)) < 0)
-		    optFatal("unrecognized option `-%c'\n", *o);
+		    optFatal("unrecognized option `-%c'", *o);
 
 		/* does this option take an argument? */
 		if (optNeedsArgument(opt_table[mi])) {
@@ -559,7 +668,7 @@ parse_short_option_token(char *argv[], const int argc, const int ai,
 		    arg = o + 1;
 		    if (!*arg) {
                 if (ai + 1 >= argc)
-			    optFatal("option `%s' requires an argument\n",
+			    optFatal("option `%s' requires an argument",
 				     optString(opt_table[mi], 0));
 			arg = argv[ai+1];
             (*tokens_consumed_p)++;
@@ -594,7 +703,7 @@ parse_long_option(char *argv[], const int argc, const int ai,
     *tokens_consumed_p = 1;  /* initial assumption */
     /* find matching option */
     if ((mi = optMatch(opt_table, &argv[ai][namepos], 1)) < 0)
-        optFatal("unrecognized option `%s'\n", argv[ai]);
+        optFatal("unrecognized option `%s'", argv[ai]);
             
     /* possibly locate the argument to this option. */
     { 
@@ -611,14 +720,14 @@ parse_long_option(char *argv[], const int argc, const int ai,
             arg = equals_arg;
         else {
             if (ai + 1 == argc)
-                optFatal("option `%s' requires an argument\n",
+                optFatal("option `%s' requires an argument",
                          optString(opt_table[mi], 1));
             arg = argv[ai+1];
             (*tokens_consumed_p)++;
         }
     } else {
         if (equals_arg)
-            optFatal("option `%s' doesn't allow an argument\n",
+            optFatal("option `%s' doesn't allow an argument",
                      optString(opt_table[mi], 1));
         else 
             arg = NULL;
@@ -700,7 +809,7 @@ zero_specified(optEntry opt_table[]) {
    Set all the "number of times specified" return values identified in the
    option table opt_table[] to zero.
 -----------------------------------------------------------------------------*/
-    int i;
+    unsigned int i;
 
     for (i = 0; opt_table[i].type != OPT_END; i++) {
         if (opt_table[i].specified)
@@ -813,3 +922,18 @@ optParseOptions3(int * const argc_p, char *argv[], const optStruct3 opt,
         }
     }
 }
+
+
+
+void
+optDestroyNameValueList(struct optNameValue * const list) {
+
+    unsigned int i;
+
+    for (i = 0; list[i].name; ++i) {
+        strfree(list[i].name);
+        strfree(list[i].value);
+    }
+
+    free(list);
+}
