@@ -25,13 +25,12 @@
 
 #include <math.h>
 
+#include "pm_config.h"
 #include "pam.h"
 #include "pammap.h"
 #include "shhopt.h"
 #include "mallocvar.h"
-
-#define MAXCOLORS 32767u
-#define MAXDEPTH 3
+#include "nstring.h"
 
 enum methodForLargest {LARGE_NORM, LARGE_LUM};
 
@@ -76,7 +75,7 @@ parseCommandLine (int argc, char ** argv,
    Note that the strings we return are stored in the storage that
    was passed to us as the argv array.  We also trash *argv.
 -----------------------------------------------------------------------------*/
-    optEntry *option_def = malloc( 100*sizeof( optEntry ) );
+    optEntry *option_def;
         /* Instructions to optParseOptions3 on how to parse our options.
          */
     optStruct3 opt;
@@ -86,6 +85,8 @@ parseCommandLine (int argc, char ** argv,
     unsigned int spreadbrightness, spreadluminosity;
     unsigned int center, meancolor, meanpixel;
 
+    MALLOCARRAY_NOFAIL(option_def, 100);
+
     option_def_index = 0;   /* incremented by OPTENT3 */
     OPTENT3(0,   "spreadbrightness", OPT_FLAG,   
             NULL,                       &spreadbrightness, 0);
@@ -164,51 +165,34 @@ parseCommandLine (int argc, char ** argv,
         }
     }
 }
-static int __inline__
-compareplane(struct tupleint ** const comparandP, 
-             struct tupleint ** const comparatorP,
-             unsigned int       const plane) {
-    return (*comparandP)->tuple[plane] - (*comparatorP)->tuple[plane];
-}
 
+
+
+#ifndef LITERAL_FN_DEF_MATCH
+static qsort_comparison_fn compareplane;
+#endif
+
+static unsigned int compareplanePlane;
+    /* This is a parameter to compareplane().  We use this global variable
+       so that compareplane() can be called by qsort(), to compare two
+       tuples.  qsort() doesn't pass any arguments except the two tuples.
+    */
 static int
-compareplane0(const void * const comparandP, const void * const comparatorP) {
-    return compareplane((struct tupleint **)comparandP,
-                        (struct tupleint **)comparatorP,
-                        0);
-}
+compareplane(const void * const arg1, 
+             const void * const arg2) {
 
-static int
-compareplane1(const void * const comparandP, const void * const comparatorP) {
-    return compareplane((struct tupleint **)comparandP,
-                        (struct tupleint **)comparatorP,
-                        1);
-}
+    const struct tupleint * const * const comparandPP  = arg1;
+    const struct tupleint * const * const comparatorPP = arg2;
 
-static int
-compareplane2(const void * const comparandP, const void * const comparatorP) {
-    return compareplane((struct tupleint **)comparandP,
-                        (struct tupleint **)comparatorP,
-                        2);
+    return (*comparandPP)->tuple[compareplanePlane] -
+        (*comparatorPP)->tuple[compareplanePlane];
 }
 
 
 
-typedef int qsort_comparison_fn(const void *, const void *);
-
-/* This ought to be an array of functions, but the compiler complains at
-   that for some reason.
-*/
-static struct {qsort_comparison_fn * fn;} samplecompare[MAXDEPTH];
-
-static void
-initSampleCompare(void) {
-    samplecompare[0].fn = compareplane0;
-    samplecompare[1].fn = compareplane1;
-    samplecompare[2].fn = compareplane2;
-}
-
-
+#ifndef LITERAL_FN_DEF_MATCH
+static qsort_comparison_fn sumcompare;
+#endif
 
 static int
 sumcompare(const void * const b1, const void * const b2) {
@@ -224,23 +208,25 @@ sumcompare(const void * const b1, const void * const b2) {
 ** Display", SIGGRAPH '82 Proceedings, page 297.
 */
 
-static tupletable
+static tupletable2
 newColorMap(unsigned int const newcolors,
             unsigned int const depth) {
 
-    tupletable colormap;
+    tupletable2 colormap;
     unsigned int i;
     struct pam pam;
 
     pam.depth = depth;
 
-    colormap = pnm_alloctupletable(&pam, newcolors);
+    colormap.table = pnm_alloctupletable(&pam, newcolors);
 
     for (i = 0; i < newcolors; ++i) {
         unsigned int plane;
         for (plane = 0; plane < depth; ++plane) 
-            colormap[i]->tuple[plane] = 0;
+            colormap.table[i]->tuple[plane] = 0;
     }
+    colormap.size = newcolors;
+
     return colormap;
 }
 
@@ -265,7 +251,7 @@ newBoxVector(int const colors, int const sum, int const newcolors) {
 
 
 static void
-findBoxBoundaries(tupletable   const colorfreqtable,
+findBoxBoundaries(tupletable2  const colorfreqtable,
                   unsigned int const depth,
                   unsigned int const boxStart,
                   unsigned int const boxSize,
@@ -278,13 +264,15 @@ findBoxBoundaries(tupletable   const colorfreqtable,
     unsigned int plane;
     unsigned int i;
 
-    for (plane = 0; plane < depth; ++plane) 
-        minval[plane] = maxval[plane] = colorfreqtable[boxStart]->tuple[plane];
+    for (plane = 0; plane < depth; ++plane) {
+        minval[plane] = colorfreqtable.table[boxStart]->tuple[plane];
+        maxval[plane] = minval[plane];
+    }
     
     for (i = 1; i < boxSize; ++i) {
         unsigned int plane;
         for (plane = 0; plane < depth; ++plane) {
-            sample const v = colorfreqtable[boxStart + i]->tuple[plane];
+            sample const v = colorfreqtable.table[boxStart + i]->tuple[plane];
             if (v < minval[plane]) minval[plane] = v;
             if (v > maxval[plane]) maxval[plane] = v;
         } 
@@ -351,7 +339,7 @@ largestByLuminosity(sample minval[], sample maxval[],
 static void
 centerBox(int          const boxStart,
           int          const boxSize,
-          tupletable   const colorfreqtable, 
+          tupletable2  const colorfreqtable, 
           unsigned int const depth,
           tuple        const newTuple) {
 
@@ -361,10 +349,10 @@ centerBox(int          const boxStart,
         int minval, maxval;
         unsigned int i;
 
-        minval = maxval = colorfreqtable[boxStart]->tuple[plane];
+        minval = maxval = colorfreqtable.table[boxStart]->tuple[plane];
         
         for (i = 1; i < boxSize; ++i) {
-            int const v = colorfreqtable[boxStart + i]->tuple[plane];
+            int const v = colorfreqtable.table[boxStart + i]->tuple[plane];
             minval = MIN( minval, v);
             maxval = MAX( maxval, v);
         }
@@ -377,7 +365,7 @@ centerBox(int          const boxStart,
 static void
 averageColors(int          const boxStart,
               int          const boxSize,
-              tupletable   const colorfreqtable, 
+              tupletable2  const colorfreqtable, 
               unsigned int const depth,
               tuple        const newTuple) {
 
@@ -390,7 +378,7 @@ averageColors(int          const boxStart,
         sum = 0;
 
         for (i = 0; i < boxSize; ++i) 
-            sum += colorfreqtable[boxStart+i]->tuple[plane];
+            sum += colorfreqtable.table[boxStart+i]->tuple[plane];
 
         newTuple[plane] = sum / boxSize;
     }
@@ -401,7 +389,7 @@ averageColors(int          const boxStart,
 static void
 averagePixels(int          const boxStart,
               int          const boxSize,
-              tupletable   const colorfreqtable, 
+              tupletable2  const colorfreqtable, 
               unsigned int const depth,
               tuple        const newTuple) {
 
@@ -413,7 +401,7 @@ averagePixels(int          const boxStart,
     /* Count the tuples in question */
     n = 0;  /* initial value */
     for (i = 0; i < boxSize; ++i)
-        n += colorfreqtable[boxStart + i]->value;
+        n += colorfreqtable.table[boxStart + i]->value;
 
 
     for (plane = 0; plane < depth; ++plane) {
@@ -423,8 +411,8 @@ averagePixels(int          const boxStart,
         sum = 0;
 
         for (i = 0; i < boxSize; ++i) 
-            sum += colorfreqtable[boxStart+i]->tuple[plane]
-                * colorfreqtable[boxStart+i]->value;
+            sum += colorfreqtable.table[boxStart+i]->tuple[plane]
+                * colorfreqtable.table[boxStart+i]->value;
 
         newTuple[plane] = sum / n;
     }
@@ -432,11 +420,11 @@ averagePixels(int          const boxStart,
 
 
 
-static tupletable
+static tupletable2
 colormapFromBv(unsigned int      const newcolors,
                boxVector         const bv, 
                unsigned int      const boxes,
-               tupletable        const colorfreqtable, 
+               tupletable2       const colorfreqtable, 
                unsigned int      const depth,
                enum methodForRep const methodForRep) {
     /*
@@ -447,7 +435,7 @@ colormapFromBv(unsigned int      const newcolors,
     ** the box - this is the method specified in Heckbert's paper.  A third
     ** method is to average all the pixels in the box. 
     */
-    tupletable colormap;
+    tupletable2 colormap;
     unsigned int bi;
 
     colormap = newColorMap(newcolors, depth);
@@ -456,15 +444,15 @@ colormapFromBv(unsigned int      const newcolors,
         switch (methodForRep) {
         case REP_CENTER_BOX: 
             centerBox(bv[bi].ind, bv[bi].colors, colorfreqtable, depth, 
-                      colormap[bi]->tuple);
+                      colormap.table[bi]->tuple);
             break;
         case REP_AVERAGE_COLORS:
             averageColors(bv[bi].ind, bv[bi].colors, colorfreqtable, depth,
-                          colormap[bi]->tuple);
+                          colormap.table[bi]->tuple);
             break;
         case REP_AVERAGE_PIXELS:
             averagePixels(bv[bi].ind, bv[bi].colors, colorfreqtable, depth,
-                          colormap[bi]->tuple);
+                          colormap.table[bi]->tuple);
             break;
         default:
             pm_error("Internal error: invalid value of methodForRep: %d",
@@ -477,15 +465,15 @@ colormapFromBv(unsigned int      const newcolors,
 
 
 static unsigned int
-freqTotal(tupletable const freqtable, unsigned int const entries) {
+freqTotal(tupletable2 const freqtable) {
     
     unsigned int i;
     unsigned int sum;
 
     sum = 0;
 
-    for (i = 0; i < entries; ++i)
-        sum += freqtable[i]->value;
+    for (i = 0; i < freqtable.size; ++i)
+        sum += freqtable.table[i]->value;
 
     return sum;
 }
@@ -495,7 +483,7 @@ static void
 splitBox(boxVector             const bv, 
          unsigned int *        const boxesP, 
          unsigned int          const bi,
-         tupletable            const colorfreqtable, 
+         tupletable2           const colorfreqtable, 
          unsigned int          const depth,
          enum methodForLargest const methodForLargest) {
 /*----------------------------------------------------------------------------
@@ -508,26 +496,29 @@ splitBox(boxVector             const bv,
    Assume the box contains at least two colors.
 -----------------------------------------------------------------------------*/
     unsigned int const boxStart = bv[bi].ind;
-    unsigned int const boxSize = bv[bi].colors;
-    unsigned int const sm = bv[bi].sum;
+    unsigned int const boxSize  = bv[bi].colors;
+    unsigned int const sm       = bv[bi].sum;
+
+    sample * minval;  /* malloc'ed array */
+    sample * maxval;  /* malloc'ed array */
 
-    sample minval[MAXDEPTH];
-    sample maxval[MAXDEPTH];
     unsigned int largestDimension;
-    /* number of the plane with the largest spread */
+        /* number of the plane with the largest spread */
     unsigned int medianIndex;
     int lowersum;
-    /* Number of pixels whose value is "less than" the median */
+        /* Number of pixels whose value is "less than" the median */
+
+    MALLOCARRAY_NOFAIL(minval, depth);
+    MALLOCARRAY_NOFAIL(maxval, depth);
 
     findBoxBoundaries(colorfreqtable, depth, boxStart, boxSize, 
                       minval, maxval);
 
-    /*
-            ** Find the largest dimension, and sort by that component.  I have
-            ** included two methods for determining the "largest" dimension;
-            ** first by simply comparing the range in RGB space, and second
-            ** by transforming into luminosities before the comparison. 
-            */
+    /* Find the largest dimension, and sort by that component.  I have
+       included two methods for determining the "largest" dimension;
+       first by simply comparing the range in RGB space, and second by
+       transforming into luminosities before the comparison.
+    */
     switch (methodForLargest) {
     case LARGE_NORM: 
         largestDimension = largestByNorm(minval, maxval, depth);
@@ -543,20 +534,23 @@ splitBox(boxVector             const bv,
        represent the final boxes 
     */
 
-    qsort((char*) &colorfreqtable[boxStart], boxSize, 
-          sizeof(colorfreqtable[boxStart]), 
-          samplecompare[largestDimension].fn);
+    /* Set the gross global variable 'compareplanePlane' as a
+       parameter to compareplane(), which is called by qsort().
+    */
+    compareplanePlane = largestDimension;
+    qsort((char*) &colorfreqtable.table[boxStart], boxSize, 
+          sizeof(colorfreqtable.table[boxStart]), 
+          compareplane);
             
     {
-        /* Now find the median based on the counts, so that
-           about half the pixels (not colors, pixels) are in
-           each subdivision. 
-                */
+        /* Now find the median based on the counts, so that about half
+           the pixels (not colors, pixels) are in each subdivision.  */
+
         unsigned int i;
 
-        lowersum = colorfreqtable[boxStart]->value; /* initial value */
+        lowersum = colorfreqtable.table[boxStart]->value; /* initial value */
         for (i = 1; i < boxSize - 1 && lowersum < sm/2; ++i) {
-            lowersum += colorfreqtable[boxStart + i]->value;
+            lowersum += colorfreqtable.table[boxStart + i]->value;
         }
         medianIndex = i;
     }
@@ -569,27 +563,28 @@ splitBox(boxVector             const bv,
     bv[*boxesP].sum = sm - lowersum;
     ++(*boxesP);
     qsort((char*) bv, *boxesP, sizeof(struct box), sumcompare);
+
+    free(minval); free(maxval);
 }
 
 
 
-static tupletable
-mediancut(tupletable            const colorfreqtable, 
-          int                   const colors, 
+static void
+mediancut(tupletable2           const colorfreqtable, 
           unsigned int          const depth,
           int                   const newcolors,
           enum methodForLargest const methodForLargest,
-          enum methodForRep     const methodForRep) {
+          enum methodForRep     const methodForRep,
+          tupletable2 *         const colormapP) {
 /*----------------------------------------------------------------------------
-   Compute a set of only 'newcolors' colors that best represent an image
-   whose pixels are summarized by the histogram 'colorfreqtable'.  There are
-   'colors' colors in that table.  Each tuple in it has depth 'depth'.
-   colorfreqtable[i] tells the number of pixels in the subject image have a
-   particular color.
+   Compute a set of only 'newcolors' colors that best represent an
+   image whose pixels are summarized by the histogram
+   'colorfreqtable'.  Each tuple in that table has depth 'depth'.
+   colorfreqtable.table[i] tells the number of pixels in the subject image
+   have a particular color.
 
    As a side effect, sort 'colorfreqtable'.
 -----------------------------------------------------------------------------*/
-    tupletable colormap;
     boxVector bv;
     unsigned int bi;
     unsigned int boxes;
@@ -598,9 +593,10 @@ mediancut(tupletable            const colorfreqtable,
            there is more splitting we can do.
         */
 
-    bv = newBoxVector(colors, freqTotal(colorfreqtable, colors), newcolors);
+    bv = newBoxVector(colorfreqtable.size, freqTotal(colorfreqtable),
+                      newcolors);
     boxes = 1;
-    multicolorBoxesExist = (colors > 1);
+    multicolorBoxesExist = (colorfreqtable.size > 1);
 
     /* Main loop: split boxes until we have enough. */
     while (boxes < newcolors && multicolorBoxesExist) {
@@ -611,132 +607,187 @@ mediancut(tupletable            const colorfreqtable,
         else 
             splitBox(bv, &boxes, bi, colorfreqtable, depth, methodForLargest);
     }
-    colormap = colormapFromBv(newcolors, bv, boxes, colorfreqtable, depth,
-                              methodForRep);
+    *colormapP = colormapFromBv(newcolors, bv, boxes, colorfreqtable, depth,
+                                methodForRep);
 
     free(bv);
-    return colormap;
 }
 
 
 
 
 static void
-computeHistogram(struct pam *   const pamP, 
-                 tupletable *   const colorfreqtableP, 
-                 unsigned int * const colorsP,
-                 sample *       const freqtableMaxvalP) {
+validateCompatibleImage(struct pam * const inpamP,
+                        struct pam * const firstPamP,
+                        unsigned int const imageSeq) {
+
+    if (inpamP->depth != firstPamP->depth)
+        pm_error("Image %u depth (%u) is not the same as Image 0 (%u)",
+                 imageSeq, inpamP->depth, firstPamP->depth);
+    if (inpamP->maxval != firstPamP->maxval)
+        pm_error("Image %u maxval (%u) is not the same as Image 0 (%u)",
+                 imageSeq,
+                 (unsigned)inpamP->maxval, (unsigned)firstPamP->maxval);
+    if (inpamP->format != firstPamP->format)
+        pm_error("Image %u format (%d) is not the same as Image 0 (%d)",
+                 imageSeq, inpamP->format, firstPamP->format);
+    if (!STREQ(inpamP->tuple_type, firstPamP->tuple_type))
+        pm_error("Image %u tuple type (%s) is not the same as Image 0 (%s)",
+                 imageSeq, inpamP->tuple_type, firstPamP->tuple_type);
+}
+
+
+
+static void
+addImageColorsToHash(struct pam *   const pamP,
+                     tuplehash      const tuplehash,
+                     unsigned int * const colorCountP) {
+
+    tuple * tuplerow;
+    unsigned int row;
+    
+    tuplerow = pnm_allocpamrow(pamP);
+
+    for (row = 0; row < pamP->height; ++row) {
+        unsigned int col;
+
+        pnm_readpamrow(pamP, tuplerow);
+
+        for (col = 0; col < pamP->width; ++col) {
+            bool firstOccurrence;
+
+            pnm_addtuplefreqoccurrence(pamP, tuplerow[col], tuplehash,
+                                       &firstOccurrence);
+
+            if (firstOccurrence)
+                ++(*colorCountP);
+        }
+    }
+    pnm_freepamrow(tuplerow);
+}
+
+
+
+static void
+computeHistogram(FILE *         const ifP,
+                 int *          const formatP,
+                 struct pam *   const freqPamP,
+                 tupletable2 *  const colorfreqtableP) {
 /*----------------------------------------------------------------------------
-  Attempt to make a histogram of the colors in the image represented by
-  *pamP, whose file is seekable and is now positioned to the raster.
+  Make a histogram of the colors in the image stream in the file '*ifP'.
+  
+  Return as *freqPamP a description of the tuple values in the histogram.
+  Only the fields of *freqPamP that describe individual tuples are
+  meaningful (depth, maxval, tuple type);
 
-  If at first we don't succeed, rescale the color resolution to half
-  the present maxval and try again, repeatedly.  This will eventually
-  terminate, with maxval at worst 16, which is half of 32, and 32^3 is
-  just slightly larger than MAXCOLORS.
+  As a fringe benefit, also return the format of the input file as
+  *formatP.
 ----------------------------------------------------------------------------*/
-    tupletable colorfreqtable;
-    sample maxval;
-    pm_filepos rasterPos;
-
+    unsigned int imageSeq;
+    struct pam firstPam;
+    tuplehash tuplehash;
+    unsigned int colorCount;
+    bool eof;
+    
     pm_message("making histogram...");
 
-    pm_tell2(pamP->file, &rasterPos, sizeof(rasterPos));
+    tuplehash = pnm_createtuplehash();
+    colorCount = 0;
 
-    colorfreqtable = NULL;  /* initial value */
+    eof = FALSE;
+
+    for (imageSeq = 0; !eof; ++imageSeq) {
+        struct pam inpam;
+        
+        pm_message("Scanning image %u", imageSeq);
+
+        pnm_readpaminit(ifP, &inpam, PAM_STRUCT_SIZE(tuple_type));
+
+        if (imageSeq == 0)
+            firstPam = inpam;
+        else
+            validateCompatibleImage(&inpam, &firstPam, imageSeq);
     
-    for (maxval = pamP->maxval;
-         colorfreqtable == NULL && maxval > 0;
-         maxval /= 2)
-    {
-        pm_seek2(pamP->file, &rasterPos, sizeof(rasterPos));
-        colorfreqtable = 
-            pnm_computetuplefreqtable2(pamP, NULL, MAXCOLORS, maxval, colorsP);
-        if (colorfreqtable)
-            *freqtableMaxvalP = maxval;
+        addImageColorsToHash(&inpam, tuplehash, &colorCount);
+
+        pm_message("%u colors so far", colorCount);
+
+        pnm_nextimage(ifP, &eof);
     }
+    colorfreqtableP->table =
+        pnm_tuplehashtotable(&firstPam, tuplehash, colorCount);
+    colorfreqtableP->size = colorCount;
 
-    /* colorfreqtable is now non-null.  The "maxval > 0" in the for loop 
-       limit above is just for robustness.
-    */
+    pnm_destroytuplehash(tuplehash);
 
-    if (*freqtableMaxvalP != pamP->maxval) 
-        pm_message("Too many colors (more than %u)!  "
-                   "scaled color resolution from maxval %lu "
-                   "to maxval %lu before choosing",
-                   MAXCOLORS, pamP->maxval, *freqtableMaxvalP);
-
-    pm_message("%u colors found", *colorsP);
-    *colorfreqtableP = colorfreqtable;
+    pm_message("%u colors found", colorfreqtableP->size);
+    
+    freqPamP->size   = sizeof(*freqPamP);
+    freqPamP->len    = PAM_STRUCT_SIZE(tuple_type);
+    freqPamP->maxval = firstPam.maxval;
+    freqPamP->bytes_per_sample = pnm_bytespersample(freqPamP->maxval);
+    freqPamP->depth  = firstPam.depth;
+    STRSCPY(freqPamP->tuple_type, firstPam.tuple_type);
+    
+    *formatP = firstPam.format;
 }
 
 
 
-
 static void
-computeColorMapFromInput(struct pam *          const pamP, 
+computeColorMapFromInput(FILE *                const ifP,
                          bool                  const allColors, 
                          int                   const reqColors, 
                          enum methodForLargest const methodForLargest,
                          enum methodForRep     const methodForRep,
-                         tupletable *          const colormapP,
-                         int *                 const colorsP,
-                         sample *              const colormapMaxvalP) {
+                         int *                 const formatP,
+                         struct pam *          const freqPamP,
+                         tupletable2 *         const colormapP) {
 /*----------------------------------------------------------------------------
    Produce a colormap containing the best colors to represent the
-   image represented by *pamP, whose file is seekable and now
-   positioned to the raster.  Figure it out using the median cut
+   image stream in file 'ifP'.  Figure it out using the median cut
    technique.
 
    The colormap will have 'reqcolors' or fewer colors in it, unless
    'allcolors' is true, in which case it will have all the colors that
    are in the input.
 
-   The colormap has the same maxval as the input, unless the input contains
-   more than MAXCOLORS colors.  In that case, we compute the map from a
-   maxval-reduced version of the input.  The colormap has that reduced 
-   maxval.  Note that in this case, 'allColors' does not cause the colormap
-   to have all the colors from the input; rather, it contains all the colors
-   from the input once the input's color resolution is reduced to the new
-   maxval.
+   The colormap has the same maxval as the input.
 
-   Put the colormap in newly allocated storage as a tupletable 
+   Put the colormap in newly allocated storage as a tupletable2 
    and return its address as *colormapP.  Return the number of colors in
    it as *colorsP and its maxval as *colormapMaxvalP.
------------------------------------------------------------------------------*/
-    tupletable colorfreqtable;
-    unsigned int colors;
-    sample colormapMaxval;
 
-    computeHistogram(pamP, &colorfreqtable, &colors, &colormapMaxval);
+   Return the characteristics of the input file as
+   *formatP and *freqPamP.  (This information is not really
+   relevant to our colormap mission; just a fringe benefit).
+-----------------------------------------------------------------------------*/
+    tupletable2 colorfreqtable;
+
+    computeHistogram(ifP, formatP, freqPamP, &colorfreqtable);
     
     if (allColors) {
         *colormapP = colorfreqtable;
-        *colorsP = colors;
     } else {
-        if (colors <= reqColors) {
+        if (colorfreqtable.size <= reqColors) {
             pm_message("Image already has few enough colors (<=%d).  "
                        "Keeping same colors.", reqColors);
             *colormapP = colorfreqtable;
-            *colorsP = colors;
         } else {
             pm_message("choosing %d colors...", reqColors);
-            *colormapP = mediancut(colorfreqtable, colors, pamP->depth, 
-                                   reqColors,
-                                   methodForLargest, methodForRep);
-            *colorsP = reqColors;
-            pnm_freetupletable(pamP, colorfreqtable);
+            mediancut(colorfreqtable, freqPamP->depth,
+                      reqColors, methodForLargest, methodForRep,
+                      colormapP);
+            pnm_freetupletable2(freqPamP, colorfreqtable);
         }
     }
-    *colormapMaxvalP = colormapMaxval;
 }
 
 
 
 static void
-sortColormap(tupletable   const colormap, 
-             sample       const depth, 
-             unsigned int const colormapSize) {
+sortColormap(tupletable2  const colormap, 
+             sample       const depth) {
 /*----------------------------------------------------------------------------
    Sort the colormap in place, in order of ascending Plane 0 value, 
    the Plane 1 value, etc.
@@ -745,11 +796,11 @@ sortColormap(tupletable   const colormap,
 -----------------------------------------------------------------------------*/
     int i;
     
-    pm_message("Sorting %d colors...", colormapSize);
+    pm_message("Sorting %u colors...", colormap.size);
 
-    for (i = 0; i < colormapSize; ++i) {
+    for (i = 0; i < colormap.size; ++i) {
         int j;
-        for (j = i+1; j < colormapSize; ++j) {
+        for (j = i+1; j < colormap.size; ++j) {
             unsigned int plane;
             bool iIsGreater, iIsLess;
 
@@ -757,16 +808,19 @@ sortColormap(tupletable   const colormap,
             for (plane = 0; 
                  plane < depth && !iIsGreater && !iIsLess; 
                  ++plane) {
-                if (colormap[i]->tuple[plane] > colormap[j]->tuple[plane])
+                if (colormap.table[i]->tuple[plane] >
+                    colormap.table[j]->tuple[plane])
                     iIsGreater = TRUE;
-                else if (colormap[i]->tuple[plane] < colormap[j]->tuple[plane])
+                else if (colormap.table[i]->tuple[plane] <
+                         colormap.table[j]->tuple[plane])
                     iIsLess = TRUE;
             }            
             if (iIsGreater) {
                 for (plane = 0; plane < depth; ++plane) {
-                    sample const temp = colormap[i]->tuple[plane];
-                    colormap[i]->tuple[plane] = colormap[j]->tuple[plane];
-                    colormap[j]->tuple[plane] = temp;
+                    sample const temp = colormap.table[i]->tuple[plane];
+                    colormap.table[i]->tuple[plane] =
+                        colormap.table[j]->tuple[plane];
+                    colormap.table[j]->tuple[plane] = temp;
                 }
             }
         }    
@@ -777,19 +831,18 @@ sortColormap(tupletable   const colormap,
 
 static void 
 colormapToSquare(struct pam * const pamP,
-                 tupletable   const colormap,
-                 unsigned int const colormapSize, 
+                 tupletable2  const colormap,
                  tuple ***    const outputRasterP) {
     {
-        unsigned int const intsqrt = (int)sqrt((float)colormapSize);
-        if (intsqrt * intsqrt == colormapSize) 
+        unsigned int const intsqrt = (int)sqrt((float)colormap.size);
+        if (intsqrt * intsqrt == colormap.size) 
             pamP->width = intsqrt;
         else 
             pamP->width = intsqrt + 1;
     }
     {
-        unsigned int const intQuotient = colormapSize / pamP->width;
-        if (pamP->width * intQuotient == colormapSize)
+        unsigned int const intQuotient = colormap.size / pamP->width;
+        if (pamP->width * intQuotient == colormap.size)
             pamP->height = intQuotient;
         else
             pamP->height = intQuotient + 1;
@@ -809,10 +862,10 @@ colormapToSquare(struct pam * const pamP,
                 unsigned int plane;
                 for (plane = 0; plane < pamP->depth; ++plane) {
                     outputRaster[row][col][plane] = 
-                        colormap[colormapIndex]->tuple[plane];
+                        colormap.table[colormapIndex]->tuple[plane];
                 }
-                if (colormapIndex < colormapSize-1)
-                    colormapIndex++;
+                if (colormapIndex < colormap.size-1)
+                    ++colormapIndex;
             }
         }
         *outputRasterP = outputRaster;
@@ -823,14 +876,13 @@ colormapToSquare(struct pam * const pamP,
 
 static void 
 colormapToSingleRow(struct pam * const pamP,
-                    tupletable   const colormap,
-                    unsigned int const colormapSize, 
+                    tupletable2  const colormap,
                     tuple ***    const outputRasterP) {
 
     tuple ** outputRaster;
     unsigned int col;
     
-    pamP->width = colormapSize;
+    pamP->width = colormap.size;
     pamP->height = 1;
     
     outputRaster = pnm_allocpamarray(pamP);
@@ -838,7 +890,7 @@ colormapToSingleRow(struct pam * const pamP,
     for (col = 0; col < pamP->width; ++col) {
         int plane;
         for (plane = 0; plane < pamP->depth; ++plane)
-            outputRaster[0][col][plane] = colormap[col]->tuple[plane];
+            outputRaster[0][col][plane] = colormap.table[col]->tuple[plane];
     }
     *outputRasterP = outputRaster;
 }
@@ -846,36 +898,35 @@ colormapToSingleRow(struct pam * const pamP,
 
 
 static void
-colormapToImage(struct pam * const inpamP, 
-                struct pam * const outpamP, 
-                tupletable   const colormap,
-                unsigned int const colormapSize, 
-                sample       const colormapMaxval,
-                bool         const sort,
-                bool         const square,
-                tuple ***    const outputRasterP) {
+colormapToImage(int                const format,
+                const struct pam * const colormapPamP,
+                tupletable2        const colormap,
+                bool               const sort,
+                bool               const square,
+                struct pam *       const outpamP, 
+                tuple ***          const outputRasterP) {
 /*----------------------------------------------------------------------------
    Create a tuple array and pam structure for an image which includes
    one pixel of each of the colors in the colormap 'colormap'.
 
    May rearrange the contents of 'colormap'.
 -----------------------------------------------------------------------------*/
-    outpamP->size = sizeof(*outpamP);
-    outpamP->len = sizeof(*outpamP);
-    outpamP->format = inpamP->format;
-    outpamP->plainformat = FALSE;
-    outpamP->depth = inpamP->depth;
-    outpamP->maxval = colormapMaxval;
-    outpamP->bytes_per_sample = inpamP->bytes_per_sample;
-    strcpy(outpamP->tuple_type, inpamP->tuple_type);
+    outpamP->size             = sizeof(*outpamP);
+    outpamP->len              = PAM_STRUCT_SIZE(tuple_type);
+    outpamP->format           = format,
+    outpamP->plainformat      = FALSE;
+    outpamP->depth            = colormapPamP->depth;
+    outpamP->maxval           = colormapPamP->maxval;
+    outpamP->bytes_per_sample = pnm_bytespersample(outpamP->maxval);
+    STRSCPY(outpamP->tuple_type, colormapPamP->tuple_type);
 
     if (sort)
-        sortColormap(colormap, outpamP->depth, colormapSize);
+        sortColormap(colormap, outpamP->depth);
 
     if (square) 
-        colormapToSquare(outpamP, colormap, colormapSize, outputRasterP);
+        colormapToSquare(outpamP, colormap, outputRasterP);
     else 
-        colormapToSingleRow(outpamP, colormap, colormapSize, outputRasterP);
+        colormapToSingleRow(outpamP, colormap, outputRasterP);
 }
 
 
@@ -884,34 +935,29 @@ int
 main(int argc, char * argv[] ) {
 
     struct cmdlineInfo cmdline;
-    FILE* ifP;
-    struct pam inpam, outpam;
-    tuple** colormapRaster;
-    int newcolors;
-    tupletable colormap;
-    sample colormapMaxval;
+    FILE * ifP;
+    int format;
+    struct pam colormapPam;
+    struct pam outpam;
+    tuple ** colormapRaster;
+    tupletable2 colormap;
 
     pnm_init(&argc, argv);
 
-    initSampleCompare();
- 
     parseCommandLine(argc, argv, &cmdline);
 
-    ifP = pm_openr_seekable(cmdline.inputFilespec);
+    ifP = pm_openr(cmdline.inputFilespec);
 
-    pnm_readpaminit(ifP, &inpam, sizeof(inpam));
-    
-    computeColorMapFromInput(&inpam,
+    computeColorMapFromInput(ifP,
                              cmdline.allcolors, cmdline.newcolors, 
                              cmdline.methodForLargest, 
                              cmdline.methodForRep,
-                             &colormap, &newcolors, &colormapMaxval);
+                             &format, &colormapPam, &colormap);
 
     pm_close(ifP);
 
-    colormapToImage(&inpam, &outpam, colormap, newcolors, 
-                    colormapMaxval, cmdline.sort,
-                    cmdline.square, &colormapRaster);
+    colormapToImage(format, &colormapPam, colormap,
+                    cmdline.sort, cmdline.square, &outpam, &colormapRaster);
 
     if (cmdline.verbose)
         pm_message("Generating %u x %u image", outpam.width, outpam.height);
@@ -920,12 +966,11 @@ main(int argc, char * argv[] ) {
     
     pnm_writepam(&outpam, colormapRaster);
     
+    pnm_freetupletable2(&colormapPam, colormap);
+
     pnm_freepamarray(colormapRaster, &outpam);
 
     pm_close(stdout);
 
-    exit(0);
+    return 0;
 }
-
-
-
